/**
 * @summary
 * 		这个是用于获取B端数据的Group 可能 存在使用C端的Group
 */
define([
	"dojo/_base/declare",
	"dojo/Deferred",
	"dojo/_base/lang",
    "cq/util/share/share",
    "cq/util/config/config",
    "bob/shop/shop",
	"../BoBase"
], function (declare, Deferred, lang, share, config, shop, BoBase) {
	var GroupB = declare('', [BoBase.B], {
		modalName: 'mat/group',
		// 可处理字段
		fields: {
			logo: {
				type: 'image',
				label: '品类图片',
				className: 'large-width',
                sizeType: 'cut',
				size: {
					width: 56,
					height: 56
				}
			},
			code: {
				type: 'text',
				label: '编码',
				placeHolder: '必填',
				notNull: true
			},
			gName: {
				type: 'text',
				label: '品类名称',
				placeHolder: '必填',
				notNull: true
			},
			isUse: {
				type: 'checkbox',
				label: '使用',
                value: true
			},
			onBusinessRegistButton: {
				type: "button",
				label: "保存",
				action: "save",
				buttonClass: "fullButton"
			},
			groupListActions: {
				type: "text",
				label: "",
				formatter: function (action, data) {
					var v = "<div class='button' cq-action='detail'>详情</div>" +
						"<div class='button' cq-action='edit'>编辑</div>";
					return v;
				},
				className: "middle-width"
			},
            share: {
				type: "button",
				label: "复制链接",
				action: "share",
				buttonClass: "fullButton"
			}
		},
		getListUrl: function () {
            // summary:
            //        加载品类的URL
            // view:
            //         mat  --商品信息录入和修改页面
            //         groupList    --品类信息列表页面
            //         group    --品类信息录入和修改页面
            // description:
            //         mat 是在列表显示的ref中使用的
			return this.getBaseUrl() + '/load';
		},
		getGroupUrl: function () {
            // summary:
            //        加载品类的URL
            // view:
            //         groupList    --品类信息列表页面
            // description:
            //      用于ref之类不能使用getListParam获得查询参数的使用
			return this.getBaseUrl() + '/load?shopId=' + this.getShopId();
		},
		getListParam: function () {
            // summary:
            //        返回路由的查询参数
            // view:
            //         mat  --商品信息录入和修改页面
            // description:
            //
			return {
				shopId: this.getShopId(),
				orderBy: "createAt"
			};
		},
		getEdtLayout: function () {
            // summary:
            //        编辑品类Layout
            // view:
            //         group    --品类信息录入和修改页面
            // description:
            //
			return [{
				type: "form",
				fields: this.getFormItems(["logo", "code", "gName", "isUse","share", "onBusinessRegistButton"], true)
			}];
		},
		loadById: function (params) {
            // summary:
            //        加载指定条件的品类
            // view:
            //         group    --品类信息录入和修改页面
            // description:
            //
            // parmas:
            //      groupId
			var def = new Deferred();
            var shopId = this.getShopId();
			if (!(params&&params.cqViewParam&&params.cqViewParam.params)) {
				def.resolve({
                    isUse: true
                });
			}else{
                this.exec('loadById', {
                    _id: params.cqViewParam.params
                }).then(function (data) {
                    if(data && data.data && data.data.ok === 1) {
                        def.resolve(data.data.item);
                    }else {
                        def.reject(data.message);
                    }
                });
            }
			return def;
		},
        share: function (data, widget) {
            share.copyUrl({
                url:shop.getCurrentShopLocation()+"#resultMatList-group_" + data._id
            });
        },
		onListClick: function (params) {
            // summary:
            //			列表点击事件
            // view:
            //			groupList   --品类信息列表页面
            // description:
            //
            // parmas:
            //			groupId
			window.moveToView("group", params._id);
		},
		onDataReady: function (widget) {
            // summary:
            //			数据加载完成后 页面处理方法
            // view:
            //			group   --品类信息录入和修改页面
            // description:
            //
            // parmas:
            //			groupId
            widget.hideByFields(["share"]);
            var data = widget.getData();
            if(data._id){
                widget.showByFields(["share"]);
            }
			/*var def = new Deferred();
            param = param.cqRefInfo;
			if (!param.cqViewParam || (param.cqViewParam && !param.cqViewParam.params)) {
				var data = {
					isUse: true
				};
                param.hideByFields(["share"]);
				def.resolve(data);
				return def;
			}
			var m = param.cqViewParam.params;
			this.loadById(m).then(lang.hitch(this, function (data) {
				// 获取商家的信息
                if(data&&data.data&&data.data.item){
                    data = data.data.item;
                    var images = [];
                    images.push(data.logo);
                    images = images.concat(data.images);
                    data.logo = images;
                    def.resolve(data);
                }else{
                    def.resolve(null);
                }
				
			}));
			return def;*/
		},
		onListDataReady: function () {},
		save: function (data, dataList) {
            // summary:
            //			保存品类
            // view:
            //			group   --品类信息录入和修改页面
            // description:
            //			在group 按钮配置中使用 onBusinessRegistButton
            // parmas:
            //			data , wiget
			var msg = this.getDataValidMsg(data, ['logo', 'code', 'gName']);
			if (msg) {
				alert(msg);
				return;
			}
			if (!data.isUse) {
				data.isUse = false;
			}
            window.busy();
			this.exec('save', data).then(function (matData) {
				if (matData.data && matData.data.ok === 1) {
                    window.free();
					history.back();
				} else {
                    window.free();
					alert("保存失败");
				}
			});
		},
		platGroupListActions: function (data, action, list) {
            // summary:
            //			商品品类点击处理
            // view:
            //			groupList   --品类信息列表页面
            // description:
            //			在网页端使用的接口
            // parmas:
            //			data , action, wiget
			if ("edit" === action) {
				window.moveToView('matGroup', data._id);
			} else if ("detail" === action) {
				window.moveToView('matGroupDetail', data._id);
			}
		},
		// 删除品类
		// 参数o:
		//      groupId
		"delete": function () {},
		getGroupExport: function () {
            // summary:
            //			显示列表信息
            // view:
            //			group   --品类信息录入和修改页面
            // description:
            //
			if (!this.matExport) {
				this.matExport = this.getFormItems(["logo", "code", "gName", "isUse", "groupListActions"]);
			}
			return this.matExport;
		},
		getGroupGridLayout: function () {
            // summary:
            //			获得商品品类显示列表
            // view:
            //
            // description:
            //      和getGroupExport 功能一样 没找到哪里使用的 暂时保留
			if (!this.matGridLayout) {
				this.matGridLayout = this.getFormItems(["logo", "code", "gName", "isUse", "groupListActions"]);
			}
			return this.matGridLayout;
		}
	});
	var groupB = new GroupB();
	groupB.Bo = GroupB;
	return groupB;
});