﻿// 创建时间: 2016/12/06
// 创建人: 杜飞军
// 创建内容:
//      获得商品的标准分类
define([
    "dojo/_base/declare",
    "dojo/Deferred",
    "dojo/_base/lang",
    "dojo/_base/array",
    "dojo/topic",
    "../BoBase"
], function (declare, Deferred, lang, array, topic, BoBase) {
    var SortBo = declare("", [BoBase.C], {
        modalName: 'mat/std/mat_std_sort',
        // 可处理的字段
        fields: {
            logo: {
                type: 'image',
                label: '分类图片',
                className: 'large-width',
                sizeType: 'cut',
                size: {
                    width: 128,
                    height: 120
                },
                notNull: true
            },
            code: {
                type: 'text',
                label: '编码',
                placeHolder: '选填'
            },
            sortName: {
                type: 'text',
                label: '分类名称',
                placeHolder: '必填',
                notNull: true
            },
            upSort: {
                type: 'ref',
                label: '上级分类',
                dataProvider: "bob/mat/stdSort",
                targetMethod: "getMatSortUrl",
                labelFields: "sortName",
                searchField: "sortName"
            },
            isUse: {
                type: 'checkbox',
                label: "使用",
                checked: true
            },
            onBusinessRegistButton: {
                type: "button",
                label: "保存",
                action: "save",
                buttonClass: "registerButton",
                className: "fullButton"
            },
            temps:{
                type:"objects",
                label:"商品模板",
                labelField:"remark",
                field:"temps",
                canEdit:true,
                canDelete:true,
                canAdd:true,
                canDnd:false,
                fields:{
                    workMode:"edit",
                    className:"editEditor",
                    layout:[{
                        field:"temp",
                        type:"ref",
                        label: "模板",
                        pageSize : 5,
                        labelFields : "title",
                        dataProvider : "bob/article/pm/richTxtTemp",
                        searchField: 'title',
                        targetMethod : "getShopIdTempUrl",
                        canEdit : true
                    }]
                }
            }
        },
        loadSortInfo: function (param) {
            // summary:
            //        加载分类内容
            // view:
            //        matSort --商品分类信息录入和修改页面
            // description:
            //
            var def = new Deferred();
            if(this.getShopId()){
                param.cqViewParam.shopId = this.getShopId();
            }
            if (!param.cqViewParam.params) {
                def.resolve({
                    isUse: true
                });
                return def;
            }
            var m = param.cqViewParam.params;
            this.loadById(m).then(lang.hitch(this, function (data) {
                // 获取商家的信息
                data = data.data.item;
                if(data.temps){
                    data.temps = array.map(data.temps,function(item){
                        return {temp:item};
                    });
                }
                def.resolve(data);
            }));
            return def;
        },
        loadById: function (data) {
            // summary:
            //        加载分类的内容
            // view:
            //
            // description:
            //        这里是提供给内部使用的方法
            return this.exec('loadById', {
                _id: data
            });
        },
        save: function (data, dataList) {
            // summary:
            //        保存标准分类
            // view:
            //        matSort --商品分类信息录入和修改页面
            // description:
            //
            var msg = this.getDataValidMsg(data, ['logo', 'code', 'sortName']);
            if (msg) {
                alert(msg);
                return;
            }
            var sortData = {
                fromShopId: this.getPlatformShopId()
            };
            sortData.data = data;
            if(sortData.data&&sortData.data.temps){
                var canSave = true;
                for(var i=0;i<sortData.data.temps.length;i++){
                    if(!sortData.data.temps[i].temp._id){
                        canSave = false;
                        alert("第"+(i+1)+"行模板不能为空");
                        break;
                    }
                }
                if(!canSave){
                    return;
                }
                sortData.data.temps = array.map(sortData.data.temps,function(item){
                    if(item.temp&&item.temp._id){
                        return item.temp._id;
                    }
                });
            }
            this.exec("save", sortData).then(function (data) {
                if (data && data.data && data.data.ok === 1) {
                    history.back();
                } else {
                    alert('添加标准分类失败');
                }
            });
        },
        
        getMatSortUrl: function () {
            // summary:
            //        获得标准分类url路径
            // view:
            //        sortList    --商品分类列表页面
            //      sortPicker  --商品分类列表页面 
            // description:
            //      sortPicker    用于主页的侧栏
            return this.getBaseUrl() + "/load?isUse=true&noSub=true&shopId=" + this.getPlatformShopId();
        },
        getSortQuery: function (params) {
            // summary:
            //        获得路由的查询参数
            // view:
            //        sortList    --商品分类列表页面
            //      sortPicker  --商品分类列表页面 
            // description:
            //      sortPicker    用于主页的侧栏
            var query = {
                orderBy: "code"
            };
            if(params.search){
                query.search = params.search;
            }
            return query;
        },
        getItemProp: function (item,itemType,list) {
            // summary:
            //        商品列表默认选择摸个分类的问题,如果存在sortId就选择某一个分类，否则选择全部
            // view:
            //        matList    --商品分类列表页面
            // description:
            //
            //如果没有监听到cqRefInfo.cqViewParam.params的话会异常，也是默认选择全部
            if(!(list&&list.cqRefInfo&&list.cqRefInfo.cqViewParam&&list.cqRefInfo.cqViewParam.params)){
                if (item._id == "0"){
                    item.selected = true;
                }
            }else{
                var sortId = list.cqRefInfo.cqViewParam.params.split("_")[1];
                if(item._id == sortId){
                    item.selected = true;
                }else if (item._id == "0"){
                    item.selected = true;
                }
            }
            return item;
        },
        onListClick: function (params) {
            // summary:
            //        列表点击事件
            // view:
            //        sortList    --商品分类列表页面
            // description:
            //
            window.moveToView("matSort", params._id);
        },
        onSortPickerClick: function (params) {
            // summary:
            //        列表点击事件
            // view:
            //        sortPicker  --商品分类列表页面 
            // description:
            //      sortPicker    用于主页的侧栏
            window.moveToView("matList", "sort_" + params._id);
        },
        getListLayout: function () {
            // summary:
            //        获得页面显示内容
            // view:
            //        matSort --商品分类信息录入和修改页面
            // description:
            //
            return [{
                type: "form",
                fields: this.getFormItems(["logo", "code", "sortName", "isUse", "temps", "onBusinessRegistButton"], true)
            }];
        },
        getSortLabel: function (data) {
            // summary:
            //        获得页面显示内容
            // view:
            //
            // description:
            //        没有找到是哪里使用的
            return {
                label: data.sortName,
                data: data
            };
        },
        showSortInDrawer: function (widget) {
            // summary:
            //        首页分类，以抽屉的形式展示
            // view:
            //        sortPicker  --商品分类列表页面
            // description:
            //      sortPicker    用于主页的侧栏
            require([
                "cq/app/widget/utils/widget",
                "cq/app/widget/utils/drawer"
            ], function (widgetUtil, drawer) {
                drawer.showDawer("sortPicker", widgetUtil.getEnclosingView(widget));
            });
        }
    });
    var sortBo = new SortBo();
    sortBo.Bo = SortBo;
    return sortBo;
});
