﻿define([
    "dojo/_base/declare",
    "dojo/_base/lang",
    "../../BoBase",
    "dojo/Deferred",
    "cq/app/config",
    "cq/util/config/config",
    "cq/util/sys/query",
    "cq/util/sys/date",
],function(declare, lang, BoBase, Deferred, appConfig, config, query, date){
    var CShopBo = declare("",BoBase.B,{
        modalName : 'mk/cs/c_shop_config',
        // 用到的字段
        fields: {
            bgImg : {
                type : 'image',
                label : '背景图片',
                sizeType : "cut",
                canEdit : true,
                size : {
                    width : 640,
                    height : 960
                },
                notNull: true
            },
            newUserAmt : {
                type : 'number',
                label : '下线奖励',
                placeHolder: "未设置",
                fixedTo: 2
            },
            oldUserAmt : {
                type : 'text',
                label : '上线奖励',
                placeHolder: "未设置",
                fixedTo: 2
            },
            exRate : {
                type : 'text',
                label : '兑换比率',
                placeHolder: "未设置",
                fixedTo: 0,
                notNull: true
            },
            reqType : { //1活动奖品，2新客奖品，3注册优惠，4其他
                type : "options",
                label : "获取条件",
                options : {
                    0 : '注册获取',
                    1 : '购买任意商品',
                    2 : '购买足够商品'
                },
                selected: 0,
                notNull: true
            },
            reqAmt : {
                type : 'text',
                label : '累计消费',
                placeHolder: "获取条件为“购买足够商品”时填写此项,累计消费X元才能获取名片"
            },
            rate1 : {
                type : 'number',
                label : '一级提成比例',
                placeHolder: "未设置",
                fixedTo: 4,
                range: "[0,1]"
            },
            rate2 : {
                type : 'number',
                label : '二级提成比例',
                placeHolder: "未设置",
                fixedTo: 4,
                range: "[0,1]"
            },
            rate3 : {
                type : 'number',
                label : '三级提成比例',
                placeHolder: "未设置",
                fixedTo: 4,
                range: "[0,1]"
            },
            isUse : {
                type : 'checkbox',
                label : '启用'
            },
            prizeId: {
                type: 'ref',
                label: '老客送抽奖',
                dataProvider: 'bob/mk/prize/b',
                targetMethod: 'getListUrl',
                searchField: 'search',
                labelFields: 'prizeName',
                notNull: true,
                query: {
                    isUse: true,
                    orderBy: "startTime",
                    state: 2
                }
            },
            discountId: {
                type: 'ref',
                label: '老客优惠券',
                dataProvider: 'bob/mk/discount/b',
                targetMethod: 'getListUrl',
                searchField: 'search',
                labelFields: 'title',
                notNull: true,
                query: {
                    orderBy: "startDate",
                    state: 4
                }
            },
            matRates: {
                type: 'objects',
                label: '例外',
                labelField: 'name',
                rightTextField: 'price',
                canEdit:true,
                canDelete:true,
                canAdd:true,
                canDnd:false,
                validFields: ["rate1","rate2","rate3"],
                fields: {
                    workMode:"edit",
                    className:"editEditor",
                    layout: [{
                        field: 'mId',
                        type: 'ref',
                        label: '例外商品',
                        dataProvider: 'bob/mat/matB',
                        targetMethod: 'getListUrlForCurrentShop',
                        searchField: 'search',
                        labelFields: 'mName',
                        notNull: true,
                        query: {
                            isUse: true
                        }
                    }, {
                        field: 'rate1',
                        type: 'number',
                        label: '一级提成比例',
                        placeHolder: "未设置",
                        fixedTo: 4,
                        range: "[0,1]"
                    }, {
                        field: 'rate2',
                        type: 'number',
                        label: '二级提成比例',
                        placeHolder: "未设置",
                        fixedTo: 4,
                        range: "[0,1]"
                    },{
                        field: 'rate3',
                        type: 'number',
                        label: '三级提成比例',
                        placeHolder: "未设置",
                        fixedTo: 4,
                        range: "[0,1]"
                    }]
                }
            },
            onBusinessRegistButton: {
				type: "button",
				label: "完成",
				action: "save"
			}
        },
        // 添加优惠活动Layout
        getEdtLayout: function(){
            return [{
                type : "form",
                fields : this.getFormItems(["isUse", "bgImg", "newUserAmt", "oldUserAmt", "exRate", "reqType", "reqAmt", "prizeId", "discountId", "rate1", "rate2", "rate3","matRates", "onBusinessRegistButton"], true)
            }];
        },
        // 保存优惠活动
        save: function(data, dataList){
            // 启用时才检查字段可空
            if(data.isUse){
                var msg = this.getDataValidMsg(data, ["bgImg", "exRate", "reqType", "rate1", "rate2", "rate3", "matRates"]);
                if (msg) {
                    alert(msg);
                    return;
                }
                if(data.reqType==2&&(!data.reqAmt||data.reqAmt<0)){
                    alert("累计消费填写不合法");
                    return;
                }
                // 验证商品设置的提成比例不能重复
                var matRates = data.matRates, hasSetted = {}, finded = false;
                for(var i=0;i<matRates.length;i++){
                    var msg = this.getDataValidMsg(matRates[i], ["mId","rate1","rate2", "rate3"]);
                    if(msg){
                        alert("第"+(i+1)+"行"+msg);
                        finded = true;
                        break;
                    }
                    if(hasSetted[matRates[i].mId._id]){
                       alert("第"+hasSetted[matRates[i].mId._id]+"行商品与第"+(i+1)+"行商品设置重复");
                       finded = true;
                       break;
                    }
                    hasSetted[matRates[i].mId._id] = i+1;
                }
                if(finded){
                    return;
                }
            }
            this.exec('save', data).then(function (result) {
				if (result&&result.data && result.data.ok === 1) {
					history.back();
				} else {
					alert(result&&result.data&&result.data.message||"保存失败");
				}
			});
        },
        // 加载店铺分销数据
        // 参数:
        //      _id
        loadByShopId: function (){
            var params = {};
            params.shopId = this.getShopId();
			return this.exec('loadByShopId', params);
        },
        loadOneShopData : function (params) {
            var def = new Deferred();
			this.loadByShopId().then(lang.hitch(this, function (data) {
				// 获取商家的信息
				data = data.data.item||{};
                if(!data._id){
                    data.exRate = 1;
                    data.isUse = true;
                    data.reqType = 0;
                }
				def.resolve(data);
			}));
			return def;
        },
        onDataReady: function(widget){
            var data = widget.getData();
            widget.hideByFields(["reqAmt"]);
            if(data.reqType==2){
                widget.showByFields(["reqAmt"]);
            }
        },
        fieldChangeMethod : function (widget, field, value) {
            if (field && field === "reqType") {
                if (value === "2") {
                    // 当选择购买足够商品时设置累计消费可编辑
                    widget.showByFields(["reqAmt"]);
                } else {
                    widget.hideByFields(["reqAmt"]);
                }
            }
        }
    });
    var cShopBo = new CShopBo();
    cShopBo.Bo = CShopBo;
	return cShopBo;
});
