﻿// 订单系统商家端
define([
    "dojo/_base/declare",
    "dojo/_base/lang",
    "dojo/Deferred",
    "cq/util/config/config",
    '../../BoBase',
    "./_Mixin",
    'cq/util/sys/query',
    "cq/util/sys/date"
], function (declare,lang,Deferred,config,BoBase,_Mixin, query, date) {
    var O = declare("", [BoBase.B, _Mixin], {
        fields: {
            alreadyBegin: {
                type: "button",
                action: "alreadyBegin",
                label: "已开始"
            },
            disabled: {
                type: "button",
                action: "disabled",
                label: "已失效"
            },
            ended: {
                type: "button",
                action: "ended",
                label: "已结束"
            },
            save: {
                type: "button",
                label: "保存",
                action: "save",
                buttonClass: "fullButton"
            },
            disableById: {
                type: "button",
                label: "使失效",
                action: "disableById",
                buttonClass: "fullButton"
            }
        },
        // 停用
        disableById : function (data, widget) {
            this.exec("disableById", {_id : data._id}).then(lang.hitch(this, function (result) {
                if (result && result.data && result.data.ok == 1) {
                    widget.cqRefresh();
                    history.back();
                } else {
                    alert(result&&result.data&&result.data.message||"使失效失败");
                }
            }));
        },
        onDataReady : function (widget) {
            widget.hideByFields(["disableById", "disabled", "alreadyBegin", "ended", "save"]); 
            widget.showByFields(["finalPayment", "ladders"]);
            var data = widget.getData();
            if(!data._id){
                widget.showByFields(["save"]);
                widget.enableByFields();
                return;
            }
            if(data.ladders&&data.ladders.length){
                widget.hideByFields(["finalPayment"]);
            }
            // 如果活动已经失效或已经开始 则不允许保存或失效
            var now = date.nowISOString();
            widget.disableByFields();
            // 活动未失效
            if(data.isUse){
                if(now<data.startTime){
                    // 未开始 显示保存和使失效按钮
                    widget.showByFields(["disableById", "save"]);
                    widget.enableByFields();
                }else if(now>=data.startTime&&now<=data.endTime){
                    // 已开始
                    widget.showByFields(["alreadyBegin"]);
                }else{
                    // 已结束
                    widget.showByFields(["ended"]);
                }
            }
            // 活动已失效
            else{
                widget.showByFields(["disabled"]);
            }
        },
        // 添加优惠活动Layout
        getEdtLayout: function(){
            return [{
                type : "form",
                fields : this.getFormItems(["mId", "startTime", "endTime", "endPayTime", "earnest",
                "finalPayment", "ladders","disableById","alreadyBegin", "disabled", "ended", "save"], true)
            }];
        },
        checkedFiled : function (data) {
            var nowDate = date.now();
            var ladders = data.ladders;
            if (new Date(data.startTime) < nowDate) {
                return "预售开始时间不能小于当前时间"
            }
            if (new Date(data.startTime) > new Date(data.endTime)) {
                return "预售开始时间不能大于预售结束时间"
            }
            if(new Date(data.endPayTime) <= new Date(data.endTime)){
                return "尾款支付结束时间不能小于或等于预售结束时间";
            }
            if (!data.finalPayment && !ladders.length) {
                return "尾款或阶梯必须选填一个"
            }
            if(ladders.length){
                var firstLadder = ladders[0].qtyFrom;
                var lastLadder = ladders[ladders.length-1].qtyTo;
                if(firstLadder!=0){
                    return "阶梯应该从0开始";
                }
                if(lastLadder!=""){
                    return "阶梯应该以空结束";
                }
                for(var i=0;i<ladders.length;i++){
                    if(Number(ladders[i].qtyFrom)<0){
                        return "第"+(i+1)+"个阶梯数量从填写不合法";
                    }
                    if(Number(ladders[i].qtyTo)<0&&i!=ladders.length-1){
                        return "第"+(i+1)+"个阶梯数量到填写不合法";
                    }
                    if(Number(ladders[i].finalPayment)<0){
                        return "第"+(i+1)+"个阶梯尾款填写不合法";
                    }
                    if(i!=ladders.length-1){
                        if(!ladders[i].qtyFrom||!ladders[i].qtyTo||!ladders[i].finalPayment){
                            return "第"+(i+1)+"个阶梯信息填写不完整"
                        }
                    }else{
                        if(!ladders[i].qtyFrom||!ladders[i].finalPayment){
                            return "第"+(i+1)+"个阶梯信息填写不完整"
                        }
                    }
                    if(ladders[i+1]){
                        if(ladders[i+1].qtyFrom-(ladders[i].qtyTo) != 1){
                            return "第"+(i+1)+"个规格数量到与第"+(i+2)+"个规格数量从不连续"
                        }
                    }
                }
            }
            return "";
        },
        // 保存预售
        save: function(data, dataList){
            window.busy();
            var msg = this.getDataValidMsg(data, ["mId", "startTime", "endTime", "endPayTime", "earnest"]);
            if (msg) {
                window.free();
                alert(msg);
                return;
            }
            // 验证合法性
            msg = this.checkedFiled(data);
            if (msg) {
                window.free();
                alert(msg);
                return;
            }
            // 尾款支付开始时间为预售活动结束时间+一分钟
            var endTime = new Date(data.endTime);
            var startPayTime = endTime.getTime()+1000*60;
            startPayTime = new Date(startPayTime);
            startPayTime = startPayTime.toISOString();
            data.startPayTime = startPayTime;
            this.exec('save', data).then(function (result) {
                window.free();
                if (result && result.data && result.data.ok === 1) {
                    alert("保存成功");
                    history.back();
                } else {
                    alert(result&&result.data&&result.data.message || "保存失败");
                }
            });
            /*var _this = this;
            // 检查商品在同一时间是否有限购活动
            require(["bob/mk/limitDiscount/b"], function(limitDiscount){
                limitDiscount.loadConflict({
                    mId: data.mId,
                    sd: data.startTime,
                    ed: data.endTime
                }).then(function(result){
                    if(result&&result.data&&result.data.items.length){
                        window.free();
                        alert("该时间段已有限购活动，请重新设置");
                        return;
                    }
                    
                });
            });
            */
        },
        // 加载预售活动的url
        getListUrl: function(){
            return this.getBaseUrl() + '/load';
        },
        // 加载预售活动的条件
        getListParam: function(cqRefInfo){
            var params = {
                shopId : this.getShopId()
            };
            var paramValue = query.getQueryValue(cqRefInfo.search);
            if (paramValue) {
                params.search = paramValue;
            }
            paramValue = query.getQueryValue(cqRefInfo.state);
            if (paramValue == 1) {
                params.isUse = true;
                params.state = 1;
            } else if (paramValue == 2) {
                params.isUse = true;
                params.state = 2;
            } else if (paramValue == 3) {
                params.isUse = true;
                params.state = 3;
            } else {
                delete params.isUse;
                delete params.state;
            }
            return params;
        },
        // 预售活动列表点击事件
        onListClick : function (item) {
            window.moveToView("preSale", item._id);
        },
        fieldChangeMethod: function (widget, field, fieldVl) {
            if (field == "ladders") {
                if (fieldVl.length) {
                    widget.hideByFields(['finalPayment']);
                } else {
                    widget.showByFields(['finalPayment']);
                }
            }
        },
        // 加载指定预售活动
        // 参数:
        //      _id
        loadById : function (params) {
            if(!params.cqViewParam.params){
                var def = new Deferred();
                def.resolve({
                    isUse: true
                });
                return def;
            }
            return this.exec('loadById', {
                _id: params.cqViewParam.params
            }).then(lang.hitch(this, function (result) {
                return result&&result.data&&result.data.item||{};
            }));
        },
        loadConflict: function(params){
            return this.exec("loadConflict", params);
        }
    });
    
    var o = new O();
    o.Bo = O;
    return o;
});