// B2C店铺：提供B2C服务的商家；
//		B2C服务需要认证后才能开展服务；
//		服务项目：
//      向运营商提交认证信息
//          需要填写的信息：
//			LOGO、联系人contactName、联系电话contactPhone、
//			经营主体名称companyName、法人姓名、法人身份证号、法人身份证正反面、营业执照、地理位置、所在省市区、详细地址

define([
	"dojo/_base/declare",
	"dojo/_base/lang",
    "cq/util/sys/common",
	"./shop"
], function(declare,lang,common,shop) {
    var B2CShopBo = declare("",shop.Bo,{
		modalName:"shop/b2cShop",
        // 可处理的字段
        fields:{
            shopAutApplyButton:{
                type:"button",
                label:"提交认证",
                action:"shopAutApply",
                className:"fullButton"
            },
            shopAutReApplyButton : {
                type:"button",
                label:"重新提交",
                action:"shopAutReApply",
                className:"fullButton"
            },
            shopCertificationThrough:{
                type:"button",
                label:"认证通过",
                action:"verifyPass",
                className:"large-width",
                canEdit: false
            },
            shopAudit : {
                type:"button",
                label:"审核中",
                action:"verify",
                className:"large-width",
                canEdit: false
            }
		},
        //  summary：
        //      获取商家列表
        //  view：
        //      searchShopList
        getSearchListUrl: function(){
           return this.getBaseUrl()+"/searchShop?";
        },

        getEdtLayoutForShopAutApply: function(){
            // summary:
            //      b2c店铺认证Layout
            // view:
            //      shopAut-B2C商家提交认证
            return [{
                type:"form",
                fields:this.getFormItems([
                    "shopCertificationThrough", //认证通过
                    "shopAudit", //审核中
                    "rejectedReason",  // 驳回原因
                    "logo","contactName","contactPhone",
                    "companyName","legalPerson","legalPersonCode",
                    "legalImg1","legalImg2","businessLicense",
                    "lon,lat","provinceId,cityId,countId,town",
                    "countId,townId,villageId,locationName",
                    "address","shopAutApplyButton", //提交认证
                    "shopAutReApplyButton"//重新提交
                ],true)
            }];
        },
        shopAutDataReady: function(widget){
            // summary:
            //      数据加载完成,可在此处处理按钮的显示和隐藏
            // view:
            //      shopAut-B2C商家提交认证            
            //description:
            //      驳回原因 始终不允许编辑
            widget.disableByFields(["rejectedReason"]);
            widget.hideByFields([
                "shopAutApplyButton",
                "rejectedReason",
                "shopAutReApplyButton",
                "shopCertificationThrough",
                "shopAudit"
            ]);
            var data = widget.getData();
            if(!data.isVerified&&0==data.isVerifySubmit){  
                widget.showByFields(["shopAutApplyButton"]);
            }
            if(!data.isVerified&&-1==data.isVerifySubmit){
                widget.showByFields(["shopAutReApplyButton"]);
            //      显示驳回原因
                widget.showByFields(["rejectedReason"]);
            }
            if(!data.isVerified&&1==data.isVerifySubmit){
                widget.showByFields(["shopAudit"]);
            }
            if(data.isVerified&&2==data.isVerifySubmit){
                widget.showByFields(["shopCertificationThrough"]);
            }
            //      若是审核中或者审核通过 要禁止所有字段的编辑
            if((!data.isVerified&&1==data.isVerifySubmit) || 
                (data.isVerified&&2==data.isVerifySubmit)){
                widget.disableByFields([                   
                    "logo","contactName","contactPhone","companyName",
                    "legalPerson","legalPersonCode","legalImg1",
                    "legalImg2","businessLicense","lon,lat",
                    "provinceId,cityId,countId,town",
                    "countId,townId,villageId,locationName","address"
                ]);
            }else{
                widget.enableByFields([                   
                    "logo","contactName","contactPhone","companyName",
                    "legalPerson","legalPersonCode","legalImg1",
                    "legalImg2","businessLicense","lon,lat",
                    "provinceId,cityId,countId,town",
                    "countId,townId,villageId,locationName","address"
                ]);
            }
            var view = widget.getView();
            if(view){
                view.resize();
            }
        },
        shopAutApply : function(data,widget){
            // summary:
            //      B2C店铺认证提交函数 
            // view:
            //      shopAut-B2C商家提交认证
            var msg = this.getDataValidMsg(data, ["logo","contactName","contactPhone",
                    "companyName","legalPerson","legalPersonCode",
                    "legalImg1","legalImg2","businessLicense",
                    "lon", "lat","provinceId,cityId,countId,town",
                    "countId,townId,villageId,locationName","address"]);
            if(msg){
                alert(msg);
                return;
            }
            if (!common.isCellPhone(data.contactPhone)) {
                alert("请输入11位电话号码");
                return;
            }
            //  去除字符串前后空格
            data.contactName = common.deleteBail(data.contactName);
            data.legalPerson = common.deleteBail(data.legalPerson);
            data.companyName = common.deleteBail(data.companyName);
            if(! data.lat || ! data.lon){
                alert("请设置地理位置");
                return;
            }
            if(!data.address){
                alert("详细地址不能为空");
                return;
            }
            //  检查身份证号码合法性
            if(!common.legIdCard(data.legalPersonCode)){
                alert("请输入正确的身份证号码");
                return;
            }
            window.busy();
            var datas = data;
            datas.provinceId = data.provinceId._id||data.provinceId;
            datas.cityId = data.cityId._id||data.cityId;
            datas.countId = data.countId._id||data.countId;
            datas.townId = data.townId._id||data.townId;
            this.exec("verifyShopSubmit", datas).then(lang.hitch(this, function(result){
                window.free();
                if(result&&result.data&&result.data.ok==1){
                    history.back();
                }else{
                    alert(result&&result.data&&result.data.message||"提交失败");
                }
            }));
        },
        shopAutReApply: function(data, widget){
            // summary:
            //      B2C店铺认证重新提交 
            // view:
            //      shopAut-B2C商家提交认证
            this.shopAutApply(data, widget);
        },
        getB2CShopListQuery:function(params){
            // summary:
            //      B2C商家列表的查询条件
            // view:
            //      b2cShopManage-B2C商家管理页面
            //description:
            //      默认加载B2C店铺
            var query = {
				shopType:"1",
				isDelete: false,
				isVerifySubmit:1,
                isVerified: false
			};
            if(params.search){
                query.search = params.search;
            }
            if(params.state&&params.state._id){
                //待认证
                if(1 == params.state._id){
                    query.isVerifySubmit=1;
                    query.isVerified=false;
                }else if(2 == params.state._id){
                    //已认证
                    query.isVerifySubmit=2;
                    query.isVerified=true;
                }else if(3 == params.state._id){
                    //未提交
                    query.isVerifySubmit= 0;
                    query.isVerified=false;
                }else if(4 == params.state._id){
                    //已驳回
                    query.isVerifySubmit=-1;
                    query.isVerified=false;
                }else{
                    delete query.isVerifySubmit;
                    delete query.isVerified;
                }
            }
            return query;
        },
        onB2CShopListClick: function(data){
            // summary:
            //      B2C商家列表点击事件  
            // view:
            //      b2cShopManage-B2C商家管理页面
            // description:
            //      点击跳转到B2C商家详情     
            window.moveToView("b2cMerDetail",data._id);
        },
        //认证帮助
        identification : function(data,widget){
            require([
                "cq/util/edit/confirmFrame2",
                "cq/util/config/config"
            ],lang.hitch(this,function(confirmIFrame, config){
                var url = location.href.split("#")[0]+"#b2cHelpVerification";
                confirmIFrame.show(url,"认证帮助");
            }));
        }
    });
	
	var b2cShopBo = new B2CShopBo();
	b2cShopBo.Bo = B2CShopBo;
    lang.setObject("bob.shop.b2cShop", b2cShopBo);
    return b2cShopBo;
});
