define([
        "dojo/_base/declare",
        "dojo/_base/lang",
        '../BoBase'
    ], function (declare, lang, BoBase) {
    var FreightTemplateBo = declare('', [BoBase.B], {
            modalName: 'shop/freightTemplate',
            fields : {
                typeId: {
                    type: "options",
                    options: {
                        1: "按件数",
                        2: "按重量"
                    },
                    label: "模板类型",
                    notNull: true,
                    selected: 1
                },
                title: {
                    type:"text",
                    label:"标题",
                    className:"large-high",
                    placeHolder : "填写模板标题",
                    notNull: true
                },
                minQty: {
                    type:"number",
                    label:"首件(重)",
                    className:"large-high",
                    placeHolder : "填写首件数量或首重重量(g)",
                    range: "(0,)",
                    notNull: true
                },
                minFee: {
                    type:"number",
                    label:"首费",
                    className:"large-high",
                    placeHolder : "填写首件或首重运送价格",
                    range: "[0,)",
                    notNull: true
                },
                incQty: {
                    type:"number",
                    label:"续件(重)",
                    className:"large-high",
                    placeHolder : "填写续件数量或续重重量(g)",
                    range: "(0,)",
                    notNull: true
                },
                incFee: {
                    type:"number",
                    label:"续费",
                    className:"large-high",
                    placeHolder : "填写续件(重)费用",
                    range: "[0,)",
                    notNull: true
                },
                achieveButton : {
                    type:"button",
                    label:"完成",
                    action:"onSaveFreight",
                    className:"large-width",
                    canEdit: false
                },
                deleteById: {
                    type: "button",
                    label: "删除",
                    action: "deleteById"
                },
                excepts : {
                    type:"text",
                    label:"例外城市运费",
                    formatter:function(action,data){
                        if(data.exceptions){
                            var exceptFreights = "";
                            var freData = data.exceptions; 
                            for(var i = 0;i < freData.length; i++){
                                if(freData[i].provinceId == null){
                                    freData[i].provinceId = {};
                                    freData[i].provinceId.distName = "";
                                }
                                if(freData[i].cityId == null){
                                    freData[i].cityId = {};
                                    freData[i].cityId.distName = "";
                                }
                                if(freData[i].countyId == null){
                                    freData[i].countyId = {};
                                    freData[i].countyId.distName = "";
                                }
                                exceptFreights += "<div>"+freData[i].provinceId.distName+","
                                    +freData[i].cityId.distName+","+freData[i].countyId.distName+":"
                                        +freData[i].minQty+"件以内"+freData[i].minFee+"元;"
                                            +"每增加"+freData[i].incQty+"件,"+"增加运费"+freData[i].incQty+"元"+"</div>"
                            }
                            return exceptFreights;
                        }
                    }
                },
                "exceptions" : {
                    type: 'objects',
                    label: '例外城市运费',
                    canEdit:true,
                    canDelete:true,
                    canAdd:true,
                    canDnd:false,
                    fields: {
                        workMode:"edit",
                        className:"editEditor",
                        layout: [{
                            field : "provinceId,cityId,countyId,remark",
                            type : "district",
                            enableAll:true,
                            label : "省-市-区(县)",
                            className:"large-width",
                            notNull: true
                        },{
                            field : "minQty",
                            type : "number",
                            label : "首件(重)",
                            placeHolder:"填写首件数量或首重重量(g)",
                            className:"middle-width"
                        }, {
                            field : "minFee",
                            type : "number",
                            placeHolder : "填写首件或首重运送价格",
                            label : "首费",
                            className:"middle-width"
                        }, {
                            field : "incQty",
                            type : "number",
                            placeHolder:"填写续件数量或续重重量(g)",
                            label : "续件(重)",
                            className:"middle-width"
                        },{
                            field : "incFee",
                            type : "number",
                            placeHolder : "填写续件(重)费用",
                            label : "续费",
                            className:"middle-width"
                        }]
                    }
                }                                                
            },
            getFreightTemplateListUrl: function () {
                return this.getBaseUrl() + '/load?' + 'shopId='+ this.getShopId() ;
            },
            onFreightTemplateListClick : function(params,node,list){
                if(node == "removeFreight"){
                    return this.removeFreight(params,list);
                }else{    
                    window.moveToView('freightDetail',params._id);
                }
            },
            getFreightDetailLayout : function(){
                return [{
                    type:"form",
                    fields:this.getFormItems([
                        "typeId","title","minQty","minFee","incQty","incFee","exceptions","achieveButton","deleteById"
                    ],true)
                }]
            },
            initFreightData : function(params){
                var _id = params&&params.cqViewParam&&params.cqViewParam.params||"";
                if(!_id){
                    return {};
                }
                return this.exec("loadById", {
                    _id: params.cqViewParam?params.cqViewParam.params:params
                }).then(lang.hitch(this, function(result){
                    return result&&result.data&&result.data.item||{};
                }));
            },
            onSaveFreight : function(data){
                var msg = this.getDataValidMsg(data, ["typeId","title","minQty","minFee","incQty","incFee"]);
                if(msg){
                    alert(msg);
                    return;
                }
                if(data.exceptions&&data.exceptions.length){
                    if(!this.verifyExcep(data.exceptions)){
                        return;
                    }
                }            
                return this.exec("save",data).then(lang.hitch(this,function(result){
                    if(result.data&&result.data.ok==1){
                        alert("保存成功")
                        history.back();
                    }else{
                        alert(result&&result.data&&result.data.message||"保存错误");
                    }
                }));
            },
            onDataReady: function(widget){
                var data = widget.getData();
                widget.hideByFields(["deleteById"]);
                if(!data._id){
                    return;
                }
                widget.showByFields(["deleteById"]);
            },
            // 检测额外的数据合法性
            verifyExcep: function(data) {
                var reg = /^\+?[1-9][0-9]*$/;
                for (var e = 0; e < data.length; e++) {
                    if (!data[e].provinceId) {
                        alert('第'+(e+1)+'个例外城市没有选择省');
                        return false;
                    } else if (data[e].minQty < 1 || !reg.test(data[e].minQty)) {
                        alert('第'+(e+1)+'个例外城市首件数量不能小于1,并且只能为整数');
                        return false;
                    } else if (data[e].minFee < 0 || !(parseFloat(data[e].minFee) == data[e].minFee)) {
                        alert('第'+(e+1)+'个例外城市首件费用不能小于0,并且只能为数字');
                        return false;
                    } else if (data[e].incQty < 1 || !reg.test(data[e].incQty)) {
                        alert('第'+(e+1)+'个例外城市续件数量不能小于1,并且只能为整数');
                        return false;
                    } else if (data[e].incFee < 0 || !(parseFloat(data[e].incFee) == data[e].incFee)) {
                        alert('第'+(e+1)+'个例外城市续件费用不能小于0,并且只能为数字');
                        return false;
                    }
                }
                return true;
            },
            _deleteById: function(_id){
                return this.exec("deleteById", {
                    _id: _id
                });
            },
            removeFreight: function(data,wiget){
                window.busy();
                if(confirm("确定删除?")){
                    this._deleteById(data._id).then(lang.hitch(this,function(result){
                        window.free();
                        if(result&&result.data&&result.data.ok == 1){
                            wiget.cqRefresh();
                        }else{
                            alert(result&&result.data&&result.data.message || "操作失败，请稍后再试！");
                            return;
                        }
                    }));
                }
                else{
                    window.free();
                }
            },
            deleteById: function(data, widget){
                window.busy();
                if(confirm("确定删除?")){
                    this._deleteById(data._id).then(lang.hitch(this,function(result){
                        window.free();
                        if(result&&result.data&&result.data.ok == 1){
                            history.back();
                        }else{
                            alert(result&&result.data&&result.data.message || "操作失败，请稍后再试！");
                            return;
                        }
                    }));
                }
                else{
                    window.free();
                }
            },
            setobjsValue:function(widget,value){
                var dist = value&&((value.cityId&&value.cityId.distName) + (value.countyId&&value.countyId.distName))||"未定义";
                var label = "<div class='title'>"+dist+"</div>"+
                    "<div class='title'>"+"首件 "+value.minQty+" ,首费 ￥"+value.minFee+" ;续件 "+value.incQty+" ,续费 ￥"+value.incFee+"</div>"
                widget.set("label",label);
            }
        });

    var freightTemplateBo = new FreightTemplateBo();
    freightTemplateBo.Bo = FreightTemplateBo;
    return freightTemplateBo;
});
