define([
	"dojo/_base/declare",
	"dojo/_base/lang",
    "dojo/DeferredList",
	"./shop"
], function(declare,lang,DeferredList,shop) {
    var ShopBalanceBo = declare("",shop.Bo,{
        modalName : "ts/pm/shopPayLog",
        fields:{
            pendingAmount: {
                type:"number",
                fixedTo: 2
            },
            availableAmount:{
                type:"number",
                fixedTo: 2,
                label:"可提现余额",
                canEdit : false
            },
            amountSum:{
                type:"number",
                label:"待审核金额",
                fixedTo: 2,
                canEdit : false
            },
            bankName:{
                type:"text",
                label:"开户行",
                canEdit : false,
                notNull: true,
                placeHolder: "收款开户行"
            },
            bankOwner:{
                type:"text",
                label:"账户名称",
                notNull: true,
                canEdit : false,
                placeHolder: "收款账户"
            },
            bankAccount:{
                type:"text",
                label:"银行账号",
                canEdit : false,
                notNull: true,
                placeHolder: "收款账号"
            },
            "fromShopId.shopName":{
                type:"text",
                label:"提现店铺",
                canEdit : false
            },
            "fromShopId.logo":{
                type:"text",
                label:"店铺图标",
                canEdit : false
            },
            "fromShopId.fiName":{
                type:"text",
                label:"联系人",
                canEdit : false
            },
            "fromShopId.fiPhone":{
                type:"text",
                label:"联系电话",
                canEdit : false
            },
            logo:{
                type:"image",
                label:"商家图标",
                clickImg:false,
                canEdit : false
            },
            amount:{
                type:"number",
                fixedTo:2,
                label:"提现金额",
                notNull: true,
                placeHolder: "填写本次提现金额"
            },
            // 确认提现
            toDeposit: {
                type: "button",
                action: "onDeposit",
                label: "确认提现"
            },
            moveToDepositRecord: {
                type: "button",
                action: "moveToDepositRecord",
                label: "提现记录"
            },
            // 审核
            onAccept: {
                type: "button",
                action: "onAccept",
                label: "审核"
            },
            accepted: {
                type: "button",
                action: "accepted",
                label: "已审核"
            },
            createAt: {
                type: "dateTime",
                label: "提交时间"
            },
            moveToTransaction: {
                type: "button",
                action: "moveToTransaction",
                label: "已审核"
            },
            isVerified: {
                type: "options",
                options: {
                    "true": "已审核",
                    "false": "未审核"
                },
                label: "状态"
            },
            receiveDate: {
                type: "dateTime",
                label: "审核时间"
            }
        },
        // 导出数据
        exportData: function(){
            require([
                "cq/util/sys/xlsx"
            ], lang.hitch(this, function(xlsx){
                var url = this.getListUrl();
                var query = this.queryInfo;
                xlsx.exportData(url, query, this.getFormItems(["fromShopId.shopName","amount","bankName","bankOwner","bankAccount","fromShopId.fiName","fromShopId.fiPhone","createAt","isVerified","receiveDate"]), "店铺充值记录"+(new Date()).toISOString());
            }));
        },
        getLayout:function(){
            return [{
                type: "form",
                fields: this.getFormItems([
                    "availableAmount",
                    "amountSum",
                    "amount",
                    "bankName",
                    "bankOwner",
                    "bankAccount",
                    "toDeposit"
                ])
            }];
        },
        _initData:function(shopId){
            var defList = new DeferredList([
                // 获取店铺的可用余额
                shop.loadMe(),
                // 获取店铺已申请提现，但未审核的总金额
                this.loadByType()
            ]);
            return defList;
        },
        initData:function(){
            return this._initData().then(lang.hitch(this,function(result){
                var shopData = result[0][1];
                var data = result[1][1].items;
                var amountSum = 0;
                if(data.length){
                    for(var i = 0; i < data.length; i ++){
                        amountSum += data[i].amount;
                    }
                }
                var item = {};
                item.availableAmount = shopData.availableAmount;
                item.bankName = shopData.bankName;
                item.bankOwner = shopData.bankAccountName;
                item.bankAccount = shopData.bankAccountCode;
                item.amountSum = amountSum;
                return item;
            }));
        },
        loadByType:function(){
            return this.exec("loadByType",{isVerified:false,ioType:1}).then(lang.hitch(this,function(data){
                return data.data;
            }));
        },
        onDataReady:function(widgets){
            widgets.disableByFields(["availableAmount","amountSum"]);
        },
        onDeposit:function(data,widgets){
            // 数据合法性检查
            var msg = this.getDataValidMsg(data, ["amount", "bankName", "bankOwner", "bankAccount"]);
            if(msg){
                alert(msg);
                return false;
            }
            if(Number(data.amount)<=0){
                alert("金额必须大于零");
                return;
            }
            if(Number(data.amount)>Number(data.availableAmount)){
                alert("金额不能大于可用余额");
                return;
            }
            window.busy();
            data.ioType = 1;
            // 保存数据
            this.exec("save",data).then(lang.hitch(this,function(result){
                window.free();
                if(result&&result.data&&result.data.ok==1){
                    //widgets.cqRefresh();
                    // 将账号保存到店铺中 以免下次再重新填写
                    shop.loadMe().then(lang.hitch(this, function(item){
                        if(item&&(!item.bankName||!item.bankAccountName||!item.bankAccountCode)){
                            if(confirm("保存账号信息?")){
                                shop.toSave({
                                    _id: item._id,
                                    bankName: data.bankName,
                                    bankAccountName: data.bankOwner,
                                    bankAccountCode: data.bankAccount
                                });
                            }
                        }
                        alert("提交成功");
                        history.back();
                    }));
                    
                }else{
                    alert(result&&result.data&&result.data.message||"提现失败");
                }
            }));
        },
        moveToDepositRecord:function(){
            window.moveToView("shopBalanceDepositRecord");
        },
        //余额提现管理列表
        getListUrl:function(){
            return this.getBaseUrl()+"/loadByType?shopId="+this.getShopId();
        },
        queryInfo: null,
        getListParam:function(params){
            var query = {
                isVerified:false,
                ioType:1
            };
            if(params.state&&params.state._id == 2){
                query.isVerified = true;
            }else if(params.state&&params.state._id == 1){
                query.isVerified = false;
            }else{
                delete query.isVerified;
            }
            this.queryInfo = query;
            return query;
        },
        onListClick:function(data){
            window.moveToView("shopBalanceDepDetail",data._id);
        },
        //余额提现管理详情
        getDetailLayout:function(){
            return [{
                type: "form",
                fields: this.getFormItems(["logo","fromShopId.shopName","fromShopId.fiName","fromShopId.fiPhone","bankName",
                "bankAccount","bankOwner","amount","accepted","onAccept","moveToTransaction"])
            }];
        },
        getDetailBLayout:function(){
            return [{
                type: "form",
                fields: this.getFormItems(["bankName",
                "bankAccount","bankOwner","amount","receiveDate"])
            }];
        },
        initDetailData:function(cqRefInfo){
            var _id = cqRefInfo.cqViewParam.params;
            // return this.exec("loadById",{
            //     _id:_id
            // }).then(lang.hitch(this,function(data){
            //     var data = data.data.item;
            //     data.logo = data.fromShopId.logo;
            //     return data;
            // }));
            return this.loadById(_id).then(lang.hitch(this,function(data){
                var data = data.data.item;
                data.logo = data.fromShopId.logo;
                // 重新获取一次账号信息 从商家的表中获取
                if(data.fromShopId&&(!data.bankOwner||!data.bankName||!data.bankAccount)){
                    return shop.loadById(data.fromShopId._id).then(lang.hitch(this, function(shopItem){
                        if(!data.bankOwner){
                            data.bankOwner = shopItem.bankAccountName;
                        }
                        if(!data.bankName){
                            data.bankName = shopItem.bankName;
                        }
                        if(!data.bankAccount){
                            data.bankAccount = shopItem.bankAccountCode;
                        }
                        return data;
                    }));
                }else{
                    return data;   
                }
            }));
        },
        loadById:function(_id){
            return this.exec("loadById",{_id:_id});
        },
        onDetailDataReady:function(widgets){
            widgets.disableByFields(["logo","fromShopId.shopName","fromShopId.fiName","fromShopId.fiPhone","bankName","bankAccount","bankOwner","amount","receiveDate"]);
            widgets.hideByFields(["accepted","onAccept"]);
            var item = widgets.getData();
            if(item._id){
                var showField = [];
                showField.push(item.isVerified?"accepted":"onAccept");
                widgets.showByFields(showField);
            }
        },
        moveToTransaction:function(data){
            window.moveToView("transactionList",data.fromShopId._id);
        },
        onAccept:function(data,widgets){
            window.busy();
            this.exec("verify",{_id:data._id}).then(lang.hitch(this,function(result){
                window.free();
                if(result&&result.data&&result.data.ok==1){
                    widgets.cqRefresh();
                }else{
                    alert(result.message||"审核失败");
                }
            }));
        },
        //已审核按钮不做任何操作
        accepted:function(){}
    });
	var shopBalanceBo = new ShopBalanceBo();
	shopBalanceBo.Bo = ShopBalanceBo;
    return shopBalanceBo;
});
