define([
	"dojo/_base/declare",
	"dojo/_base/lang",
    "dojo/DeferredList",
    "./shop"
], function(declare,lang,DeferredList,shop) {
    var ShopBalanceBo = declare("",shop.Bo,{
        modalName : "ts/pm/shopPayLog",
        fields:{
            shopBankName:{
                type:"text",
                label:"收款方开户行",
                canEdit : false,
                notNull:true
            },
            shopAccountName:{
                type:"text",
                label:"收款方账户名",
                canEdit : false,
                notNull:true
            },
            shopAccountCode:{
                type:"text",
                label:"收款方开户账号",
                canEdit : false,
                notNull:true
            },
            bankName:{
                type:"text",
                label:"付款方开户行",
                canEdit : false,
                placeHolder:"例：招商银行 北京 朝阳区支行",
                notNull:true
            },
            bankOwner:{
                type:"text",
                label:"付款方账户名",
                canEdit : false,
                placeHolder:"请填写开户名称",
                notNull:true
            },
            bankAccount:{
                type:"text",
                label:"付款方开户账号",
                canEdit : false,
                placeHolder:"银行卡号",
                notNull:true
            },
            amount:{
                type:"text",
                label:"充值金额",
                placeHolder:"请填写要缴纳的金额",
                notNull:true
            },
            tsCode:{
                type:"text",
                label:"交易流水号",
                placeHolder : "请填写银行的流水单号",
                notNull:true
            },
            remark:{
                type:"text",
                label:"备注信息",
                placeHolder : "请填写备注信息"
            },
            "fromShopId.shopName":{
                type:"text",
                label:"商家名称",
                canEdit : false,
                notNull:true
            },
            "fromShopId.fiName":{
                type:"text",
                label:"联系人",
                canEdit : false,
                notNull:true
            },
            "fromShopId.fiPhone":{
                type:"text",
                label:"联系电话",
                canEdit : false,
                notNull:true
            },
            "fromShopId.logo":{
                type:"text",
                label:"商家图标",
                canEdit : false,
                notNull:true
            },
            logo:{
                type:"image",
                label:"商家图标",
                clickImg:false,
                canEdit : false,
                notNull:true
            },
            // 确认充值
            toSaveStore: {
                type: "button",
                action: "onSaveStore",
                label: "确认充值"
            },
            // 审核
            onAccept: {
                type: "button",
                action: "onAccept",
                label: "审核"
            },
            accepted: {
                type: "button",
                action: "accepted",
                label: "已审核"
            },
            moveToTransaction: {
                type:"button",
                action:"moveToTransaction",
                label:"交易明细"
            },
            lastMod: {
                type: "dateTime",
                label: "审核时间"
            }
        },
        getLayout:function(){
            return [{
                type: "form",
                fields: this.getFormItems([
                    "shopBankName", 
                    "shopAccountName",
                    "shopAccountCode",
                    "bankName",
                    "bankOwner",
                    "bankAccount",
                    "amount",
                    "tsCode",
                    "remark",
                    "toSaveStore"
                ])
            }];
        },
        _initData:function(shopId){
            var defList = new DeferredList([
                shop.loadMe(),
                shop.loadByIdFromPlt(shopId)
            ]);
            return defList;
        },
        initData:function(){
            var platformShopId = this.getPlatformShopId();
            return this._initData(platformShopId).then(lang.hitch(this,function(result){
                var nowShopData = result[0][1];
                var platShopData = result[1][1];
                var data = {}
                data.shopBankName = platShopData.bankName;
                data.shopAccountName = platShopData.bankAccountName;
                data.shopAccountCode = platShopData.bankAccountCode;
                data.bankName = nowShopData.bankName;
                data.bankOwner = nowShopData.bankAccountName;
                data.bankAccount = nowShopData.bankAccountCode;
                return data;
            }));
        },
        onDataReady:function(widgets){
            widgets.disableByFields(["shopBankName", "shopAccountName","shopAccountCode"]);
        },
        onSaveStore:function(data,widgets){
            if(!(data.amount > 0)){
                alert("金额必须大于零");
                return;
            }
            var fields = [];
            //线下转账
            delete data.shopBankName
            delete data.shopAccountName
            delete data.shopAccountCode
            data.ioType = 0;
            for(var k in data){
                if(k != "remark")
                fields.push(k);
            }
            var msg = this.getDataValidMsg(data,fields);
            if(msg){
                alert(msg);
                return;
            }
            window.busy();
            this.exec("save",data).then(lang.hitch(this,function(result){
                window.free();
                if(result&&result.data&&result.data.ok==1){
                    history.back();
                }else{
                    alert(result.message||"充值失败");
                }
            }));
        },
        //余额充值管理列表
        getListUrl:function(){
            return this.getBaseUrl()+"/loadByType?shopId="+this.getShopId();
        },
        getListParam:function(params){
            var query = {isVerified:false,ioType:0};
            if(params.state&&params.state._id == 1){
                query.isVerified = true;
            }
            return query;
        },
        onListClick:function(data){
            window.moveToView("shopBalanceTraDetail",data._id);
        },
        //余额充值管理详情
        getDetailLayout:function(){
            return [{
                type: "form",
                fields: this.getFormItems(["logo","fromShopId.shopName","fromShopId.fiName","fromShopId.fiPhone","bankName","bankAccount","bankOwner","amount","tsCode","remark","accepted","onAccept","moveToTransaction"])
            }];
        },
        getDetailBLayout:function(){
            return [{
                type: "form",
                fields: this.getFormItems(["bankName","bankAccount","bankOwner","amount","tsCode","remark","lastMod"])
            }];
        },
        initDetailData:function(cqRefInfo){
            var id = cqRefInfo.cqViewParam.params;
            return this.exec("loadById",{
                _id:id
            }).then(lang.hitch(this,function(data){
                var data = data.data.item;
                data.logo = data.fromShopId.logo;
                return data;
            }))
        },
        onDetailDataReady:function(widgets){
            widgets.disableByFields(["logo","fromShopId.shopName","fromShopId.fiName","fromShopId.fiPhone","bankName","bankAccount","bankOwner","amount","tsCode","remark", "lastMod"])
            widgets.hideByFields(["accepted","onAccept"])
            var item = widgets.getData();
            if(item._id){
                var showField = [];
                showField.push(item.isVerified?"accepted":"onAccept");
                widgets.showByFields(showField);
            }
        },
        moveToTransaction:function(data){
            window.moveToView("transactionList",data.fromShopId._id);
        },
        onAccept:function(data,widgets){
            window.busy();
            this.exec("verify",{_id:data._id}).then(lang.hitch(this,function(result){
                window.free();
                if(result&&result.data&&result.data.ok==1){
                    widgets.cqRefresh();
                }else{
                    alert(result.message||"审核失败");
                }
            }))
        },
        //已审核按钮不做任何操作
        accepted:function(){}
    });
	var shopBalanceBo = new ShopBalanceBo();
	shopBalanceBo.Bo = ShopBalanceBo;
    return shopBalanceBo;
});
