// 服务商：提供商品或者服务的商家；
//		服务商需要认证后才能开展服务；
//		服务项目：信息类；交易类；集成类；
//		公益服务：生产经营
//      向运营商提交认证信息
//          需要填写的信息：
//			LOGO、联系人contactName、联系电话contactPhone、
//			经营主体名称companyName、法人姓名、法人身份证号、法人身份证正反面、营业执照、组织机构代码、地理位置、所在省市区、详细地址、服务商简介、
//			销售区域

define([
	"dojo/_base/declare",
	"dojo/_base/lang",
	"./shop",
    "whz/util/config"
], function(declare,lang,shop, config) {
    var ShopBo = declare("",shop.Bo,{
        modalName : "shop/serviceShop",
        fields:{
            "tranRegions" : {
                type:"objects",
                label:"服务地域",
                labelField:"remark",
                canEdit:true,
                canDelete:true,
                canAdd:true,
                canDnd:true,
                defaultData:{
                    cityId:null,
                    countyId:null,
                    provinceId:null,
                    remark:""
                },
                fields:{
                    layout:[{
                        field:"provinceId,cityId,countyId,remark",
                        type:"district",
                        enableAll:true,
                        placeHolder:"请选择地址",
                        label:"区划"
                    }]
                },
                className: "large-width"
            },
			"services" : {
                type:"objects",
                label:"服务项目",
                labelField:"serviceType",
                canEdit:true,
                canDelete:true,
                canAdd:true,
                canDnd:true,
                defaultData:{
                    serviceType:null,
                    remark:""
                },
                fields:{
                    layout:[{
                        field:"serviceType",
                        type:"options",
						label:"服务项目",
						options:config.shopService/*{
							"2001":"涉农政策",
							"3001":"水电缴纳"
						}*/
                    },{
                        field:"remark",
                        type:"text",
						label:"备注"
                    }]
                },
                className: "large-width"
            },
            "serviceType": {
                type:"options",
                label:"服务项目",
                options:config.shopService
            },
            "brief": {
                type:"textarea",
                label:"服务商简介",
                className:"large-width",
                placeHolder:"请填写简介"
            },
            sevAutApplyButton:{
                type:"button",
                label:"提交认证",
                action:"sevAutApply",
                className:"fullButton"
            },
            revokeSevAutButton : {
                type:"button",
                label:"撤销认证",
                action:"revokeVerify",
                className:"fullButton"               
            },
            sevAutReApplyButton : {
                type:"button",
                label:"重新提交",
                action:"sevAutReApply",
                className:"fullButton"
            },
            sevCertificationThrough:{
                type:"button",
                label:"认证通过",
                action:"verifyPass",
                className:"large-width",
                canEdit: false
            },
            sevAudit : {
                type:"button",
                label:"审核中",
                action:"verify",
                className:"large-width",
                canEdit: false
            }
		},
        getEdtLayoutForSevAutApply: function(){
            // summary:
            //      服务商店铺认证Layout
            // view:
            //      sevAut——服务商提交认证页面           
            return [{
                type:"form",
                fields:this.getFormItems([
                    "sevCertificationThrough",  //认证通过
                    "sevAudit",  //审核中
                    "rejectedReason",  // 驳回原因
                    "logo","contactName","contactPhone",
                    "companyName","legalPerson","legalPersonCode",
                    "legalImg1","legalImg2","businessLicense",
                    "lon,lat","provinceId,cityId,countId,town",
                    "address","brief","tranRegions",
					"services","sevAutApplyButton",  //提交认证
                    "revokeSevAutButton",  //撤销认证  
                    "sevAutReApplyButton"  //重新提交
                ],true)
            }]
        },
        sevAutDataReady: function(widget){
            // summary:
            //      数据加载完成,可在此处处理按钮的显示和隐藏
            // view:
            //      sevAut——服务商提交认证页面       
            //description:
            //      驳回原因 始终不允许编辑
            widget.disableByFields(["rejectedReason"]);
            widget.hideByFields([
                "sevAutApplyButton",
                "revokeSevAutButton",
                "rejectedReason",
                "sevAutReApplyButton",
                "sevCertificationThrough",
                "sevAudit"]);
            var data = widget.getData();
            if(!data.isVerified&&0==data.isVerifySubmit){
                widget.showByFields(["sevAutApplyButton"]);
            }
            if(!data.isVerified&&-1==data.isVerifySubmit){
                widget.showByFields(["sevAutReApplyButton"]);
                // 显示驳回原因
                widget.showByFields(["rejectedReason"]);
            }
            if(!data.isVerified&&1==data.isVerifySubmit){
                //widget.showByFields(["revokeSevAutButton"]);
                widget.showByFields(["sevAudit"]);
            }
            //  若店铺isVerify为true且isVerifySubmit为2,则显示认证通过
            if(data.isVerified&&2==data.isVerifySubmit){
                widget.showByFields(["sevCertificationThrough"]);
            }
            // 若是审核中或者审核通过 要禁止所有字段的编辑
            if((!data.isVerified&&1==data.isVerifySubmit) || 
                (data.isVerified&&2==data.isVerifySubmit)){
                widget.disableByFields([                   
                    "logo","contactName","contactPhone",
                    "companyName","legalPerson","legalPersonCode",
                    "legalImg1","legalImg2","businessLicense",
                    "lon,lat","provinceId,cityId,countId,town",
                    "address","brief","tranRegions","services"
                ]);
            }else{
                widget.enableByFields([                   
                    "logo","contactName","contactPhone",
                    "companyName","legalPerson","legalPersonCode",
                    "legalImg1","legalImg2","businessLicense",
                    "lon,lat","provinceId,cityId,countId,town",
                    "address","brief","tranRegions","services"
                ]);
            }
        },
        sevAutApply : function(data,widget){
            // summary:
            //      服务商认证提交函数
            // view:
            //      sevAut——服务商提交认证页面   
            var msg = this.getDataValidMsg(data, ["logo","contactName","contactPhone",
                    "companyName","legalPerson","legalPersonCode",
                    "legalImg1","legalImg2","businessLicense",
                    "lon","lat","provinceId,cityId,countId,town",
                    "address","brief","tranRegions","services"]);
            if(msg){
                alert(msg);
                return;
            }
            window.busy();
            var datas = data;
            datas.provinceId = data.provinceId._id|| data.provinceId;
            datas.cityId = data.cityId._id || data.cityId;
            datas.countId = data.countId._id || data.countId;
            delete datas.__v;
            this.exec("verifyShopSubmit", datas).then(lang.hitch(this, function(result){
                window.free();
                if(result&&result.data&&result.data.ok==1){
                    history.back();
                }else{
                    alert("提交失败");
                }
            }));
        },
        sevAutReApply: function(data, widget){
            // summary:
            //      服务商认证重新提交函数
            // view:
            //      sevAut——服务商提交认证页面  
            this.sevAutApply(arguments);
        },
        

        //获取服务（主要用作引用）
        //  创建：周超
        getServList: function(){
            return this.getBaseUrl()+"/getServiceList?shopId="+this.getShopId();
        },
        getItemPropForService: function(data){
            data.serericeName = config.shopService[data.serviceType];
            return data;
        },
        //服务商列表的查询条件 
        getSevListQuery:function(params){
            // summary:
            //      服务商列表的查询条件  
            // view:
            //      sevManage-服务商列表            
            //description:
            //      默认加载服务商     
            var query = {shopType:4};
            if(params.search){
                query.search = params.search;
            }
            if(params.state&&params.state._id){
                //待认证
                if(1 == params.state._id){
                    query.isVerifySubmit=1;
                    query.isVerified=false;
                }else if(2 == params.state._id){
                    //已认证
                    query.isVerifySubmit=2;
                    query.isVerified=true;
                }else if(3 == params.state._id){
                    //未提交
                    query.isVerifySubmit= 0;
                    query.isVerified=false;
                }else if(4 == params.state._id){
                    //已驳回
                    query.isVerifySubmit=-1;
                    query.isVerified=false;
                }else{
                    delete query.isVerifySubmit;
                    delete query.isVerified;
                }
            }
            return query;
        },
        onSevListClick: function(data){
            // summary:
            //      服务商列表点击事件  
            // view:
            //      sevManage-服务商列表
            // description:
            //      点击跳转到服务商详情 
            window.moveToView("sevMerDetail",data._id);
        }
    });
	
	var shopBo = new ShopBo();
	shopBo.Bo = ShopBo;
    lang.setObject("bob.shop.sevShop", shopBo);
    return shopBo;
});
