define([
	"dojo/_base/declare",
	"dojo/_base/lang",
    "dojo/DeferredList",
	"./shop"
], function(declare,lang,DeferredList,shop) {
    var ShopBalanceBo = declare("",shop.Bo,{
        modalName : "ts/shopPayLog",
        fields:{
            availableAmount:{
                type:"text",
                label:"店铺可用余额",
                canEdit : false
            },
            amountSum:{
                type:"text",
                label:"店铺待审核金额",
                canEdit : false
            },
            bankName:{
                type:"text",
                label:"付款方开户行",
                canEdit : false
            },
            bankOwner:{
                type:"text",
                label:"付款方账户名",
                canEdit : false
            },
            bankAccount:{
                type:"text",
                label:"付款方开户账号",
                canEdit : false
            },
            "fromShopId.shopName":{
                type:"text",
                label:"商家名称",
                canEdit : false
            },
            "fromShopId.address":{
                type:"text",
                label:"商家地址",
                canEdit : false
            },
            "fromShopId.logo":{
                type:"text",
                label:"商家图标",
                canEdit : false
            },
            logo:{
                type:"image",
                label:"商家图标",
                clickImg:false,
                canEdit : false
            },
            amount:{
                type:"text",
                label:"提现金额"
            },
            // 确认提现
            toDeposit: {
                type: "button",
                action: "onDeposit",
                label: "确认提现"
            },
            // 审核
            onAccept: {
                type: "button",
                action: "onAccept",
                label: "审核"
            },
            accepted: {
                type: "button",
                action: "accepted",
                label: "已审核"
            },
            moveToTransaction: {
                type: "button",
                action: "moveToTransaction",
                label: "已审核"
            }
        },
        getLayout:function(){
            return [{
                type: "form",
                fields: this.getFormItems([
                    "availableAmount",
                    "amountSum",
                    "amount",
                    "bankName",
                    "bankOwner",
                    "bankAccount",
                    "toDeposit"
                ])
            }];
        },
        _initData:function(shopId){
            var defList = new DeferredList([
                // 获取店铺的可用余额
                shop.loadMe(),
                // 获取店铺已申请提现，但未审核的总金额
                this.loadByType()
            ]);
            return defList;
        },
        initData:function(){
            return this._initData().then(lang.hitch(this,function(result){
                var shopData = result[0][1];
                var data = result[1][1].items;
                var amountSum = 0;
                if(data.length){
                    for(var i = 0; i < data.length; i ++){
                        amountSum += data[i].amount;
                    }
                }
                var item = {};
                item.availableAmount = window.fixedTo(shopData.availableAmount, 3);
                item.bankName = data.bankName;
                item.bankOwner = data.bankAccountName;
                item.bankAccount = data.bankAccount;
                item.amountSum = window.fixedTo(amountSum, 3);
                return item;
            }))
        },
        loadByType:function(){
            return this.exec("loadByType",{isVerified:false,ioType:1}).then(lang.hitch(this,function(data){
                return data.data;
            }));
        },
        onDataReady:function(widgets){
            widgets.disableByFields(["availableAmount","amountSum"]);
            widgets.hideByFields(["bankName","bankAccount","bankOwner"]);
        },
        onDeposit:function(data,widgets){
            data.ioType = 1;
            if(!(Number(data.amount)>0)){
                alert("金额必须大于零")
                return;
            }
            if(Number(data.amount)>Number(data.availableAmount)){
                alert("金额不能大于可用余额")
                return;
            }
            this.exec("save",data).then(lang.hitch(this,function(result){
                if(result&&result.data&&result.data.ok==1){
                    widgets.cqRefresh();
                }else{
                    alert(result.message||"提现失败");
                }
            }))
        },
        //余额提现管理列表
        getListUrl:function(){
            return this.getBaseUrl()+"/loadByType?shopId="+this.getShopId();
        },
        getListParam:function(params){
            var query = {isVerified:false,ioType:1};
            if(params.state&&params.state._id == 1){
                query.isVerified = true;
            }
            return query;
        },
        onListClick:function(data){
            window.moveToView("shopBalanceDepDetail",data._id);
        },
        //余额提现管理详情
        getDetailLayout:function(){
            return [{
                type: "form",
                fields: this.getFormItems(["logo","fromShopId.shopName","fromShopId.address","bankName","bankAccount","bankOwner","amount","accepted","onAccept","moveToTransaction"])
            }];
        },
        initDetailData:function(cqRefInfo){
            var id = cqRefInfo.cqViewParam.params;
            return this.exec("loadById",{
                _id:id
            }).then(lang.hitch(this,function(data){
                var data = data.data.item;
                data.logo = data.fromShopId.logo;
                return data;
            }));
        },
        onDetailDataReady:function(widgets){
            widgets.disableByFields(["logo","fromShopId.shopName","fromShopId.address","bankName","bankAccount","bankOwner","amount"])
            widgets.hideByFields(["accepted","onAccept"])
            var item = widgets.getData();
            if(item._id){
                var showField = [];
                showField.push(item.isVerified?"accepted":"onAccept");
                widgets.showByFields(showField);
            }
        },
        moveToTransaction:function(data){
            window.moveToView("transactionList",data.fromShopId._id);
        },
        onAccept:function(data,widgets){
            this.exec("verify",{_id:data._id}).then(lang.hitch(this,function(result){
                if(result&&result.data&&result.data.ok==1){
                    widgets.cqRefresh();
                }else{
                    alert(result.message||"审核失败");
                }
            }));
        },
        //已审核按钮不做任何操作
        accepted:function(){}
    });
	var shopBalanceBo = new ShopBalanceBo();
	shopBalanceBo.Bo = ShopBalanceBo;
    return shopBalanceBo;
});
