define([
    "dojo/_base/declare",
    "dojo/_base/lang",
    "dojo/_base/json",
    "dojo/Deferred",
    "dojo/_base/array",
    "dojo/DeferredList",
    "bob/user/consumer",
    "cq/util/sys/date"
],function(declare,lang,json,Deferred,array,DeferredList,consumer,date){
    return declare("",[],{
        shopBo:null,
        type:"",
        //购物车公共操作、对外暴露的接口
        //商品数量相关的操作
        //获取购物车商品、数量信息
        getCartInfo:function(){
            return json.fromJson(localStorage.getItem(this.getShopId()+this.type+"cartmn")||"{}");
        },
        //保存购物车商品、数量信息
        setCartInfo:function(info){
            localStorage.setItem(this.getShopId()+this.type+"cartmn",json.toJson(info));
        },
        //购物车中指定商品数量加一
        addMat:function(mid){
            var info = this.getCartInfo();
            info[mid] = info[mid]||0;
            info[mid]++;
            this.setCartInfo(info);
            return info;
        },
        //商品详情添加到购物车,
        addMatAndQty:function(mid,qty,params){
            //params里面param.specs和配送此时
            if(!mid){
                return;
            }
            if(params&&params.specs&&params.specs._id){
                this._addLastSpecs(params.specs);
            }
            if(qty>0){
                var info = this.getCartInfo();
                info[mid] = (info[mid]||0)+parseInt(qty);
                this.setCartInfo(info);
                return info;
            }
        },
        //添加接口保存上次添加的规格，有可能是被删除了的，需要在是小商品列表显示规格问题
        _addLastSpecs:function(specs){
            var spec = this.getCartSpecs();
            spec[specs._id] = specs;
            this.setCartSpecs(spec);
        },
        //保存添加商品的规格
        setCartSpecs:function(specs){
            localStorage.setItem(this.getShopId()+this.type+"specs",json.toJson(specs));
        },
        //获取添加商品的规格
        getCartSpecs:function(){
            return json.fromJson(localStorage.getItem(this.getShopId()+this.type+"specs")||"{}");
        },
        //修改购物车中指定商品的数量   qty为0 时删除该商品
        setMatQty:function(mid,qty){
            var info = this.getCartInfo();
            qty = qty-0;
            info[mid] = qty;
            this.setCartInfo(info);
            //购物车图片数量变更
            this.addMatAndQty(mid,"set");
            return info;
        },
        //清除失效商品时候，执行删除操作
        deleteMid:function(mid){
            //如果存在，需要删除相应保存的那个规格的信息
            var specs = this.getCartSpecs(),info = this.getCartInfo();
            if(specs[mid.split("_")[1]]){
                delete specs[mid.split("_")[1]];
            }
            this.setCartSpecs(specs);
            delete info[mid];
            this.setCartInfo(info);
            this.addMatAndQty(mid,"set");
        },
        //根据商品id查看当前在购物车的数量
        getNumById:function(id){
            var allCart = this.getCartInfo();
            for(var mid in allCart){
                if(mid==id){
                    return allCart[mid];
                }
            }
        },
        //设置当前mid的选中
        updateMatSelected:function(mid,selected){
            if(!mid){
                return;
            }
            var mids = this.getSelected();
            mids = mids.replace(","+mid+",",",");
            if(selected){
                mids += (mids?"":",")+mid+",";
            }
            this.setSelected(mids);
        },
        //获取选中的商品
        getSelectedMids:function(){
            var mids = this.getSelected();
            if(mids){
                mids = mids.split(",");
                mids.shift();
                mids.pop();
                return mids;
            }else{
                return [];
            }
        },
        //需要修改规格问题,清除结算的该商品的某个规格的商品
        clearSelected:function(){
            var mids = this.getSelectedMids();
            var info = this.getCartInfo();
            for(var i=0;i<mids.length;i++){
                delete info[mids[i]];
            }
            this.setCartInfo(info);
            this.setSelected("");
            //触发变更购物车图标事件
            this.addMatAndQty("","");
        },
        setSelected:function(matInfo){
            localStorage.setItem(this.getShopId()+this.type+"cartms",matInfo);
        },
        getSelected:function(){
            return localStorage.getItem(this.getShopId()+this.type+"cartms")||"";
        },
        //获取选中的商品进行购买
        //订单结算使用 需要修改规格问题
        //若改商品是周期产品，需要保存周期
        /*
        deliveryCycle:配送方式，（周月日）
        deliveryCycleDetail:配送的具体详情（如果是每周13,0101000，如果是每月，则是31位字符串）
        deliveryCycleEachNum:每次配送数量
        totalDeliveryTimes:总的配送次数
        */
        getMatQtys:function(){
            var mids = this.getSelectedMids(),info = this.getCartInfo(),cycle = this.getCartCycle(),matQtys = [];
            for(var i=0;i<mids.length;i++){
                if(info[mids[i]]){
                    if(cycle[mids[i]]){
                        matQtys.push({
                            mid:mids[i],
                            qty:info[mids[i]],
                            deliveryCycle:cycle[mids[i]].deliveryCycle,
                            deliveryCycleDetail:cycle[mids[i]].deliveryCycleDetail,
                            deliveryCycleEachNum:cycle[mids[i]].trgNum,
                            totalDeliveryTimes:cycle[mids[i]].totalDeliveryTimes
                        });
                    }else{
                        matQtys.push({
                            mid:mids[i],
                            qty:info[mids[i]]
                        });
                    }
                }
            }
            return matQtys;
        },
        //保存添加商品的规格
        setCartCycle:function(cycle){
            localStorage.setItem(this.getShopId()+this.type+"cycle",json.toJson(cycle));
        },
        //获取添加商品的规格
        getCartCycle:function(){
            return json.fromJson(localStorage.getItem(this.getShopId()+this.type+"cycle")||"{}");
        },
        //直接购买
        //需要修改规格问题
        //需要处理配送周期
        /*
            本对象使用的周期配送信息
            deliveryCycle：配送方式（日周月）
            totalDeliveryTimes：总的配送次数
            userDelivery：立即购买选择的配送方式
            trgNum :每次配送数量

        */
        //获取配送周期
        getOrderCycle:function(cycle){
            var orderCycle = {};
            orderCycle.deliveryCycle = cycle.deliveryCycle;
            orderCycle.trgNum = cycle.trgNum;
            orderCycle.totalDeliveryTimes = cycle.totalDeliveryTimes||1;
            //每天配送的是周末的话，转换成没周6周7
            if(cycle.deliveryCycle==1){
                orderCycle.deliveryCycleDetail = cycle.userDelivery.substring(0,1);
                if(cycle.userDelivery.substring(0,3) == "001"){
                    //周日周6
                    orderCycle.deliveryCycleDetail = "1000001";
                    orderCycle.deliveryCycle = 2;
                }else if(cycle.userDelivery.substring(0,3) == "010"){
                    //周一到周5 配送
                    orderCycle.deliveryCycleDetail = "0111110";
                    orderCycle.deliveryCycle = 2;
                }
            }
			// 每周或每两周配送
			else if(cycle.deliveryCycle==2||cycle.deliveryCycle==4){
                orderCycle.deliveryCycleDetail = cycle.userDelivery.substring(0,7);
            }else if(cycle.deliveryCycle==3){
                orderCycle.deliveryCycleDetail = cycle.userDelivery;
            }
            return orderCycle;
        },
        addBuyNow:function(mid,qty,params){
            var info = this.getCartInfo();
            
            //为了设置购物车图标所做的变更，2017/2/13 徐阿雪
            info[mid] = 0;
            //设置购物车该商品的qty为0，不知道合理不合理，但是不会导致立即购买和购物车里面的同种商品累加
            this.setCartInfo(info);
            this.addMatAndQty(mid,qty,params);
            //直接选中
            var mids = ","+mid+",";
            this.setSelected(mids);
            if(params&&params.specs&&params.specs._id){
                this._addLastSpecs(params.specs);
            }
            if(params&&params.trgMode){
                this.setCartCycle({});
                var cycle = this.getCartCycle();
                cycle[mid] = this.getOrderCycle(params.trgMode);
                this.setCartCycle(cycle);
            }
            // 处理拼团订单的团队信息
            if(params&&params.pacTeamId){
                localStorage.setItem(this.getShopId()+this.type+"pacbuy","");
                localStorage.setItem(this.getShopId()+this.type+"pacbuy",params.pacTeamId);
            }else{
                localStorage.setItem(this.getShopId()+this.type+"pacbuy","");
            }
            return this.getCartInfo();
        },
        //获取所有商品ID
        getIds:function(){
            var ids = [],info = this.getCartInfo();
            for(var mid in info){
                if(mid){//&&info[mid]
                    ids.push(mid.split("_")[0]);
                }
            }
            return ids;
        },
        //如果其他地方购物同一个商品导致购物车里面商品的数量
        _updateCartMats:function(mid,qty){
            var matInfo = this.getCartInfo();
            if(matInfo[mid]>qty){
                matInfo[mid] = qty;
            }
            this.setCartInfo(matInfo);
        },
        //重新区分整理matList
        _distinguishMat:function(mats){
            var specId = this._getMidAndSpecId(),matInfo = this.getCartInfo(),mat={},matList = [],
                currentClientLevel = consumer.getCurrentUserFromSession().clientLevel&&consumer.getCurrentUserFromSession().clientLevel._id||"";
            if(currentClientLevel){
                for(var i=0;i<mats.length;i++){
                    if(mats[i].fromShopId==this.getShopId()){
                        var currentRate = 10;
                        for(var j=0;j<mats[i].prices.length;j++){
                            if(currentClientLevel==(mats[i].prices[j].clientLevel._id||mats[i].prices[j].clientLevel)){
                                currentRate = mats[i].prices[j].price;
                                break;
                            }
                        }
                        mats[i].retailPrice = mats[i].retailPrice*currentRate/10;
                        for(var m=0;m<mats[i].specs.length;m++){
                            mats[i].specs[m].price = mats[i].specs[m].price*currentRate/10;
                        }
                    }
                } 
            }
            //根据matInfo来定量matList的长度
            for(var cartMid in matInfo){
                for(var i=0;i<mats.length;i++){
                    mat = mats[i];
                    if(mat._id == cartMid.split("_")[0]){
                        var spec = null,sId = cartMid.split("_")[1],changeMat = lang.clone(mat);
                        //有规格查找规格信息
                        if(cartMid.split("_")[1]){
                            //商品没有有规格，肯定是删除了
                            if(!changeMat.specs.length){
                                //删除了商品规格
                                changeMat.isUse = false;
                                spec = this.getCartSpecs()[cartMid.split("_")[1]];
                            }else{
                                //判断是否发生变化
                                spec = this._getMsgBySpecId(cartMid.split("_")[1],changeMat.specs);
                                //spec为空，就是发生变化，去localStroage 里面取值
                                if(!spec){
                                    changeMat.isUse = false;
                                    spec = this.getCartSpecs()[cartMid.split("_")[1]];
                                }
                            }
                            if(!spec){
                                changeMat.isUse = false;
                            }else{
                                //商品的mid修改
                                changeMat._id = changeMat._id+"_"+sId;
                                changeMat.qty = spec.qty||0;//库存
                                changeMat.retailPrice = spec.price||0;//价格
                                changeMat.spec = spec||{};
                            }
                        }else{
                            //只考虑新增
                            if(changeMat.specs.length){
                                //该商品新增规格
                                changeMat.isUse = false;
                            }
                        }
                        matList.push(changeMat);
                        break;
                    }
                }
            }
            return matList;
        },
        //得到这个规格的库存和规格信息
        _getMsgBySpecId:function(specId,specs){
            var specMsg = null;
            for(var i=0;i<specs.length;i++){
                if(specs[i]._id == specId){
                    specMsg = specs[i];
                    break;
                }
            }
            return specMsg;
        },
        _getMidAndSpecId:function(){
            var specId = {},info = this.getCartInfo(),specs=[];
            for(var mid in info){
                if(!specId[mid.split("_")[0]]||!specId[mid.split("_")[0]].length){
                    specs = [];
                }
                if(mid.split("_")[1]){
                    specs.push(mid.split("_")[1]);
                }
                specId[mid.split("_")[0]] = specs;
                
            }
            return specId;
        },
        //检查当前购物车是否有活动的商品，根据活动修改，qty,limitQty,price
        _updateMatList:function(mids){
            var matInfo = this.getCartInfo(),_this=this;
            var info={};
            var def = new Deferred();
            require([
                "bob/mk/limitDiscount/c"
            ], lang.hitch(this, function (limitDiscount) {
                // 获取商家的数据
                limitDiscount.loadByMids(mids).then(lang.hitch(this, function (data) {
                    items = data.data.items;
                    array.forEach(items,function(item){
                        var mid = "";
                        if(item.mId&&item.mId._id){
                            mid = item.mId._id;
                        }else{
                            mid = item.mId;
                        }
                        if(matInfo[mid]){
                            info[mid] = {
                                price:item.limitPic,
                                limitQty:item.limitQty,
                                qty:item.qty
                            };
                        }
                    },this);
                    def.resolve(info);
                }));
            }));
            return def;
        },
        //判断当前商品是否在预售活动中
        isMatPresale:function(preSale){
            var isPresale = false;
            //判断presale的ed和sd是否在当前时间范围，是则返回true反之fasle
            for(var i=0;i<preSale.length;i++){
                //date.formatISODateMinutes(preSale[i].sd)<=date.formatISODateMinutes(date.nowISOString())<=date.formatISODateMinutes(preSale[i].ed)
                if(preSale[i].sd<=date.nowISOString()&&date.nowISOString()<=preSale[i].ed){
                    isPresale = true;
                    break;
                }
            }
            return isPresale;
        },
        modalName:"mat/mat",
        loadByIds:function(ids){
            return this.exec("loadByIds",{
                ids:ids
            });
        },
        loadCartData:function(){   
            var def = new Deferred();
            this.loadByIds(this.getIds()).then(lang.hitch(this,function(mats){
                mats = mats&&mats.data&&mats.data.items||[];
                if(!(mats&&mats.length)){
                    def.resolve([]);
                }else{
                    //失效商品和无库存的商品
                    var unUseMats = [],unUse=[];
                    //按照商家分类
                    var items = [],
                        mat = null,
                        matInfo = this.getCartInfo(),
                        mids = this.getSelectedMids(),
                        selectedInfo = {},
                        shopInfo = null,
                        shopHash = {},
                        shopIds = [],
                        usable = [],
                        i;
                    for(i=0;i<mids.length;i++){
                        selectedInfo[mids[i]] = true;
                    }
                    mats = this._distinguishMat(mats);
                    for(i=0;i<mats.length;i++){
                        mat = mats[i];
                        //qty为0 要删除localStorage的cart,qty小于localStorage的数量需要改变
                        var qty = mat.limitedQty?Math.min(mat.qty,mat.limitedQty):mat.qty;
                        if(matInfo[mat._id]>qty){//mat.qty||!mat.qty
                            this._updateCartMats(mat._id,qty);
                        }
                        var isPresale = false;
                        //如果是预售的商品
                        if(mat.preSale&&mat.preSale.length){
                            isPresale = this.isMatPresale(mat.preSale);
                        }
                        //失效商品和无库存的商品
                        if(mat.isUse && mat.qty && mat.fromShopId&&mat.minQty<=mat.qty && !mat.isCycle && !mat.isDelete && !isPresale){
                            shopInfo = shopHash[mat.fromShopId];
                            if(!shopInfo){
                                shopInfo = {
                                    mats:[]
                                };
                                shopHash[mat.fromShopId] = shopInfo;
                                items.push(shopInfo);
                                shopIds.push(mat.fromShopId);
                            }
                            //库存，//商品在购物车里面的数量
                            this._updateMatsQty(mat,matInfo);
                            mat.selected = !!selectedInfo[mat._id];
                            //修改在活动中的商品的价格
                            shopInfo.mats.push(mat);
                            usable.push(mat._id.split("_")[0]);
                        }else{
                            //已经下架和失效的商品整理到一起
                            mat.stockQty = mat.qty;
                            mat.qty = 0;
                            unUse.push(mat);
                            //把失效的商品在购物车里面的数量修改为0
                            this._updateCartMats(mat._id,0);
                        }
                    }
                    //修改价格限购和库存
                    this._updateMatList(usable).then(lang.hitch(this,function(activityMats){
                        //修改的是shopHash{}里面的两个值
                        for(i=0;i<items.length;i++){
                            for(var j=0;j<items[i].mats.length;j++){
                                var newMat = items[i].mats[j];
                                if(activityMats[newMat._id]){
                                    //qty限购库存，limitQty限购数
                                    items[i].mats[j].stockQty = Math.min(activityMats[newMat._id].qty,newMat.stockQty);
                                    items[i].mats[j].limitedQty = activityMats[newMat._id].limitQty;
                                    //价格是不是需要根绝规格做对比
                                    items[i].mats[j].retailPrice = activityMats[newMat._id].price;
                                    //比较过后的商品库存与限购库存做对比
                                    items[i].mats[j].qty = Math.min(activityMats[newMat._id].limitQty,newMat.qty);
                                    items[i].mats[j].minQty = 1;
                                    if(matInfo[newMat._id]>items[i].mats[j].qty||!items[i].mats[j].qty){//||!items[i].mats[j].qty
                                        this._updateCartMats(items[i].mats[j]._id,items[i].mats[j].qty);
                                    }
                                }
                                /*else{
                                    //根据等级修改商品
                                    var userInfo = consumer.getCurrentUserFromSession();
                                        level = this._editPriceByClientLevel(newMat,userInfo);
                                    if(level){
                                        items[i].mats[j].retailPrice = items[i].mats[j].retailPrice*level*0.1;
                                    }
                                }*/
                            }
                        }
                        //加载商家信息,需要商家的商家名称
                        if(shopIds.length||unUse.length){
                            this.shopBo.loadByIds(shopIds).then(lang.hitch(this,function(shops){
                                shops = shops&&shops.data&&shops.data.items||[];
                                for(i=0;i<shops.length;i++){
                                    shopHash[shops[i]._id].shopInfo = shops[i];
                                }
                                var a = [];
                                //有失效的商品就创建失效的商家列表
                                if(unUse.length){
                                    var unUseShopInfo={shopName:"失效的商品",_id:"111"};
                                    unUseMats.push({mats:unUse,shopInfo:unUseShopInfo});
                                    a = items.concat(unUseMats);
                                }else{
                                    a = items;
                                }
                                //将商家和商家对应的商品分租
                                var shop = [];
                                array.forEach(items,function(item){
                                    var mids = [],fromShopId = "";
                                    array.forEach(item.mats,function(mat){
                                        mids.push(mat._id.split("_")[0]);
                                        //mids.push(mat._id);
                                    },this);
                                    fromShopId = item.shopInfo._id;
                                    shop.push({mids:mids,shopId:fromShopId});
                                },this);
                                //加载商家可能存在的满减活动
                                this._getFullAvtivity(shop).then(lang.hitch(this,function(activityInfo){
                                    for(i=0;i<shops.length;i++){
                                        shopHash[shops[i]._id].shopInfo = shops[i];
                                        if(activityInfo[shops[i]._id]&&activityInfo[shops[i]._id].length){
                                            shopHash[shops[i]._id].shopActivityInfo = activityInfo[shops[i]._id];
                                        }
                                    }
                                    def.resolve(a);
                                }));
                            }));
                        }else{
                            def.resolve([]);
                        }
                    }));
                }
            }));
            return def;
        },
        //根据等级修改购物车商品的价格，预售和抢购不处理
        _editPriceByClientLevel:function(item,userInfo){
            // 有可能取到多行数据
            var limitDiscountItem = item.limitDiscount||[], 
                findedLimitDis = null,
                isPreSale=false,
                level = null,
                now = date.nowISOString();
            for(var i=0;i<limitDiscountItem.length;i++){
                if(limitDiscountItem[i].ed<now){
                    continue;
                }
                if(!findedLimitDis||findedLimitDis.sd>limitDiscountItem[i].sd){
                    findedLimitDis = limitDiscountItem[i];
                }
            }
            //看当前商品是否是在预售活动中,看当前时间和预售里面时间是否有重合的
            if(item.preSale.length){
                item.preSale.forEach(function(presale){
                    if(presale.sd<now&&presale.ed>now){
                        isPreSale = true;
                    }
                },this);
            }
            if(!findedLimitDis){
                if(!isPreSale&&item.prices.length){
                    item.prices.forEach(function(price){
                        if((price.userLevel._id||price.userLevel) == (userInfo.userLevel._id||userInfo.userLevel)){
                            level = price.price;
                        }
                    },this);
                }
            }
            return level;
        },
        //加载满减活动
        _getFullAvtivity:function(shop){
            var def = new Deferred(),_this = this,defList;
            require(["bob/mk/reward/c"], lang.hitch(this, function (reward) {
                var shopActivityInfo = {};
                defList = new DeferredList(shop.map(function(params){
                    // 获取商家的数据
                    return reward.loadByMids({mids:params.mids,targetShopId:params.shopId}).then(lang.hitch(this, function (data) {
                        //如果该商品有多个规格的话，需要将mid = mid_specId
                        var midAndSpecId  = _this._getMidAndSpecId();
                        var items = data.data.items;//
                        for(var i =0;i<items.length;i++){
                            //var item = items[i];//修改items[].mids这个数组
                            var specList = lang.clone(items[i].mids);
                            var newSpecList = lang.clone(specList);
                            //遍历所有活动的商品，如果购物车有该商品，而且该商品是多规格的情况，修改mid
                            //如果又该商品，不是多规格，mids不做修改
                            array.forEach(specList,function(specMsg){
                                if(midAndSpecId[specMsg.mId]&&midAndSpecId[specMsg.mId].length){
                                    array.forEach(midAndSpecId[specMsg.mId],function(mid){
                                        var spec = lang.clone(specMsg);
                                        spec.mId = spec.mId+"_"+mid;
                                        newSpecList.push(spec);
                                    },this);
                                }
                            },this);
                            items[i].mids = newSpecList;
                        }
                        shopActivityInfo[params.shopId] = items||[];
                    }));
                }));
                defList.then(lang.hitch(this,function(data){
                    def.resolve(shopActivityInfo);
                }));
            }));
            return def;
        },
        
        /*
        如果有限购数量
            忽略最小起订量的控制(购物车加减不受最小起订量影响)
            购物车数量=Math.min(库存,限购数量,购物车数量)
        如果没有限购数量
            库存<最小起订量,将商品从购物车中直接删除，放到失效的产品里面
            库存>=最小起订量
            购物车数量=Math.min(库存,购物车数量)
        */
        //修改商品的最小起订量
        _updateMatsQty:function(mat,matInfo){
            //显示的库存
            mat.stockQty = mat.qty;
            //处理商品的数量和选中
            //查看localStorage里面保存的购物车数据和库存的大小
            //商品在购物车里面的数量，取localStorage和商品库存的最小值（其他地方买了会导致商品存款减少）
            if(matInfo[mat._id]){
                mat.qty = Math.min(matInfo[mat._id],mat.qty);
            }
            if(mat.qty<mat.minQty){
                mat.qty = mat.minQty;
            }
        },
        getCartQtyInfo:function(){
            var info = this.getCartInfo(),qtyInfo = {qty:0};
            for(var mid in info){
                qtyInfo.qty += info[mid]||0;
            }
            return qtyInfo;
        },
        
        //购物车中商品加载完成之后的处理，设置购物车中商品数量
        getItemParam:function(data){
            data.cartQty = this.getCartInfo()[data._id]||0;
            return data;
        },
        // 获取当前的团队ID
        getPacTeamId: function(){
            return localStorage.getItem(this.getShopId()+this.type+"pacbuy");
        }
    });
});