//订单系统公共基类
// 1、提供订单系统的基本属性
// 2、提供公用的方法
define([
    "dojo/_base/declare",
    "dojo/_base/lang",
    "dojo/_base/array",
    "dojo/Deferred",
    "cq/util/image/blob",
    "bob/ts/deliveryLog/b",
    "cq/util/sys/date"
], function (declare, lang,array,Deferred, blob,deliveryLog, date) {
    return declare("",null,{
        modalName:"ts/order",
        //字段名
        fields:{
            _id:{
                type:"text",
                label:"订单号",
                canEdit : false,
            },
            state:{
                type:"text",
                label:"订单状态",
                canEdit : false
            },
            fromShopName:{
                type:"text",
                label:"供应商名称",
                canEdit : false,
            },
            couponAmount:{
                type:"text",
                fixedTo: 2,
                label:"优惠金额",
                canEdit : false,
            },
            amount:{
                type:"text",
                label:"总金额(含运费)",
                fixedTo:2,
                className:"large-width",
                canEdit : false,
            },
            preAmount:{
                type:"text",
                label:"预付金额",
                fixedTo:2,
                className:"large-width",
                canEdit : false,
            },
            prePayAmount:{
                type:"text",
                label:"在线支付定金",
                fixedTo:2,
            },
            preGold:{
                type:"text",
                label:"金元宝支付定金",
                fixedTo:2,
            },
            prePoint:{
                type:"text",
                label:"积分支付定金",
                fixedTo:2,
            },
            restAmount:{
                type:"text",
                label:"尾款",
                fixedTo:2,
                className:"large-width",
                canEdit:false
            },
            gold: {
                type: "Number",
                fixedTo: 2,
                label: "金元宝"
            },
            remark: {
                type: "text",
                label: "备注"
            },
            deliveryTimesMsg: {
                type: "text"
            },
            cycleModeMsg: {
                type: "text"
            },
            tAmount:{
                type:"text",
                label:"运费",
                fixedTo:2,
                className:"large-width",
                canEdit : false,
            },
            payAmount:{
                type:"text",
                label:"应付金额",
                fixedTo:2,
                className:"large-width",
                canEdit : false
            },
            _dName:{
                type:"text",
                label:"收货人",
                canEdit : false,
            },
            _dTel:{
                type:"text",
                label:"电话",
                className: "large-width",
                canEdit : false,
            },
            _dAddress:{
                type:"text",
                label:"收货地址",
                className:"large-width",
                canEdit : false,
            },
            //列表没有以下三个字段，用formatter
            dName:{
                type:"text",
                label:"收货人姓名",
                formatter:function(action,data){
                    return v = "<div>"+data.details[0].dName+"</div>";
                }
            },
            dTel:{
                type:"text",
                label:"收货人电话",
                formatter:function(action,data){
                    return v = "<div>"+data.details[0].dTel+"</div>";
                }
            },
            dAddress:{
                type:"text",
                label:"收货人地址",
                className:"large-width",
                formatter:function(action,data){
                    return v = "<div>"+data.details[0].dAddress+"</div>";
                }
            },
            point:{
                type:"text",
                label:"村币/优惠券抵扣",
                fixedTo:2,
                className:"large-width",
                canEdit : false,
            },
            score: {
                type: "score",
                canEdit: true,
            },
            "details.0.dName": {
                type: "text"
            },
            "details.0.dTel": {
                type: "text"
            },
            "details.0.dAddress": {
                type: "text"
            },
            "startPayTime":{
                type:"dateTime"
            },
            "endPayTime":{
                type:"dateTime"
            },
            discountAmount: {
                type: "text",
                fixedTo: 2
            },
            "mId.mName": {
                type: "text"
            },
            price: {
                type: "Number",
                fixedTo:2
            },
            qty: {
                type: "Number"
            },
            mInfo:{
                type:"objects",
                label:"",
                canEdit : false,
                canDelete : false,
                canAdd:false,
                fields : {
                    layout : [{
                        field : "mLogo",
                        type : "image",
                        label : "商品图标",
                        className :　"large-width large-height"
                    },{
                        field : "mId.mName",
                        type : "text",
                        label : "名称"
                    }, {
                        field : "price",
                        type : "text",
                        label : "金额"
                    }, {
                        field : "qty",
                        type : "text",
                        label : "商品数量",
                        className: "large-width"
                    }]
                },
            },
            eventBtns:{
                type:"text",
                label:"操作",
                formatter:function(action,data){
                    var v = "<div class='button' cq-action='detail'>订单详情</div>";
                    return v;
                }
            },
            createAt:{
                type:"dateTime",
                label: "创建时间",
                className: "large-width",
                canEdit : false,
            },
            //支付时间
            payTime:{
                type:"dateTime",
                label: "支付时间",
                className: "large-width",
                canEdit : false,
            },
            //定金支付时间
            prePayTime:{
                type:"dateTime",
                label: "定金支付时间"
            },
            delivTime:{
                type:"dateTime",
                label: "发货时间",
                className: "large-width",
                canEdit : false,
            },
            recepiptTime:{
                type:"dateTime",
                label: "收货时间",
                className: "large-width",
                canEdit : false,
            },
            // 物流查询
            toQryLog: {
                type: "button",
                action: "QryLog"
            },
            //送达日历
            deliveryCalendar:{
                type:"button",
                action:"checkDeliveryCalendar"
            },
            getPayTime: {
                type: "button",
                action: "getPayTime"
            },
            getDelivTime: {
                type: "button",
                action: "getDelivTime"
            },
            getRecepiptTime: {
                type: "button",
                action: "getRecepiptTime"
            },
            //退款字段可以变化
            refundMsg:{
                type:"button",
                label:"退款中"
            },
            refund:{
                type:"button",
                label:"退款"
            },
            payType:{
                type:"options",
                label:"支付方式",
                options:{
                    "weixin": "微信",
                    "wxApp": "微信",
                    "wxNative": "微信",
                    "aliwap": "支付宝",
                    "aliapp": "支付宝",
                    "point": "村币"
                }
            },
            prePayType:{
                type:"options",
                label:"定金支付方式",
                options:{
                    "weixin": "微信",
                    "wxApp": "微信",
                    "wxNative": "微信",
                    "aliwap": "支付宝",
                    "aliapp": "支付宝",
                    "point": "村币"
                }
            },
            payTypeTxt: {
                type: "text"
            },
            showPayType: {
                type: "button",
                action: "showPayType"
            },
            specName: {
                type: "text",
                label: "规格"
            },
            nextDeliveryTime: {
                type: "date"
            },
            preAmount: {
                label:"预付定金",
                type: "number",
                fixedTo: "2"
            },
            preGold: {
                type: "number",
                fixedTo: "2"
            },
            prePoint: {
                type: "number",
                fixedTo: "2"
            },
            preCoupon: {
                type: "number",
                fixedTo: "2"
            },
            // 定金+尾款部分金额
            realAmount: {
                type: "number",
                fixedTo: "2"
            },
            userShopName:{
                type:"text",
                label:"下单店铺"
            },
            toShowPrePayedDetails: {
                type: "button",
                action: "toShowPrePayedDetails"
            },
            toShowFinalPayedDetails: {
                type: "Button",
                action: "toShowFinalPayedDetails"
            },
            deliveryType: {
                type: "options",
                options: {
                    "1": "快递",
                    "2": "到店自提"
                },
                label: "配送方式"
            },
            payShopName : {
                type : "text",
                label : "收款商家"
            },
            settleDate : {
                type:"dateTime",
                label: "收款时间",
                className: "large-width",
                canEdit : false,
            },
            prePayShopName : {
                type : "text",
                label : "定金收款商家"
            },
            preSettleDate : {
                type:"dateTime",
                label: "定金收款时间",
                className: "large-width",
                canEdit : false,
            }
        },
        hasShowPrePayedDetails: false,
        hasShowFinalPayedDetails: false,
        toShowFinalPayedDetails: function(data, widget){
            if(this.hasShowFinalPayedDetails){
                widget.hideByFields(["showFinalPayedDetails"]);
                this.hasShowFinalPayedDetails = false;
            }else{
                widget.showByFields(["showFinalPayedDetails"]);
                this.hasShowFinalPayedDetails = true;
            }
        },
        toShowPrePayedDetails: function(data, widget){
            if(this.hasShowPrePayedDetails){
                widget.hideByFields(["showPrePayedDetails"]);
                this.hasShowPrePayedDetails = false;
            }else{
                widget.showByFields(["showPrePayedDetails"]);
                this.hasShowPrePayedDetails = true;
            }
        },
        getUnionOrders: function(unionId){
            return this.exec("getUnionOrders", {
                unionId: unionId
            }).then(function(result){
                return result&&result.data&&result.data.items||[];
            });
        },
        //退货退款
        onCancel: function(data, widget){
            if(confirm("确认取消？")){
                window.busy();
                if(data.unionId){
                    this.getUnionOrders(data.unionId).then(lang.hitch(this, function(orders){
                        if(confirm("本订单将与订单编号为"+array.map(orders, function(order){
                            return order._id!=data._id?order._id:"";
                        }).join("、")+"的订单一起取消，确定取消？")){
                            var len = orders.length;
                            var msg = "";
                            var _this = this;
                            var cancelById = function(){
                                var def = new Deferred();
                                if(len<=0){
                                    return def.resolve();
                                }
                                return _this.exec("cancelById",{
                                    _id: orders[len-1]
                                }).then(function(result){
                                    msg = result&&result.data&&result.data.message||"";
                                    len--;
                                    cancelById();
                                });
                                return def;
                            }
                            cancelById().then(function(){
                                if(msg){
                                    alert(msg);
                                }else{
                                    widget.cqRefresh();
                                }
                            });
                        }
                    }));
                }else{
                    this.exec("cancelById",{
                        _id: data._id,
                        shopId:this.getShopId()
                    }).then(lang.hitch(this, function(result){
                        window.free();
                        if(result&&result.data&&result.data.ok==1){
                            widget.cqRefresh();
                        }else{
                            alert(result.message||"取消失败");
                        }
                    }));
                }
            }
        },
        //页面查询条件
        _getListQuery:function(params){
            var query = {orderBy:'-createAt'};
            //搜索
            if(params.search){
                query.search = params.search;
            }
            return query;
        },
        //web端列表相关
        //web端列表布局
        getListLayout: function(){
            return this.getFormItems(["_id","dName","dAddress","dTel","mName","amount","couponAmount","eventBtns"],false);
        },

        //web端详情相关
        //web端详情页面布局
        getEditLayout: function(){
            return [{
                type: "form",
                fields: this.getFormItems(["_id","fromShopName","_dName","state","_dTel","_dAddress","mInfo","tAmount","amount","point","payAmount","createAt"])
            }];
        },
        getOrderStateTxt: function(data){
            return "";
        },
        loadById:function(tsId){
            return this.exec("loadById",{
                _id:tsId
            })
        },
        _getPayTypeTxt: function(payType){
            var o = {
                "weixin": "微信：",
                "wxApp": "微信：",
                "wxNative": "微信：",
                "aliwap": "支付宝：",
                "aliapp": "支付宝：",
                "point": "村币："
            }
            return o[payType];
        },
        _loadWithDetails:function(tsId){
            return this.exec("loadWithDetails",{
                _id:tsId
            }).then(lang.hitch(this, function(data){
                if(data){
                    for (var i=0;i<data.data.items[0].details.length;i++){
                        data.data.items[0].details[i].mId.logo = blob.getImageUrl(data.data.items[0].details[i].mId.logo,"80");
                    }
                }
                if(data.data){
                    data = data.data;
                }
                if(data.items){
                    data = data.items[0];
                    // 重写订单状态
                    data.state = this.getOrderStateTxt(data);
                    data.amount = data.amount + data.discountAmount;
                    data.realAmount = data.amount+(data.preAmount||0);
                    data.discountAmount = data.discountAmount*-1;
                    data.gold = data.gold||0;
                    data.remark = data.remark||"无";
                    // 显示现金支付部分
                    data.payTypeTxt = this._getPayTypeTxt(data.payType);
                    data.finalPayTypeTxt = this._getPayTypeTxt(data.payType);
                    data.finalPayAmount = data.payAmount;
                    data.prePayTypeTxt = this._getPayTypeTxt(data.prePayType);
                    data.preOnlineAmount = data.preAmount-data.preGold-data.prePoint-data.preCoupon;
                    //计算周期订单发过几次货
                    //加载deliveryLog里面所有的记录count
                    if(data.isCycle){
                        //周期订单只有一个明细，周期配送消息
                        var delivery = data.details[0];
                        data.cycleModeMsg = this._getCycleModeMsg(delivery.deliveryCycleDetail, delivery.deliveryCycle);
                        data.deliveryTimesMsg = this._getDeliveryTimesMsg(delivery.deliveryCycleEachNum, delivery.totalDeliveryTimes, delivery.deliveredTimes);
                        data.nextDeliveryTime = delivery.nextDeliveryTime;
                    }
                    return data;
                }  
            }));
        },
        _getDeliveryTimesMsg: function(num, total, delivered){
            return "每次"+num+"，共"+total+"次，已送"+delivered+"次";
        },
        _getCycleModeMsg: function(cycleModeStr, cycleMode){
            var msg = ""
            if(!cycleModeStr||!cycleMode){
                return msg;
            }
            if(cycleMode==1){
                msg = "每天配送";
            }else if((cycleMode==2||cycleMode==4)){
                //每周
                if(cycleMode==4){
                    msg = "每两周";
                }else{
                    msg = "每周";    
                }
                var week = this._getCycleOrderTime(cycleModeStr.split(""));
                array.forEach(week,function(day){
                    msg += this._week[day]+"、";
                },this);
                msg = msg.substr(0, msg.length-1);
                msg += "配送";
            }else{
                //每月
                msg = "每月";
                var month = this._getCycleOrderTime(cycleModeStr.split(""));
                array.forEach(month,function(day){
                    msg += (day+1)+"号、";
                },this);
                msg = msg.substr(0, msg.length-1);
                msg += "配送";
            }
            return msg;
        },
        _week: {
            0: "日",
            1: "一",
            2: "二",
            3: "三",
            4: "四",
            5: "五",
            6: "六"
        },
        //判断是否是周期订单
        _isCycleOrder:function(cycle,num,total,alreadyNum){
            var msg = ""
            if(cycle.split("").length==1){
                msg = "每天配送"+num+"个，共"+total+"次，已送"+alreadyNum+"次";
            }else if(cycle.split("").length==7){
                //每周
                msg = "每周";
                var week = this._getCycleOrderTime(cycle.split(""));
                array.forEach(week,function(day){
                    msg += this._week[day]+"、";
                },this);
                msg = msg.substr(0, msg.length-1);
                msg += "各配送"+num+"个，共"+total+"次，已送"+alreadyNum+"次";
            }else{
                //每月
                msg = "每月";
                var month = this._getCycleOrderTime(cycle.split(""));
                array.forEach(month,function(day){
                    msg += (day+1)+"号、";
                },this);
                msg = msg.substr(0, msg.length-1);
                msg += "各配送"+num+"个，共"+total+"次，已送"+alreadyNum+"次";
            }
            return msg;
        },
        //找到某几天
        _getCycleOrderTime:function(arr){
            var day = [];
            for(var i=0;i<arr.length;i++){
                if(arr[i]==1){
                    //该天配送
                    day.push(i);
                }
            }
            return day;
        },
        loadWithDetails:function(params){
            if (!params.cqViewParam.params) {
                return;
            }
            var tsId = params.cqViewParam.params,def = new Deferred();;
            this._loadWithDetails(tsId).then(lang.hitch(this,function(data){
                data.details[0].pacTeamId = data.pacTeamId;
                var now = date.nowISOString();
                if(data.preSaleId&&data.isPreAmountPayed&&!data.isPayed&&!data.isCanceled){
                    if(now<data.details[0].startPayTime){
                        data.finalPaymentTitle = "距离尾款支付还有";
                        data.finalPaymentTime = data.details[0].startPayTime;
                    }else if(now>=data.details[0].startPayTime&&now<=data.details[0].endPayTime){
                        data.finalPaymentTitle = "距离尾款支付结束还有";
                        data.finalPaymentTime = data.details[0].endPayTime;
                    }else{
                        
                    }
                }
                data.earnestState = data.isPreAmountPayed?"已完成":(data.isCanceled?"已取消":"未支付");
                data.finalPaymentState = data.isPayed?"已完成":(data.isCanceled?"已取消":"未支付");
                //计算周期订单发过几次货
                //加载deliveryLog里面所有的记录count
                if(data.isCycle){
                    deliveryLog.countTotalNum({tsId:tsId}).then(lang.hitch(this,function(result){
                        var items = result&&result.data&&result.data.item||"";
                        //计算商家端第几次发货
                        data.details[0].newdeliveredTimes = items.totalNum;
                        def.resolve(data);
                    }));
                }else{
                    def.resolve(data);
                }
            }));
            return def;
        },
        //订单状态栏选中按钮
        getStateItemProps:function(data,itemType,list){
            var refInfo = list&&list.cqRefInfo||{};
            var state = refInfo.cqViewParam&&refInfo.cqViewParam.params||"";
            data.selected = data._id==state;
            return data;
        },
        // 订单明细加载数据
        matInfoReload: function(widget){
            var item = widget.getData();
            widget.hideByFields(["reNew", "specName","recovery", "pause","showCycleTag","deliveryPackage", "showPreSaleTag","showPacBuyTag","showIsGift"]);
            if(!item._id){
                return;
            }
            if(item.isGift){
                widget.showByFields(["showIsGift"]);
            }
            if(item.specName){
                widget.showByFields(["specName"]);
            }
            if(item.isCycle&&item.isPayed){
                widget.showByFields(["reNew"]);
            }
            if(item.isCycle){
                widget.showByFields(["showCycleTag"]);
            }
            if(item.preSaleId){
                widget.showByFields(["showPreSaleTag"]);
            }
            if(item.pacTeamId){
                widget.showByFields(["showPacBuyTag"]);
            }
            // 周期订单相关配送显示
            if(item.isCycle&&item.isPayed&&item.isCanceled==false&&(!item.refundState||item.refundState<=1)&&item.isDelived&&
            Number(item.totalDeliveryTimes)-Number(item.deliveredTimes)>0){
                if(item.isPause){
                    widget.showByFields(["recovery"]);
                }else{
                    widget.showByFields(["pause"]);
                    widget.showByFields(["deliveryPackage"]);
                    //, "deliveryPackage"
                    // 如果未来两天有待配送的订单 则可以选菜
                    //var now = new Date();
                    //var t = new Date(now.setDate(now.getDate()+1));
                    //var a = new Date(now.setDate(now.getDate()+1));
                    //var n = new Date(item.nextDeliveryTime);
                    //if(item.nextDeliveryTime&&(n.toLocaleDateString()==t.toLocaleDateString()||
                    //n.toLocaleDateString()==a.toLocaleDateString())){
                    //    widget.showByFields(["deliveryPackage"]);
                    //}
                }
            }
        },
        originLoadWithDetails: function(tsId){
            return this.exec("loadWithDetails", {
                _id: tsId
            });
        }
    });
});
