﻿define([
    "dojo/_base/declare",
    "./_Mixin",
    '../../BoBase',
    "dojo/_base/lang",
    "cq/util/sys/userAgent",
    "cq/util/config/config",
    "dojo/Deferred",
    "dojo/dom-construct"
], function (declare,_Mixin,BoBase, lang, userAgent, config, Deferred, domConstruct) {
    var O = declare('', [BoBase.C, _Mixin], {
        getListUrl: function(){
            return this.getBaseUrl() + '/loadMyData';
        },
        getListParam: function(cqRefInfo){
            var params = {
                shopId : this.getShopId()
            };
            return params;
        },
        getListParamByDays: function(cqRefInfo){
            var params = {
                shopId : this.getShopId(),
                fvType : "days"
            };
            return params;
        },
        getItemProp: function(item){
            item.payTime = new Date(item.lastMod);
            return item;
        },
        createOrder: function(params){
            return this.exec("createOrder", params);
        },
        getPayParams: function(params){
            return this.exec("getPayParams", params);
        },
        getValidPayType: function(ts){
            if(ts&&ts.payAmount<=0){
                return [{
                    name: "余额",
                    type:"point",
                    logo:"./images/temp/payicon/pointpay.png"
                }];
            }else if(userAgent.isApp()){
                return [{
                    name: "微信",
                    type:config.appWxPay?"wxApp":"wxNative",
                    logo:"./images/temp/payicon/weixinpay.png"
                },{
                    name: "支付宝",
                    type:cordova&&cordova.plugins&&cordova.plugins.AliPay&&config.appAliPay?"aliapp":"aliwap",
                    logo:"./images/temp/payicon/alipay.png"
                }];
            }else if(userAgent.isWeixin()){
                return [{
                    name: "微信",
                    type:"weixin",
                    logo:"./images/temp/payicon/weixinpay.png"
                },{
                    name: "支付宝",
                    type:"aliwap",
                    logo:"./images/temp/payicon/alipay.png"
                }];
            }else{
                return [{
                    name: "微信",
                    type:"wxNative",
                    logo:"./images/temp/payicon/weixinpay.png"
                },{
                    name: "支付宝",
                    type:"aliwap",
                    logo:"./images/temp/payicon/alipay.png"
                }];
            }
        },
        payOrder: function (param) {
            var def = new Deferred();
            //若不是微信环境或者商家的是订阅号，通过微信扫码支付
            if ("weixin" == param.payType && (!window.WeixinJSBridge || 1 == g_sys_info.wxType || !g_sys_info.wxVerification)) {
                param.payType = "wxNative";
            }
            // 如果是支付宝支付，判断是否是APP支付，是否安装支付宝插件，如果没有则继续网页支付，否则试用插件支付
            if ("aliwap" == param.payType) {
                if (userAgent.isApp() && cordova.plugins.AliPay) {
                    param.payType = "aliapp";
                } else {
                    param.payType = "aliwap";
                }
            }
            param.returnUrl = location.href;
            this.getPayParams(param).then(lang.hitch(this, function (result) {
                window.free();
                if (result.data.ok) {
                    this.invokePayParam(lang.mixin({}, param, result.data.item), def, result);
                } else {
                    // 统一由前台提示错误信息
                    def.resolve(result);
                }
            }));
            return def;
        },
        // 获取支付参数
        invokePayParam: function (params, def, result) {
            if ("weixin" == params.payType) {
                //微信网页支付
                WeixinJSBridge.invoke("getBrandWCPayRequest", params, function (res) {
                    def.resolve(result);
                });
            } else if ("wxNative" == params.payType) {
                //微信扫码支付
                this._payWithWeiXinQr(params, def, result);
            } else if ("wxApp" == params.payType) {
                // 微信APP支付
                var payParams = {
                    partnerid: params.partnerid,
                    prepayid: params.prepayid,
                    noncestr: params.noncestr,
                    timestamp: params.timestamp,
                    sign: params.paySign,
                };
                Wechat.sendPaymentRequest(payParams, function () {
                    alert("支付成功");
                    def.resolve(result);
                }, function (reason) {
                    alert("支付失败: " + reason);
                    def.resolve(result);
                });
            } else {
                if (params.payType === "aliwap") {
                    this._payWithAliWapParam(params, def, result);
                } else {
                    var payInfo = decodeURIComponent(params.payQueryString);
                    cordova.plugins.AliPay.pay(payInfo, function success(e) {
                        def.resolve(result);
                    }, function error(e) {
                        alert("支付失败: " + e.memo);
                        def.resolve(result);
                    });
                }
            }
        },
        // 微信支付生成支付二维码
        _payWithWeiXinQr: function (params, def, result) {
            //生成二维码
            require([
                "cq/util/image/qrCode"
            ], function (qrCode) {
                if (paymentQrDiv && paymentQrDiv.parentNode) {
                    paymentQrDiv.parentNode.removeChild(paymentQrDiv);
                }
                var div1 = paymentQrDiv = domConstruct.create("div", {
                    "class": "paymentQrDiv"
                }, document.body), img1 = domConstruct.create("div", {
                    innerHTML: window.WeixinJSBridge ? "请长按二维码支付喔~~" : "微信扫一扫支付",
                    style: "font-size:20px;color:green;margin-top: 3%;"
                }, div1);
                img1 = domConstruct.create("img", {
                    "class": "paymentQrImg",
                    src: qrCode.generatePNG(params.code_url, {
                        modulesize: 5,
                        margin: 2,
                        ecclevel: "M"
                    })
                }, div1);
                img1 = domConstruct.create("span", {
                    innerHTML: "完成",
                    "class": "paymentQrBtn"
                }, div1);
                img1.onclick = function () {
                    div1.parentElement.removeChild(div1);
                    div1 = null;
                    def.resolve(result);
                };
                img1 = domConstruct.create("span", {
                    innerHTML: "取消",
                    "class": "paymentQrBtn"
                }, div1);
                img1.onclick = function () {
                    div1.parentElement.removeChild(div1);
                    div1 = null;
                    def.resolve(result);
                };
            });
        },
        // 支付宝支付打开浏览器
        _payWithAliWapParam: function (params, def, result) {
            if (params.payUrl) {
                if (window.WeixinJSBridge) {
                    return window._AP ? _AP.pay(params.payUrl) : domConstruct.create("script", {
                        type: "text/javascript",
                        src: "/static/alipay/ap.js",
                        onload: function () {
                            setTimeout(function(){
                                _AP.pay(params.payUrl);
                            },300);
                        }
                    }, document.body);
                } else if (userAgent.isMobile()) {
                    if (window.cordova) {
                        var ref = window.open(params.payUrl, "_blank");
                        ref.addEventListener("exit", function () {
                            def.resolve(result);
                            ref.removeEventListener("exit", function () { });
                        });
                        return;
                    }
                    require(["cq/util/edit/confirmFrame"], function (frame) {
                        frame.show(params.payUrl).then(function () {
                            def.resolve(result);
                        });
                    });
                } else {
                    window.location.href = params.payUrl;
                }
            } else {
                def.resolve(result);
            }
        },
        onListClick:function(){
            
        }
    });
    var o = new O();
    o.Bo = O;
    return o;
});