/**
 * Time 2017/04/07
 * Name 杜飞军
 * Docs
 *      用于处理周期配送设置
 */
define([
    'dojo/_base/declare',
    'dojo/_base/lang',
    'dojo/_base/array',
    "dojo/Deferred",
    'dojo/query',
    'dojo/dom-construct',
    'dojo/dom-class',
    '../BoBase'
], function (declare, lang, array, Deferred, query, domConstruct, domClass, BoBase) {
    var TrgBo = declare("", [BoBase.B], {
        modalName: 'ts/trgMode',
        fields: {
            deliveryCycle: {
                type: "options",
                label: "配送周期",
                options: {
                    1: "日",
                    2: "周",
                    3: "月",
					4: "每两周"
                },
                canEdit: true,
                notNull: true
            },
            // 配送时间
            deliveryStatus: {
                type: 'button',
                label: '配送时间',
                action: 'getDelivery'
            },
            onSave: {
                type: 'button',
                label: '保存',
                action: 'onSave'
            }
        },
        getEdtLayout: function () {
            return [{
                type: 'form',
                fields: this.getFormItems(['deliveryCycle', 'deliveryStatus', 'onSave'], true)
            }];
        },
        loadOne: function (params) {
            return this.exec('loadOne', params || {});
        },
        initData: function () {
            var def = new Deferred();
            this.loadOne({}).then(function (data) {
                if (data && data.data && data.data.ok == 1) {
                    def.resolve(data.data.item);
                } else {
                    def.resolve({});
                }
            });
            return def;
        },
        onChangeField: function (widget, field, fieldVl) {
            if (field == 'deliveryCycle') {
                this.initDelivery(widget);
            }
        },
        // 初始化周期运送时间
        initDelivery: function (widget) {
            var data = widget.getData();
            if (!data.deliveryCycle) {
                alert('请先选择配送周期');
                return;
            }
            var node = widget.getElementByField('deliveryStatus');
            domConstruct.empty(node);
            switch (data.deliveryCycle.toString()) {
                case '1':
                    domConstruct.place(this.createEveryDay(widget), node, 'last');
                    break;
                case '2':
                    domConstruct.place(this.createWeek(widget), node, 'last');
                    break;
                case '3':
                    domConstruct.place(this.createLuna(widget), node, 'last');
                    break;
				case '4':
					domConstruct.place(this.createWeek(widget), node, "last");
					break;
                default:
                    node.innerHTML = '配送时间';
            }
        },
        // 创建每天的显示内容
        createEveryDay: function (widget) {
            var node;
            node = domConstruct.create('div', {
                "class": 'trg-pack'
            });
            domConstruct.create('div', {
                "class": 'trg-list',
                innerHTML: '每天'
            }, node);
            domConstruct.create('div', {
                "class": 'trg-list',
                innerHTML: '工作日'
            }, node);
            domConstruct.create('div', {
                "class": 'trg-list',
                innerHTML: '周末'
            }, node);
            widget.connect(node, 'click', lang.hitch(this, function (e) {
                var dom = e.target;
                if (domClass.contains(dom, 'trg-list')) {
                    query('.trg-list', node).forEach(function (item) {
                        domClass.remove(item, 'trg-selector');
                    });
                    domClass.add(dom, 'trg-selector');
                }
            }));
            return node;
        },
        onDataReady: function (widget) {
            var data = widget.getData();
            if (data.deliveryStatus) {
                this.initDelivery(widget);
                var node = widget.getElementByField('deliveryStatus');
                var deliveryData = data.deliveryStatus.split('');
                query('.trg-list', node).forEach(function (item, key) {
                    if (deliveryData[key] == 1) {
                        domClass.add(item, 'trg-selector');
                    }
                });
            }
        },
        // 创建每周的显示内容
        createWeek: function (widget) {
            var node;
            node = domConstruct.create('div', {
                "class": 'trg-pack'
            });
            var week = ['周日', ' 周一', '周二', '周三', '周四', '周五', '周六'];
            array.forEach(week, function (item, key) {
                domConstruct.create('div', {
                    "class": 'trg-list',
                    innerHTML: item
                }, node);
            });
            widget.connect(node, 'click', this.trgSelector);
            return node;
        },
        // 创建每月显示的内容
        createLuna: function (widget) {
            var node;
            node = domConstruct.create('div', {
                "class": 'trg-pack'
            });
            array.forEach(Array.apply(null, {
                length: 31
            }), function (item, key) {
                domConstruct.create('div', {
                    "class": 'trg-list',
                    innerHTML: key + 1 + '号'
                }, node);
            });
            widget.connect(node, 'click', this.trgSelector);
            return node;
        },
        // 选中的处理
        trgSelector: function (e) {
            var node = e.target;
            if (domClass.contains(node, 'trg-list')) {
                if (domClass.contains(node, 'trg-selector')) {
                    domClass.remove(node, 'trg-selector');
                } else {
                    domClass.add(node, 'trg-selector');
                }
            }
        },
        // 获得初始化内容
        getDelivery: function (data, widget) {
            if (!data.deliveryCycle) {
                alert('请先选择配送周期');
                return;
            }
        },
        onSave: function (data, widget) {
            var msg = this.getDataValidMsg(data, ['deliveryCycle']);
            if (msg) {
                alert(msg);
                return false;
            }
            var node = widget.getElementByField('deliveryStatus');
            // 初始化配送事件
            this.deliveryData = '0000000000000000000000000000000';
            this.deliveryData = this.deliveryData.split('');
            query('.trg-list', node).forEach(lang.hitch(this, function (item, key) {
                if (domClass.contains(item, 'trg-selector')) {
                    this.deliveryData[key] = 1;
                }
            }));
            var selector = array.some(this.deliveryData, function (item) {
                return item == 1;
            });
            this.deliveryData = this.deliveryData.join('');
            if (!selector) {
                alert('请选择配送时间');
                return;
            }
            window.busy();
            data.deliveryStatus = this.deliveryData;
            this.exec("save", data).then(function (data) {
				window.free();
                if (data && data.data && data.data.ok === 1) {
                    alert('保存成功');
					history.back();
                } else {
                    alert(data&&data.data&&data.data.message||'保存失败');
                }
            });
        }
    });
    var trgBo = new TrgBo();
    trgBo.Bo = TrgBo;
    return trgBo;
});