﻿//会员系统公共基类
// 1、提供会员系统的基本属性
// 2、提供会员注册、登录、注销、修改密码、找回密码、个人信息维护的UI和逻辑
// 3、提供当前用户是否登录的判断，提供当前用户的个人身份信息
define([
		"dojo/_base/declare",
		"dojo/_base/lang",
        "dojo/Deferred",
		"dojo/topic",
		"../BoBase",
		"cq/app/config",
		"cq/util/config/config"
	], function (declare, lang, Deferred, topic, BoBase, appConfig, config) {
	appConfig.session = {};
	var UserBaseBo = declare("", BoBase, {
        modalName: "user/user",
        //是否登录
        _hasLogin: false,
        //当前用户ID
        userId: null,
        //当前用户名
        userName: null,
        //当前用户信息
        info: null,
        //当前用户在不同店铺的信息
        shopUserInfo: {},
        fields: {
            // 手机号
            userName: {
                type: "text",
                placeHolder: "请填写手机号",
                label: "手机号",
                //icon: "images/username.png",
                className: "cq-item-username",
                notNull: true
            },
            // 密码输入框
            password: {
                type: "password",
                placeHolder: "必须输入密码",
                label: "密码",
                //icon: "images/password.png",
                //rightIcon: "images/eye.png",
                className: "cq-item-password",
                notNull: true
            },
            repassword: {
                type: "password",
                placeHolder: "请确认密码",
                label: "确认密码",
                className: "cq-item-password",
                notNull: true
            },
            // 登录按钮
            loginButton: {
                type: "button",
                label: "登录",
                action: "login",
                buttonClass: "fullButton"
            },
            // 去注册
            toRegisterViewButton: {
                type: "button",
                label: "去注册",
                action: "moveToRegister",
                buttonClass: "fullButton"
            },
            toLoginViewButton: {
                type: "button",
                label: "去登录",
                action: "moveToLogin",
                buttonClass: "fullButton"
            },
            // 找回密码
            toFindPwdViewButton: {
                type: "button",
                label: "找回密码",
                action: "moveToFindPwd",
                buttonClass: "txtButton"
            },
            // 确认找回密码
            findPwdButton: {
                type: "button",
                label: "确定",
                action: "findPwd",
                buttonClass: "fullButton"
            },
            // 确认注册
            registerButton: {
                type: "button",
                label: "注册",
                action: "register",
                buttonClass: "fullButton"
            },
            // 验证码
            "userName,vcode": {
                type: "vcode",
                errorMsg: "请输入正确的11位电话号码",
                unExists: "账号已经存在,请直接登录",
                notNull: true
            },
            // 确认密码
            confirmPassword: {
                type: "password",
                placeHolder: "请再次输入密码",
                label: "确认密码",
                //icon: "images/password.png",
                //rightIcon: "images/eye.png",
                className: "cq-item-password",
                notNull: true
            },
            // 店铺名称
            shopName: {
                type: "text",
                placeHolder: "取一个好听的店铺名吧",
                label: "店铺名",
                className: "shopName",
                notNull: true
            },
            // 添加店铺确认按钮
            addShopSave : {
                type:"button",
                label:"确认",
                action:"createShopByUser",
                className:"fullButton"                
            },
            logo: {
                type: "image",
                label: "头像",
                sizeType: "cut",
                size: {
                    width: 128,
                    height: 128
                }
            },
            pName: {
                type: "text",
                placeHolder: "请输入姓名",
                label: "姓名"
            },
            nickName: {
                type: "text",
                placeHolder: "",
                label: "昵称"
            },
            amount: {
                type: "number",
                placeHolder: "",
                label: "余额"
            },
            phone: {
                type: "tel",
                placeHolder: "",
                label: "手机号"
            },
            birthDay: {
                type: "date",
                placeHolder: "",
                label: "生日"
            },
            sex: {
                type: "options",
                label: "性别",
                options: {
                    0: "男",
                    1: "女",
                    2: "保密"
                },
                canEdit: true
            },
            userDesc: {
                type: "textarea",
                className: "large-width large-high3",
                placeHolder: "",
                label: "简介"
            },
            address: {
                className: "large-width",
                type: "text",
                label: "地址"
            },
            score: {
                type: "options",
                placeHolder: "",
                label: "星级",
                options: {
                    1: "1星级客户",
                    2: "2星级客户",
                    3: "3星级客户",
                    4: "4星级客户",
                    5: "5星级客户"
                }
            },
            mdfUserInfoButton: {
                className: "fullButton",
                type: "button",
                label: "确认",
                action: "mdfUserInfo"
            },
            oldPassword: {
                type: "password",
                placeHolder: "请输入原密码",
                label: "原密码"
            },
            newPassword: {
                type: "password",
                placeHolder: "请输入新密码",
                label: "新密码"
            },
            confirmPassword: {
                type: "password",
                placeHolder: "请再次输入密码",
                label: "确认密码",
                icon: "images/password.png",
                //rightIcon: "images/eye.png",
                className: "cq-item-password"
            },
            changePasswordButton: {
                className : "fullButton",
                type: "button",
                label: "确定",
                action: "onChange"
            },
            contactName:{
                className: "large-width",
                type: "text",
                label: "姓名"
            },
            tel:{
                className: "large-width",
                type: "text",
                label: "电话"
            },
            detailAddress:{
                className: "large-width",
                type: "text",
                label: "详细地址"
            },
            "provinceId,cityId,countyId,distAddr": {
                type : "district",
                label : "区划",
                canEdit : true
            },
            amont:{
                className: "large-width",
                type: "text",
                label: "充值金额"
            },
            "userId.userName":{
                type: "text",
                placeHolder: "请填写手机号",
                label: "手机号",
                className: "cq-item-username",
                notNull: true
            },
            isDefault:{
                className: "large-width",
                type: "checkbox",
                label: "设为默认地址"
            }
        },
        // 登录Layout
        getLoginLayout: function () {
            // 手机号
            // 密码
            // 确认登录
            if (!this.loginLayout) {
                this.loginLayout = [{
                        type: "form",
                        fields: this.getFormItems([
                                "userName", "password", "loginButton", "toRegisterViewButton", "toFindPwdViewButton"
                            ])
                    }
                ];
            }
            return this.loginLayout;
        },
        // 获取用户上次登录信息
        getUserInfo: function () {
            var obj = {};
            var shopId = this.getShopId();
            if (localStorage.getItem(shopId + "userName")) {
                obj.userName = localStorage.getItem(shopId + "userName");
            }
            if (localStorage.getItem(shopId + "password")) {
                obj.password = localStorage.getItem(shopId + "password");
                obj.rememberPassword = true;
            }
            if (localStorage.getItem(shopId + "rememberPassword")) {
                this.login(obj);
            }
            return obj;
        },
        //登录成功的后续处理
        _onLoginSuccess: function (info) {
            if (!appConfig.session)
                appConfig.session = {};
            appConfig.session.userId = info._id;
            appConfig.session.userName = info.userName;
            appConfig.session.userInfo = info;
            if (!appConfig.session.shopUserInfo)
                appConfig.session.shopUserInfo = {};
            appConfig.session.shopUserInfo[this.getShopId()] = info;
            setTimeout(function () {
                topic.publish("/cq/biz/user/user/hasLogin", true);
            }, 100);
        },
        // 执行登录
        doLogin: function (data) {
            return this.exec("loginByPassword", {
                userName: data.userName,
                password: data.password
            }).then(lang.hitch(this, function (result) {
                    if (result.data.ok == 1) {
                        this._onLoginSuccess(result.data.item);
                    } else {
                        delete appConfig.session.userId;
                    }
                    return result;
                }));

        },
        // 登录
        login: function (data, widgets) {
            var msg = this.getDataValidMsg(data, ["userName", "password"]);
            if (msg) {
                alert(msg);
                return;
            }
            return this.doLogin(data).then(lang.hitch(this, function (datas) {
                    if (datas.data.ok == 1) {
                        window.moveToView(appConfig.nextPageOnLogin || "shopList-auto");
                    } else {
                        alert(datas.data.msg || "无效的用户名或密码");
                    }
                }));
        },
        //获取当前用户ID
        getCurrentUserId: function () {
            return appConfig.session.userId;
        },
        // 检查当前用户的登录状态
        // 返回值:
        //      true/false
        hasLogin: function () {
            if (!appConfig.session.userId) {
                return this.exec("getCurrentUser", {}).then(lang.hitch(this, function (data) {
                        if (data && data.data && data.data.item &&
                            data.data.item._id) {
                            this._onLoginSuccess(data.data.item);
                        }
                        return !!appConfig.session.userId;
                    }));
            } else {
                var def = new Deferred();
                def.resolve(!!appConfig.session.userId);
                return def;
            }
        },
        // 找回密码Layout
        getFindPwdLayout: function () {
            // 手机号
            // 验证码
            // 密码
            // 确认密码
            if (!this.findPwdLayout) {
                this.findPwdLayout = [{
                        type: "form",
                        fields: this.getFormItems([
                                "userName,vcode", "password", "confirmPassword", "findPwdButton", "toLoginViewButton"
                            ])
                    }
                ];
                this.findPwdLayout[0].fields[0].canToLogin = false;
            }
            return this.findPwdLayout;
        },
        // 找回密码
        findPwd: function (data, widget) {
            // 检查输入合法性
            var msg = this.getDataValidMsg(data, ["userName", "vcode", "password", "confirmPassword"]);
            if (msg) {
                alert(msg);
                return;
            } else if (data.password != data.confirmPassword) {
                alert("两次密码不一致");
                return;
            }
            // 找回密码
            this.exec("resetPassword", {
                "userName": data.userName,
                "password": data.password,
                "vCode": data.vcode,
                "shopId": this.getShopId()
            }).then(lang.hitch(this, function (result) {
                    if (result.data.ok === 1) {
                        widget.setData("");
                        window.moveToView("login");
                    } else {
                        alert(result.data.msg);
                    }
                }));
        },
        //消费者端，通过id加载用户详情
        loadUser:function(params){
            var _id = params && params.cqViewParam && params.cqViewParam.params;
            if(_id) return this.loadById(_id);
            else return {};
        },
        loadById: function (id) {
            return this.exec("loadById", {
                _id:id
                }).then(lang.hitch(this, function (userData) {
                return userData.data.item;
            }));
        },
        //获取用户数据
        getMdfUserInfoData:function(){
            var _id = appConfig.session.userId
            return this.loadById(_id);
        },
        // 修改密码数据处理
        onChange: function (data, widgets) {
            // 新密码和老密码不能一样
            if (data.oldPassword === data.newPassword) {
                alert("原密码和新密码不能一样");
                return;
            }
            if (!data.newPassword || data.newPassword !== data.confirmPassword) {
                alert("新密码必须输入，且两次输入的密码需一致");
                return;
            }
            this.exec("modify_password_by_id", {
                _id: this.userId,
                password: data.oldPassword,
                newPassword: data.newPassword,
                shopId: config.getPlatformShopId()
            }).then(lang.hitch(this, function (result) {
                    if (result.data.ok === 1) {
                        widgets.setData("");
                        //清除原先的密码
                        localStorage.removeItem(this.loginShopId + "password");
                        history.back();
                    } else {
                        widgets.updateByField("oldPassword");
                        alert("修改失败，请检查原密码是否正确");
                    }
                }));
        },
        // 添加店铺管理员Layout
        addShopAdminLayout: function () {},
        // 加载当前店铺店铺管理员
        getShopAdminUrl: function () {},
        // 添加店铺管理员
        addShopAdmin: function () {},
        // 注销登录
        logout: function () {
            localStorage.removeItem(this.loginShopId + "password");
            localStorage.removeItem(this.loginShopId + "rememberPassword");
            topic.publish("whz/shop/changed", {
                _id: this.loginShopId
            });
            delete this.userId;
            delete this.info;
            setTimeout(function () {
                topic.publish("/cq/biz/user/user/hasLogin", false);
            }, 100);
            return this.exec("logout", {
                shopId: this.getShopId()
            });
            //user.logout().then(function () {
            //    window.moveToView("login");
            //});
        },
        // 当前用户个人信息Layout
        userCLayout: function () {
            // 我的头像
            // 我的
        },
        // 修改个人信息Layout
        getMdfUserInfoLayout: function () {
            // 头像
            // 昵称
            // 性别
            // 生日
            // 地区
            // 电话
            return [{
                type: "form",
                fields: this.getFormItems([
                    "logo",
                    "pName",
                    "phone",
                    "birthDay",
                    "sex",
                    "address",
                    "mdfUserInfoButton"
                ], true)
            }];
        },
        //依据模版字符串获取用户登录提示信息
        getAppUserTitle: function (cqRefInfo, options) {
            var format = options && options.format || "[客户名称]您好";
            return this.deferred(format.replace("[客户名称]", appConfig.session.userInfo.pName || ""));
        },
        //依据登录状态显示登录、注册、注销和我的信息
        getLoginButtonsForWeb: function (cqRefInfo, options) {
            var def = new Deferred();
            this.hasLogin().then(lang.hitch(this,function(userId){
                if(!userId) def.resolve("<a href='#login' class='menu-item'>登录</a><a href='#register' class='menu-item'>注册</a>") ;
                else def.resolve("<a href='#user' class='menu-item'>我的信息</a><a href='javascript:window.logout();' class='menu-item'>注销</a>");
            }));
            return def;
        },
        // 修改个人信息
        mdfUserInfo: function () {
            var msg = this.getDataValidMsg(data, ["pName", "phone"]);
            if (msg) {
                alert(msg);
                return;
            }
            user.save(data).then(function (dat) {
                user.info.logo = data.logo || user.info.logo;
                user.info.pName = data.pName || user.info.pName;
                user.info.phone = data.phone || user.info.phone;
                if (dat.data.ok === 1) {
                    alert("保存成功");
                } else {
                    alert("保存失败");
                }
            });
        },
        moveToLogin: function () {
            window.moveToView("login");
        },
        moveToFindPwd: function () {
            window.moveToView("findPwd");
        },
        moveToRegister: function () {
            window.moveToView("register");
        },
        // 登录状态切换
        onLoginChanged: function (hasLogin) {
            this._hasLogin = hasLogin;
        },
        getCurrentUser : function(){
            return this.exec("getCurrentUser",{})
                .then(lang.hitch(this, function(result){
                    return result.data.item;
                }));
        },
        // 消费者端我的收入
        loadUserInformation : function (userId) {
            var params = {};
            if (userId) {
                params._id = userId;
            }
            return this.exec("loadUserInformation", params);
        },
    });
	var userBaseBo = new UserBaseBo();
	userBaseBo.Bo = UserBaseBo;
	topic.subscribe("/cq/biz/user/user/hasLogin", function (hasLogin) {
		userBaseBo.onLoginChanged(hasLogin);
	});
	userBaseBo.hasLogin();
	return userBaseBo;
});