define([
    "dojo/_base/declare",
    "dojo/_base/lang",
    "dojo/topic",
    "cq/util/config/config",
    "../consumer",
    "./_ShopAdminMixin",
    "../../shop/shop"
],function(declare,lang, topic,config,consumer,_ShopAdminMixin,shop){
    config.session = config.session||{};
    // 功能：店铺管理员
    //      我的店铺列表，切换店铺，创建店铺
    var ShopAdmin = declare("",[consumer.Bo,_ShopAdminMixin],{
        fields:{
            addAdminButton:{
                type:"button",
                label:"完成",
                action:"addAdminAction",
                className:"fullButton"
            },
            // 新增店铺
            shopName: {
                type:"text",
                label:"店铺名称",
                className:"middle-width",
                placeHolder:"取个好听的名字",
                notNull: true
            },
            // 添加店铺确认按钮
            addShopSave : {
                type:"button",
                label:"确认",
                action:"createShopByUser",
                buttonClass:"fullButton"                
            },
            shopType: {
                type: "options",
                label: "店铺类型",
                options: {
                    "1": "B2C商家",
                    "4": "服务商",
                    "3": "服务中心",
                    "2": "服务站",
                    "5": "运营商"
                },
                selected: "1",
                notNull: true
            },
            shopTypeName: {
                type: "text"
            }
        },
        addShopReload : function(widget) {
            // 判断config中是否配置了商家协议，配置了则显示，没有则隐藏
            var shopProtocolUrl = config.shopProtocol;
            widget.hideByFields(["registerProtocol"]);
            // 如果有一个没有配置则隐藏掉对应的按钮
            if (shopProtocolUrl) {
                widget.showByFields(["registerProtocol"]);
            } 
        },
        onRegisterProtocol : function(data, widget, evt){
            window.open(config.shopProtocol);
        },
        // 我的店铺列表相关接口
        // 加载店铺列表的地址
        shopListUrl:function(){
            return this.getBaseUrl()+"/loadShopsByUser";
        },
        //加载店铺列表的查询条件
        shopListQuery:function(e){
            var query = {};
            query.platformShopId = this.getPlatformShopId();
            query.userId = this.getCurrentUserId();
            if(e&&e.search){
                query.search = e.search;
            }
            return query;
        },
        //设置根据店铺列表数据应该跳转的页面
        toShopView:function(){
            // 若是在微信环境 不允许进我是掌柜
            require(["cq/util/sys/userAgent"], lang.hitch(this, function(userAgent){
                if(userAgent.isWeixin()){
                    //alert("请下载在村头APP管理商家端功能");
                    // 直接跳转到下载页面
                    if(userAgent.isIos()&&config.IOSLoadUrl){
                        window.open(config.IOSLoadUrl);
                    }else if(userAgent.isAndroid()&&config.AndroidUrl){
                        window.open(config.AndroidUrl);
                    }else{
                        alert("请下载APP管理商家端功能");
                    }
                    return;
                }
                window.busy();
                this.exec("loadShopsByUser", {
                    platformShopId: this.getPlatformShopId(),
                    userId: consumer.getCurrentUserId()
                }).then(lang.hitch(this,function(data){
                    window.free();
                    if(data.data.items.length == 1){
                        var shopData =data.data.items[0];
                        this.shopListClick(shopData);
                    }else if(data.data.items.length == 0){
                        window.moveToView('addShop');
                    }else{
                        window.moveToView('shopList');
                    }
                }));
            }));
        },
        //点击店铺列表之后的响应——切换店铺
        shopListClick:function(data,list){
            window.busy();
            return this.setCurrentShopLists(data).then(function(data){
                if(data){
                    window.moveToView("mainB");
                }
                window.free();
            });
        },
        // 设置当前店铺
        setCurrentShopLists:function(shopData){
            return this.setCurrentShop(shopData.shopId).then(function(data){
                if(data){
                    config.session.userInfo = shopData.user || config.session.userInfo;
                    if (config.session.userInfo) {
                        config.session.userInfo.role = shopData.role;
                    }
                }
                return data;
            });
        },
        // 设置当前店铺
        setCurrentShop: function (shop) {
            return this.exec("autoLoginTo", {
                platformShopId: this.getPlatformShopId(),
                targetShopId: shop._id
            }).then(lang.hitch(this,function(data){
                if(data&&data.data&&data.data.item){
                    window.__shop = shop;
                    topic.publish("whz/shop/changed", shop);
                    return data.data.item;
                }else{
                    return null;
                }
            }));
        },
        
        
        //添加店铺相关接口
        // 添加店铺界面
        getEdtLayoutForUser: function(){
            // 店铺名称
            // 确认按钮 
            return [{
                type : "form",
                fields : this.getFormItems([
                    "shopName",
                    // 第一版本发布时没有这么多商家类型 后面版本叠加时再开放出来
                    //"shopType",
                    "addShopSave"
                ],true)
            }];
        },
        // 获取当前用户信息
        getCurrentUser: function () {
            return config.session;
        },
        // 提交创建店铺
        createShopByUser : function(data){
            var msg = this.getDataValidMsg(data, ["shopName"/*, "shopType"*/]);
            if(msg){
                alert(msg);
                return ;
            }
            window.busy();
            // 添加店铺时 默认为一般商家 2017-2-7 刘轩
            data.shopType = 1;
            data.platformShopId = this.getPlatformShopId();
            this.exec("createShop",data).then(lang.hitch(this,function(result){
                window.free();
                if(result&&result.data&&result.data.ok==1){
                    // 创建店铺成功
                    // 设置当前店铺信息
                    this.setCurrentShop(result.data.item.shop).then(lang.hitch(this, function(currentShop){
                        if(currentShop){
                            window.moveToView("mainB");
                        }else{
                            alert("初始化当前店铺信息失败");
                        }
                    }));
					/*if(result2&&result2.data&&result2.data.ok==1){
						window.moveToView("mainB");
					}else{
						alert(result2&&result2.data&&result2.data.result.message||"创建店铺失败");
					}*/
                }else{
                    window.free();
                    alert(result&&result.data&&result.data.message||"创建店铺失败");
                }
            }));
        },
        staffInfo:{},
        getStaffInfo:function(){
            var shopId = shop.getShopId();
            if(this.staffInfo.hasOwnProperty(shopId)){
                return this.defTest(this.staffInfo[shopId]);
            }else if(this.getCurrentUserId()){
                return this.exec("isStaff",{
                    targetShopId:shopId
                }).then(lang.hitch(this,function(data){
                    if(data&&data.data&&data.data.item&&data.data.item.isStaff){
                        this.staffInfo[shopId] = data.data.item;
                    }else{
                        this.staffInfo[shopId] =  null;
                    }
                    return this.staffInfo[shopId];
                }));
            }else{
                return this.defTest(null);
            }
        }
    });
    var shopAdmin = new ShopAdmin();
    shopAdmin.Bo = ShopAdmin;
    return shopAdmin;
});