﻿define([
    "dojo/_base/declare",
    "dojo/_base/lang",
    "dojo/_base/array",
    "dojo/dom-attr",
    "dojo/dom-construct",
    "dojo/query",
    "dojo/Deferred",
    "dijit/registry",
    "dojox/mobile/parser"
],function(declare,lang,array,domAttr,domConstruct,query,Deferred,registry,parser){
    /*
    组件或指定类型的html片段中的关键字：
        {}:代表需要通过属性替换的内容
        {j:xxx}:做属性替换时，需要将xxx转为json字符串之后再做替换
        {j:idMap}:系统保留字段，替换id对照表
        {id}:本模块需要指定id属性，如果没有自定，系统会自动分配
        data-dojo-type="xxx":本节点对应的widget模块id，中间路径以 / 分割
        class为cq-view-container的节点为下级的容器，其他功能组件不能使用这个class
        每个用到的组件原则上都应该通过htmlTemp指定html片段
    layout中的关键字：
        dojoType:本节点对应的模块id，规则同html片段中的data-dojo-type
        fields:编辑字段清单，会先做类型翻译，然后翻译成html
        waitfor:对象之间的联动关系，等待了对象和影响的cqRefInfo中的属性
        id为0的对象是容器对象(包含了本layout定义的所有内容的对象)
    页面build的过程：
        加载生成html片段依赖的模块
        依据layout生成html片段
        加载html片段依赖的模块
        解析html
        将解析之后的结果放到对应的对象中
        build对象之间的关系
    */
    
    // module:
    //      cq/app/builder/_Builder
    
    return declare("",null,{
        // summary:
        //      由layout生成页面内容的对象
        
        // htmls: Object
        //      layout中内容和html模版的对照表
        htmls:null,
        
        // models: Object
        //      依据layout生成页面内容以来的模块的模块id和模块对象对照表
        models:null,
        
        constructor:function(params){
            // summary:
            //      构造一个本对象的新实例，依据参数设置属性
            // tags:
            //      private
            lang.mixin(this,params);
        },
        requireHtmls:function(layout){
            // summary:
            //      依据layout解析以来的模块
            //      加载layout依赖但是还没有加载的模块
            // tags:
            //      private
            var def = new Deferred();
            var modelStrs = [],handledType = {},layoutStr = JSON.stringify(layout);
            array.forEach(layoutStr.match(/("dojoType":"[a-z\/A-Z0-9]+")/g),function(type){
                type = type.replace('"dojoType":"',"");
                type = type.substr(0,type.length-1);
                if(this.htmls[type]||handledType[type]){

                }else{
                    handledType[type] = true;
                    modelStrs.push(type);
                }
            },this);
            if(modelStrs.length){
                require(modelStrs,lang.hitch(this,function(){
                    for(var index=0;index<modelStrs.length;index++){
                        this.models[modelStrs[index]] = arguments[index];
                        if(arguments[index].prototype&&arguments[index].prototype.htmlTemp){
                            this.htmls[modelStrs[index]] = arguments[index].prototype.htmlTemp;
                        }
                    }
                    def.resolve();
                }));
            }else{
                def.resolve();
            }
            return def;
        },
        
        // maxId:Number
        //      当前可用的系统最大id
        // tags:
        //      private
        maxId:0,
        
        getNewId:function(){
            // summary:
            //      系统自动分配一个可用的id
            //      用来为新创建的widget分配id
            return "cq_auto_"+(this.maxId++).toString(36);
        },
        //依据layout生成html片段，并建立id的对照关系，对象之间的联动关系
        buildHtml:function(layout,refMap,idMap,idLayout,idProps){
            // summary:
            //      依据layout生成html片段
            //      依据layout建立layout中定义的id和真实id的对照关系
            //      依据layout建立widget之间的关联关系信息
            // tags:
            //      private
            if(lang.isArray(layout)){
                //数组元素，拼装每一个成员的内容，然后join
                return array.map(layout,function(item){
                    return this.buildHtml(item,refMap,idMap,idLayout,idProps);
                },this).join("");
            }else if(layout instanceof Object){
                var str = this._getLayoutStr(layout,refMap,idMap,idLayout,idProps);
                // 替换属性
                //      需要替换的属性用{}标记
                //      需要将属性转成json字符串之后再替换的，属性名前加j:
                //      {j:idMap}不做替换，当layout生成html后再做替换
                //      若属性为数组时，遍历每一个成员分别生成html片段，链接到一起替换
                //      若属性值不是数字或字符串，依据属性类型找到html模版，递归替换
                return str.replace(/\{[a-z0-9A-Z.\:]+\}/g,lang.hitch(this,function(str){
                    //系统中的id对照关系，DataLayout可能需要控制所有界面元素
                    if(str==="{j:idMap}"){
                        return str;
                    }else if(str==="{id}"){
                        return idMap[layout.id];
                    }
                    var field = str.substr(1);
                    field = field.substr(0,field.length-1);
                    
                    //处理转json的问题
                    var value = "";
                    if(0===field.indexOf("j:")){
                        field = field.substr(2);
                        value = JSON.stringify(layout[field]);
                    }else if(0<="stepsTags,stepsFields,stepsBtns".indexOf(field)){
                        value = [];
                        if(field != "stepsFields"){
                            array.forEach(layout.steps[field], lang.hitch(this, function(ff){
                                this.decorateField(ff, value);
                            }));
                        }else{
                            array.forEach(layout.steps[field], lang.hitch(this, function(ff){
                                // 先解析ff.fields
                                var value1 = [];
                                array.forEach(ff.fields, lang.hitch(this, function(fff){
                                    this.decorateField(fff, value1);
                                }));
                                ff.fields = value1;
                                this.decorateField(ff, value);
                                // console.log(value);
                            }));
                        }
                    }else{
                        value = layout[field];
                    }
                    if(lang.isObject(value)||lang.isArray(value)){
                        return this.buildHtml(value,refMap,idMap,idLayout,idProps);
                    }else{
                        return value||0===value||false===value?value:"";
                    }
                }));
            }else{
                return layout;
            }
        },
        _getLayoutStr:function(layout,refMap,idMap,idLayout,idProps){
            //只要
            layout.type = layout.type||layout.dojoType;
            //识别data-dojo-props
            var str = this.htmls[layout.type]||"";
            var props = str.match(/data-dojo-props='[a-zA-Z0-9\/,"\.\{\}:]+'/);
            if(!(props&&props.length)){
                props = str.match(/data-dojo-props="[a-zA-Z0-9\/,'\.\{\}:]+"/);
            }
            if(props){
                props = props[0];
            }
            if(props){
                if(-1===str.indexOf("{id}")){
                    str = str.replace(props,'id="{id}"');
                }else{
                    str = str.replace(props,"");
                }
            }
            if(0<str.indexOf("{id}")||layout.waitFor){
                if(layout.id){
                    idMap[layout.id] = this.getNewId();
                }else{
                    layout.id = this.getNewId();
                    idMap[layout.id] = layout.id;
                }
                if(layout.waitFor){
                    refMap[layout.id] = layout.waitFor;
                }
                
                idLayout[layout.id] = layout;
                if(props&&props.length){
                    props = props.substr(17);
                    idProps[layout.id] = props.substr(0,props.length-1);
                }
            }
            return str;
        },
        requireModels:function(html){
            // summary:
            //      依据html加载依赖的模块，防止parse时出现异常
            //      识别依赖的模块的方式是匹配 data-dojo-type="moduleId"
            //          注意双引号和moduleId的格式为a/b/c的形式
            // tags:
            //      private
            var def = new Deferred();
            //获取layout依赖的外部模块
            var modelStrs = [],handledModels = {};
            
            array.forEach(html.match(/(data-dojo-type=")([a-z\/A-Z0-9]+)(")/g),function(type){
                type = type.replace('data-dojo-type="',"").replace('"',"");
                if(this.models[type]||handledModels[type]){
                }else{
                    handledModels[type] = true;
                    modelStrs.push(type);
                }
            },this);
            if(modelStrs.length){
                require(modelStrs,lang.hitch(this,function(){
                    for(var index=0;index<modelStrs.length;index++){
                        this.models[modelStrs[index]] = arguments[index];
                    }
                    def.resolve();
                }));
            }else{
                def.resolve();
            }
            return def;
        },
        buildRelation:function(view,refMap,idMap){
            // summary:
            //      依据id对照表和对象关系表，建立对象之间的关系
            // tags:
            //      private
            var widget = null,target = null,targetInfo = null;
            for(var widgetId in refMap){
                //哪些对象受其他对象的影响
                widget = registry.byId(idMap[widgetId]);
                targetInfo = refMap[widgetId];
                if(widget&&targetInfo){
                    for(var targetId in targetInfo){
                        target = registry.byId(idMap[targetId]);
                        if(target){
                            widget.observe(target,targetInfo[targetId]);
                        }
                    }
                }
            }
        },
        //将html拼装为完整的html内容
        joinHtmls:function(htmls,layout){
            // summary:
            //      将顶层html片段拼装成完整的html，移动版和桌面版的拼装方式可能不同
            var html = "",header = "",body = "",footer = "";
            for(var index=0;index<htmls.length;index++){
                if(layout[index].type=="splitterVR"||layout[index].type=="viewContainer"){
                    body = htmls[index];
                }else if(body){
                    footer += htmls[index];
                }else{
                    header += htmls[index];
                }
            }
            if((header||footer)&&body){
                html = '<div data-dojo-type="dojox/mobile/FixedSplitter" data-dojo-props="orientation:\'V\',variablePane:'+(header?1:0)+'">'+(
                    header&&('<div data-dojo-type="dojox/mobile/Pane">'+header+'</div>')
                )+(
                    body&&('<div data-dojo-type="dojox/mobile/Pane">'+body+'</div>')
                )+(
                    footer&&('<div data-dojo-type="dojox/mobile/Pane">'+footer+'</div>')
                )+'</div>';
            }else{
                html = header||body||footer;
            }
            return html;
        },
        //修改layout的属性，使适合不同的界面布局、移动版和桌面版可能是不同的.
        decorateField:function(field,contents){
            // summary:
            //      处理编辑方式，移动版和桌面版的编辑方式不同，重写本接口实现
            //      为了保证程序的通用性，所有子类支持的编辑方式应该完全相同
        },
        decorateLayout:function(layout){
            // summary:
            //      过滤到所有字段信息，调用decorateField方法
            // tags:
            //      private
            if(layout&&layout.fields&&lang.isArray(layout.fields)){
                var fields = layout.fields;
                layout.contents = [];
                array.forEach(fields,function(field){
                    this.decorateField(field,layout.contents);
                },this);
            }else if(lang.isArray(layout)){
                array.forEach(layout,this.decorateLayout,this);
            }else if(lang.isObject(layout)){
                for(var k in layout){
                    this.decorateLayout(layout[k]);
                }
            }
        },
        //构建界面的总入口
        build:function(layout,view){
            // summary:
            //      依据layout，生成界面，并填充到指定的对象中
            var def = new Deferred();
            this.decorateLayout(layout);
            //加载依赖的html
            this.requireHtmls(layout).then(lang.hitch(this,function(){
                var refMap = {},idMap = {
                    "0":view.get("id")
                },idLayout = {},idProps = {};
                //依据layout生成html
                var htmls = array.map(layout,function(item){
                    return this.buildHtml(item,refMap,idMap,idLayout,idProps);
                },this);
                var html = this.joinHtmls(htmls,layout,view.preventScroll);
                html = html.replace(/\{j\:idMap\}/,JSON.stringify(idMap));
                //依据html加载依赖的模块
                this.requireModels(html).then(lang.hitch(this,function(){
                    //var tempDiv = domConstruct.create("div",{
                    //    style:"width:100%;height:100%;position:absolute;left:-9999px;"
                    //},document.body),
                    setTimeout(lang.hitch(this,function(){
                        var widgets = [];
                        view.containerNode.innerHTML = html;
                        this.parseContent(view.containerNode,widgets,idLayout,idProps,idMap);
                        widgets.forEach(function(widget){
                            widget.startup();
                        });
                        //while(tempDiv.childNodes.length){
                        //    view.containerNode.appendChild(tempDiv.childNodes[0]);
                        //}
                        //tempDiv.parentNode.removeChild(tempDiv);
                        //依据生成html时建立的联动关系，建立对象之间的联动关系
                        this.buildRelation(view,refMap,idMap);
                        view.viewContainerNode = query(".cq-view-container",view.domNode)[0];
                        
                        if(view.onContentReady){
                            view.onContentReady();
                        }
                        def.resolve();
                    }),0);
                }));
            }));
            return def;
        },
        // 将html中的dojo对象实例化
        parseContent:function(tempDiv,widgets,idLayout,idProps,idMap){
            //parser.parse(tempDiv);
            //return;
            var mapId = {};
            for(var id in idMap){
                mapId[idMap[id]] = id;
            }
            array.forEach(query("[data-dojo-type]",tempDiv),function(node){
                var id = domAttr.get(node,"id"),
                    props = idProps[mapId[id]];
                if(!props){
                    props = domAttr.get(node,"data-dojo-props");
                    domAttr.remove(node,"data-dojo-props");
                }
                props = this._getProps(props,idLayout[mapId[id]]||{},idMap);
                var tp = domAttr.get(node,"data-dojo-type");
                props["class"] = (props["class"]||"")+" "+domAttr.get(node,"class");
                widgets.push(new this.models[tp](props,node));
            },this);
            /*array.forEach(query("[dojoType]",tempDiv),function(node){
                var id = domAttr.get(node,"id"),props = this._getProps(idProps[id],idLayout[id],idMap);
                var tp = domAttr.get(node,"dojoType");
                widgets.push(new this.models[tp](props,node));
            },this);*/
        },
        // 依据属性定义和layou定义解析对象的属性
        _getProps:function(props,layout,idMap){
            var res = {};
            if(props){
                var vs = [],idx = 0,getStrProp = function(str){
                    return getProp(str.substr(1).substr(0,str.length-2));
                },getProp = function(str){
                    var key = str.substr(1).substr(0,str.length-2);
                    if(key.indexOf(":")>=0){
                        if(0===key.indexOf("j:")){
                            key = key.substr(2);
                        }
                    }
                    var value = layout[key];
                    value = value||false===value||0===value?value:"";
                    //变量名过长会在编译时被重写，导致eval出错
                    vs.push(value instanceof Object?lang.clone(value):value);
                    return "vs["+(vs.length-1)+"]";
                };
                layout.idMap = idMap;
                props = props.replace(/'\{[a-z0-9A-Z.\:]+\}'/g,getStrProp).replace(/"\{[a-z0-9A-Z.\:]+\}"/g,getStrProp).replace(/\{[a-z0-9A-Z.\:]+\}/g,getProp);
                
                try{
                    res = eval("({"+props+"})");
                }catch(e){
                    console.log(arguments);
                    console.error(e);
                }
            }
            return res;
        }
    });
});
