﻿//依据layout配置处理界面的生成，同时处理对象之间的关联刷新问题
define([
    "./_Builder"
],function(_Builder){
    //由类型或data-dojo-type建立的html片段
    return new _Builder({
        //一些固定的html片段：
        //  依赖的模块在不同的界面风格下不同
        //  不依赖任何模块
        //编辑区域相关的模块的属性要求参考cq/app/widget/_LayoutMixin
        htmls:{
            //顶部工具栏，可以水平和垂直，默认是水平的
            "topBar":'<div class="top-items" data-dojo-type="dojox/mobile/Heading" data-dojo-props="label:\'{label}\'"><div style="float:left;">{left}</div><div style="float:right;">{right}</div></div>',
            //带图标的按钮
            "iconMenuItem":'<span id="{id}" class="menu-item {className}" data-dojo-type="cq/app/widget/mobile/menu/ToolBarButton" data-dojo-props="label:\'{label}\',icon:\'{icon}\',arrow:\'{arrow}\',cqHref:\'{href}\'"></span>',
            //左右分栏对象，右边宽度可变
            "splitterVR":'<div id="{id}" data-dojo-type="dojox/mobile/FixedSplitter" data-dojo-props="orientation:\'H\', variablePane:1"><div data-dojo-type="dojox/mobile/Pane">{left}</div><div data-dojo-type="dojox/mobile/Pane">{right}</div></div>',
            //放置子view的容器
            "viewContainer":'<div id="{id}" class="cq-view-container"></div>',
            //大字体文本，欢迎页的示例
            "bigText":'<div id="{id}" style="font-size:36px;">{content}</div>',
            //底部工具栏类型
            "footerBar":'<div id="{id}" class="hflex {className}">{buttons}</div>',
			//直接放图片
            "img":'<a href="{href}"><img id="{id}" class="{className}" src="{src}" class="{className}"/></a>',
            //可以在任何地方注入html片段的对象
            "content":"{content}{contents}",
            //DIV容器
            "div":'<div class="{className}">{content}</div>',
            //底部工具栏类型
            "flexBar":'<div id="{id}" class="{className} top-items hflex">{buttons}</div>',
            

            //支持数据绑定的节点需要定义的内容
            //  data-cq-field:对应的字段名
            //  _LayoutMixin能够识别到对应类型的节点，具体做法参考文本框
            //      处理并实现以下两个方法——执行的上下文为节点本身
            //          setValue:设置节点的值，依据当前值刷新界面
            //          getValue:依据当前状态获取对应的值，只读对象可以不实现
            //      侦听变更事件，只读对象无需侦听变更
            //          发生变更之后更新_LayoutMixin中保存的数据
            //          并刷新其他相关元素的显示
            //支持数据绑定的组件widget需要提供的属性和方法
            //  field:对应的字段名
            //  setValue:设置值，需要修改自己的值，并刷新显示
            //  getValue:获取值，依据组件当前状态获取编辑对象的值，只读对象不提供这个方法
            //class说明
            //  cq-form:form表单区域
            //  cq-field-input:数据绑定的节点或对象
            //      直接用来显示数据的对象：文本框、checkbox、textarea、select、label(只读的对象显示)等
            //      必须提供data-cq-field属性——要显示或编辑的字段名
            //      若响应数据的变化，需要提供setValue方法
            //      若支持编辑内容，需要提供getValue方法
            //  cq-form-item:form中的一个字段显示对象，包含标签和显示组件
            //      以后可能会依据其他字段的值显示隐藏，这个时字段的根节点
            //  cq-form-label:form中的标签对象，独占一行
            //      只用来显示字段的标题
            //  cq-form-input:form中的输入框对象
            //      输入框(文本框、密码框、搜索框、电话号码、数字等)
            //  cq-form-button:form中的按钮
            //  cq-action:可以点击的对象
            //节点属性说明
            //  data-cq-field:本节点对应的字段
            //  cq-action:点击之后执行操作的操作名
            //TODO：以后扩展应该支持逗号分割的多个字段
            //编辑区域相关
            "form":'<form id="{id}" class="cq-form {formClass}" onsubmit="return false">'+
                '<ul data-dojo-type="dojox/mobile/EdgeToEdgeList">{contents}</ul>'+
            '</form>',
            //一项内容，内部包含标签和输入框内容
            "formItem":'<li id="{id}" tabIndex="-1" class="cq-form-item {className}" data-dojo-type="dojox/mobile/ListItem" data-dojo-props="rightIcon:\'{rightIcon}\',label:\'{label}\',icon:\'{icon}\'" data-cq-field="{field}">{contents}</li>',
            //输入框
            "formInput":'<input id="{id}" class="cq-form-input cq-field-input" data-cq-field="{field}" placeHolder="{placeHolder}" type="{inputType}"/>',
            //按钮
            //"formButton":'<li data-dojo-type="dojox/mobile/ListItem" data-dojo-props="variableHeight:true" style="text-align:center;">'+
            //    '<div id="{id}" class="cq-action cq-form-button {buttonClass}" cq-action="{action}">{label}</div>'+
            //"</li>"
            "buttonItem":'<li id="{id}" tabIndex="-1" style="text-align:center;" class="cq-form-item {className}" data-dojo-type="dojox/mobile/ListItem" data-dojo-props="variableHeight:true">{contents}</li>',
            "formButton":'<div id="{id}" class="cq-action cq-form-button {buttonClass}" data-cq-field="{field}" cq-action="{action}">{label}</div>',
            "group":'<div id="{id}" class="cq-group {groupClass}" groupId="{groupId}" >{contents}</div>'
        },
        //由data-dojo-type到对应的js模块
        models:{},
        //将html拼装为完整的html内容，处理可拖动区域
        joinHtmls:function(htmls,layout,preventScroll){
            // summary:
            //      将多个html片段拼装为一个完整的html片段
            //      依据view的配置，判断中间区域是否可以拖动
            var html = "",header = "",body = "",footer = "",absolute="";
            for(var index=0;index<htmls.length;index++){
                if(layout[index].fixed==="top"){
                    header += htmls[index];
                }else if(layout[index].fixed==="bottom"){
                    footer += htmls[index];
                }else if(layout[index].fixed==="absolute"){
                    absolute += htmls[index];
                }else{
                    body += htmls[index];
                }
            }
            //凡是不是锁定在顶部或底部的都是中间可拖动区域的
            if(body){
                if(-1===body.indexOf("cq-view-container")&&(!preventScroll)){
                    body = '<div data-dojo-type="dojox/mobile/ScrollablePane" data-dojo-props="height:\'100%\'">'+body+'</div>';
                }
                if(header||footer){
                    html = '<div data-dojo-type="dojox/mobile/FixedSplitter" data-dojo-props="orientation:\'V\',variablePane:'+(header?1:0)+'">'+(
                        header&&('<div data-dojo-type="dojox/mobile/Pane">'+header+'</div>')
                    )+(
                        body&&('<div data-dojo-type="dojox/mobile/Pane">'+body+'</div>')
                    )+(
                        footer&&('<div data-dojo-type="dojox/mobile/Pane">'+footer+'</div>')
                    )+'</div>';
                }else{
                    html = body;
                }
            }else{
                html = header||body||footer;
            }
            return (absolute || '') + html;
        },
        
        //已处理的编辑方式：文本框、checkbox、action
        decorateField:function(field,fields){
            var fieldType = ","+field.type+",",fieldItem = null;
            //所有支持的编辑方式都要在这里再写一遍，很奇怪
            if(fieldType===",action,"){
                //按钮
                // 判断fields中最后一个是否为按钮
                // 如果是按钮，则判断btns中个数，大于3则新增一行，否则追加到btns中
                // 如果不是按钮，则新增一行
                //return fields.push({
                //    type: "formButton",
                //    label: field.label,
                //    canEdit:field.canEdit,
                //    action: field.action
                //});
                if(fields.length&&fields[fields.length-1].type=="buttonItem"&&fields[fields.length-1].contents.length<3){
                    fields[fields.length-1].contents.push({
                        type:"formButton",
                        label:field.label,
                        canEdit:field.canEdit,
                        field:field.field,
                        action:field.action,
                        buttonClass:field.buttonClass
                    });
                }else{
                    fieldItem = {
                        type:"buttonItem",
                        className:field.className,
                        contents:[{
                            type:"formButton",
                            label:field.label,
                            field:field.field,
                            canEdit:field.canEdit,
                            action:field.action,
                            buttonClass:field.buttonClass
                        }]
                    };
                }
            }else if(",link,"===fieldType){
                delete field.type;
                field.dojoType = "cq/app/widget/mobile/edit/LinkItem";
                return fields.push(field);
            }else if(-1<",text,tel,password,number,".indexOf(fieldType)){
                //文本
                fieldItem = {
                    type: "formItem",
                    field: field.field,
                    className:field.className,
                    contents: [{
                        type: "formInput",
                        inputType: field.type,
                        field: field.field,
                        canEdit:field.canEdit,
                        placeHolder:field.placeHolder,
                        label:field.label||"",
                        icon:field.icon||""
                    }]
                };
            }else if(",date,"===fieldType){
                //日期
                return fields.push({
                    dojoType:"cq/app/widget/mobile/edit/Date",
                    field:field.field,
                    canEdit:field.canEdit,
                    label:field.label,
                    defaultDate:field.defaultDate||"",
                });
            }else if (",dateTime,"===fieldType){
                //日期
                return fields.push({
                    dojoType:"cq/app/widget/mobile/edit/DateTime",
                    field:field.field,
                    canEdit:field.canEdit,
                    label:field.label
                });
            }else if(-1<",image,".indexOf(fieldType)){
                //单图
                return fields.push({
                    dojoType:"cq/app/widget/mobile/edit/Image",
                    field:field.field,
                    label:field.label,
                    canEdit:field.canEdit,
                    clickImg:field.clickImg,
                    size:field.size||{},
                    sizeType:field.sizeType,
                    cqWidth:field.cqWidth
                });
            }else if(-1<",images,".indexOf(fieldType)){
                //图片
                return fields.push({
                    dojoType:"cq/app/widget/mobile/edit/Images",
                    field:field.field,
                    label:field.label,
                    canEdit:field.canEdit,
                    size:field.size||{},
                    sizeType:field.sizeType,
                    isSetLabel:field.isSetLabel,
                    maxImages:field.maxImages,
                    multiple:field.multiple
                });
            }else if(",location,"===fieldType){
                //地理位置
                return fields.push({
                    canEdit:field.canEdit,
                    label:field.label,
                    dojoType: "cq/app/widget/mobile/edit/Location",
                    "className":field.className,
                    field: field.field,
                    distFields:field.distFields,
                    distFieldsStr:field.distFieldsStr,
                    detailAddrField:field.detailAddrField
                });
            }else if(",areaEditor,"===fieldType){
                // 区域编辑
                return fields.push({
                    canEdit:field.canEdit,
                    label:field.label,
                    dojoType: "cq/app/widget/mobile/edit/AreaEditor",
                    "className":field.className,
                    field: field.field
                });
            }else if(",vcode,"===fieldType){
                //手机号、验证码
                return fields.push({
                    dojoType:"cq/app/widget/mobile/edit/VCode",
                    field: field.field,
                    canEdit:field.canEdit,
                    errorMsg:field.errorMsg,
                    unExists:field.unExists,
                    noIcon:!!field.noIcon,
                    dataProvider:field.dataProvider,
                    smsMethod:field.smsMethod
                });
            }else if(",textarea,"===fieldType){
                return fields.push({
                    dojoType: "cq/app/widget/mobile/edit/TextArea",
                    label: field.label,
                    canEdit:field.canEdit,
                    field: field.field
                });
            }else if(",checkbox,"===fieldType){
                //开关
                fieldItem = {
                    type:"formItem",
                    className:field.className,
                    field:field.field,
                    canEdit:field.canEdit,
                    contents:[{
                        dojoType:"cq/app/widget/mobile/edit/CheckBox",
                        field:field.field,
                        canEdit:field.canEdit,
                        selected:field.selected,
                        className:field.className
                    }]
                };
            }else if(",checkbox1,"===fieldType){
                //复选框
                fieldItem = {
                    type:"formItem",
                    className:field.className,
                    field:field.field,
                    canEdit:field.canEdit,
                    contents:[{
                        dojoType:"cq/app/widget/mobile/edit/CheckBox1",
                        field:field.field,
                        canEdit:field.canEdit,
                        label:field.label,
                        className:field.className
                    }]
                };
            }else if(",options,"===fieldType){
                //枚举值
                return fields.push({
                    dojoType:"cq/app/widget/mobile/edit/Options",
                    field:field.field,
                    canEdit:field.canEdit,
                    options:field.options,
                    selected:field.selected,
                    label:field.label
                });
            }else if(",ref,"===fieldType){
                //引用
                return fields.push({
                    dojoType:"cq/app/widget/mobile/edit/Ref",
                    field:field.field,
                    keyField:field.keyField,
                    label:field.label,
                    canEdit:field.canEdit,
                    searchAttr:field.searchField,
                    pageSize:field.pageSize||20,
                    multiselect:field.multiselect,
                    labelAttr:field.labelFields,
                    dataProvider:field.dataProvider,
                    targetMethod:field.targetMethod,
                    queryMethod:field.queryMethod,
                    setRigthTextMethod:field.setRigthTextMethod,
                    query:field.query,
                    refQuery:field.refQuery
                });
            }else if(",richText,"===fieldType){
                return fields.push({
                    dojoType: "cq/app/widget/mobile/edit/RichText",
                    label: field.label,
                    canEdit:field.canEdit,
                    field: field.field,
                    tempType:field.tempType,
                    isAdvance:field.isAdvance
                });
            }else if(",district,"===fieldType){
                return fields.push({
                    dojoType:"cq/app/widget/mobile/edit/District",
                    label:field.label,
                    canEdit:field.canEdit,
                    selectType:field.selectType,
					rootField:field.rootField,
                    field:field.field,
                    enableAll:field.enableAll
                });
            }else if(",village,"===fieldType){
                return fields.push({
                    dojoType:"cq/app/widget/mobile/edit/Village",
                    label:field.label,
                    canEdit:field.canEdit,
                    selectType:field.selectType,
					rootField:field.rootField,
                    field:field.field,
                    enableAll:field.enableAll
                });
            }else if(",object,"===fieldType){
                return fields.push({
                    dojoType:"cq/app/widget/mobile/edit/ObjEditor",
                    label:field.label,
                    labelField:field.labelField,
                    rightTextField:field.rightTextField,
                    canEdit:field.canEdit,
                    field:field.field,
                    fields:field.fields,
                    canAdd:field.canAdd,
                    dataProvider:field.dataProvider,
                    reloadMethod:field.reloadMethod,
                    fieldChangeMethod:field.fieldChangeMethod,
                    userDefinedValue:field.userDefinedValue,
                });
            }else if(",objects,"==fieldType){
                return fields.push({
                    dojoType:"cq/app/widget/mobile/edit/ObjsEditor",
                    labelField:field.labelField,
                    label:field.label,
                    
                    //displayPage 测试
                    //displayPage:field.displayPage,
                    rightTextField:field.rightTextField,
                    canEdit:field.canEdit,
                    canAdd:field.canAdd,
                    canDelete:field.canDelete,
                    canDnd:field.canDnd,
                    field:field.field,
                    fields:field.fields,
                    defaultData:field.defaultData||{},
                    dataProvider:field.dataProvider,
                    reloadMethod:field.reloadMethod,
                    fieldChangeMethod:field.fieldChangeMethod,
                    userDefinedValue:field.userDefinedValue,
                    arrLength:field.arrLength
                });
            }else  if(",newobjs,"==fieldType){
                return fields.push({
                    dojoType:"cq/app/widget/mobile/edit/ObjsNewTest",
                    field:field.field,
                    canEdit:field.canEdit,
                    items:field.items
                });
            }else if(",tabBox,"==fieldType){
                return fields.push({
                    dojoType:"cq/app/widget/mobile/edit/TabBox",
                    tabItems:field.tabItems
                });
            }else if(",file,"==fieldType){
                return fields.push({
                    dojoType:"cq/app/widget/mobile/edit/FileEditor",
                    field:field.field,
                    canEdit:field.canEdit,
                    label:field.label,
                    value:field.value||{}
                });
            }else if(",score," == fieldType){
                fieldItem = {
                    type:"formItem",
                    className:field.className,
                    field:field.field,
                    canEdit:field.canEdit,
                    contents:[{
                        dojoType:"cq/app/widget/mobile/edit/Score",
                        field:field.field,
                        canEdit:field.canEdit,
                        className:field.className,
                        image:field.image,
                        numStars:field.numStars,
                        value:field.value
                    }]
                };
            }else if(",scored," == fieldType){
                return fields.push({
                    dojoType:"cq/app/widget/mobile/edit/Score",
                    field:field.field,
                    canEdit:field.canEdit,
                    className:field.className,
                    image:field.image,
                    numStars:field.numStars,
                    value:field.value
                });
            }else if(",shelfEditor," == fieldType){
                return fields.push({
                    dojoType:"cq/app/widget/mobile/edit/ShelfEditor",
                    field:field.field,
                    canEdit:field.canEdit
                });
            }else if(",shelf," == fieldType){
                return fields.push({
                    dojoType:"cq/app/widget/mobile/item/Shelf",
                    field:field.field,
                    dataProvider:field.dataProvider,
                    initMethod:field.initMethod
                });
            }else if(",groupEdit," == fieldType){
                return fields.push({
                    dojoType:"cq/app/widget/mobile/edit/GroupEditor",
                    field:field.field,
                    canEdit:field.canEdit,
                    label:field.label,
                    rightText:field.rightText,
                    group:field.group,
                    intervalTop:field.intervalTop,
                    intervalBottom:field.intervalBottom,
                    
                });
            }else if(",materialTags," == fieldType){
                return fields.push({
                    dojoType:"cq/app/widget/mobile/edit/MaterialTags",
                    field:field.field,
                    label:field.label,
                    rightText:field.rightText
                });
            }else if(",imageRef," == fieldType){
                return fields.push({
                    dojoType:"cq/app/widget/mobile/edit/ImageRef",
                    field:field.field,
                    label:field.label,
                    dataProvider:field.dataProvider,
                    tagDataProvider:field.tagDataProvider,
                    targetMethod:field.targetMethod,
                    tagTargetMethod:field.tagTargetMethod,
                    showTags:field.showTags,
                    showType:field.showType,
                    pageSize:field.pageSize||20,
                    addMaterial:field.addMaterial
                });
            }else{
                fields.push(field);
            }
            
            if(fieldItem){
                //添加标签项
                if(field.label){
                    fieldItem.label = field.label;
                }
                if(field.className){
                    fieldItem.className = field.className;
                }
                if(field.icon){
                    fieldItem.icon = field.icon;
                    //delete fieldItem.label;
                }
                if(field.rightIcon){
                    fieldItem.rightIcon = field.rightIcon;
                }
                //添加到编辑列表中
                fields.push(fieldItem);
            }
        }
    });
});
