﻿//本轮变更需要达到的目标
//  view嵌套的问题
//  界面风格可配置性
//  开发和配置分离，系统的可配置性
//  部分操作标准化，列表、编辑界面等
//  样式标准化
//需要处理的问题
//  每个view配置文件的加载机制，提供几种标准的加载配置的方式
//  view和子view的联动机制，通过_RouteMixin来实现，子view的创建，子view的刷新
//  view和view下的组件的联动机制
//  同一view下组件之间的消息传递机制
define([
    "dojo/_base/lang",
    "dojo/_base/xhr",
    "dojo/_base/json",
    "dojo/ready",
    "dojo/Deferred",
    "dojo/dom-class",
    "dojo/dom-style",
    "dojo/topic",
    "dijit/registry",
    "cq/util/config/config",
    "cq/util/sys/log"
],function(lang,xhr,json,ready,Deferred,domClass,domStyle,topic,registry,configUtil,log){
    var config = {},manager = {};
    //APP配置对象，提供配置并启动APP的程序
    lang.mixin(config,{
        //赵春辉   是否微信登陆，默认为false，需要手工指定
        enableWxLogin:false,
        //赵春辉   禁用biz模块，新版本在村头启用了bob模块，判断登陆的方式不同    2017-01-11
        disableBiz:false,
        //界面配置
        view:{},
        //通用界面生成器
        layoutBuilder:null,
        //界面view对象
        ViewRender:null,
        nextPageOnLogin:"",

        //方法
        //加载layout的方式
        loadLayout:null,
        //依据配置生成新的view对象
        createView:function(viewInfo,viewId,parentNode){
            var def = new Deferred();
            config.loadLayout(viewInfo,viewId).then(function(layout){
                var view = new config.ViewRender({
                    subViews:viewInfo.subViews,
                    defaultView:viewInfo.defaultView,
                    toTopInfo:viewInfo.toTopInfo,
                    autoRefresh:viewInfo.autoRefresh,
                    topOpacity:viewInfo.topOpacity,
                    refreshBeforeTop:viewInfo.refreshBeforeTop,
                    lazyOn:viewInfo.lazyOn,
                    canHideTop:viewInfo.canHideTop,
                    title:viewInfo.title||""
                });

                                manager.layoutInfo[viewId].view = view;
                if(viewInfo.preventScroll){
                    view.preventScroll = viewInfo.preventScroll;
                }
                if(viewInfo["class"]){
                    domClass.add(view.domNode,viewInfo["class"]);
                }
                if(viewInfo.className){
                    domClass.add(view.domNode,viewInfo.className);
                }
                parentNode.appendChild(view.domNode);
                view.startup();
                config.layoutBuilder.build(layout,view).then(function(){
                    def.resolve(view);
                });
            });
            return def;
        },
        //依据viewId获取对应view对象，显示和位置由调用方控制
        //      处理动态显示部分信息到浮动出来的容器中
        getLayoutView:function(viewId,parentNode){
            var def = new Deferred(),info = manager.layoutInfo[viewId];
            if(info.view){
                setTimeout(function(){
                    def.resolve(info.view);
                },10);
            }else{
                this.createView(info,viewId,parentNode).then(function(view){
                    info.view = view;
                    def.resolve(view);
                });
            }
            return def;
        }
    });
    
    
    //管理器
    lang.mixin(manager,{
        viewRoads:{
            "":[]
        },
        layoutInfo:{},
        mustLoginView:{},
        mustStaffView:{},
        initWindowMethod:function(){
            //切换到指定view的接口
            window.moveToView = function(view,param,target){
                if("_blank"===target){
                    return window.open("#" + view + (param || 0 === param ? "-" + param : ""),target);
                }
                var url = "#"+view+(param||0===param||false===param?"-"+param:"");
                if(location.hash===url){
                    manager.showView();
                }else{
                    location.href = url;
                }
            };
            window._clearBusyTimer = window._clearBusyTimer||function() {
                if (window._busyTimer) {
                    clearTimeout(_busyTimer);
                    window._busyTimer = null;
                }
            };
            window._initBusyTimer = window._initBusyTimer||function() {
                window._busyTimer = setTimeout(window.free, configUtil.maxBusy||15000);
            };
            var prog = registry.byId("__loaderProgress");
            var busyStackLength = 0;
            if(prog){
                prog.removeOnStop = false;
                prog._start = prog.start;
                prog.start = function(){
                    clearInterval(this.timer);
                    this._start();
                };
            }
            var isFreeing = false;
            window.busy = window.busy||function() {
                busyStackLength++;
                isFreeing = false;
                window._clearBusyTimer();
                if(prog){
                    prog.start();
                }
                domStyle.set("__loader", "display", "");
                window._initBusyTimer();
            };
            window.free = window.free||function() {
                isFreeing = true;
                setTimeout(function() {
                    if(isFreeing){
                        busyStackLength = 0;
                        domStyle.set("__loader", "display", "none");
                        if(prog){
                            prog.stop();
                        }
                    }
                }, 300);
            };
            window.checkFree = window.checkFree||function(){
                if(busyStackLength==1){
                    window.free();
                }
            };
            topic.subscribe("cq/sys/forceRefresh",function(){
                manager.showView();
            });
            //重新定义A标签的href跳转，长按图片出菜单
            require(["cq/util/sys/fixeEnv"]);
        },
        initLoginInfo:function(view,id){
            if(view.mustLogin){
                this.mustLoginView[id] = true;
            }
            if(view.mustStaff){
                this.mustStaffView[id] = true;
            }
        },
        genViewRoads:function(view,id){
            //viewId和layout配置的对照关系
            this.layoutInfo[id] = view;
            //处理登录控制信息
            this.initLoginInfo(view,id);
            if(view.subViews){
                var road = null;
                for(var viewId in view.subViews){
                    //若上级有登录权限控制，下级同样受到控制
                    view.subViews[viewId].mustLogin = view.subViews[viewId].mustLogin||view.mustLogin;
                    view.subViews[viewId].mustStaff = view.subViews[viewId].mustStaff||view.mustStaff;
                    
                    road = this.viewRoads[viewId] = [].concat(this.viewRoads[id]);
                    road.push(viewId);
                    this.genViewRoads(view.subViews[viewId],viewId);
                }
            }
        },
        clearCreatedViews:function(clearHash){
            var view = null;
            //若当前有已打开的页面，删除要销毁的已打开页面记录
            if(this.subViewsId){
                var len = this.subViewsId.length;
                while(len--){
                    if(clearHash[this.subViewsId[len]]){
                        this.subViewsId.splice(len,1);
                    }
                }
            }
            //便利要销毁的页面，若已创建，执行销毁
            for(var viewId in clearHash){
                view = this.layoutInfo[viewId];
                if(view&&view.view){
                    view.view.destroyRecursive();
                    delete view.view;
                }
            }
        },
        _showView:function(canShow,roads,param){
            if(canShow){
                var view = roads[roads.length-1];
                // 调用清理view列表的接口
                this.cleanUpSubViews(view||"main");
                config.view.view.showView(roads,param).then(function(){
                    config.view.view.resize();
                    domStyle.set(config.view.view.domNode,"opacity",1);
                }).then(function(){
                    if(false!==config.enableLog){
                        log.onViewChange({});
                    }
                    //赵春辉   保持兼容性，不能改变参数的含义，以后应该吧参数当做对象使用   2017-08-31
                    var params = new String(view);
                    params.viewId = view;
                    params.viewInfo = manager.layoutInfo[view];
                    topic.publish("widgets/show/view",params);
                });
            }else{
                configUtil.session = config.session||{};
                configUtil.session.loginBackLevel = 2;
                window.moveToView("login");
            }
        },
        _isStaff:function(){
            var def = new Deferred();
            require([
                config.staffModel
            ],function(user){
                user[config.isStaffMethod]().then(function(canShow){
                    def.resolve(!!canShow);
                });
            });
            return def;
        },
        _hasLogin:function(){
            var def = new Deferred();
            require([
                config.userModel
            ],function(user){
                user[config.hasLoginMethod]().then(function(canShow){
                    def.resolve(canShow);
                });
            });
            return def;
        },
        showView:function(){
            domStyle.set(config.view.view.domNode,"opacity",0);
            var hash = location.hash.substr(1).split("-");
            var view = hash.shift()||"main";
            var roads = this.viewRoads[view],param = hash.join("-");
            if(window.viewParam&&window.viewParam[view||"main"]){
                return;
            }
            //window.busy();
            var viewId = roads[roads.length-1];
            //将viewId记录到全局   在处理返回键时用到
            window.curView = viewId||"";
            //显示内容之后重算界面内元素大小
            if(this.mustStaffView[viewId]){
                this._isStaff().then(lang.hitch(this,function(isStaff){
                    this._showView(isStaff,roads,param);
                }));
            }else if(this.mustLoginView[viewId]){
                this._hasLogin().then(lang.hitch(this,function(hasLogin){
                    this._showView(hasLogin,roads,param);
                }));
            }else{
                this._showView(true,roads,param);
            }
        },
        startApp:function(){
            // 张松   解决微信自动登录问题  2017/2/8
            require([
                "dojo/cookie",
                "cq/util/sys/userAgent"
            ],function(cookie,userAgent,shop){
                // 只有微信环境才做自动登录
                if(config.enableWxLogin&&userAgent.isWeixin()&&window.g_sys_info&&window.g_sys_info.wxVerification && window.g_sys_info.wxAutoLogin){
                    if(!cookie(window.g_sys_info._id + "_openid")){
                        window.location = "/w/init_openid/" + window.g_sys_info._id +
                            "?returnUri=" + encodeURIComponent(location.href);
                        return;
                    }
                    if(window.__wxjs_environment!=='miniprogram'&&window.g_sys_info.wxPayShopId && !cookie(window.g_sys_info.wxPayShopId + "_openid")){
                        window.location = "/w/init_openid/" + window.g_sys_info.wxPayShopId +
                            "?returnUri=" + encodeURIComponent(location.href);
                        return;
                    }
                }
            });
            require([
                "dojo/_base/config"
            ],function(config){
                config.parseOnLoad = false;
            });
            this.initWindowMethod();
            //初始化登录控制对象
            if(!config.userModel){
                config.userModel = config.disableBiz?"bob/user/consumer":"cq/biz/user/user";
                config.hasLoginMethod = "hasLogin";
            }
            if(!this.staffModel){
                config.staffModel = config.disableBiz?"bob/user/shopAdmin/c":"cq/biz/user/user";
                config.isStaffMethod = config.disableBiz?"getStaffInfo":"hasLogin";
            }
            
            //处理view之间的嵌套关系，生成每个view到达的路径
            this.genViewRoads(config.view,"");
            lang.mixin(this.mustLoginView,this.mustStaffView);
            var topPane = registry.byId("topPane");
            //加载根view对应的程序
            config.createView(config.view,"",topPane&&topPane.containerNode||document.body).then(function(view){
                //view的类型从布局对象那里读取，一种布局只能有一种view类型
                config.view.view = view;
                //监听地址栏的变化，从根view发起响应
                config.view.view.connect(window,"onhashchange",function(){
                    manager.showView();
                });
                //当切换店铺时，需要销毁与员工相关的view
                config.view.view.subscribe("whz/shop/changed",function(){
                    manager.clearCreatedViews(manager.mustStaffView);
                });
				if(config.disableBiz){
					//当切换登陆时，销毁登陆相关的view
					config.view.view.subscribe("/cq/biz/user/user/hasLogin",function(){
						manager.clearCreatedViews(manager.mustLoginView);
					});
				}
                //执行view的第一次切换响应
                manager.showView();
            });
        },
        //已创建的需要缓存的页面
        cackedPages:{},
        // 清理历史view
        cleanUpSubViews:function(viewId){
            // 张松  2018/10/15
            // 暂时不启用该功能
            // 因为每次进入页面时，没有做缓存的页面都会重新做一次初始化，这个操作不合理
            return;
            
            
            if(viewId==='main'){
                if(!this.layoutInfo[viewId]){
                    viewId = "login";
                }
            }
            if(this.layoutInfo[viewId].cachePage){
                this.cackedPages[viewId] = true;
                return;
            }
            
            // 如果不是"uniqName_1_0"，直接返回
            if(!config.view.subViews){
                return;
            }
            // 创建一个队列保存viewId
            if(!this.subViewsId){
                this.subViewsId = [];
                this.subViewsId.push(viewId);
            }
            
            // 1、若新view需要缓存，直接返回
            // 2、若新的view不是view队列中的最后一个，向view队列中追加新的view的view
            // 3、否则，直接返回
            if(this.subViewsId[0]===viewId){
                // A---B---A
                // A---A---A
                this.subViewsId.shift();
                this.subViewsId.push(viewId);
                return;
            }
            if(this.subViewsId.length===1){
                this.subViewsId.push(viewId);
                return;
            }
            if(this.subViewsId.length>1&&this.subViewsId[1]===viewId){
                return;
            }
            this.subViewsId.push(viewId);
            if(this.subViewsId.length>2){
                // 4、若view队列的长度===3，获取view队列中的第一个view(要删除的view)，并从view队列中删除
                var destroyViewId = this.subViewsId.shift();
                // 判断页面是否被销毁
                if(destroyViewId===''){
                    return;
                }
                // 5、若要删除的view不是新的view，调用销毁view的接口销毁对应view
                this.destroyView(destroyViewId);
            }
        },
        destroyView:function(destroyViewId){
            var destroyRoad = lang.clone(this.viewRoads[destroyViewId]);
            var rouds = [this.viewRoads[this.subViewsId[0]],this.viewRoads[this.subViewsId[1]]];
            for(var view in this.cachePage){
                rouds.push(this.viewRoads[view]);
            }
            var views = ","+rouds.map(function(roud){
                return roud.join(",");
            }).join(",")+",";
            var len = destroyRoad.length;
            while(len--){
                if(-1===views.indexOf(","+destroyRoad[len]+",")){
                    this.layoutInfo[destroyRoad[len]].view.destroyRecursive();
                    delete this.layoutInfo[destroyRoad[len]].view;
                }else{
                    break;
                }
            }
        }
    });
    
    ready(function(){
        manager.startApp();
    });
    
    return config;
});
