define([
    "dojo/_base/declare",
    "dojo/_base/lang",
    "dojo/dom-construct",
    "dojo/dom-class",
    "dojo/dom-style",
    "dojo/query",
    "dojo/topic",
    "dijit/registry",
    "dojox/mobile/View",
    "cq/util/sys/userAgent",
    "../_ViewBase"
],function(declare,lang,domConstruct,domClass,domStyle,query,topic,registry,View,userAgent,_ViewBase){
    return declare("",[View,_ViewBase],{
        baseClass:"mblView cq-view-fs",
        // 标记当前页面是否可见
        isDisplay:false,
        //获取scrollPane
        getSubVScroll:function(){
            var widget = null,scrollNodes = query(".mblScrollablePane",this.domNode);
            for(var index=0;index<scrollNodes.length;index++){
                widget = registry.byNode(scrollNodes[index]);
                if(widget&&widget.scrollDir==="v"){
                    return widget;
                }
            }
            return null;
        },
        //当view里面的内容创建完毕后调用
        onContentReady:function(){
            this.scrollable = this.getSubVScroll();
            if(!this.scrollable){
                return;
            }
            if(this.toTopInfo||this.refreshBeforeTop){
                if(this.toTopInfo){
                    //初始化回到顶部的按钮和事件
                    this.toTopNode = domConstruct.create("img",{
                        className:this.toTopInfo.iconClass,
                        src:this.toTopInfo.src,
                        style:"display: none;position: absolute;bottom: 60px;right: 40px;width: 50px;height: 50px;"
                    },this.scrollable.domNode);
                    this.connect(this.toTopNode,"onclick",function(){
                        this.scrollable.scrollTo({
                            x:0,
                            y:0
                        });
                    });
                }
                if(this.refreshBeforeTop){
                    //初始化拖动刷新更多的节点
                    this.topReloadNode = domConstruct.create("div",{
                        style:"display:none;position: absolute;top:10px;width:100%;height: 80px;text-align: center;"
                    },this.scrollable.domNode,"first");
                }
                //监听滑动，并作出响应
                this.connect(this.scrollable,"slideTo",this.onSubScroll);
                this.connect(this.scrollable,"scrollTo",this.onSubScroll);
            }
            if(this.lazyOn){
                // 监听view的onAfterTransitionIn和onAfterTransitionOut事件，分别做标记，以确定是否需要加载scrollable中可见区域的图片
                this.connect(this, "onAfterTransitionIn", lang.hitch(this, function(){
                    this.isDisplay = true;
                }));
                this.connect(this, "onAfterTransitionOut", lang.hitch(this, function(){
                    this.isDisplay = false;
                }));
                // 监听resize    ---    页面第一次进入时，加载可见区域内的图片
                this.connect(this.scrollable,"resize",this.loadLazyImgs);
                // 监听onBeforeScroll     ---     拖动时，加载可见区域内的图片
                this.connect(this.scrollable,"onBeforeScroll",this.loadLazyImgs);
                // 监听slideTo    ---     当鼠标或手指离开页面，scrollablePane做滚动的动画效果时，加载可见区域内的图片
                this.connect(this.scrollable,"slideTo",this.loadLazyImgs);
                // 如果scrollablepane内部通过TypeContainer做切换，则需要监听tag切换事件
                var tc = query(".cq-typePane", this.scrollable.containerNode);
                if(tc && tc[0]){
                    this.tc = registry.byNode(tc[0]);
                    this.connect(this.tc, "clickTag", this.loadLazyImgs);
                }
            }
            this.hideTop();
            this.changeTop();
        },
        // 加载可见区域内的图片
        // evt 
            // 1 执行resize时evt为undefined
            // 2 执行面板滚动时，evt为滚动事件{x:int,y:int}
            // 3 执行tag切换时，evt为tag节点
        loadLazyImgs:function(evt){
            // 当前view可见，加载图片
            if(this.isDisplay){
                // 判断是否tag切换
                if(evt && evt.tagName){
                    evt = this.tc.getTag(evt);
                    // 当前view
                    for(var index = 0;index < this.tc.tags.length;index++){
                        if(this.tc.tags[index].domNode == evt){
                            this.tcCurrView = this.tc.views[index];
                            break;
                        }
                    }
                }
                if(this._lazyTimer){
                    clearTimeout(this._lazyTimer);
                }
                this._lazyTimer = setTimeout(lang.hitch(this,this._loadLazyImg),300);
            }
        },
        _loadLazyImg:function(){
            var visibilityHeight = this.scrollable.domNode.clientHeight+(this.lazyOn&&this.lazyOn.cacheHeight-0||0);
            visibilityHeight -= this.lastScrollTo&&this.lastScrollTo.y||0;
            // 图片距离containerNode顶部的offsetTop值
            var offsetTop = 0,node;
            var imgs = query("img[data-src]", this.tcCurrView&&this.tcCurrView.domNode||this.scrollable.containerNode);
            // 遍历所有图片，判断图片是否在可见区域内，或者缓冲区域内
            for(var index = 0;index < imgs.length;index++){
                // 计算offsetTop值
                // 循环遍历img的父节点   ---   这里的父节点是img相对定位的父节点，而不是它的直接父节点
                node = imgs[index];
                offsetTop = 0;
                while(node && node != this.scrollable.containerNode){
                    offsetTop += node.offsetTop;
                    node = node.offsetParent;
                }
                if(!node){
                    continue;
                }
                // 如果图片超出缓冲区，则跳出循环  ---    break会出错，因为query查询出的图片不是按照先后顺序排的，所以直接break会出错
                if(offsetTop <= visibilityHeight){
                    // 修改图片src，并且删除data-src属性
                    imgs[index].src = imgs[index].getAttribute("data-src");
                    imgs[index].removeAttribute("data-src");
                }else{
                    break;
                }
            }
        },
        //是否正在刷新，默认为false
        refreshing:false,
        //滑动的响应函数，处理了下拉刷新 和 返回顶部按钮
        onSubScroll:function(to){
            this.lastScrollTo = to;
            //要拖动刷新
            if(this.refreshBeforeTop){
                if(to.y>0&&!this.istooltip){
                    this.istooltip = true;
                    clearTimeout(this._hidePenddingTimer);
                    this.topReloadNode.innerHTML = "下拉刷新...";
                    domStyle.set(this.topReloadNode,"display","");
                }else if(to.y>100){
                    if(false===this.refreshing&&!this._refreshTimer){
                        this.topReloadNode.innerHTML = "正在刷新...";
                        this._refreshTimer = setTimeout(lang.hitch(this,function(){
                            this.refreshing = true;
                            setTimeout(lang.hitch(this,function(){
                                this.refreshing = false;    
                            }),10000);
                            this.onCqChanged(this,this.viewParam);
                        }),600);
                    }
                }else if(to.y<=0&&this.istooltip){
                    this.istooltip = false;
                    this._hidePenddingTimer = setTimeout(lang.hitch(this,function(){
                        domStyle.set(this.topReloadNode,"display","none");
                    }),600);
                    clearTimeout(this._refreshTimer);
                    this._refreshTimer = null;
                }
            }
            //返回顶部按钮
            if(this.toTopNode){
                clearTimeout(this._toTopTimer);
                this._toTopTimer = setTimeout(lang.hitch(this,function(){
                    var matrix = 0;
                    if(this.lastScrollTo&&this.lastScrollTo.y<0){
                        matrix = this.lastScrollTo.y;
                    }
                    if((parseFloat(matrix,10)||0)<this.scrollable.domNode.clientHeight*-1.5){
                        domStyle.set(this.toTopNode,"display","");
                    }else{
                        domStyle.set(this.toTopNode,"display","none");
                    }
                }),300);
            }
        },
        opacitytoTop:function(to){
            if(to.y>=0&&this.isOpacity){
                this.isOpacity = false;
                domStyle.set(this.topNode,"background","rgba(255,255,255,0)");
                if(this.topChangeNodes.length){
                    for(var i=0;i<this.topChangeNodes.length;i++){
                        domStyle.set(this.topChangeNodes[i],{
                            borderRadius:"20px",
                            background:"rgba(255,255,255,0.7)"
                        });
                    }
                }
                if(this.topOpacityNodes.length){
                    for(var i=0;i<this.topOpacityNodes.length;i++){
                        domStyle.set(this.topOpacityNodes[i],{
                            opacity:0
                        });    
                    }
                }
                return;
            }
            if(!this.isOpacity&&to.y<=0){
                this.viewHeight = Math.floor(this.domNode.clientHeight*0.8);
                var opacity = Math.floor(Math.abs(to.y)/this.viewHeight*100) / 100; 
                if(opacity>1){
                    opacity = 1;
                }else{
                    this.opaque = false;
                }
                if(this.opaque){
                    return;
                }
                domStyle.set(this.topNode,"background","rgba(255,255,255,"+opacity+")");
                if(this.topChangeNodes.length){
                    for(var i=0;i<this.topChangeNodes.length;i++){
                        domStyle.set(this.topChangeNodes[i],{
                            borderRadius:"20px",
                            background:"rgba(255,255,255,"+(opacity>0.7?0:(1-opacity))+")"
                        });
                    }
                }
                if(this.topOpacityNodes.length){
                    for(var i=0;i<this.topOpacityNodes.length;i++){
                        domStyle.set(this.topOpacityNodes[i],{
                            opacity:opacity
                        });  
                    }
                }
                if(opacity===1){
                    this.opaque = true;
                }
            }
        },
        // 隐藏头部
        hideTop:function(){
            if(this.canHideTop){
                if(!userAgent.isWeixin()){
                    return;
                }
                var fixedSplit = this.getChildren();
                var topPane = null;
                if(!fixedSplit.length){
                    return;
                }else{
                    topPane = fixedSplit[0];
                }
                var headPane = query('.top-items ',topPane.domNode);
                
                if(!headPane.length){
                    console.log('no header');
                    return;
                }else{
                    headPane = headPane[0];
                }
                domStyle.set(topPane.domNode,{
                    position:"static"
                });
                domStyle.set(headPane,{
                    position:"absolute",
                    display:"none"
                });
                if(this.topOpacity){
                    this.topOpacity = false;
                }
            }
        },
        // 头部渐变
        changeTop:function(){
            if(this.topOpacity){
                var fixedSplit = this.getChildren();
                if(!fixedSplit.length){
                    return;
                }else{
                    fixedSplit = fixedSplit[0];
                }
                var topPane = fixedSplit.getChildren();
                this.topNode  = null;
                if(!topPane.length){
                    return;
                }else{
                    topPane = topPane[0];
                    this.topNode = topPane.domNode.childNodes[0];
                }
                if(!domClass.contains(this.topNode,"top-items")){
                    return;
                }
                this.topChangeNodes = [];
                this.topOpacityNodes = [];
                this.topChangeNodes = query('.top-change',this.topPane);
                if(this.topChangeNodes.length){
                    for(var i=0;i<this.topChangeNodes.length;i++){
                        domStyle.set(this.topChangeNodes[i],{
                            borderRadius:"20px",
                            background:"rgba(255,255,255,0.7)"
                        });    
                    }
                }
                this.topOpacityNodes = query('.top-opacity',this.topPane);
                if(this.topOpacityNodes.length){
                    for(var i=0;i<this.topOpacityNodes.length;i++){
                        domStyle.set(this.topOpacityNodes[i],{
                            opacity:"0"
                        });    
                    }
                }
                domStyle.set(topPane.domNode,{
                    height:'0px',
                    position:'static'
                });
                domStyle.set(this.topNode,{
                    zIndex:"1",
                    position:"absolute",
                    width:"100%",
                    background:"rgba(255,255,255,0)"
                });
                this.isOpacity = true;
                this.connect(this.scrollable,"slideTo",this.opacitytoTop);
                this.connect(this.scrollable,"scrollTo",this.opacitytoTop);
            }
        },
        // 更改title
        changeTitle:function(title){
            if(!document.originTitle){
                document.originTitle = document.title;
            }
            if(title){
                this.title = title    
            }
            if(!this.title){
                this.title = window.g_sysg_sys_info&&g_sys_info.shopName||document.originTitle;
            }
            topic.publish("cq/app/view/view/titleChanged",this);
        }
    });
});
