﻿define([
    "dojo/_base/declare",
    "dojo/_base/lang",
    "dojo/_base/array",
    "dojo/dom-attr",
    "dojo/dom-construct",
    "dojo/dom-class",
    "dojo/dom-style",
    "dijit/registry",
    "./_WidgetBase",
    "./utils/listLayout",
    "cq/util/image/blob"
],function(declare,lang,array,domAttr,domConstruct,domClass,domStyle,registry,_CqWidgetBase,listLayout,blob){
    // module:
    //      cq/app/widget/_ListMixin
    
    //对象属性
    //  dataProvider:数据提供者
    //  targetMethod:获取数据url的方法
    //  itemPropMethod:数组准备好后依据数据生成列表项参数的方法名称
    //  queryParamMethod:查询参数方法
    //  itemType:列表项类型
    //  workMode:工作模式
    //  pageSize:分页行数，所有的动态数据列表都要分页
    //  clickMethod:点击事件方法名
    
    //列表支持layout、layout标题和没有数据时隐藏
    //  layoutMethod:读取列表结构的方法
    //  hideWhenNoData:没有数据时隐藏
    var itemClick = function(e){
        this.getParent().onItemClick(this,e);
        return false;
    };
    var props = "dataMethod:{dataMethod},storeType:{storeType},autoScroll:{autoScroll},baseQueryInfo:{baseQueryInfo},itemOptions:{j:itemOptions},allData:{j:allData},items:{j:items},selected:{selected},label:'{label}',dataLoadedMethod:'{dataLoadedMethod}',hideWhenNoData:'{hideWhenNoData}',layoutMethod:'{layoutMethod}',dataProvider:'{dataProvider}',targetMethod:'{targetMethod}',itemPropMethod:'{itemPropMethod}',queryParamMethod:'{queryParamMethod}',itemType:'{itemType}',workMode:'{workMode}',selectMode:'{selectMode}',pageSize:'{pageSize}',showPager:'{showPager}',clickMethod:'{clickMethod}',exportLayoutMethod:'{exportLayoutMethod}'";
    //列表对象，支持选择
    return declare("",[_CqWidgetBase],{
        // summary:
        //      提供一个可配置的列表
        // 实现列表通常要处理的问题：
        //      数据分页
        //      加载
        //      显示
        //      刷新
        //      销毁
        //      单选
        //      多选
        //      点击事件处理
        
        // htmlTemp:string
        //      本对象的html模版，每个通过layout配置显示在界面上的widget都要实现本属性
        htmlTemp:'<div id="{id}" class="cq-list {listClass}" data-dojo-type="cq/app/widget/web/List" data-dojo-props="'+props+'"></div>',
        
        // items:Array
        //      生成列表的原始数据，没有提供数据源时，通过本属性指定列表的固定内容
        items:null,
        
        selected:"",
        
        // label:string
        //      显示在所有列表项之前的提示信息，可以不设置
        label:"",
        
        // allData:object
        //      显示动态获取的备选项之前的固定备选项数据
        allData:null,
        
        // dataProvider:string
        //      提供操作的对象的模块id
        dataProvider:"",
        dataMethod:"",
        
        // targetMethod:string
        //      dataProvider提供的获取读取数据地址的方法的方法名
        targetMethod:"",
        
        // storeType:String
        //      提供特殊处理的store
        storeType:"",
        
        // baseQueryInfo:object
        //      dataProvider加载数据时的基本参数，内容可以被queryParam覆盖
        baseQueryInfo:null,
        
        // dataLoadedMethod:string
        //      dataProvider提供的数据加载之后的事件处理函数
        dataLoadedMethod:"",
        
        // itemPropMethod:string
        //      dataProvider提供的生成获取列表项的参数的方法的方法名
        itemPropMethod:"",
        
        // queryParamMethod:string
        //      dataProvider提供的依据cqRefInfo生成查询条件的方法的方法名
        queryParamMethod:"",
        
        // clickMethod:string
        //      dataProvider提供的处理列表项点击事件方法的方法名
        clickMethod:"",
        
        // itemType:string
        //      依据数据生成的列表项的类型
        //      需要在item目录下实现对应的模块
        itemType:"",
        
        // workMode:string
        //      工作模式,
        //          "single":单选模式
        //          "multi":多选模式
        //          "action":点击响应
        workMode:"",
        
        // 选中方式，默认点击后选中
        //        "hover"：鼠标hover后选中
        //        "click"：鼠标点击后选中
        selectMode:"click",
        //获取列表结构的方法
        layoutMethod:"",
        //没有数据时是否隐藏
        hideWhenNoData:false,
        //列表项配置属性
        itemOptions:null,
        
        //分页页面大小
        pageSize:20,
        
        //当前页序
        start:0,
        
        //列表项原型对象
        // tags:
        //      private
        _itemRender:null,
        
        // refreshOnReady:boolean
        //      创建之后是否要指定执行一次刷新
        refreshOnReady:false,
        
        // isExport:String
        //      获取导出列表layout的方法
        exportLayoutMethod:"",
        
        //重写_widgetBase.observe方法，用于绑定外部按钮
        observe:function(widget,field){
            if(this.dataProvider&&this.exportLayoutMethod&&this.targetMethod&&field === 'exportMethod'){
                this.connect(widget.domNode,"onclick",lang.hitch(this,function(){
                    if(this.dataProvider){
                        //有数据提供者，初始化数据提供者
                        if(lang.isString(this.dataProvider)){
                            require([this.dataProvider],lang.hitch(this,this.exportMethod));
                        }else{
                            this.exportMethod()
                        }
                    }
                }));
            }else{
                this.inherited(arguments);
            }
        },
        exportMethod:function(){
            if(this.dataProvider[this.exportLayoutMethod]){
                this.exportLayout = null;
                this.exportLayout = this.dataProvider[this.exportLayoutMethod]()
                if(this.exportLayout&&this.exportLayout.length){
                    window.busy();
                    require([
                        "cq/util/sys/xlsx"
                    ], lang.hitch(this, function(xlsx){
                        xlsx.exportData(this.dataProvider[this.targetMethod](),this.queryParam||{},this.exportLayout,Date.now().toString());
                    }));
                }
            }else{
                console.log("获取layout的方法配置出错");
                return;
            }
        },
        // 自动滚动
        autoScroll:false,
        createLabelNode:function(){
            // summary:
            //      创建标签节点，部分列表不需要有这个节点，所有提供了接口
            this.labelNode = domConstruct.create("div",{
                "class":"cq-list-text-label",
                innerHTML:this.label
            },this.containerNode,"first");
        },
        //执行初始化
        buildRendering:function(){
            this.inherited(arguments);
            if(this.autoScroll){
                var scrollNode = domConstruct.create("div",{
                    "class":"list-scroll-pane"
                },this.domNode,"first");
                this.containerNode = scrollNode;
            }
            if(this.label){
                this.createLabelNode();
            }
            this.initPageNode();
            
            this.pageSize = this.pageSize-0;
            this.pageSize = this.pageSize||20;
            if(this.items&&lang.isString(this.items)){
                this.items = JSON.parse(this.items);
            }
            this.hideWhenNoData = true===this.hideWhenNoData||"true"===this.hideWhenNoData;
            this.selectedItems = [];
            //初始化列表的显示隐藏
            this.updateDataHideState([]);
            if(this.dataProvider){
                //有数据提供者，初始化数据提供者
                if(lang.isString(this.dataProvider)){
                    require([this.dataProvider],lang.hitch(this,this._onDataProviderReady));
                }else{
                    this._onDataProviderReady(this.dataProvider);
                }
            }else{
                this._initItemRender();
            }
        },
        initPageNode:function(){
            // summary:
            //      初始化分页区域节点
        },
        onDataProviderReady:function(){},
        _onDataProviderReady:function(dataProvider){
            // summary:
            //      加载完数据提供者之后的处理
            // tags:
            //      private
            this.dataProvider = dataProvider;
            this.onDataProviderReady();
            this._initItemRender();
        },
        getItemTypeId:function(itemType){
            return "cq/app/widget/web/item/"+itemType;
        },
        _initItemRender:function(){
            // summary:
            //      加载列表项对应模块，加载完成之后，初始化列表
            if(this.itemType&&"string"===typeof(this.itemType)){
                require([this.itemType.indexOf("/")>0?this.itemType:this.getItemTypeId(this.itemType)],lang.hitch(this,this._onItemRenderReady));
            }else{
                this._onItemRenderReady(this.itemType);
            }
        },
        _onItemRenderReady:function(itemRender){
            this._itemRender = itemRender;
            if(this.dataProvider&&this.itemOptions){
                if((typeof(this.itemOptions)!=="string")){
                    this.itemOptions = JSON.stringify(this.itemOptions);
                }
                var fields = this.itemOptions.match(/\{[\w\.]+\}/g)||[];
                this.fieldFormatterHash = {};
                if(fields.length){
                    fields = listLayout.initFieldFormatter(this.dataProvider.getFormItems(fields.map(function(f){
                        return f.substr(1).substr(0,f.length-2);
                    })));
                    for(var i=0;i<fields.length;i++){
                        this.fieldFormatterHash[fields[i].field] = fields[i];
                    }
                }
                this.itemOptions = JSON.parse(this.itemOptions);
                this.getFieldValue = lang.hitch(this,this.getFieldValue);
            }
            if(this.dataProvider){
                if(this.targetMethod){
                    if(this.layoutMethod){
                        this.layout = listLayout.initLayoutFormatter(this.dataProvider[this.layoutMethod]());
                        this.fieldItem = new this._itemRender({
                            layout:this.layout
                        });
                    }
                    //使用store
                    require([this.storeType||"dojo/store/JsonRest"],lang.hitch(this,function(JsonRest){
                        this.store = new JsonRest({
                            target:this.dataProvider[this.targetMethod]()
                        });
                        if(this.refreshOnReady){
                            this.refresh();
                        }
                    }));
                }else if(this.dataMethod&&this.refreshOnReady){
                    this.refresh();
                }
            }else{
                this.onDataReady(this.items||[]);
            }
            this.initSelected();
        },
        //依据字段读取字段值的方法
        getFieldValue:function(field,data){
            if(this.fieldFormatterHash[field].type==="image"&&this.fieldFormatterHash[field].cqWidth){
                return blob.getImageUrl(this.fieldFormatterHash[field].formatter(data[field],data),this.fieldFormatterHash[field].cqWidth);    
            }
            return this.fieldFormatterHash[field].formatter(data[field],data);
        },
        //刷新界面，从第一页开始显示
        refresh:function(){
            // summary:
            //      安全的刷新列表的接口
            if(false===this.initList()){
                return;
            }
            if(!this.canReload()){
                return;
            }
            this.clearList();
            if(this.dataProvider&&this.targetMethod){
                if(this.store){
                    //有可能前一次请求已发出，未得到结果，条件发生了改变，需要重新刷新列表
                    if(this._loadQueryResult&&!this._loadQueryResult.isResolved()){
                        this._loadQueryResult.cancel();
                    }
                    this._loadQueryResult = this.loadMore();
                }else{
                    this.refreshOnReady = true;
                }
            }else if(this.dataMethod){
                var res = this.dataProvider[this.dataMethod](this.cqRefInfo,this,this.dataOptions);
                if(res){
                    if(res.then){
                        res.then(lang.hitch(this,function(result){
                            if(result){
                                this.onDataReady(result.items||result);
                            }
                        }));
                    }else{
                        this.onDataReady(res.items||res);
                    }
                }
            }else{
                this.onDataReady(this.items||[]);
            }
        },
        _lastQueryStart:-1,
        _lastQueryTime:0,
        //_lastQueryParam:"",
        _lastQueryParam:{},
        loadMore:function(index){
            if(this.enableBusy){
                window.busy();
            }
            if(index>=0){
                this.start = this.pageSize*index;
            }
            this._lastQueryStart = this.start;
            //this._lastQueryParam = JSON.stringify(this.queryParam);
            this._lastQueryParam = this.queryParam;
            this._lastQueryTime = new Date();
            var queryResult = this.store.query(this.queryParam,{
                start:this.start,
                count:this.pageSize
            });
            queryResult.then(lang.hitch(this,this.onDataReady));
            queryResult.total.then(lang.hitch(this,this.onTotalReady));
            return queryResult;
        },
        getCqQueryInfo:function(){
            return {
                target:this.store&&this.store.target,
                query:this.queryParam
            };
        },
        canReload:function(){
            // summary:
            //      判断是否可以刷新，解决短时间内多重刷新问题
            //开始页相同、时间间隔很短时才有可能忽略
            if(this._lastQueryStart===this.start&&2000>(new Date())-this._lastQueryTime){
                var equal = function(params1,params2){
                    var field = "";
                    for(field in params1){
                        if((params1[field]+"")!==(params2[field]+"")){
                            return false;
                        }
                    }
                    for(field in params2){
                        if((params1[field]+"")!==(params2[field]+"")){
                            return false;
                        }
                    }
                    return true;
                };
                if(equal(this.queryParam||{},this._lastQueryParam||{})){
                    return false;
                }
            }
            return true;
        },
        //初始化列表
        initList:function(){
            // summary:
            //      初始化列表，内容的清理，从第几页加载，查询条件等
            this.start = 0;
            this.selectedItems = [];
            if(this.store){
                var query = lang.clone(this.baseQueryInfo||{});
                if(this.dataProvider[this.queryParamMethod]){
                    var query2 = this.dataProvider[this.queryParamMethod](this.cqRefInfo);
                    if(false===query2){
                        return false;
                    }
                    query = lang.mixin(query,query2);
                }
                this.queryParam = query;
                this.store.target = this.dataProvider[this.targetMethod]();
            }
        },
        clearList:function(){
            if(this.fieldItem&&this.fieldItem.domNode.parentNode){
                this.fieldItem.domNode.parentNode.removeChild(this.fieldItem.domNode);
            }
            this.oldChildrenNode = this.getChildren();
            
            if(this.fieldItem){
                this.addChild(this.fieldItem);
            }
            if(this.allData){
                //this.addChild(new this._itemRender(this.getItemParam(this.allData)));
                this._createItem(this.allData);
            }
        },
        destroyChildren:function(){
            if(this.oldChildrenNode&&this.oldChildrenNode.length){
                while(this.oldChildrenNode.length){
                    this.oldChildrenNode.pop().destroyRecursive();
                }    
                this.oldChildrenNode = [];
            }
        },
        getItemParam:function(data){
            if(this.dataProvider&&this.itemPropMethod){
                data = this.dataProvider[this.itemPropMethod](data,this.itemType,this);
            }
            if(!data.data){
                data.data = data;
            }
            data.layout = this.layout;
            if(this.itemOptions){
                var item = {
                    clickable:true,
                    data:data.data,
                    onClick:itemClick
                },replace = lang.hitch(this,function(field){
                    return this.getFieldValue(field.substr(1).substr(0,field.length-2),data);
                });
                for(var f in this.itemOptions){
                    value = this.itemOptions[f];
                    if("string"===typeof(value)){
                        value = value.replace(/\{[\w\.]+\}/g,replace);
                    }
                    item[f] = value;
                }
                data = item;
            }
            return data;
        },
        //数据接在完成
        onDataReady:function(datas){
            if(datas.items){
                datas = datas.items;
            }
            this.onCqChanged(this,null);
            // 没有数据时，提示暂无数据   郭周全  2017/05/05
            if(datas.length===0||datas.message){
                // 判断提示节点是否存在，避免刷新时多次创建
                if(!this.noDataTooltip){
                    // 创建一个节点作为提示信息的容器
                    this.noDataTooltip = domConstruct.create("div",{
                        className:"noDataTooltip",
                        style:"font-size:25px;"+
                              "text-align:center;"+
                              "color: #aaa;"+
                              "line-height: 200px;"+
                              "background: rgba(0,0,0,0);"
                    },this.domNode);
                    this.noDataTooltip.innerHTML = "暂时没有数据~";
                }else{
                    domStyle.set(this.noDataTooltip, "display", "block");
                }
                // 给list添加一个className 用于控制没有数据是list的样式
                this.noDataTooltip.parentNode&&domClass.add(this.noDataTooltip.parentNode,"noDataMesage");
            }else if(this.noDataTooltip){
                // 刷新后删除控制无数据时样式的class 并删除提示节点
                this.noDataTooltip.parentNode&&domClass.remove(this.noDataTooltip.parentNode,"noDataMesage");
                //domConstruct.destroy(this.noDataTooltip);
                //domConstruct.removeChild(this.noDataTooltip);
                domStyle.set(this.noDataTooltip, "display", "none");
            }
            if(this.dataProvider&&this.dataLoadedMethod&&this.dataProvider[this.dataLoadedMethod]){
                this.dataProvider[this.dataLoadedMethod](datas,this);
            }
            array.forEach(datas,function(data){
                this._createItem(data);
            },this);
            if(this._started){
                var view = this.getParent();
                while(view&&(!view.show)){
                    view = view.getParent();
                }
                if(true===this.hideWhenNoData){
                    this.updateDataHideState(datas);
                }
                if(view){
                    view.resize();
                }
            }
            this.destroyChildren();
            if(this.autoScroll)this.autoScrollMethod();
        },
        updateDataHideState:function(datas){
            var hide = !(datas.length||this._lastQueryStart);
            domStyle.set(this.domNode,"display",hide?"none":"");
            if(this.labelNode&&this.label){
                domStyle.set(this.labelNode,"display",hide?"none":"");
            }
            if(this.pagerNode){
                domStyle.set(this.pagerNode,"display",hide?"none":"");
            }
        },
        //处理分页，数据总行数
        onTotalReady:function(total){
            
        },
        //初始化选中的列表项
        _initSelected:function(item){
            this.selectedItems = this.selectedItems||[];
            if(item&&item===this.selectedItems[0]){
                return;    
            }else{
                while(this.selectedItems.length){
                    this.selectedItems.pop().set("checked",false);
                }    
            }
        },
        //获取节点对应的操作方法
        getActionByNode:function(item,node){
            // summary:
            //      依据节点获取操作类型，clickMethod中可以依据不同的操作类型做不同的相应
            
            var action = "";
            while(node&&node.parentNode&&node!==item.domNode){
                action = domAttr.get(node,"cq-action");
                if(action){
                    break;
                }
                node = node.parentNode;
            }
            //干劲：少return
            return action;
        },
        //列表项点击之后的响应
        onItemClick:function(item,e){
            // summary:
            //      点击的处理函数，处理了列表在不同工作模式下的不同响应方式
            //不是多选，清空选中
            this.clickable = true;
            if("multi"!=this.workMode){
                this._initSelected(item);
            }
            //依据工作模式做出不同的响应
            switch(this.workMode){
                case "single":
                    //设置新的选中
                    if(this.selectedItems.length===0){
                        this.selectedItems.push(item);
                        item.set("checked",true);
                    }
                    //单选
                    this.onCqChanged(this,item.params.data);
                    break;
                case "multi":
                    //设置新的选中
                    this.selectedItems.push(item);
                    item.set("checked",true);
                    //多选
                    break;
                default:
                    //点击响应
                    if(this.dataProvider && this.clickMethod){
                        // 赵春辉  允许listItem直接传入操作的意图action   2017-01-17
                        // 我关注的货，右侧应有图标，图标点击后底部浮起overlay
                        var action = "";
                        if(e){
                            if("string"===typeof(e)){
                                action = e;
                            }else{
                                action = this.getActionByNode(item,e.target);
                            }
                        }
                        this.dataProvider[this.clickMethod](item.params.data,action,this);
                    }
                    break;
            }
            //return false;
        },
        removeItemByData:function(data){
            // summary:
            //      依据数据销毁对应的列表项
            if(data){
                array.some(this.getChildren(),function(item){
                    if(item.params.data&&item.params.data._id === data._id){
                        item.destroyRecursive();
                        // BUG:如果有多个ITEM的ID一样，则全部删除
                        //return true;
                    }
                },this);
            }
        },
        getItemById:function(_id){
            var item = null;
            array.some(this.getChildren(),function(i){
                if(i.params.data&&i.params.data._id === _id){
                    item = i;
                    return true;
                }
            },this);
            return item;
        },
        refreshItemByData:function(data){
            // summary:
            //      依据数据刷新对应的列表项
            if(data){
                var item = this.getItemById(data._id);
                data = this.getItemParam(data);
                (new this._itemRender(data)).placeAt(item.domNode,"before");
                item.destroyRecursive();
                this.initSelected();
            }
        },
        cqRefresh:function(){
            //      当监听到其他对象变化时，刷新本对象
            this.refresh();
            
            /*if(this.class==="cq-list "&&this.domNode.childNodes.length){
                this.onItemClick(registry.byId(this.domNode.childNodes[0].id));
            }*/
        },
        getSelectedItems:function(){
            // summary:
            //      获取当前选中的列表项的数据
            if("single"===this.workMode){
                return this.selectedItems&&this.selectedItems.length?this.selectedItems[0].params.data:null;
            }else{
                return array.map(this.selectedItems,function(item){
                    return item.params.data;
                });
            }
        },
        initSelected:function(){
            var isSelect = false;
            array.forEach(this.getChildren(),lang.hitch(this,function(item){
                if(item.get("checked")){
                    return;
                }
                if(domClass.contains(item.domNode,"selected")){
                    this.selectedItems.push(item);
                }
            }));
        },
        _createItem:function(data){
            var item = this.getItemParam(data);
            item = new this._itemRender(item);
            this.addChild(item);
            if(data.selected){
                this.onItemClick(item,{
                    target:item.domNode
                });
            }
            return item;
        },
        createItem:function(data){
            return this._createItem(data);
        },
        autoScrollMethod:function(){
            if(this.scrollH === undefined){
                this.scrollH = 0;
            }
            this.cPaneH = this.domNode.clientHeight;
            this.scrollMethod();
        },
        scrollMethod:function(){
            this.containerH = this.containerNode.clientHeight;
            
            if(this.containerH+this.scrollH<0){
                this.scrollH = this.cPaneH;
            }
            this.scrollHandle = setTimeout(lang.hitch(this,function(){
                if(!this.isStop){
                    domStyle.set(this.containerNode,"transform","translate3d(0px,"+(this.scrollH--)+"px,0px)");    
                }
                
                clearTimeout(this.scrollHandle);
                this.scrollHandle = null;
                this.scrollMethod();
            }),10);
        }
    });
});