define([
    "dojo/_base/declare",
    "dojo/_base/lang",
    "dojo/_base/connect",
    "dojo/_base/array",
    "dojo/dom-attr",
    "dojo/dom-style",
    "dojo/dom-class",
    "dojo/dom-construct",
    "dojo/Deferred",
    "dojox/mobile/ScrollablePane",
    "dijit/_WidgetBase",
    "dijit/registry",
    "dojox/mobile/FixedSplitter",
    "dojox/mobile/Pane",
    "dojox/mobile/EdgeToEdgeList",
    "dojox/mobile/ListItem",
    "cq/biz/shop/bs/district"
],function(declare, lang, connect, array, domAttr, domStyle, domClass, domConstruct, Deferred, ScrollablePane, _WidgetBase, registry, FixedSplitter, Pane, EdgeToEdgeList, ListItem, district){
    var Selector = declare("cq.app.widget.mobile.edit.DistrictSelector", [_WidgetBase], {
        // 区划缓存，区划不会随时变动，每次加载之后将加载过的数据缓存到cache中，可以减少加载次数
        distCache:{country:null,dist:{},allVillage:null},
        buildRendering:function(){
            this.inherited(arguments);
            this.indexElement();
            this.initEvent();
            this.resize();
        },
        resize:function(){
            this.rootPane.resize();
        },
        indexElement:function(){
            this.domNode = this.containerNode = domConstruct.create("div", {
                style:"width:100%;height:100%;position:absolute;top:0;display:none;opacity:1;background-color:white;z-index:2;",
                className:"distselector"
            });
            var rootPane = this.rootPane = new FixedSplitter({
                variablePane:1,
                orientation:"V"
            });
            rootPane.placeAt(this.domNode);
            // 添加顶部区域，左边显示区划选择，右边显示选中的商家名称
            this.rootPane.addChild(this.headerPane = new Pane());
            var headLabel = domConstruct.create("div", {
                style:"margin-left:12px;height:40px;display:inline-block;line-height:40px;",
                innerHTML:"区划选择",
                className:"distselector-headlabel"
            }, this.headerPane.domNode);
            var currDistPane = this.currDistPane = domConstruct.create("div", {
                style:"margin-right:12px;height:40px;display:inline-block;float:right;line-height:40px;",
                className:"distselector-currdistpane"
            }, this.headerPane.domNode);
            var tabPane = this.tabPane = domConstruct.create("div", {
                style:"display:flex;height:40px;line-height:40px;",
                className:"distselector-tabpane"
            }, this.headerPane.domNode);
            domConstruct.create("div", {
                style:"height:100%;flex:1;text-align:center;",
                innerHTML:"区/县",
                className:"distselector-tabpane1"
            }, this.tabPane);
            domConstruct.create("div", {
                style:"height:100%;flex:1;text-align:center;",
                innerHTML:"镇/街道",
                className:"distselector-tabpane2"
            }, this.tabPane);
            domConstruct.create("div", {
                style:"height:100%;flex:1;text-align:center;",
                innerHTML:"村/商家",
                className:"distselector-tabpane3"
            }, this.tabPane);
            // 添加中间选择区域，中间区域分三部分，都是ScrollablePane,ScrolloabelPane中是EdgeToEdgeList
            // left:区/县，center:镇/街道,right:村/商家
            this.rootPane.addChild(this.centerPane = new Pane({className:"hflex mblPane"}));
            this.centerPane.startup();
            domStyle.set(this.centerPane, "display", "flex");
            domStyle.set(this.centerPane, "height", "100%");
            var leftPanel = this.leftPanel = domConstruct.create("div", {
                style:"flex:1;height:100%;display:inline-block;"
            }, this.centerPane.domNode);
            var centerPanel = this.centerPanel = domConstruct.create("div", {
                style:"flex:1;height:100%;display:inline-block;"
            }, this.centerPane.domNode);
            var rightPanel = this.rightPanel = domConstruct.create("div", {
                style:"flex:1;height:100%;display:inline-block;"
            }, this.centerPane.domNode);
            var leftSP = new ScrollablePane({height:"100%"});
            leftSP.placeAt(leftPanel);
            var centerSP = new ScrollablePane({height:"100%"});
            centerSP.placeAt(centerPanel);
            var rightSP = new ScrollablePane({height:"100%"});
            rightSP.placeAt(rightPanel);
            var countryList = this.countryList = new EdgeToEdgeList({isEdge:true});
            countryList.placeAt(leftSP.containerNode);
            var townList = this.townList = new EdgeToEdgeList({isEdge:true});
            townList.placeAt(centerSP.containerNode);
            var villageList = this.villageList = new EdgeToEdgeList({isEdge:true});
            villageList.placeAt(rightSP.containerNode);
        },
        initEvent:function(){
            // 加载区县信息并初始化区县列表
            this.loadDataByUpper().then(lang.hitch(this, function(data){
                array.forEach(data, lang.hitch(this, function(item, index){
                    var li = new ListItem({label:item.distName,distValue:item,noArrow:false,moveTo:"#"});
                    this.countryList.addChild(li);
                }));
                this.resize();
            }));
            // 列表点击事件
            connect.connect(this.countryList.domNode, "onclick", lang.hitch(this, this.onCountryListClick));
            connect.connect(this.townList.domNode, "onclick", lang.hitch(this, this.onTownListClick));
            connect.connect(this.villageList.domNode, "onclick", lang.hitch(this, this.onVillageListClick));
        },
        startEdit:function(params){
            var def = this.currDef = new Deferred();
            domStyle.set(this.domNode, "display", "");
            this.setValue(params);
            return def;
        },
        loadDataByUpper:function(upId){
            var def = new Deferred();
            if(!upId){
                // 加载区县
                if(!this.distCache["country"]){
                    district.loadByLevel(4).then(lang.hitch(this, function(data){
                        if(data&&data.data&&data.data.items){
                            this.distCache["country"] = data.data.items;
                            def.resolve(data.data.items);
                        }
                    }));
                }else{
                    def.resolve(this.distCache["country"]);
                }
            }else if(upId == "allVillage"){
                if(!this.distCache["allVillage"]){
                    district.loadByLevel(6).then(lang.hitch(this, function(data){
                        if(data&&data.data&&data.data.items){
                            this.distCache["allVillage"] = data.data.items;
                            def.resolve(data.data.items);
                        }
                    }));
                }else{
                    def.resolve(this.distCache["allVillage"]);
                }
            }else{
                if(this.distCache["dist"][upId]){
                    def.resolve(this.distCache["dist"][upId]);
                }else {
                    district.loadByUpId(upId).then(lang.hitch(this, function(data){
                        if(data&&data.data&&data.data.items){
                            this.distCache["dist"][upId] = data.data.items;
                            def.resolve(data.data.items);
                        }
                    }));
                }
            }
            return def;
        },
        onCountryListClick:function(evt){
            var countryes = this.countryList.getChildren();
            array.forEach(countryes, function(country){
                domClass.remove(country.domNode, "selected");
            });
            var target = evt.target;
            var widget = registry.getEnclosingWidget(target);
            domClass.add(widget.domNode, "selected");
            this.value["country"] = widget.distValue;
            this.townList.destroyDescendants();
            this.loadDataByUpper(widget.distValue._id).then(lang.hitch(this, function(data){
                array.forEach(data, lang.hitch(this, function(item, index){
                    var li = new ListItem({label:item.distName,distValue:item,noArrow:false,moveTo:"#"});
                    this.townList.addChild(li);
                }));
                this.resize();
            }));
        },
        onTownListClick:function(evt){
            var townes = this.townList.getChildren();
            array.forEach(townes, function(town){
                domClass.remove(town.domNode, "selected");
            });
            var target = evt.target;
            var widget = registry.getEnclosingWidget(target);
            domClass.add(widget.domNode, "selected");
            this.value["town"] = widget.distValue;
            this.villageList.destroyDescendants();
            this.loadDataByUpper("allVillage").then(lang.hitch(this, function(data){
                array.forEach(data, lang.hitch(this, function(item, index){
                    var li = new ListItem({label:item.distName,distValue:item});
                    this.villageList.addChild(li);
                }));
                this.resize();
            }));
        },
        onVillageListClick:function(evt){
            var villages = this.villageList.getChildren();
            array.forEach(villages, function(village){
                domClass.remove(village.domNode, "selected");
            });
            var target = evt.target;
            var widget = registry.getEnclosingWidget(target);
            domClass.add(widget.domNode, "selected");
            this.value["village"] = widget.distValue;
            this.currDistPane.innerHTML = this.value["country"].distName||"";
            this.currDistPane.innerHTML += this.value["town"].distName||"";
            this.currDistPane.innerHTML += this.value["village"].distName||"";
            this.currDef.resolve(this.value);
            domStyle.set(this.domNode, "display", "none");
        },
        setValue:function(value){
            if(value){
                this.value = {
                    country:value["country"]||null,
                    town:value["town"]||null,
                    village:value["village"]|null
                };
            }else{
                this.value = {};
            }
            setTimeout(lang.hitch(this, this.refresh), 100);
        },
        // 设置镇/街道  村/商家选中
        refresh:function(type){
            if(type == "town"){
                var towns = this.townList.getChildren();
                if(towns&&towns.length){
                    if(this.value["town"]){
                        array.forEach(towns, lang.hitch(this, function(town){
                            domClass.remove(town.domNode, "selected");
                            if(town.distValue._id == this.value["town"]._id){
                                domClass.add(town.domNode, "selected");
                                this.onTownListClick({target:town.domNode});
                            }
                        }));
                    }else{
                        array.forEach(towns, lang.hitch(this, function(town, index){
                            domClass.remove(town.domNode, "selected");
                            if(index == 0){
                                domClass.add(town.domNode, "selected");
                                this.value["town"] = town.distValue;
                                this.onTownListClick({target:town.domNode});
                            }
                        }));
                    }
                }else{
                    setTimeout(lang.hitch(this, function(){
                        this.refresh("town");
                    }), 100);
                }
                setTimeout(lang.hitch(this, function(){
                    this.refresh("village");
                }), 100);
            }else if(type == "village"){
                var villages = this.villageList.getChildren();
                if(villages && villages.length){
                    if(this.value["village"]){
                        array.forEach(villages, lang.hitch(this, function(village){
                            domClass.remove(village.domNode, "selected");
                            if(village.distValue._id == this.value["village"]._id){
                                domClass.add(village.domNode, "selected");
                                this.currDistPane.innerHTML = this.value["country"].distName||"";
                                this.currDistPane.innerHTML += this.value["town"].distName||"";
                                this.currDistPane.innerHTML += this.value["village"].distName||"";
                            }
                        }));
                    }else{
                        array.forEach(villages, lang.hitch(this, function(village, index){
                            domClass.remove(village.domNode, "selected");
                            if(index == 0){
                                domClass.add(village.domNode, "selected");
                                this.value["village"] = village.distValue;
                                this.currDistPane.innerHTML = this.value["country"].distName||"";
                                this.currDistPane.innerHTML += this.value["town"].distName||"";
                                this.currDistPane.innerHTML += this.value["village"].distName||"";
                            }
                        }));
                    }
                }else{
                    setTimeout(lang.hitch(this, function(){
                        this.refresh("village");
                    }), 100);
                }
            }else{
                var countryes = this.countryList.getChildren();
                if(this.value["country"]){
                    array.forEach(countryes, lang.hitch(this, function(country){
                        domClass.remove(country.domNode, "selected");
                        if(country.distValue._id == this.value["country"]._id){
                            domClass.add(country.domNode, "selected");
                            this.onCountryListClick({target:country.domNode});
                        }
                    }));
                }else{
                    array.forEach(countryes, lang.hitch(this, function(country, index){
                        domClass.remove(country.domNode, "selected");
                        if(index == 0){
                            domClass.add(country.domNode, "selected");
                            this.value["country"] = country.distValue;
                            this.onCountryListClick({target:country.domNode});
                        }
                    }));
                }
                setTimeout(lang.hitch(this, function(){
                    this.refresh("town");
                }), 100);
            }
        },
        getValue:function(){
            return {
                country:this.value["country"],
                town:this.value["town"],
                village:this.value["village"]
            };
        }
    });
    var selector = new Selector();
    selector.placeAt(document.body);
    return selector;
});