﻿define([
	"dojo/_base/lang",
	"cq/util/sys/date"
], function (lang, date) {
    var getMultiFieldValue = function(fields,data){
        if(fields.length){
            data = (data||{})[fields.shift()];
        }else{
            return data||false===data||0===data?data:"";
        }
        return getMultiFieldValue(fields,data);
    };
    
	return {
		typeFormatters: {
			"options": function (v,data) {
                if(!data){
                    return this.options[v] || "";
                }
                this.fields = this.fields || this.field.split(".");
                
                var fields = [];
                this.fields.forEach(function(field){
                    fields.push(field);
                });
                var value = getMultiFieldValue(fields,data);
                
				return this.options[value] || "";
			},
			"date": function (v) {
                if(v){
                    if(this.formatStr){
                        v = date.parse(v,this.formatStr);
                    }
                    v = date.formatISODate(v);
                }else{
                    v = "";
                }
                return v;
			},
			"dateTime": function (v) {
                if(v){
                    if(this.formatStr){
                        v = date.parse(v,this.formatStr);
                    }
                    v = date.formatISODateMinutes(v);
                }else{
                    v = "";
                }
                return v;
			},
			"ref": function (v, data) {
				return (v && v[this.labelFields]) || "";
			},
			"checkbox": function (v, data) {
				return v ? "是" : "否";
			},
			"multiFields": function (v, data) {
                if(!data){
                    return v||0===v||false==v?v:"";
                }
				this.fields = this.fields || this.field.split(".");
                var fields = [];
                this.fields.forEach(function(field){
                    fields.push(field);
                });
                return getMultiFieldValue(fields,data);
			},
			"image": function (v, data) {
				return v ? '<image class="' + (this.iconClass || "dgrid-icon") + '" src="' + v + '"/>' : "";
			},
			"dateMin": function (v, data) {
				return v ? date.formatISODateMinutes(v) : "";
			},
			"none": function (v,data) {
                if(v){
                    return v;
                }
                if(!data){
                    return getMultiFieldValue([],v);
                }
                if(!this.fields){
                    //多字段拼装
                    this.fields = this.field.split(",");
                }
                return this.fields.map(function(field){
                    return getMultiFieldValue(field.split("."),data);
                }).join(",");
			},
            "formatNum":function(v){
                if(!v){
                    return v;
                }
                var splitter = "";
                //格式化字符串："xxx xxx.xx"，"xxx,xxx,xx"，"xx.xx万"
                if(this.formatStr==="xx.xx万"){
                    if(v>10000){
                        v = ((v/10000).toFixed(2))+"万";
                    }else{
                        //v = (v-1).toFixed(2)-0;
                        // 20190424干劲，减一是错误的
                        v = v.toFixed(2)-0;
                    }
                    return v;
                }else if(this.formatStr==="xxx xxx.xx"){
                    splitter = " ";
                }else if(this.formatStr==="xxx,xxx.xx"){
                    splitter = ",";
                }else{
                    return v;
                }
                v = (v-0).toFixed(2);
                var idx = v.indexOf(".")-3;
                while(idx>0){
                    v = v.substr(0,idx)+splitter+v.substr(idx);
                    idx = idx-3;
                }
                v = v.replace(".00","");
                return v;
            },
            "fxedTo0": function(v) {
                return v || 0 === v ? window.fixedTo(v, 1) : "";
            },
            "fxedTo1": function(v) {
                return v || 0 === v ? window.fixedTo(v, 2) : "";
            },
            "fixedTo2": function(v) {
                return v || 0 === v ? window.fixedTo(v, 3) : "";
            },
            "fixedTo3": function(v) {
                return v || 0 === v ? window.fixedTo(v, 4) : "";
            },
            "fixedTo4": function(v) {
                return v || 0 === v ? window.fixedTo(v, 5) : "";
            },
            "fixedTo5": function(v) {
                return v || 0 === v ? window.fixedTo(v, 6) : "";
            },
            "fixedTo6": function(v) {
                return v || 0 === v ? window.fixedTo(v, 7) : "";
            }
		},
        _getFixedToFormatter:function(fixedTo){
            return 
        },
		fieldFormatter: {
			"image": function (v, data) {
				return v || "";
			}
		},
		initFieldFormatter: function (layout) {
			return this._initFormatter(layout,this.fieldFormatter);
		},
		initLayoutFormatter: function (layout) {
			return this._initFormatter(layout,{});
		},
        _initFormatter: function(layout,extFormatter){
			var cols = [];
			for (var i = 0; i < layout.length; i++) {
				if (!layout[i].formatter) {
					cols[i] = lang.clone(layout[i]);
					if (extFormatter[cols[i].type]) {
						cols[i].formatter = extFormatter[cols[i].type];
					} else if (this.typeFormatters[cols[i].type]) {
						cols[i].formatter = this.typeFormatters[cols[i].type];
					} else if (-1 < cols[i].field.indexOf(".")&&-1 == cols[i].field.indexOf(",")) {
						cols[i].formatter = this.typeFormatters.multiFields;
					} else if(cols[i].formatStr){
                        cols[i].formatter = this.typeFormatters.formatNum;
                    } else if (cols[i].fixedTo) {
                        cols[i].formatter = this.typeFormatters["fixedTo"+cols[i].fixedTo];
                    } else {
						cols[i].formatter = this.typeFormatters.none;
					}
				} else {
					cols[i] = layout[i];
				}
			}
			return cols;
        },
		exportCsvFormatter: function (s) {
			if (("" + s).match(/[\s,]/g)) {
				return '"' + s.replace(/"/g, '""') + '"';
			} else {
				return s;
			}
		}
	};
});