define([
    "dojo/_base/declare",
    "dojo/_base/lang",
    "dojo/_base/array",
    "cq/biz/object",
    "cq/util/config/config",
    "dojo/Deferred"
], function (declare, lang, array, object, config, Deferred) {
    var O = declare("", object.Schema, {
        modalName: "shop/shop",
        shopId: null,
        loadByAid: function (aid) {
            return this.exec("load_by_aid", { aid: aid });
        },
        //获取当前AID的ShopId
        getCurrentShopId: function () {
            return this.loadByAid(this.getCurrentAid());
        },
        //当前账户AID
        getCurrentAid: function () {
            return window.g_d && window.g_d.websocket && g_d.websocket.appId;
        },
        //当前店铺汇总数据
        summary: function (param) {
            return this.execInCurrentShop("summary", param);
        },
        //保存
        save: function (param) {
            var data = this.filterSaveData(param);
            if (data.tranRegions) {
                data.tranRegions = array.map(data.tranRegions, this.filterSaveData, this);
            }
            return this.execInCurrentShop("save", data);
        },
        loadByIdsFromServiceShop: function (ids) {
            var i = ids.length;
            while (i--) {
                if (!ids[i]) {
                    ids.splice(i, 1);
                }
            }
            if (ids.length) {
                return this.execInCurrentShop("load_from_service_shop_by_ids", {
                    ids: ids,
                    serviceShopId: config.serviceShopId
                });
            } else {
                return this.defTest({ data: { ok: 1, items: [] } });
            }
        },
        //加载指定商家,肖芳圆:2016.07.20
        loadByIds: function (ids) {
            var i = ids.length;
            while (i--) {
                if (!ids[i]) {
                    ids.splice(i, 1);
                }
            }
            if (ids.length) {
                return this.exec("load_by_ids", { ids: ids, shopId: config.getPlatformShopId() });
            } else {
                return this.defTest({ data: { ok: 1, items: [] } });
            }
        },
        //指定ID的商家
        loadById: function (shopId) {
            return this.exec("load_by_id", {
                shopId: shopId
            });
        },
        // 在平台库中获取指定ID的商家信息  潘云
        loadShopByPlatform: function (shopId) {
            return this.exec("load_by_id", {
                shopId: config.getPlatformShopId(),
                _id: shopId
            });
        },
        //当前登录商家信息
        loadCurrentShop: function () {
            return this.loadById(this.getShopId());
        },
        loadCurrentFromPaltShop: function () {
            return this.loadById(config.getPlatformShopId());
        },
        //指定类型和乡镇的商家
        loadRecShop: function (type) {
            return this.execInCurrentShop("load_rec_shop_by_type", type);
            //return this.defTest([{APP_TITLE:"测试数据",M_BUSS:"二级hi发七台河io儿人",PHONE:123456478,ADDR:"只能乖乖",LOGO:"30014",shopId:"45555",appendClass:"ok"}]);
        },
        //指定类型和乡镇的商家
        loadTypeAndTownShop: function (param, page) {
            var hash = {
                "shopType": param.type || "",
                "townId": param.town || "",
                "shopId": config.getPlatformShopId()
            };
            /*if (undefined !== param.isVerified) {
                hash.isVerified = param.isVerified;
            }*/
            if (page) {
                hash.pageSize = page.pageSize || "";
                hash.pageIndex = page.pageIndex || "";
                hash.orderBy = page.orderBy || "";
            }
            return this.exec("load_by_typeandtown", hash);
        },
        getShopUrl: function () {
            return config.getDataOrigin() + '/w/home/' + this.getShopId();
        },
        getPlatformShopUrl: function () {
            return config.getDataOrigin() + '/w/home/' + config.getPlatformShopId();
        },
        //商家实名认证
        //参数：params.targetShopId
        verifyShop: function (params) {
            return this.execInCurrentShop("verify_shop", params);
        },
        // 店铺认证流程中用到的三个接口 独立出来 2016-6-21 刘轩
        // 审核店铺认证
        verifyShopAut: function (params) {
            return this.execInCurrentShop("verify_shop_aut", params);
        },
        // 驳回店铺认证
        rejectShopAut: function (params) {
            return this.execInCurrentShop("reject_shop_aut", params);
        },
        // 取消店铺认证
        cancelShopAut: function (params) {
            return this.execInCurrentShop("cancel_shop_aut", params);
        },
        //保存
        platformSave: function (param) {
            return this.execInCurrentShop("platform_save", param);
        },
        //复制店铺信息
        clone: function (param) {
            return this.exec("clone", param);
        },
        //复制店铺信息并提交审核
        cloneAndSubmit: function (param) {
            return this.exec("clone_submit", param);
        },
        //删除店铺
        removeShop: function (targetShopId) {
            return this.execInCurrentShop("delete_by_shop_id", {
                targetShopId: targetShopId
            });
        },
        // 恢复店铺
        reverseShop:function(targetShopId) {
            return this.execInCurrentShop("reverse_by_shop_id", {
                targetShopId: targetShopId
            });
        },
        getShopByPlatform: function () {
            //从config中读取platformID
            return this.getBaseUrl() + "/load_shops_by_platform?shopId=" + config.getPlatformShopId();
        },
        getAllShopsByPlatform:function(params){
          return this.exec("load_shops_by_platform",params);  
        },
        load_verifyshops_by_platform: function () {
            return this.getBaseUrl() + "/load_shops_by_platform?shopId=" + config.getPlatformShopId() + "&isVerified=true";
        },
        load_saleshops_by_platform: function () {
            return this.getBaseUrl() + "/load_shops_by_platform?shopId=" + config.getPlatformShopId() + "&isVerified=true&shopTypeIn=true";
        },        
        //获取登陆者所有店铺列表
        getShopListUrl: function () {
            return this.getBaseUrl() + "/load_platform_shops?shopId=" + config.getPlatformShopId();
        },
        //获取指定店铺详情
        getShopDetailUrl: function () {
            return this.getBaseUrl() + "/load_by_id";
        },
        //获取平台商家数
        getShopQtyByPlatform: function (params) {
            if (!params) {
                params = {};
            }
            params.shopId = config.getPlatformShopId();
            return this.exec("load_shops_by_platform", params);
        },
		//获取指定区域的平台商家数
		getRegionShopQtyByPlatform: function (params) {
            if (!params) {
                params = {};
            }
            params.shopId = config.getPlatformShopId();
            return this.exec("region_shop_num", params);
        },
        getRegionShopNum:function(params){
            if (!params) {
                params = {};
            }
            params.shopId = config.getPlatformShopId();
            return this.exec("shop_num_by_lon_lat", params);
        },
        getRegionShops:function(params){
            if (!params) {
                params = {};
            }
            params.shopId = config.getPlatformShopId();
            return this.exec("load_shops_by_lon_lat", params);
        },
		//加载指定区域内的区县内的商家数量
		getRegionCountyShopQtyByPlatform: function (params) {
            if (!params) {
                params = {};
            }
            params.shopId = config.getPlatformShopId();
            return this.exec("region_county_shop_num", params);
        },
		//加载指定商家的分销商数
		getCustomerNum: function (params) {
            if (!params) {
                params = {};
            }
            return this.execInCurrentShop("customer_num", params);
        },
        //提交注册信息
        onSubmitShop: function (params) {
            params.platformShopId = config.getPlatformShopId();
            var def = new dojo.Deferred();
            dojo.xhr("POST", {
                url: config.getDataOrigin() + "/biz/shop/storage_shop/storage_shop/create_shop",
                handleAs: "json",
                content: params,
                load: lang.hitch(this, function (result) {
                    def.resolve(result);
                })
            });
            return def;
        },
        //注销
        logout: function () {
            var def = new Deferred();
            delete this.__shop;
            delete localStorage.shopId;
            def.resolve(true);
            return def;
        },
        getShopMsg: function () {
            return window.__shop;
        },
        //修改推荐商家
        setRecommend: function (params) {
            return this.execInCurrentShop("rec_shop", params);
        },
        // 查找下级商家
        getChildShops: function (params) {
            params.shopId = config.getPlatformShopId();//'5703ca7d0ccae34b2b40ab8c'
            params.platformShopId = config.getPlatformShopId();
            return this.exec("get_child_shops", params);
            //return this.getBaseUrl()+"/get_child_shops";
        },
        //查找下级商家URL
        getChildShopsURL: function () {
            return this.getBaseUrl() + "/get_child_shops?shopId=" + config.getPlatformShopId() + "&platformShopId=" + config.getPlatformShopId();
        },
        getQrCodeByPlatform: function () {
            var params = {};
            params.platformShopId = config.getPlatformShopId();
            return this.exec("get_qrcode_by_platformShopId", params);
        },
        getSummaryShop: function (townId) {
            var params = {};
            params.townId = townId || "";
            params.shopId = config.getPlatformShopId();
            return this.exec("summary_by_townid", params);
        },
        getSummaryShopByParams : function (params) {
            var params = params || {};
            params.shopId = config.getPlatformShopId();
            return this.exec("summary_by_params", params);
        },
        getShopSummaryUrl: function () {
            return this.getBaseUrl() + "/summary_by_townid?shopId=" + config.getPlatformShopId();
        },
        getPartnerUrl: function () {
            return this.getBaseUrl() + "/load_partner?shopId=" + this.getShopId();
        },
        loadVillage: function () {
            return this.exec("load_village_by_id", { shopId: config.getPlatformShopId() });
        },
        loadCompany: function () {
            return this.exec("load_company_by_id", { shopId: config.getPlatformShopId() });
        },
        // 获取商家等级的总数
        getShopGradesCount: function (condition) {
            return this.execInCurrentShop("load_count_by_level", condition);
        },
        createShopNoCode: function (params) {
            params.platformShopId = config.getPlatformShopId();
            return this.exec("create_shop", params);
        },
        verifySubmit: function (params) {
            params = this.filterSaveData(params);
            return this.execInCurrentShop("verify_shop_submit", params);
        },
        //查询商家的粉丝数
        loadFansNum: function (shopId) {
            return this.exec("load_fans_num", {
                shopId: config.getPlatformShopId(),
                targetShopId: shopId
            });
        },
        //支付方式
        getPayType: function () {
            return this.defTest(g_sys_info);
        },
        //获取B2B订单的支付方式
        getB2BPayType: function () {
            return this.exec("load_by_id", {
                //干劲20160805：处理语法错误
                shopId: config.getPlatformShopId()
            }).then(function (data) {
                return data.data;
            });
        },
        getVerifiedCount: function () {
            return this.exec("load_verified_shop_num", {
                shopId: config.getPlatformShopId()
            });
        },
        getPlatformMatNum: function (param) {
			if(!param){
				param = {};
			}
            if(!param.shopId){
                param.shopId = config.getPlatformShopId();
            }
            return this.getShopMatNum(param.shopId);
        },
        getShopMatNum: function (shopId) {
            return this.exec("load_mat_num", {
                shopId: shopId
            });
        },
        // 检查备案号的合法性 酉阳数据统计
        checkShopByIds: function (shopdata) {
            var params = {};
            params.shopId = config.getPlatformShopId();
            params.ids = shopdata;
            return this.exec("check_shop_by_ids", params);
        },
        //同意添加为分销商
        agreeAdd: function (params) {
            return this.exec("agree", {
                shopId: this.getShopId(),
                customerShopId: params
            });
        },
        disAgree: function (params) {
            return this.exec("disagree", {
                shopId: this.getShopId(),
                customerShopId: params
            });
        },
        getSummaryByVerifyUrl: function () {
            return this.getBaseUrl() + "/summary_by_verify?shopId=" + config.getPlatformShopId();
        },
        //修改合作伙伴客户等级
        modifyPartner: function (params) {
            return this.exec("modifyPartner", {
                shopId: this.getShopId(),
                targetShopId: params.targetShopId,
                shopLevel: params.shopLevel,
				inviter: params.inviter
            });
        },
        summaryByInviter: function () {
            return this.getBaseUrl() + "/summary_by_inviter?shopId=" + config.getPlatformShopId();
        },
        summaryByInviters: function () {
            return this.exec("summary_by_inviter", { shopId: config.getPlatformShopId() });
        },
        shopsByInviterUrl : function(){
        	return this.getBaseUrl() + "/shopsByInviter?platformShopId="+config.getPlatformShopId();
        },
        getSubShopFromServiceUrl: function () {
            return this.getBaseUrl() + "/load_sub_from_service?shopId=" + this.getShopId() + "&serviceShopId=" + config.serviceShopId;
        },
        updateCountyInService: function (targetShopId, shopLevel) {
            return this.execInCurrentShop("update_shop_county_in_service", {
                serviceShopId: config.serviceShopId,
                shopLevel: shopLevel,
                targetShopId: targetShopId
            });
        },
        loadInfoFromSevice: function () {
            return this.execInCurrentShop("load_info_from_service", {
                serviceShopId: config.serviceShopId
            });
        },
        getCustomerCount: function (params) {
            return this.execInCurrentShop("get_customer_count",params);
        },
		setOpShop:function(condition){
			return this.exec("set_op_shop", {
                _id: condition._id,
				shopId:config.platformShopId
            });
		},
        getUserNum:function(){
            return this.execInCurrentShop("get_user_num",{});
        },
        execRegistDiscount:function(params){
            return this.execInCurrentShop("execRegistDiscount",params);
        },
        //替换业务经理
        replaceInviter:function(params){
            return this.execInCurrentShop("replaceInviter",params);
        }
    });
    var o = new O();
    lang.setObject("cq.biz.shop.shop", o);
    o.Schema = O;
    return o;
});
