﻿define([
    "dojo/_base/declare",
    "dojo/_base/lang",
    "dojo/_base/array"
],function(declare,lang,array){
    return declare("",null,{
        constructor:function(params){
            lang.mixin(this,params);
        },
        //field的内容
        /**
            检查机制
            所有类型的字段
                notNull
            编辑方式为数字的字段
                type:number，非数字时报错
                range:
                    [0,10]，0到10之间，包含0和10
                    (0,10)，0到10之间，不包含0和10
                    (,0)，小于0
                    (0,)，大于零
            编辑方式为电话的字段
                type:tel，内容不是电话时报错
        **/
        fields:null,
        _getFieldItem:function(field,canEdit){
            var fieldItem = null;
            if(lang.isString(field)){
                fieldItem = lang.clone(this.fields[field]);
            }else{
                fieldItem = lang.clone(this.fields[field.field]);
            }
            if("button"===fieldItem.type){
                fieldItem.type = "action";
            }else if("link"===fieldItem.type){
            }else{
                fieldItem.canEdit = false===fieldItem.canEdit||false===canEdit||false===field.canEdit?false:true;
            }
            fieldItem.field = field&&field.field||field;
            return fieldItem;
        },
        getFormItems:function(fields,canEdit){
            return [].concat(array.map(fields,function(field){
                return this._getFieldItem(field,canEdit);
            },this));
        },
        
        _isNull:function(value){
            return !(value||0===value||false===value);
        },
        _isNumber:function(value){
            return value==parseFloat(value);
        },
        isEmail:function(value){
            return /^([a-zA-Z0-9_-])+@([a-zA-Z0-9_-])+(.[a-zA-Z0-9_-])+/.test(value);
        },
        isUrl:function(value){
            var strRegex = '^((https|http|ftp|rtsp|mms)?://)'
                + '?(([0-9a-z_!~*\'().&=+$%-]+: )?[0-9a-z_!~*\'().&=+$%-]+@)?' //ftp的user@ 
                + '(([0-9]{1,3}.){3}[0-9]{1,3}' // IP形式的URL- 199.194.52.184 
                + '|' // 允许IP和DOMAIN（域名） 
                + '([0-9a-z_!~*\'()-]+.)*' // 域名- www. 
                + '([0-9a-z][0-9a-z-]{0,61})?[0-9a-z].' // 二级域名 
                + '[a-z]{2,6})' // first level domain- .com or .museum 
                + '(:[0-9]{1,4})?' // 端口- :80 
                + '((/?)|' // a slash isn't required if there is no file name 
                + '(/[0-9a-z_!~*\'().;?:@&=+$,%#-]+)+/?)$'; 
            var re=new RegExp(strRegex); 

            return re.test(value);
        },
        isTel:function(value){
            return !!(/^(\(\d{3,4}\)|\d{3,4}-|\s)?\d{7,14}$/.test(value));
        },
        isCellPhone:function(value){
            return !!(/^1\d{10}$/.test(value));
        },
        getDataValidMsg:function(data,fields){
            var msg = "";
            array.some(fields,function(field){
                var fieldItem = this.fields[field],value = data[field];
                if(!fieldItem){
                    return;
                }
                if(fieldItem.notNull&&this._isNull(value)){
                    msg = fieldItem.label+"不能为空";
                    return true;
                }
                if(!this._isNull(value)){
                    if(fieldItem.type==="tel"){
                        if(!(this.isTel(value)||this.isCellPhone(value))){
                            msg = fieldItem.label+"不合法，请输入正确的电话号码";
                            return true;
                        }
                    }
                    if(fieldItem.type==="number"){
                        if(!this._isNumber(value)){
                            msg = fieldItem.label+"不合法，请填写正确的数字";
                            return true;
                        }
                        if(fieldItem.range){
                            var range = fieldItem.range.split(/[\[\(\),\]]/g);
                            if(range[1]){
                                if("["===fieldItem.range[0]){
                                    if(value-range[1]<0){
                                        msg = fieldItem.label+"不能小于"+range[1];
                                        return true;
                                    }
                                }else{
                                    if(value-range[1]<=0){
                                        msg = fieldItem.label+"必须大于"+range[1];
                                        return true;
                                    }
                                }
                            }
                            if(range[2]){
                                if("]"===fieldItem.range[fieldItem.range.length-1]){
                                    if(value-range[1]>0){
                                        msg = fieldItem.label+"不能大于"+range[2];
                                        return true;
                                    }
                                }else{
                                    if(value-range[2]>=0){
                                        msg = fieldItem.label+"必须小于"+range[2];
                                        return true;
                                    }
                                }
                            }
                        }
                    }
                    if(fieldItem.type==="email"){
                        if(!this.isEmail(value)){
                            msg = fieldItem.label+"格式错误，请填写正确的邮箱地址";
                            return true;
                        }
                    }
                    if(fieldItem.type==="url"){
                        if(!this.isUrl(value)){
                            msg = fieldItem.label+"格式错误，请填写正确的网址";
                            return true;
                        }
                    }
                    if(fieldItem.pattern){
                        if(!fieldItem.pattern.test(value)){
                            msg = fieldItem.label+"格式错误";
                            return true;
                        }
                    }
                }
            },this);
            return msg;
        }
    });
});
