﻿define([
    "../../BoBase",
    "dojo/Deferred",
    "cq/biz/old/ncc/districtSpecials",
    "cq/biz/shop/bs/district",
    "cq/biz/shop/partner"
],function(BoBase,Deferred,districtSpecials,district,partner){
    return new BoBase({
        fields : {
            title : {
                type: "text",
                label: "名称",
                editable: true,
                notNull:true,
                placeHolder:"填写特色农产品"
            },
            mid : {
                type:"ref",
                field:"mid",
                label:"商品",
                searchField:"mName",
                pageSize:5,
                labelFields:"mName",
                dataProvider:"cq/biz/mat/mat",
                targetMethod:"getOnSaleUrl",
                canEdit:true,
                notNull:true,
				className:"middle-width"
            },
            "mid.mName":{
                type:"text",
                label:"商品",
                editable: false
            },
            logo : {
                type: "image",
                label: "图标",
                sizeType:"cut",
                notNull:true,
                size:{
                    width:128,
                    height:128
                },
                editable: true
            },
            num : {
                type: "number",
                range:"(0,)",
                label: "关注数量",
                notNull:true,
                placeHolder:"必须为大于0的正数"
            },
            "provinceId,cityId,countyId":{
                type:"district",
                label:"重庆辖区",
                notNull:true,
                fixedUpDis:{
                    upDis:[
                        "57a936a0e026905d4c92f647",
                        "57a936a0e026905d4c92f648"
                    ]
                }
            },
            // countyId : {
                // type:"district",
                // label:"重庆辖区",
                // notNull:true,
                // fixedUpDis:{
                    // upDis:[
                        // "57a936a0e026905d4c92f647"
                    // ]
                // }
            // },
            // "countyId.distName":{
                // type:"text",
                // label:"重庆辖区",
                // editable: false
            // },
            createAt : {
                type : "date",
                label : "操作时间"                
            },
            saveButton:{
				type:"button",
				action:"onSave",
				label:"保存"
			},
            listActions:{
				type:"text",
				label:"",
				formatter:function(action,data){
					var v = "<div class='button' cq-action='delete'>删除</div><div class='button' onclick='window.moveToView(\"nccDistSpecialEdit\",\""+data._id+"\");'>编辑</div>";
					return v;
				}
            }
        },
        
        //列表需要的方法
        getListLayout:function(){
            return this.getFormItems(["title","num","mid.mName","createAt","listActions"],false);
        },
        getSaleStatisticExportLayout : function(){
            return this.getFormItems(["title","num","mid.mName","createAt"]);
        },
        getListUrl:function(){
            return districtSpecials.getLoadUrl();
        },
        
        getListQuery:function(refInfo){
            // if(refInfo&&refInfo.title){
                // return {title:refInfo.title};
            // }else{
                // return {};
            // }
            var query = {};
            if(refInfo.queryInfo){
                var k,queryInfo = refInfo.queryInfo,value;
                for(k in queryInfo){
                    value = queryInfo[k];
                    if(value||0===value||false===value){
                        if(value){
                            if((typeof value == "object")&& value) {
                                if(value._id){
                                    query[k] = value._id;
                                }else if(k!="mid"){
                                    query[k] = value;
                                }
                            }
                            if(typeof value == "string" && value) {
                                query[k] = value;
                            }
                        }else{
                            query[k] = value;
                        }
                    }
                }
            }
            if(refInfo.search){
                query.title = refInfo.search;
            }
            return query;
        },
        // onGroupListClick:function(data,action,list){
            // if(data&&data._id){
                // window.moveToView("nccDistSpecialEdit",data._id);
            // }
        // },
        //查询条件区域结构
        getQueryLayout:function(){
            return [{
                type: "form",
                fields: this.getFormItems(["title","mid","createAt"])
            }];
        },
        getEditLayout:function(){
            return [{
                type: "form",
                fields: this.getFormItems(["title","mid","logo","num","provinceId,cityId,countyId","createAt","saveButton"])
            }];
        },
        loadEditData:function(refInfo){
            if(refInfo&&refInfo.cqViewParam&&refInfo.cqViewParam.params){
                return districtSpecials.loadById(refInfo.cqViewParam.params).then(function(data){
                    var datas =data.data;
                    datas["provinceId,cityId,countyId"] = [datas.provinceId||'',datas.cityId||'',datas.countyId||''];
                    // for(var i in datas) {
                        // if(i == "provinceId,cityId,countyId") {
                            // datas.item[i] = datas[i];
                        // }
                    // }
                    datas.item["provinceId,cityId,countyId"] = datas["provinceId,cityId,countyId"];
                    return datas.item||{};
                });
            }
            return {};
        },
        onListAction:function(data,action,list){
			if(action == "delete"){
				if(!confirm("确定删除吗？")) return;
                data.isDelete = true;
				districtSpecials.save(data).then(function(){list.removeItemByData(data);});
			}
		},
        onChangeData : function(widget,field,value){
            if(field=="mid"){
                widget.updateByField("title",value&&value.mName||"");
                widget.updateByField("logo",value&&value.logo||"");
            }
        },
        canSaveData : function(data){
            var msg = "";
            if(!data.title){
                msg = "请填写名称";
            }else if(!data.mid){
                msg = "请选择商品";
            }else if(!data.logo){
                msg = "请选择图标";
            }else if(!data.num){
                msg = "请填写数量";
            }else if(isNaN(data.num)){
                msg = "数量必须是数字";
            }else if(data.num<0){
                msg = "数量不能为负数，请重新输入";
            }else if(!data.createAt){
                msg = "请选择操作时间";
            }else if(!data.countyId){
                msg = "请选择区县";
            }
            return msg;
        },
		onSave:function(data,dataList){
            var isSave = this.canSaveData(data);
            if(isSave){
                alert(isSave);
                return;
            }
			districtSpecials.save(data).then(function(){history.back();});
		}
    })
});