﻿define([
	"dojo/_base/lang",
	"dojo/Deferred",
    "../../BoBase",
    "cq/biz/old/ncc/nccPrice",
	"cq/biz/old/ncc/nccProduct",
	"cq/biz/old/ncc/nccMarket",
	"cq/biz/old/ncc/nccRegion",
	"cq/biz/user/user",
	"cq/util/config/config",
	"cq/util/sys/date"
],function(lang,Deferred,BoBase,nccPrice,nccProduct,nccMarket,nccRegion,user,config,date){
    return new BoBase({
        //字段定义
        fields:{
            mid:{
                type:"ref",
                label: "品种",
                filed:"mid",
                dataProvider:"cq/bo/old/ncc/nccProduct",
                targetMethod:"getListUrl",
                searchField:"title",
                labelFields:"title",
                pageSize:5,
                canEdit:true,
                className:"middle-width",
                notNull:true
            },
            "mid.title":{
                type:"text",
                label:"品种"
            },
            tp:{
                type: "options",
                label: "销售类型",
                options:{
                    "500001":"零售",
                    "500002":"批发",
                    "500003":"产地",
                    "500004":"收购",
                    "500005":"进场"
                },
                canEdit:true,
                className:"middle-width",
                notNull:true
            },
            marketId:{
                type:"ref",
                label: "市场",
                filed:"marketId",
                dataProvider:"cq/bo/old/ncc/nccMarket",
                targetMethod:"getListUrl",
                pageSize:5,
                searchField:"title",
                labelFields:"title",
                canEdit:true,
                className:"middle-width",
                notNull:true
            },
            "marketId.title":{
                type:"text",
                label:"市场"
            },
            makeDate:{
                type: "date",
                notNull:true,
                label: "录入日期"
            },
            maker:{
                type:"text",
                label:"录入人",
                notNull:true
            },
            price:{
                type:"number",
                label:"单价",
                notNull:true,
                range:"(0,)"
            },
            regionId:{
                type: "ref",
                label: "产地",
                filed:"regionId",
                dataProvider:"cq/bo/old/ncc/nccRegion",
                targetMethod:"getListUrl",
                canEdit:true,
                pageSize:5,
                searchField:"title",
                labelFields:"title",
                className:"middle-width",
                notNull:true
            },
            "regionId.title":{
                type:"text",
                label:"产地"
            },
            isDelete:{
                type:"text",
                label:"",
                formatter:function(action,data){
                    var v = (!data.isDelete?"<div class='button' cq-action='delete'>删除</div><div class='button' cq-action='edit'>编辑</div>":"");
                    return v;
                }
            },
            saveButton:{
                type:"button",
                action:"onSave",
                label:"保存"
            }
        },
        //列表需要的方法
        getListLayout:function(){
            return this.getFormItems(["mid.title","tp","marketId.title","makeDate","maker","price","regionId.title","isDelete"],false);
        },
        getExportLayout:function(){
            return this.getFormItems(["mid.title","tp","marketId.title","makeDate","maker","price","regionId.title"],false);
        },
        getListUrl:function(){
            return nccPrice.getLoadUrl();
        },
        getListQuery:function(params){
            var query = {};
            if(params.queryInfo){
                var k,queryInfo = params.queryInfo,value;
                for(k in queryInfo){
                    value = queryInfo[k];
                    if(value||0===value||false===value){
                        if(value){
                            if((typeof value == "object")&& value) {
                                if(value._id){
                                    query[k] = value._id;
                                }else if(JSON.stringify(value) === "{}"){
                                    delete query[k];
                                }else{
                                    query[k] = value;
                                }
                            }
                            if(typeof value == "string" && value) {
                                query[k] = value;
                            }
                        }else{
                            query[k] = value;
                        }
                    }
                }
            }
            return query;
        },
        getQueryLayout:function(){
            return [{
                type: "form",
                fields: this.getFormItems(["mid","tp","marketId","regionId"])
            }];
        },
        onPriceTrendListClick:function(data,action,list){
            if(data&&data._id){
                if(action == "delete"){
                    var deleteConfirm = confirm("是否删除?")
                    if(!deleteConfirm) return;
                    data.isDelete = true;
                    nccPrice.save(data).then(function(){list.removeItemByData(data);});
                    return;
                }
                if(action == "edit"){
                    window.moveToView("nccPriceTrendEdit",data._id);
                }
            }
        },
        getEditLayout:function(){
            return [{
                type: "form",
                fields: this.getFormItems(["marketId","regionId","mid","tp","price","makeDate","maker","saveButton"])
            }];
        },
        onPriceReload:function(widget){
            //widget.disableByFields(["makeDate"]);
        },
        loadEditData:function(refInfo){
            if(refInfo&&refInfo.cqViewParam&&refInfo.cqViewParam.params){
                return nccPrice.loadById(refInfo.cqViewParam.params).then(function(data){
                    return data.data.item||{};
                });
            }
			var userInfo = user.getCurrentShopUserData();
            return {
				makeDate:date.newDateISOString(),
				maker:userInfo&&(userInfo.pName||userInfo.nickName)||"管理员"
			};
        },
        onSave:function(data,dataList){
            var fields = [];
            for ( var k in this.fields){
                fields.push(k);
            }
            var msg = this.getDataValidMsg(data,fields);
            if(msg){
                alert(msg);
                return;
            }
            nccPrice.save(data).then(function(){history.back();});
        },
		
		
		
		importPrice:function(list){
			//品种	销售类型	市场  录入日期	单价	产地
			config.promptText({
				title:"粘贴价格数据",
				placeHolder:"依次为：品种，销售类型，市场，录入日期(yyyy-MM-dd或yyyy/MM/dd)，单价，产地",
				callback:lang.hitch(this,function(str){
					var result = this._importData(str);
					result.then(function(success){
						if(success){
							list.refresh();
						} m
					});
					return result;
				})
			});
		},
		_matHash:{},
		_loadMatInfo:function(matHash){
			var matNames = [];
			var def = new Deferred();
			for(var k in matHash){
				if(this._matHash[k]){
					matHash[k] = this._matHash[k];
				}else{
					matNames.push(k);
				}
			}
			if(matNames.length){
				return nccProduct.execInCurrentShop("load_by_names",{
					names:matNames
				}).then(lang.hitch(this,function(data){
					var items = data.data&&data.data.items||[];
					for(var i=0;i<items.length;i++){
						this._matHash[items[i].title] = items[i]._id;
					}
					for(var k in matHash){
						if(matHash[k]===true){
							if(!this._matHash[k]){
								return def.resolve(k);
							}else{
								matHash[k] = this._matHash[k];
							}
						}
					}
					def.resolve();
				}));
			}else{
				def.resolve();
			}
			return def;
		},
		_regionHash:{},
		_loadRegionInfo:function(regionHash){
			var regionNames = [];
			var def = new Deferred();
			for(var k in regionHash){
				if(this._regionHash[k]){
					regionHash[k] = this._regionHash[k];
				}else{
					regionNames.push(k);
				}
			}
			if(regionNames.length){
				nccRegion.execInCurrentShop("load_by_names",{
					names:regionNames
				}).then(lang.hitch(this,function(data){
					var items = data.data&&data.data.items||[];
					for(var i=0;i<items.length;i++){
						this._regionHash[items[i].title] = items[i]._id;
					}
					for(var k in regionHash){
						if(regionHash[k]===true){
							if(!this._regionHash[k]){
								return def.resolve(k);
							}else{
								regionHash[k] = this._regionHash[k];
							}
						}
					}
					def.resolve();
				}));
			}else{
				def.resolve();
			}
			return def;
		},
		_marketHash:{},
		_loadMarketInfo:function(marketHash){
			var marketNames = [];
			var def = new Deferred();
			for(var k in marketHash){
				if(this._marketHash[k]){
					marketHash[k] = this._marketHash[k];
				}else{
					marketNames.push(k);
				}
			}
			if(marketNames.length){
				nccMarket.execInCurrentShop("load_by_names",{
					names:marketNames
				}).then(lang.hitch(this,function(data){
					var items = data.data&&data.data.items||[];
					for(var i=0;i<items.length;i++){
						this._marketHash[items[i].title] = items[i]._id;
					}
					for(var k in marketHash){
						if(marketHash[k]===true){
							if(!this._marketHash[k]){
								return def.resolve(k);
							}else{
								marketHash[k] = this._marketHash[k];
							}
						}
					}
					def.resolve();
				}));
			}else{
				def.resolve();
			}
			return def;
		},
		_importData:function(textStr){
			var def = new Deferred(),rows = lang.trim(textStr).split("\n");
			if(rows.length){
				var i,mNames = {},marketNames = {},tps = {},regionNames = {};
				for(var i=0;i<rows.length;i++){
					rows[i] = rows[i].split("\t");
					// if(rows[i].length===5){
						// rows[i][0] = lang.trim(rows[i][0]);	  		
						// rows[i][1] = lang.trim(rows[i][1]);
						// rows[i][2] = lang.trim(rows[i][2]);
						// rows[i][3] = lang.trim(rows[i][3]);
						// rows[i][4] = lang.trim(rows[i][4]);
						// mNames[rows[i][0]] = true;
						// tps[rows[i][1]] = true;
						// marketNames[rows[i][2]] = true;
						// regionNames[rows[i][4]] = true;
						// if(!(0<=rows[i][3])){
							// alert("第" +(i+1)+" 行价格错误");
						// }
					// }else{
						// alert("第" +(i+1)+" 行数据不合法");
						// return false;
					// }					
                    if(rows[i].length===6){
						rows[i][0] = lang.trim(rows[i][0]);//品种
						rows[i][1] = lang.trim(rows[i][1]);//销售类型
						rows[i][2] = lang.trim(rows[i][2]);//市场
						rows[i][3] = lang.trim(rows[i][3]);//录入日期
						rows[i][4] = lang.trim(rows[i][4]);//单价
                        rows[i][5] = lang.trim(rows[i][5]);//产地
						mNames[rows[i][0]] = true;
						tps[rows[i][1]] = true;
						marketNames[rows[i][2]] = true;
						regionNames[rows[i][5]] = true;
						if(!(0<=rows[i][4])){
							alert("第" +(i+1)+" 行价格错误");
                            return;
						}
                        //限制录入日期格式(yyyy-MM-dd或yyyy-M-d)
                        if(!new RegExp("^[1-2]\\d{3}(-||/)(0?[1-9]||1[0-2])(-||/)(0?[1-9]||[1-2][0-9]||3[0-1])$").test(rows[i][3])) {
                            alert("录入日期不合法，请按照yyyy-MM-dd或yyyy/MM/dd格式输入合法日期");
                            return;
                        }
					}else{
						alert("第" +(i+1)+" 行数据不合法");
						return false;
					}
				}
				var k;
				if(!this._tpHash){
					this._tpHash = {};
					var options = this.fields.tp.options;
					for(k in options){
						this._tpHash[options[k]] = k;
					}
				}
				for(k in tps){
					if(!this._tpHash[k]){
						alert("无效的销售类型："+k);
						def.resolve();
						return def;
					}
				}
				this._loadMatInfo(mNames).then(lang.hitch(this,function(mName){
					if(mName){
						alert("系统中没有品种："+mName);
						return def.resolve();
					}
					this._loadMarketInfo(marketNames).then(lang.hitch(this,function(market){
						if(market){
							alert("无效的市场："+market);
							return def.resolve();
						}
						this._loadRegionInfo(regionNames).then(lang.hitch(this,function(region){
							if(region){
								alert("无效的产地："+region);
								return def.resolve();
							}
							var userInfo = user.getCurrentShopUserData();
							var maker = userInfo&&(userInfo.pName||userInfo.nickName)||"管理员";
							var makeDate = date.newDateISOString();
							nccPrice.execInCurrentShop("batch_create",{
								rows:rows.map(lang.hitch(this,function(data){
									// return {
										// mid:this._matHash[data[0]],
										// tp:this._tpHash[data[1]],
										// marketId:this._marketHash[data[2]],
										// price:data[3],
										// regionId:this._regionHash[data[4]],
										// maker:maker,
										// makeDate:makeDate
									// };
									return {
										mid:this._matHash[data[0]],
										tp:this._tpHash[data[1]],
										marketId:this._marketHash[data[2]],
										price:data[4],
										regionId:this._regionHash[data[5]],
										maker:maker,
										makeDate:data[3]
									};
								}))
							}).then(function(data){
								if(data&&data.data&&data.data.ok){
									def.resolve(true);
								}else{
									def.resolve();
								}
							});
						}));
					}));
				}));
			}else{
				def.resolve(true);
			}
			return def;
		}
    });
});