define([
    "cq/biz/shop/shop",
    "cq/biz/ts/pm/shopPayLog",
    "cq/biz/ts/transaction",
    "cq/biz/shop/freightTemplate",
    "cq/biz/user/user",
    "dojo/_base/lang",
    "cq/util/config/config",
    "cq/util/sys/xlsx",
    "./BoBase"
],function(shop,shopPayLog,transaction,freightTemplate,user,lang,config,xlsx,BoBase){
    var banlanceBo =  new BoBase({
        fields:{
            "fromShopId.shopName": {
                type: "text",
                label: "商家名称"
            },
            "fromShopId.address": {
                type: "text",
                label: "商家地址"
            },
            ioType : {
                type: "options",
                label: "操作类型",
                options : {
                    0 : "充值",
                    1 : "提现"
                },
                className:"min-width"
            },
            isVerified : {
                type: "options",
                label: "状态",
                options : {
                    false : "待审核",
                    true : "已审核"
                },
                className:"min-width"
            },
            shoppe : {
                type:"ref",
                label : "商家",
                dataProvider: "cq/biz/shop/shop",
                targetMethod: "load_saleshops_by_platform",
                className:"middle-width",
                searchField: "shopName",
                labelFields: "shopName"          
            },
            // logo: {
                // type: "image",
                // sizeType:"cut",
                // size:{
                    // width:12,
                    // height:12
                // },
                // label: "店铺图标",
                // className:"middle-width"
            // },
            shopName: {
                type: "text",
                label: config.shopFieldLabel&&config.shopFieldLabel.shopName||"店铺名称",
				className:"middle-width"
            },
            saleAmount : {
                type: "text",
                label: "销售额",
				className:"min-width"                
            },
            address: {
                type: "text",
                label: "详细地址",
				className:"middle-width"
            },
            bankAccount : {
                type: "text",
                label: "银行账号",
                className:"middle-width"                
            },
            bankName: {
                type: "text",
                label: "开户行",
                className:"middle-width"  
            },
            amount:{
                type : "text",
                label: "金额",
                className:"min-width" 
            },
            tsCode : {
                type: "text",
                label: "流水号",
                className:"middle-width"     
            },
            fiName: {
                type: "text",
                label: "财务联系人",
                className:"middle-width" 
            },
            fiQQ: {
                type: "text",
                label: "财务联系人QQ",
                className:"middle-width" 
            },
            fiPhone: {
                type: "text",
                label: "财务联系人电话",
                className:"middle-width" 
            },
            remark :{
                type: "text",
                label: "备注",
                className:"large-width"                
            },
            timeAt : {
                type : "date",
                label : "时间" 
            },
            getAt : {
                type : "date",
                label : "时间"               
            },
            tsState : {
                type: "options",
                label: "交易状态",
                options : {
                    1:"进行中",
                    2:"已结算",
                    3:"暂停",
                    4:"取消"
                }                
            },
            listActions:{
				type:"text",
				label:"操作",
				formatter:function(action,data){
					var v = 
                        (data.isVerified == false?"<div class='dgrid-button' cq-action='verify'>审核</div>":"")+
						"<div class='dgrid-button' onclick='window.moveToView(\"balanceDetail\",\""+data.fromShopId._id+"\");'>往来明细</div>";
					return v;
				},
				className:"large-width"
            },
            tsType :{
                type:"options",
                label : "类型",
                options:{
                    "" : "暂无交易",
                    "1": "会员购买(在线支付金额)",
                    "2":"支付手续费",
                    "3":"交易手续费",
                    "4":"交易补贴",
                    "5":"提现",
                    "6":"充值"
                }
            },
            payType :{
                type:"text",
                label : "支付方式"
            },
            payLogId :{
                type:"text",
                label : "交易单号"
            },
            tsAmount :{
                type:"text",
                label : "金额"
            },
            createAt : {
                type : "date",
                label : "日期"                
            },
            freightName : {
                type : "text",
                label : "模板名称:"
            },            
            freightTitle : {
                type:"text",
                placeHolder:"填写模板标题",
                label:"标题",
				className:"large-width"
            },
            minQty : {
                type:"text",
                placeHolder:"填写首件数量",
                label:"首件",
				className:"large-width"              
            },
             minFee : {
                type:"text",
                placeHolder:"填写首件运送价格",
                label:"首费",
				className:"large-width"              
            },
            incQty : {
                type:"text",
                placeHolder:"填写续件数量",
                label:"续件",
				className:"large-width"              
            },
            incFee : {
                type:"text",
                placeHolder:"填写续件费用",
                label:"续费",
				className:"large-width" 
            },
			"addException" : {
				type : "objects",
				label:"例外城市运费",
				canEdit : true,
				canDelete : true,
				canAdd : true,
				fields : {
					layout : [{
                        field : "address",
					    type : "district",
					    label : "区划",
					    className:"large-width"
                    },{
						field : "minQty",
						type : "text",
						label : "首件",
                        placeHolder:"1",
                        className:"middle-width"
					}, {
						field : "minFee",
						type : "text",
						placeHolder : "填写首件运送价格",
						label : "首费",
                        className:"middle-width"
					}, {
						field : "incQty",
						type : "text",
                        placeHolder:"1",
						label : "续件",
						className:"middle-width"
					},{
						field : "incFee",
						type : "text",
                        placeHolder : "填写续件费用",
						label : "续费",
                        className:"middle-width"
					}]
				},
                tag:"申报信息"
			}
        },
        getBalanceDetailLayout : function(){
            if(!this.balanceDetailLayout){
                this.balanceDetailLayout = [{
                    type: "form",
                    fields: this.getFormItems(["shopName","address","bankAccount","bankName","amount","tsCode","fiName","fiPhone","fiQQ","remark"],false)
                }];
                this.balanceDetailLayout.push();
            }
            return this.balanceDetailLayout;
        },
        loadShopById:function(data){
            var id = data.cqViewParam.params;
			return shopPayLog.loadById(id).then(lang.hitch(this,function(data){
				for(var i in data.data.item.fromShopId) {
                    data.data.item[i] = data.data.item.fromShopId[i];
                }
                return data.data.item;
			}));
		},
        getBalanceDetailQueryLayout : function(){
            return [{
                type: "form",
                fields: this.getFormItems(["timeAt","tsState"])
            }];            
        },
        getPlatformUrl : function(){
            return  shopPayLog.loadByPlatformId(); 
        },
        getBalanceQueryParam:function(params){
            var query = {};
            if(params.queryInfo){
                var k,queryInfo = params.queryInfo,value;
                for(k in queryInfo){
                    
                    value = queryInfo[k];
                    if(value||0===value||false===value){
                        if(value){
                            if((typeof value == "object")&& value) {
                                if(value._id){
                                    query[k] = value._id;
                                }else if(JSON.stringify(value) === "{}"){
                                    delete query[k];
                                }else{
                                    query[k] = value;
                                }
                            }
                            if(typeof value == "string" && value) {
                                query[k] = value;
                            }
                        }else{
                            query[k] = value;
                        }
                    }
                }
            }
			if(params&&params.state&&params.state._id){                
                if(1 == params.state._id){                    
                    query.isVerified = true;
                }
				if(0 == params.state._id){                    
                    query.isVerified = false;
                }
            }
            return query;
        },
        getGridLayout: function() {
            if(!this.balanceGridLayout){
                this.balanceGridLayout = this.getFormItems(["fromShopId.shopName","fromShopId.address","amount","ioType","isVerified","listActions"],false);
            }
            return this.balanceGridLayout;
        },
        balanceAction : function(data,action,list){
            if(action == "verify"){
				// shopPayLog.verify({_id : data._id}).then(function(){
                    // list.removeItemByData(data); 
                // });
                shopPayLog.verify({_id : data._id}).then(lang.hitch(this,function(backData){
                    if(1 == backData.data.ok){
                        alert("审核成功");
                        list.refreshItemByData(backData&&backData.data&&backData.data.item||backData);
                    }else{
                        alert("服务器繁忙，请稍后再试");
                    }                    
                }));
			}           
        },
        //平台结算查询
        getBalanceQueryLayout:function(){
            return [{
                type: "form",
                fields: this.getFormItems(["getAt","shoppe","ioType"])
            }];
        },
        getShopBalanceExportLayout : function(){
            return this.getFormItems(["fromShopId.shopName","fromShopId.address","amount","ioType","isVerified"]);
        },
        getTrantransactionUrl : function(){          
            return  transaction.loadTshop()+"shopId="+window.location.href.split("-")[1];
        },
        detailLayout : function(){
            if(!this.detailGridLayout){
                this.detailGridLayout = this.getFormItems(["tsType","payType","payLogId","createAt","tsAmount"]);
            }  
            return this.detailGridLayout;
        },
        addFreightLayout : function(){
            return  this.freightAddLayout =  this.getFormItems(["freightName"],true);
        },
        freightLayout : function(){
            return [{
                type: "form",
                fields: this.getFormItems(["freightTitle","minQty","minFee","incQty","incFee","addException"])
            }];              
        },
        getfreightDetailUrl : function(){
            return  freightTemplate.getLoadUrl();
        },
        getFreightLayout: function() {
            if(!this.FreightLayout){
                this.FreightLayout = this.getFormItems(["minQty","minFee"],false);
            }
            return this.FreightLayout;
        },
        getfreightDetailParam : function(params){
            var query = {};
            if(params.search){
                query.search = params.search;
            }
            return query;
        },
        getSaleQueryLayout : function(){
            return [{
                type: "form",
                fields: this.getFormItems(["getAt"])
            }];
        },
        getShopsUrl : function(){
            var windowDid = window.location.href.split("-")[1];
            if(windowDid){
                 return shop.shopsByInviterUrl()+"&inviter="+window.location.href.split("-")[1];        
            }else{
                return shop.shopsByInviterUrl();
            }
        },
        getSaleGridLayout: function() {
            if(!this.saleGridLayout){
                this.saleGridLayout = this.getFormItems(["shopName","saleAmount","createAt"],false);
            }
            return this.saleGridLayout;
        },
        getLayoutMethod:function(){
            if(!this.layoutItem){
                this.layoutItem = this.getFormItems(["fromShopId.shopName","fromShopId.address",'ioType'],false);
            }
            return this.layoutItem;
        },
        getDataMethod:function(){
            var data ={
                "fromShopId.shopName":20,
                "fromShopId.address":30,
                ioType:50
            };
            return data;
        },
        getDateQueryLayout:function(){
            if(!this.layoutItem){
                this.layoutItem = this.getFormItems(["fromShopId.shopName","fromShopId.address",'ioType'],false);
            }
            return this.layoutItem;
        }
    });
    return banlanceBo;
});