/**
 * @time 2017/01/09
 * @user 杜飞军
 * @note
 * 		增加一个操作日志，只有管理员和高级管理员查看时才显示
 */
define([
	"dojo/_base/declare",
	"dojo/_base/lang",
	"dojo/store/JsonRest",
	"dijit/registry",
	"dojox/mobile/View",
	"dojox/mobile/EdgeToEdgeStoreList",
	"cq/biz/log/operationLog",
	"cq/ui/view/_ViewMixin",
	"cq/util/sys/date",
	"cq/util/sys/xlsx",
], function (declare, lang, JsonRest, registry, View, EdgeToEdgeStoreList, operationLog, _ViewMixin, date, xlsx) {
	return declare("cq.ui.b.s.OperationLog", [View, _ViewMixin], {
		//为列表设置的数据开始默认值
		start: 0,
		pageSize: 10,
		onAfterShow: function () {
			if (!this.store) {
				this.storeList.setStore(this.store = new JsonRest({
					target: this.getListTarget()
				}), {
					count: this.pageSize
				});
				//domClass.add(query('.tab-selector', this.selectBox)[0], "selected");
			} else {
				//this._selectTabAndFetch(query(".tab-selector", this.selectBox)[0], 0);
			}
		},
		onFirstShow: function () {
			var option = {
				left: {
					iconButton: ' ',
					arrow: "left",
					label: "",
					onClick: function () {
						history.back();
					}
				},
				title: "操作日志"
			};
			this.addHeader(option);
			option = {
				isHeader: true,
				placeHolder: "请输入要查询的条件，按回车查询",
				class: "cqSearchBox",
				type: "search",
				onInput: lang.hitch(this, function (evt) {
					if (evt.keyCode == 13) {
						this.params.search = evt.target.value;
						this.setQuery(this.params);
						this.params.search = '';
						evt.target.value = "";
					}
				})
			};
			// 日志列表
			this.storeList = new EdgeToEdgeStoreList({
				append: true,
				constraint: false,
				query: {},
				class: "icon-list-item",
				variableHeight: true,
				//获取数据完成后调用的的处理方法
				onComplete: lang.hitch(this, function (data) {
					for (var i = 0; i < data.length; i++) {
						data[i]._label = "<div class='column-1 col user-list-margin'>时间："+date.formatISODateMinutes(data[i].operationDate) +"</div>" +
							"<div class='column-1 col user-list-margin'>操作人：" + (data[i].operatorName || '') + (data[i].operatorTel || '') + "</div>" +						
							"<div class='column-1 col user-list-margin' >操作事项：" + data[i].content + "</div>" +
							"<div class='column-1 col user-list-margin' >结果：" + (data[i].result ? '成功。' : '失败。') +(data[i].message||'')+ "</div>";
					}

					this.storeList.generateList(data);
					this.end = this.start + data.length - 1;
					this.busy = false;
				}),
                // 查询条件
                queryOptions: {
                    count: this.pageSize,
                    start: this.start
                },
				itemMap: {
					_label: "label",
					logo: "icon"
				}
			});
			// 监控couList的点击
			this.storeList.on("click", lang.hitch(this, this.onListClick));
			this.storeList.placeAt(this.centerPanel.containerNode, "last");
			//绑定事件--加载更多
			this.centerPanel.on("afterscroll", lang.hitch(this, this.onLoadMore));
			this.addTextBox(option);
			// 添加底部功能
			option = {
				columns: [{
					label: '导出',
					onClick: this.exportList
				}],
				labelClass: 'flex2 footer-tab-highlight noverify',
				isFooter: true
			};
			this.addColumns(option);
			this.onAfterShow();
		},
		//加载列表的url
		getListTarget: function () {
			return operationLog.getLoadUrl();
		},
		// 设置查询条件
		setQuery: function (query) {
			this.storeList.destroyDescendants();
			this.start = 0;
			this.end = -1;
            this.setStoreQuery(query);
			this.centerPanel.resize();
		},
        setStoreQuery:function(query){
            return this.storeList.setQuery(query, {
				start: this.start,
				count: this.pageSize
			});
        },
		// 列表点击处理
		onListClick: function (e) {
			var li = registry.getEnclosingWidget(e.target);
			if (!li.params || !li.params.operand) return;
			var param = {
				'Orderh': 'orderDetailB',
				'Mat': 'matDetailB',
				'Shop':'supplyBusDetail'
			};
			var data = li.params.operand;
			if (param[data.modelName]&&data.operand&&data.operand._id) {
				window.moveToView(param[data.modelName], data.operand._id);
			}
		},
		//加载更多，参数evt，返回值：无
		onLoadMore: function (evt) {
			if (!this.busy && evt.afterBottom) {
				this.start = this.start + this.pageSize;
				if (this.canLoadMore(this.start)) {
					this.busy = true;
					setTimeout(lang.hitch(this, function () {
                        var queryResult = this.setStoreQuery(this.storeList.get("query"));
						this.onLoadListData(queryResult, this.start, this.pageSize);
					}), 1000);
				}
			}
		},
		// 导出功能
		exportList: function () {
			var query = this.storeList.query;
			var url = this.getListTarget();
			//var cols = orderListSale.getExportAppLayout();
			//xlsx.exportData(url,query,cols,Date.now().toString());
			//依据url和query加载所有要到处的订单数据
			xlsx.loadAllData(url, query).then(lang.hitch(this, function (items) {
				//设置表头
				var datas = [
					["时间", "操作人", "操作事项", "结果", "原因", "操作对象"]
				];
				//初始列表数据
				for (var i = 0; i < items.length; i++) {
					var data = items[i];
					data.operationDate = date.formatISODateMinutes(data.operationDate);
					if(data.operand.modelName == "Orderh" && data.operand.operand && data.operand.operand._id){
						data.objectMsg = data.operand.operand._id;
					}
					if(data.operand.modelName == "Mat" && data.operand.operand && data.operand.operand.mName){
						data.objectMsg = data.operand.operand.mName;
					}					
					datas.push([data.operationDate, data.operatorName, data.content, (data.result ? '成功' : '失败'), data.message, data.objectMsg]);
				}
				//导出成xlsx文件
				xlsx.exportMatrix(datas, Date.now().toString());
			}));
		}
	});
});