﻿define([
    "dojo/_base/declare",
    "dojo/_base/lang",
    "dojo/dom-style",
    "dojo/dom-construct",
    "dojo/Deferred",
    "dojox/mobile/View",
    "dojox/mobile/FixedSplitter",
    "dojox/mobile/Pane",
    "dojox/mobile/ScrollablePane",
    "dojox/mobile/Heading",
    "dojox/mobile/Button",
    "dojox/mobile/ToolBarButton",
    "cq/app/widget/mobile/List",
    "cq/app/widget/web/DataLayout",
    "bob/user/consumer"
],function(declare,lang,domStyle,domConstruct,Deferred,
    View,FixedSplitter,Pane,ScrollablePane,Heading,Button,ToolBarButton,
    List,DataLayout,consumer){
    //公共基类，处理上中下划分区间，类似于cq/ui/view/EditBase
    var Base = declare("",[View],{
        //内容部分是否可以拖动
        contentScroll:true,
        //初始化界面元素或界面元素的接口
        buildRendering:function(){
            this.inherited(arguments);
            domStyle.set(this.domNode,"min-height","480px");
            //划分界面，并切分界面，提供接口
            this.domNode.style.height = "100%";
            this.addChild(this.splitter = new FixedSplitter({
                variablePane:1,
                orientation:"V"
            }));
            this.splitter.addChild(this.headerPane = new Pane());
            // 中间区域不能使用ScrollablePane ，否则contenteditable属性失效
            this.splitter.addChild(this.contentPane = new Pane());
            if(this.contentScroll){
                var scrollablePane = new ScrollablePane({
                    height:"100%"
                });
                scrollablePane.placeAt(this.contentPane.domNode);
                this.contentPane = scrollablePane;
            }
            this.splitter.addChild(this.bottomPane = new Pane({
                "class": "hflex"
            }));
            this.onAfterAddedPane();
        },
        onAfterAddedPane:function(){
            //处理顶部按钮栏
            //处理中间内容部分
            //处理底部内容
        },
        //开始编辑 
        startEdit:function(params,refView){
            this.refView = refView;
            this._editDef = new Deferred();
            this.placeAt(this.refView.domNode,"before");
            this.initEdit(params);
            this.show();
            this.resize();
            return this._editDef;
        },
        //完成按钮的响应事件
        finish:function(){
            this._finish(this.getValue());
        },
        //获取编辑之后的数据
        getValue:function(){
            
        },
        //取消按钮的响应事件
        cancel:function(){
            this._finish(this.getOriginValue());
        },
        //返回编辑前的数据
        getOriginValue:function(){
            
        },
        //真正执行完成操作
        _finish:function(value){
            this.refView.show();
            this._editDef.resolve(value);
			setTimeout(lang.hitch(this,function(){
				this.refView.resize();
			}),100);
        }
    });
    //地址编辑对象
    var AddrEditor = declare("",[Base],{
        //初始化界面元素并绑定事件
        onAfterAddedPane:function(){
            //顶部工具条
            this.header = new Heading({
                label:"收货地址编辑"
            });
            this.header.placeAt(this.headerPane.domNode);
            //左侧按钮
            this.header.addChild(new ToolBarButton({
                arrow : "left",
                label:"",
                onClick:lang.hitch(this,this.cancel)
            }));
            //右侧按钮
            //this.header.addChild(new ToolBarButton({
            //    style:"float:right;",
            //    label:"完成",
            //    onClick:lang.hitch(this,this._onFinishClick)
            //}));
            //编辑区域
            this.editLayout = new DataLayout({
                layout:[{
                    "type":"form",
                    "fields":[{
                        "className":"large-width",
                        "type":"text",
                        "label":"收货人",
                        "placeHolder": "填写收货人",
                        "canEdit":true,
                        "field":"contactName"
                    },{
                        "className":"large-width",
                        "type":"text",
                        "label":"收货电话",
                        "placeHolder": "填写收货电话",
                        "canEdit":true,
                        "field":"tel"
                    },{
                        "type":"district",
                        "label":"区划",
                        "canEdit":true,
                        "field":"provinceId,cityId,countyId,distAddr"
                    },{
                        "className":"large-width",
                        "type":"text",
                        "label":"收货地址",
                        "placeHolder": "填写收货地址",
                        "canEdit":true,
                        "field":"detailAddress"
                    },{
                        "className":"large-width no-border",
                        "type":"checkbox",
                        "label":"默认地址",
                        "canEdit":true,
                        "field":"isDefault"
                    },{
                        "type": "location",
                        "distFields":"provinceId,cityId,countyId",
                        "distFieldsStr":"distAddr",
                        "className": "none",
                        "label": "获取当前位置",
                        "field":"lon,lat"
                    }]
                }]
            });
            this.editLayout.placeAt(this.contentPane.containerNode);
            // 添加一个按钮
            this.contentPane.containerNode.appendChild(domConstruct.create("div", {
                "class": "fullButton",
                innerHTML: "完成",
                style: "margin-top:10px;",
                onclick: lang.hitch(this, this._onFinishClick)
            }));
        },
        _onFinishClick:function(){
            //读取编辑对象中数据
            var data = this.editLayout.getData();
            //检查数据合法性
            var msg = this.addr.validAddr(data);
            //若不合法，报错并返回
            if(msg){
                alert(msg);
                return;
            }
            window.busy();
            //执行保存操作
            this.addr.saveUserAddr(data).then(lang.hitch(this,function(savedData){
                window.free();
                if(savedData&&savedData.data&&savedData.data.item){
                    //将保存结构设置为当前值
                    this.editLayout.setData(savedData.data.item);
                    //执行完成操作
                    this.finish();
                }else{
                    alert("保存失败");
                    this.cancel();
                }
            }));
        },
        //开始编辑
        initEdit:function(params){
            var dom;
            //不确定是编辑还是新增，每次清除获取位置和隐藏当前位置
            if(this.getLocationBt){
                this.getLocationBt.destroyRecursive();
            }
            if(params){
                window.busy();
                this.addr.getAddrById(params).then(lang.hitch(this,function(data){
                    if(data&&data.data&&data.data.item){
                        this.originData = data.data.item;
                        this.editLayout.setData(lang.clone(data.data.item));
                    }else{
                        this.originData = null;
                        this.editLayout.setData(null);
                    }
                    window.free();
                }));
            }else{
                this.originData = null;
                //显示最下方的button获取当前位置
                this.editLayout.setData(null);
                //如果是新增地址，创建底部获取当前位置按钮
                if(!dom){
                    dom = domConstruct.create("div",{
                        style:"text-align: center"
                    });
                }
                this.getLocationBt = new Button({
                    className:"mini-btn",
                    label:"获取当前位置",
                    onClick:lang.hitch(this,function(){
                        consumer.getCurrentUser().then(lang.hitch(this,function(userData){
                            var lon = userData.lon;
                            var lat = userData.lat;
                            var loc = this.editLayout.getEditorByField(["lon,lat"]);
                            loc.setValue([userData.lon,userData.lat]);
                        }));
                    })
                });
                dom.appendChild(this.getLocationBt.domNode);
                this.contentPane.containerNode.appendChild(dom);
            }
        },
        //获取当前数据，保存之后的数据
        getValue:function(){
            return this.editLayout.getData();
        },
        //获取原始值
        getOriginValue:function(){
            return this.originData;
        }
    });
    //地址选择和管理对象
    //      列表选择和地址管理列表界面
    //      地址列表
    return new declare("",Base,{
        //初始化界面元素并绑定事件
        onAfterAddedPane:function(){
            //顶部工具条
            this.header = new Heading({
                label:"我的收货地址"
            });
            this.header.placeAt(this.headerPane.domNode);
            //左侧按钮
            this.header.addChild(new ToolBarButton({
                arrow : "left",
                label:"",
                onClick:lang.hitch(this,this._onCancelClick)
            }));
            //右侧按钮
            this.header.addChild(this.manageButton = new ToolBarButton({
                style:"float:right;",
                label:"管理",
                onClick:lang.hitch(this,function(){
                    this.toMode("manage");
                })
            }));
            //两个列表
            //编辑区域
            this.selectList = new List({
                itemType:"IconLabel",
                items:[],
                dataProvider:this,
                clickMethod:"onSelectListClick"
            });
            this.selectList.placeAt(this.contentPane.containerNode);
            this.manageList = new List({
                itemType:"IconLabel",
                items:[],
                dataProvider:this,
                clickMethod:"onManageListClick"
            });
            this.manageList.placeAt(this.contentPane.containerNode);
            
            this.addButton = new Button({
                className:"addrBt flex1",
                label:"添加收货地址",
                onClick:lang.hitch(this,function(){
                    //添加时依然是空的
                this._editAddr().then(lang.hitch(this,function(){
                    this.toMode("select");
                }));
                })
            });
            this.addButton.placeAt(this.bottomPane.domNode);
        },
        toMode:function(mode){
            this.mode = mode;
            domStyle.set(this.manageButton.domNode,"display","manage"===this.mode?"none":"");
            domStyle.set(this.selectList.domNode,"display","manage"===this.mode?"none":"");
            domStyle.set(this.manageList.domNode,"display","manage"===this.mode?"":"none");
            return this.addr.getAllAddr().then(lang.hitch(this,function(data){
                //如果没有地址的话直接跳转到编辑地址的view
                if(data&&data.data&&data.data.items.length){
                    if(data&&data.data){
                        data = data.data;
                    }
                    if(data&&data.items){
                        data = data.items;
                    }
                    data = data||[];
                    var list = "manage"===this.mode?this.manageList:this.selectList;
                    list.getChildren().forEach(function(item){
                        item.destroyRecursive();
                    });
                    list.onDataReady(data.map(lang.hitch(this,function(item){
                        return {
                            label:this["manage"===this.mode?"getManageLabel":"getSelectLabel"](item),
                            variableHeight:true,
                            data:item
                        };
                    })));
                    this.contentPane.resize();
                    return data;
                }else{
                    this._editAddr().then(lang.hitch(this,function(){
                        this.toMode("select");
                    }));
                }
            }));
        },
        getManageLabel:function(item){
            return '<div class="addr-list-item" style="display: block;">'+
                '<h5 class="title">'+
                    item.contactName+
                    '<span style="float:right;">'+item.tel+'</span>'+
                '</h5>'+
                '<p class="dtl">'+
                    (item.isDefault?"<span>[默认地址]</span>":"")+(item.distAddr||"")+(item.detailAddress||"")+
                '</p>'+
                '<div class="button-area">'+/*(
                    item.isDefault?"":'<div class="button" cq-action="setDefault">设为默认地址</div>'
                )+*/
                    '<div class="button" cq-action="deleteAddr">删除</div>'+
                    '<div class="button" cq-action="editAddr">编辑</div>'+
                '</div>'+
            '</div>';
        },//+item.provinceId.distName+item.cityId.distName+item.countyId.distName+
        getSelectLabel:function(item){
            return '<li class="addr-list-item'+(
                this.originData&&this.originData._id===item._id?" selected":""
            )+'"><h5 class="title">'+item.contactName+
            '<span style="float:right;">'+item.tel+
            '</span></h5><p class="dtl"><span class="default">'+(
                item.isDefault?"[默认地址]":""
            )+'</span>'+(item.distAddr||"")+(item.detailAddress||"")+'</p></li>';
        },
        onSelectListClick:function(data,action,list){
            this.setValue(data);
            this.finish();
        },
        //列表管理界面点击事件
        onManageListClick:function(data,action,list){
            var def = null,needBusy = false;
            if("setDefault"===action){
                //设为默认地址
                needBusy = true;
                def = this.addr.setDefaultAddr(data._id);
            }else if("deleteAddr"===action){
                //删除收货地址
                needBusy = true;
                def = this.addr.deleteAddress(data._id);
            }else if("editAddr"===action){
                //编辑收货地址
                def = this._editAddr(data);
            }
            if(def){
                //需要处理数据
                if(needBusy){
                    window.busy();
                }
                def.then(lang.hitch(this,function(){
                    this.toMode("manage");
                }));
            }
        },
        _onCancelClick:function(){
            if(this.mode==="manage"){
                //若处理管理模式，返回到选择模式
                this.toMode("select");
            }else{
                this.cancel();
            }
        },
        _editAddr:function(addr){
            if(!this.editor){
                this.editor = new AddrEditor();
            }
            this.editor.addr = this.addr;
            return this.editor.startEdit(addr,this);
        },
        //开始编辑
        initEdit:function(params){
            this.originData = params;
            this.setValue(params);
            this.toMode("select").then(lang.hitch(this,function(items){
                if(items&&items.length){
                    return;
                }
                this._editAddr().then(lang.hitch(this,function(data){
                    this.setValue(data);
                    this.finish();
                }));
            }));
        },
        setValue:function(value){
            this.value = value;
        },
        //获取添加或选中的值
        getValue:function(){
            return this.value;
        },
        //获取原始值
        getOriginValue:function(){
            return this.originData;
        }
    })();
});