﻿define([
    "dojo/_base/declare",
    "dojo/_base/lang",
    "dojo/dom-construct"
],function(declare,lang,domConstruct){
    return {
        // 画布圆角区域内填充内容
        fillRoundRect:function(g, x, y, width, height, radius, pattern){
            g.save();
            g.translate(x, y);
            return this.drawRoundRectPath(g, width, height, radius).then(function(){
                g.fillStyle = pattern;
                g.fill();
                g.restore();
            });
        },
        // 画布上画出圆角区域
        drawRoundRectPath:function(g, width, height, radius){
            g.beginPath(0);
            g.arc(width-radius,height-radius,radius,0,Math.PI/2);
            g.lineTo(radius,height);
            g.arc(radius,height-radius,radius,Math.PI/2,Math.PI);
            g.lineTo(0,radius);
            g.arc(radius,radius,radius,Math.PI,Math.PI*3/2);
            g.lineTo(width-radius,0);
            g.arc(width-radius,radius,radius,Math.PI*3/2,Math.PI*2);
            g.lineTo(width,height-radius);
            g.closePath();
            return Promise.resolve();
        },
        // 画直角不透明图片
        drawAngleImage:function(g,imageObj){
            return new Promise(function(resolve,reject){
              var img = new Image();
img.setAttribute("crossOrigin",'Anonymous');
                img.onload = function(){
                    // 如果宽高比正好，则直接绘制
                    if((img.width/img.height) == (imageObj.width/imageObj.height)){
                        g.drawImage(img, imageObj.left, imageObj.top, imageObj.width, imageObj.height);
                    }else{// 如果宽高比不一致，则需要计算，保证图片居中显示在背景框中
                        // 如果是绘制背景图片
                        // 如果图片本身小于区域，直接居中
                        // 如果图片大于区域，则需要裁剪
                        var width,height;
                        if(imageObj.isBgImg&&img.width>imageObj.width&&img.height>imageObj.height){
                            // 如果较宽，裁剪左右
                            // 如果较高，裁剪上下
                            width = imageObj.width;
                            height = imageObj.height;
                            var x = imageObj.left,y = imageObj.top,sx = 0,sy = 0,swidth = 0,sheight = 0;
                            if((img.width/img.height) > (imageObj.width/imageObj.height)){// 教宽
                                sheight = img.height;
                                swidth = width/height*sheight;
                                sx = (img.width - swidth)/2;
                            }else{// 教高
                                swidth = img.width;
                                sheight = height/width*swidth;
                                sy = (img.height - sheight)/2;
                            }
                            g.drawImage(img, sx, sy, swidth, sheight, x, y, width, height);
                        }else{
                            var left = imageObj.left;
                            var top = imageObj.top;
                            width = imageObj.width;
                            height = imageObj.height;
                            if((img.width/img.height) > (imageObj.width/imageObj.height)){// 教宽
                                height = img.height * width/img.width;
                                top = top + (imageObj.height - height)/2;
                            }else{// 教高
                                width = img.width * height / img.height;
                                left = left + (imageObj.width - width)/2;
                            }
                            g.drawImage(img, left, top, width, height);
                        }
                    }
                    resolve();
                };
                img.src = imageObj.src;
            });
        },
        // 画圆角图片或半透明图片
        drawRoundImage:function(g,imageObj){
            var _this = this;
            return new Promise(function(resolve, reject){
                // 调整小图片大小
                // var cvs = new Canvas(imageObj.width, imageObj.height);
                var cvs = domConstruct.create("canvas", {
                    width:imageObj.width,
                    height:imageObj.height
                });
                var g1 = cvs.getContext("2d");
                var img = new Image();
                img.setAttribute("crossOrigin",'Anonymous');
                img.onload = function(){
                    g1.drawImage(img, 0, 0, imageObj.width, imageObj.height);
                    var img1 = new Image();
                    img1.onload = function(){
                        // 设置不透明度
                        g.globalAplha = imageObj.opacity||1;
                        resolve(g.createPattern(img1, "no-repeat"));
                    };
                    img1.src = cvs.toDataURL();
                };
                img.src = imageObj.src;
            }).then(function(pattern){
                return _this.fillRoundRect(g, imageObj.left, imageObj.top, imageObj.width, imageObj.height, imageObj.radius*1||0, pattern);
            }).then(function(){
                g.globalAplha = 1;
            });
        },
        // 画布上画图片
        drawImage:function(g,imageObj){
            imageObj.radius = imageObj.radius||0;
            imageObj.opacity = imageObj.opacity||1;
            if(imageObj.radius||imageObj.opacity<1){
                return this.drawRoundImage(g,imageObj);
            }else{
                return this.drawAngleImage(g,imageObj);
            }
        },
        // 绘制线条
        // TODO 添加线条颜色变换
        drawLine:function(g, lineObj){
            g.beginPath(0);
            g.moveTo(lineObj.left, lineObj.top);
            g.lineTo(Number(lineObj.left) + Number(lineObj.width), Number(lineObj.top) + Number(lineObj.height));
            g.closePath();
            g.lineWidth = lineObj.lineWidth;
            g.lineCap = lineObj.lineCap||"butt";
            g.strokeStyle = lineObj.color;
            g.stroke();
            return Promise.resolve();
        },
        // 绘制边框
        // TODO 添加线条颜色变换
        drawBorder:function(g, borderObj){
            g.beginPath(0);
            g.moveTo(borderObj.left, borderObj.top);
            // 绘制顶部线条
            g.lineTo(Number(borderObj.left) + Number(borderObj.width), borderObj.top);
            // 绘制右部线条
            g.lineTo(Number(borderObj.left) + Number(borderObj.width), Number(borderObj.top) + Number(borderObj.height));
            // 绘制底部先天
            g.lineTo(borderObj.left, Number(borderObj.top) + Number(borderObj.height));
            // 绘制左侧线条
            g.lineTo(borderObj.left, borderObj.top);
            g.closePath();
            g.lineWidth = borderObj.lineWidth;
            g.lineJoin = borderObj.lineJoin;
            g.strokeStyle = borderObj.color;
            g.stroke();
            return Promise.resolve();
        },
        // 向画布写字
        // TODO 添加线条颜色变换
        drawText:function(g, textObj, width, height){
            var font = "" + (textObj.fontStyle||"normal") + " " +
                (textObj.fontVariant||"normal") + " " +
                (textObj.fontWeight||"normal") + " " +
                (textObj.fontSize||12) + "px " +
                (textObj.fontFamily||'"arial"');
            // 文字竖排时，需要做旋转且charCode不小于256的符号
            var zhSymbol = [];
            // 文字竖排时，不需要做旋转的符号
            var excludeSymbol = [',','，','《','》','"','“','”',"'","‘","’",".","。","!","！","?","？",";","；",":","："];
            if(font != g.font){
                g.font = font;
            }
            // 当前文字绘制的坐标
            var fillStyle = textObj.fontColor||"#000000";
            if(fillStyle != g.fillStyle){
                g.fillStyle = fillStyle;
            }
            var lastCoordinate = null;
            for(var index = 0;index < textObj.content.length;index++){
                var coordinate = this.caculateCoordinate(g, textObj, index, lastCoordinate);
                if(!coordinate){
                    break;
                }
                if(textObj.content[index]==="\n"||textObj.content[index]==="\t"){
                    lastCoordinate = coordinate;
                    continue;
                }
                // 如果是英文字母，或者符号，选择90度
                if((textObj.content.charCodeAt(index) <= 256 || zhSymbol.indexOf(textObj.content.charAt(index)) >= 0) && textObj.direction == "V" && excludeSymbol.indexOf(textObj.content.charAt(index)) < 0){
                    // 英文字母设置基线为bottom，否则英文左边会因为旋转而发生变换
                    if(g.textBaseline != "bottom"){
                        g.textBaseline = "bottom";
                    }
                    g.translate(coordinate.left, coordinate.top);
                    g.rotate(Math.PI/2);
                    g.translate(-coordinate.left, -coordinate.top);
                    g.fillText(textObj.content[index], coordinate.left, coordinate.top);
                    // 还原旋转坐标系到初始状态
                    g.setTransform(1, 0, 0, 1, 0, 0);
                }else{
                    // TODO 设置文本的基线为顶部，默认为bottom，这是为了坐标计算的方便
                    if(g.textBaseline != "top"){
                        g.textBaseline = "top";
                    }
                    g.fillText(textObj.content[index], coordinate.left, coordinate.top);
                }
                lastCoordinate = coordinate;
            }
            return Promise.resolve();
        },
        caculateTextNumPerLine:function(g, textObj){
            var nArr = [];
        },
        caculateCoordinate:function(g, textObj, index, lastCoordinate){
            // 当前文字左上角坐标
            var left,top;
            if(!lastCoordinate){// 第一个字
                if(textObj.direction == "H"){
                    if(textObj.orientation == "l-r"){
                        if(textObj.firstIndent){
                            left = textObj.x + textObj.firstIndent;
                        }else{
                            left = textObj.x;
                        }
                        top = textObj.y;
                    }else{
                        if(textObj.firstIndent){
                            left = Number(textObj.x) - g.measureText(textObj.content[index]).width - textObj.firstIndent;
                        }else{
                            left = Number(textObj.x) - g.measureText(textObj.content[index]).width;
                        }
                        top = textObj.y;
                    }
                }else{
                    if(textObj.orientation == "l-r"){
                        left = textObj.x;
                        if(textObj.firstIndent){
                            top = textObj.y + textObj.firstIndent;
                        }else{
                            top = textObj.y;
                        }
                    }else{
                        left = Number(textObj.x) - g.measureText(textObj.content[index]).width;
                        if(textObj.firstIndent){
                            top = textObj.y + textObj.firstIndent;
                        }else{
                            top = textObj.y;
                        }
                    }
                }
            }else{
                if(textObj.direction == "H"){
                    if(textObj.orientation == "l-r"){
                        if(textObj.content[index] == "\n"){
                            // 如果是换行符，换行
                            top = lastCoordinate.top + Number(textObj.lineHeight||20);
                            left = textObj.x + (textObj.indentation||0)*(textObj.y - top)/(textObj.lineHeight||20);
                        }else{
                            left = Number(lastCoordinate.left);
                            if(textObj.content[index-1]!=="\n"&&textObj.content[index-1]!=="\t"){
                                left += g.measureText(textObj.content[index - 1]).width + Number(textObj.letterspacing);
                            }
                            top = lastCoordinate.top;
                        }
                    }else{
                        if(textObj.content[index] == "\n"){
                            top = lastCoordinate.top + Number(textObj.lineHeight||20);
                            left = textObj.x - (textObj.indentation||0)*(textObj.y - top)/(textObj.lineHeight||20);
                        }else{
                            left = Number(lastCoordinate.left) - g.measureText(textObj.content[index]).width - Number(textObj.letterspacing);
                            top = lastCoordinate.top;
                        }
                    }
                }else {
                    if(textObj.content[index] == "\n"){
                        if(textObj.orientation == "l-r"){
                            left = lastCoordinate.left + Number(textObj.lineHeight||20);
                            top = textObj.y + (textObj.indentation||0) * (left - textObj.x)/(textObj.lineHeight||20);
                        }else{
                            left = lastCoordinate.left - Number(textObj.lineHeight||20);
                            top = textObj.y + (textObj.indentation||0) * (textObj.x - left)/(textObj.lineHeight||20);
                        }
                    }else{
                        left = lastCoordinate.left;
                        top = Number(lastCoordinate.top);
                        if(textObj.content[index-1]!=="\n"&&textObj.content[index-1]!=="\t"){
                            top += g.measureText(textObj.content[index - 1]).width + Number(textObj.letterspacing);
                        }
                    }
                }
            }
            if(textObj.content[index]==="\n"||textObj.content[index]==="\t"){
                return {left:left,top:top};
            }

            // 计算当前文字是否超出限制的宽高
            // 如果超出，则换行，需要重新计算坐标
            var isOutOfWidth = false;
            var lineHeight,lineNum,bottom;
            if(textObj.direction == "H"){
                if(textObj.orientation == "l-r"){
                    // 是否超出总宽度
                    if(textObj.maxWidth){
                        if((left + g.measureText(textObj.content[index]).width) > textObj.maxWidth){
                            isOutOfWidth = true;
                        }
                    }else if(textObj.lineMaxWidth){// 是否超出单行最大宽度
                        lineHeight = Number(textObj.lineHeight||20);
                        lineNum = (top - textObj.y)/(lineHeight);
                        // 该行右上角x坐标
                        var right = Number(textObj.x) + Number(textObj.indentation||0) * lineNum + Number(textObj.lineMaxWidth);
                        if((left + g.measureText(textObj.content[index]).width) > right){
                            isOutOfWidth = true;
                        }
                    }
                    if(isOutOfWidth){
                        // 换行
                        top = top + Number(textObj.lineHeight||20);
                        left = Number(textObj.x) + Number(textObj.indentation||0) * (top - Number(textObj.y))/Number(textObj.lineHeight||20);
                    }
                    // 如果仍然超出最大宽度，最大高度，表示文本超出限制，直接返回
                    if(textObj.maxWidth && 
                        (left + g.measureText(textObj.content[index]).width) > (Number(textObj.x) + Number(textObj.maxWidth))){
                        return false;
                    }
                    if(textObj.maxHeight && 
                        (top + g.measureText(textObj.content[index].width)) > (Number(textObj.y) + Number(textObj.maxHeight))){
                        return false;
                    }
                }else{
                    isOutOfWidth = false;
                    if(textObj.maxWidth && left < (Number(textObj.x) - Number(textObj.maxWidth))){
                        isOutOfWidth = true;
                    }else if(textObj.lineMaxWidth){
                        lineHeight = Number(textObj.lineHeight||20);
                        lineNum = (top - textObj.y)/(lineHeight);
                        // 该行左上角x坐标
                        var left1 = Number(textObj.x) - Number(textObj.indentation||0) * lineNum + Number(textObj.lineMaxWidth);
                        if(left < left1){
                            isOutOfWidth = true;
                        }
                    }
                    if(isOutOfWidth){
                        top = top + Number(textObj.lineHeight||20);
                        left = Number(textObj.x) - Number(textObj.indentation||0) * (top - Number(textObj.y))/Number(textObj.lineHeight||20) - g.measureText(textObj.content[index]).width;
                    }
                    if(textObj.maxWidth && 
                        left < (Number(textObj.x) - Number(textObj.maxWidth))){
                        return false;
                    }
                    if(textObj.maxHeight && 
                        (top + g.measureText(textObj.content[index].width)) > (Number(textObj.y) + Number(textObj.maxHeight))){
                        return false;
                    }
                }
            }else{// 竖排
                if(textObj.orientation == "l-r"){
                    if(textObj.maxHeight && (top + g.measureText(textObj.content[index]).width) > (Number(textObj.y) + Number(textObj.maxHeight))){
                        isOutOfWidth = true;
                    }else if(textObj.lineMaxHeight){
                        lineHeight = Number(textObj.lineHeight||20);
                        lineNum = (left - Number(textObj.x))/lineHeight;
                        // 该行左下角y坐标
                        bottom = Number(textObj.y) + Number(textObj.indentation||0) * lineNum + Number(textObj.lineMaxHeight);
                        if((top + g.measureText(textObj.content[index]).width) > bottom){
                            isOutOfWidth = true;
                        }
                    }
                    if(isOutOfWidth){
                        left = left + Number(textObj.lineHeight||20);
                        top = Number(textObj.y) + Number(textObj.indentation||0) * (left - Number(textObj.x))/Number(textObj.lineHeight||20);
                    }
                    if(textObj.maxWidth && 
                        (left + g.measureText(textObj.content[index]).width) > (Number(textObj.x) + Number(textObj.maxWidth))){
                        return false;
                    }
                    if(textObj.maxHeight && 
                        (top + g.measureText(textObj.content[index]).width) > (Number(textObj.y) + Number(textObj.maxHeight))){
                        return false;
                    }
                }else{
                    isOutOfWidth = false;
                    if(textObj.maxHeight && 
                        (top + g.measureText(textObj.content[index]).width) > (Number(textObj.y) + Number(textObj.maxHeight))){
                        isOutOfWidth = true;
                    }else if(textObj.lineMaxHeight){
                        lineHeight = Number(textObj.lineHeight||20);
                        lineNum = (Number(textObj.x) - g.measureText(textObj.content[0]).width - left)/lineHeight;
                        // 该行底部Y坐标
                        bottom = Number(textObj.y) + Number(textObj.indentation||0) * lineNum + Number(textObj.lineMaxHeight);
                        if((top + g.measureText(textObj.content[index]).width) > bottom){
                            isOutOfWidth = true;
                        }
                    }
                    if(isOutOfWidth){
                        left = left - Number(textObj.lineHeight||20);
                        top = Number(textObj.y) + Number(textObj.indentation||0) * (Number(textObj.x) - g.measureText(textObj.content[0]).width - left)/Number(textObj.lineHeight||20);
                    }
                    if(textObj.maxWidth && 
                        left < (Number(textObj.x) - Number(textObj.maxWidth))){
                        return false;
                    }
                    if(textObj.maxHeight && 
                        (top + g.measureText(textObj.content[index]).width) > (Number(textObj.y) + Number(textObj.maxHeight))){
                        return false;
                    }
                }
            }
            
            return {
                left: left,
                top: top
            };
        },
        merge:function(options,elements){
            // let cvs = new Canvas(options.width, options.height);
            var cvs = domConstruct.create("canvas", {
                height:options.height,
                width:options.width
            });
            var g = cvs.getContext("2d");
            g.fillStyle = options.bgColor||"#FFF";
            g.fillRect(0, 0, options.width, options.height);
            var _this = this;
            var next = function(index){
                if(index>=elements.length){
                    return;
                }else {
                    if(elements[index].type == "line"){
                        return _this.drawLine(g,elements[index]).then(function(){
                            return next(++index);
                        });
                    }else if(elements[index].type == "border"){
                        return _this.drawBorder(g,elements[index]).then(function(){
                            return next(++index);
                        });
                    }else if(elements[index].type == "image"){
                        return _this.drawImage(g,elements[index]).then(function(){
                            return next(++index);
                        });
                    }else if(elements[index].type == "text"){
                        return _this.drawText(g,elements[index]).then(function(){
                            return next(++index);
                        });
                    }else{
                        return next(++index);
                    }
                }
            };
            return Promise.resolve().then(function(){
                return next(0);
            }).then(function(){
                var base64 = cvs.toDataURL(/*options.mineType||"image/jpeg",options.quality||0.92*/);//.split(",")[1];
                //return new Buffer(base64,"base64");
                return base64;
            });
        }
    };
});
