﻿define([
    "dojo/_base/lang",
    "dojo/Deferred",
    "cq/util/sys/userAgent"
],function(lang,Deferred,userAgent) {
    var realImage = {};
    lang.mixin(realImage, {
        detectVerticalSquash: function(img) {
            var ih = img.naturalHeight;
            var canvas = document.createElement("canvas");
            canvas.width = 1;
            canvas.height = ih;
            var ctx = canvas.getContext("2d");
            ctx.drawImage(img, 0, 0);
            var data = ctx.getImageData(0, 0, 1, ih).data;
            var sy = 0;
            var ey = ih;
            var py = ih;
            while (py > sy) {
                var alpha = data[(py - 1) * 4 + 3];
                if (alpha === 0) {
                    ey = py;
                } else {
                    sy = py;
                }
                py = (ey + sy) >> 1;
            }
            var ratio = (py / ih);
            return (ratio === 0) ? 1 : ratio;
        },
        drawImageIOSFix: function(ctx, img, sx, sy, sw, sh, dx, dy, dw, dh) {
            var vertSquashRatio = this.detectVerticalSquash(img); 
            ctx.drawImage(img, sx * vertSquashRatio, sy * vertSquashRatio, sw * vertSquashRatio, sh * vertSquashRatio, dx, dy, dw, dh);
        },
        _readRealData: function(img, exifInfo) {
            var opt = {
                width: img.naturalWidth,
                height: img.naturalHeight,
                canvasWidth: img.width,
                canvasHeight: img.height,
                transX: 0,
                transY: 0,
                XDimension: exifInfo.PixelXDimension,
                YDimension: exifInfo.PixelYDimension,
                Orientation: exifInfo.Orientation
            };
            if (!opt.Orientation) {
                opt.Orientation = 1;
            }
            if (1 !== opt.Orientation) {
                opt.transX = parseInt(opt.width / 2);
                opt.transY = parseInt(opt.height / 2);
            }
            /*
            var tmpC = document.createElement("canvas");
            tmpC.width = opt.width;
            tmpC.height = opt.height;
            var tmpCtx = tmpC.getContext("2d");
            tmpCtx.clearRect(0, 0, tmpC.width, tmpC.height);
            
            
            this.drawImageIOSFix(tmpCtx, img, 0, 0, opt.canvasWidth, opt.canvasHeight, 0, 0, opt.canvasWidth, opt.canvasHeight);
            */
            var tmpC = img;
            var _canvas = document.createElement("canvas");
            var _context = _canvas.getContext("2d");
            switch (opt.Orientation) {
            case 8:
                _canvas.width = opt.YDimension;
                _canvas.height = opt.XDimension;
                _context.translate(parseInt(_canvas.width / 2), parseInt(_canvas.height / 2));
                _context.rotate( - 0.5 * Math.PI);
                _context.drawImage(tmpC, 0, 0, tmpC.width, tmpC.height, 0 - opt.transX, 0 - opt.transY, tmpC.width, tmpC.height);
                break;
            case 3:
                _canvas.width = opt.XDimension;
                _canvas.height = opt.YDimension;
                _context.translate(parseInt(_canvas.width / 2), parseInt(_canvas.height / 2));
                _context.rotate(Math.PI);
                _context.drawImage(tmpC, 0, 0, tmpC.width, tmpC.height, 0 - opt.transX, 0 - opt.transY, tmpC.width, tmpC.height);
                break;
            case 6:
                _canvas.width = opt.YDimension;
                _canvas.height = opt.XDimension;
                _context.translate(parseInt(_canvas.width / 2), parseInt(_canvas.height / 2));
                _context.rotate(0.5 * Math.PI);
                _context.drawImage(tmpC, 0, 0, tmpC.width, tmpC.height, 0 - opt.transX, 0 - opt.transY, tmpC.width, tmpC.height);
                break;
            case 1:
                _canvas.width = img.width;
                _canvas.height = img.height;
                _context.drawImage(tmpC, 0, 0, img.width, img.height, 0 - opt.transX, 0 - opt.transY, tmpC.width, tmpC.height);
                break;
            }
            //赵春辉   处理图片格式，默认为png，若原始图片为png，读取为jpg，北京可能会读取为黑色
            if(-1<img.src.split(".").pop().toLowerCase()==="png"||-1<img.src.toLowerCase().indexOf("data:image/png;base64,")){
                return _canvas.toDataURL();
            }else{
                return _canvas.toDataURL("image/jpeg");
            }
        },
        getRealUurl: function(url) {
            var def = new Deferred();
            var img = new Image();
            if(userAgent.isIos()){
                img.onload = function() {
                    require(["cq/util/sys/exif/exif"],function(exif) {
                        exif.getData(img,function() {
                            def.resolve(realImage._readRealData(img, exif.getAllTags(this)));
                        });
                    });
                };
            }else{
                def.resolve(url);
            }
            img.src = url;
            return def;
        }
    });
    return realImage;
});