﻿define([
    "dojo/_base/declare",
    "dojo/_base/lang",
    "dojo/dom-attr",
    "dojo/dom-construct",
    "dojo/dom-style",
    "dojo/Deferred",
    "dijit/_WidgetBase",
    "dojo/dom-class",
    "dojo/query",
    "dojo/topic",
    "cq/util/edit/overlay",
    "dojo/_base/array",
    "cq/util/image/blob",
    "../config/config",
    "../sys/userAgent"
],function(declare,lang,domAttr,domConstruct,domStyle,Deferred,_WidgetBase, domClass, query, topic, overlay, array,blob,config,userAgent){
    var inputNode = domConstruct.create("input",{
        type:"text",
        style:"position:absolute;left:-1000px;top:-1000px;"
    },document.body);
    var successHandler = function(result,params){
        result = result||{};
        result.params = params;
        topic.publish("/cq/util/share/shared",result);
    };
    var getSuccessHandler = function(params){
        return function(res){
            successHandler(res,params);
        };
    };
    
    var Share = declare("",[_WidgetBase],{
        baseClass:"share",
        options: null,
        
        //显示提示微信分享的指示图
        showWeixinShare:function(){
            if(!this.coverNode){
                this.coverNode = domConstruct.create("div",{
                    style:"position:absolute;width:100%;height:100%;top:0;left:0;background-color:rgba(0,0,0,0.7);",
                    innerHTML:'<img style="position:absolute;top:0;right:0;" src="./images/wx/sharedirect.png" />',
                    "class": "cover"
                },document.body);
                //点击指示覆盖层，隐藏cover
                this.connect(this.coverNode,"onclick",this.hideCover);
                //微信分享成功，隐藏cover
                this.subscribe("/cq/util/share/shared",this.hideCover);
                //失败或取消，隐藏cover
                this.subscribe("/cq/util/share/shared/result",this.hideCover);
            }
            domStyle.set(this.coverNode,"display","");
        },
        hideCover:function(){
            if(this.coverNode){
                domStyle.set(this.coverNode,"display","none");
            }
        },
        //summary:本方法为分享对象向外提供的唯一个api接口,
        //      功能：将传入的信息内容展现在浮动窗内，根据用户的行为做出相应的分享处理，
        //      目前支持分享到微信聊天界面(weixin),朋友圈(friendCircle),复制链接(copy),分享到村友圈(timeLine)
        //      参数：options 数组
        /*        options 结构
                [{
                 weixin: {
                    title: "",// 分享标题
                    description:"",//简介
                    thumb: ""// logo,
                    url: "" // url
                    moveTo: "" // 跳转页面
                    param: "" // 参数
                    }
                },{
                 friendCircle: {
                    title: "",// 分享标题
                    description:"",//简介
                    thumb: ""// logo,
                    url: "" // url
                    moveTo: "" // 跳转页面
                    param: "" // 参数
                    }
                }]
            返回值：无
        */
        showShare: function(options){
            var def = this.def = new Deferred();
            if(userAgent.isWeixin()){
                this.showWeixinShare();
                return def;
            }
            if(!this.shareNode){
                this.createElement();
            }
            var ops = {};
            if(options instanceof Array){
                array.forEach(options,function(item){
                    for(var k in item){
                        ops[k] = item[k];
                        for(var j in ops[k]){
                            if(j==="thumb"){
                                ops[k][j] = blob.getImageUrl(ops[k][j],"160");
                            }
                        }
                    }
                });
            }else{
                ops = options;
                if(ops.thumb){
                    ops.thumb = blob.getImageUrl(ops.thumb,"160");
                }
            }
            this.options = ops;
            this._showShare();
            return def;
        },
        _shareNodes:null,
        _showShare:function(){
            for(var k in this._shareNodes){
                domStyle.set(this._shareNodes[k],"display",this.options[k]?"":"none");
            }
            overlay.showElement(this.shareNode);
        },
        validType:["weixin","friendCircle","timeLine","copy"],
        //summary:创建浮动窗,绑定相应的点击事件,
        createElement:function(){
            this.shareNode = domConstruct.create("div", {
                "class": "hflex"
            }, this.domNode);
            this._shareNodes = {};
            array.forEach(this.validType,function(type){
                var shareMethod = this._getActionName(type);
                this._shareNodes[type] = domConstruct.create("div", {
                    "class": "share-item flex1",
                    innerHTML: "<div>"+
                            "<img src='"+this._getIcon(type)+"'/>"+
                        "</div>"+
                        "<div class='title'>"+this._getTitle(type)+"</div>"
                }, this.shareNode);
                this.connect(this._shareNodes[type], "onclick", function(){
                    if(overlay){
                        overlay.hide();
                    }
                    this[shareMethod](this.options&&this.options[type]);
                });
            },this);
        },
        //summary:获取浮动窗相应模块的名称
        _getTitle: function(opt){
            var titles = {
                "weixin": "微信",
                "friendCircle": "朋友圈",
                "copy": "复制链接",
                "timeLine": "村友圈"
            };
            return titles[opt];
        },
        
        
        //summary：获取浮动窗相应的模块
        _getIcon: function(opt){
            return "./images/temp/shareicon/"+opt+".png";
        },
        
        
        //获取浮动窗点击事件的方法名
        _getActionName: function(opt){
            if(opt=="weixin"){
                return "shareToWeixin";
            }else if(opt=="friendCircle"){
                return "shareToFriendCircle";
            }else if(opt=="copy"){
                return "copyUrl";
            }else if(opt=="timeLine"){
                return "shareToTimeLine";
            }else {
                return "";
            }
        },
        
        
        //处理传入的数据，返回符合要求的数据格式
        _dataHandle:function(data,type){
            if(type ==='weixin'){
                //微信聊天界面数据处理
                return this.weChartData(data,'SESSION');
            }else if(type === 'friendCircle'){
                //朋友圈的分享数据处理
                return this.weChartData(data,'TIMELINE');
            }else if(type === 'copy'){
                //复制的数据处理
                return data.url||'';
            }
            
        },
        
        
        //微信相关数据处理
        weChartData:function(data,param){
            return {
                    'message': {
                            title: data.title||"",// 分享标题
                            description:data.description||data.title||"",
                            thumb: data.thumb||"",// 图片
                            media: {
                                type: window.Wechat.Type.WEBPAGE,//分享的内容为网页
                                webpageUrl: data.url||''
                            }
                    },
                    'scene': window.Wechat.Scene[param]   // 分享到聊天界面
                };
        },
        
        
        
        // 分享到微信好友
        shareToWeixin: function(data){
            //检验是否支持微信分享
            this.isWechatEnable().then(lang.hitch(this,function(datas){
                if(datas){
                    var weixinData = this._dataHandle(data,'weixin');
                    window.Wechat.share(weixinData,lang.hitch(this,function(){
                        alert('分享成功');
                        successHandler({},weixinData);
                    }),lang.hitch(this,function(){
                        alert('分享失败');
                        this.def.resolve({wxShare:false});
                    }));
                }else{
                    overlay.hide();
                    alert('不支持微信分享');
                    this.def.resolve({wxShare:false});
                    return;
                }
            }));
        },
        
        
        
        
        //检查是否支持微信分享
        isWechatEnable:function(){
            var def = new Deferred();
            if(window.Wechat){
                window.Wechat.isInstalled(function (installed) {
                    def.resolve(installed?true:false);
                }, function (reason) {
                    def.resolve(false);
                });
            }else{
                def.resolve(false);
            }
            return def;
        },
        
        
        
        
        // 分享到朋友圈
        shareToFriendCircle: function(data){
            //检查是否支持微信分享
            this.isWechatEnable().then(lang.hitch(this,function(datas){
                if(datas){
                    var friendData = this._dataHandle(data,'friendCircle');
                    window.Wechat.share(friendData,function(){
                        alert('分享成功');
                        successHandler({},friendData);
                    },function(reason){
                        alert('分享失败,原因:'+reason);
                    });
                }else{
                    overlay.hide();
                    alert('不支持朋友圈分享');
                    return;
                }
            }));
        },
        
        // 复制链接
        copyUrl: function(data){
            var url = this._dataHandle(data,'copy');
            if(!url){
                alert('没有要复制的内容，请检查');
                return;
            }
            //浏览器上的支持,只支持高级浏览器
            if(this.isBrowser()){
                this._browserCopy(url);
            }else if(cordova&&cordova.plugins&&cordova.plugins.clipboard){
                //移动端设备的剪切板支持
                this._cordovaCopy(url);
                alert('复制成功');
            }else {
                prompt("请复制以下内容",url);
            }
        },
        
        //网页上剪切复制
        _browserCopy:function(url){
            inputNode.value = url;
            inputNode.select();
            try{
                var isTrue = document.execCommand("Copy");
                if(isTrue){
                    alert('复制成功');    
                }else{
                    prompt("请复制以下内容",url);    
                }
            }catch(e){
                prompt("请复制以下内容",url);
            }
        },
        //移动端设备的剪切复制
        _cordovaCopy:function(url){
            cordova.plugins.clipboard.copy(url);
        },
        //判断当前环境是否是浏览器
        isBrowser:function(){
            return !(window.device && device.uuid);
        },
        //分享到村友圈
        shareToTimeLine: function(data){
            require([
                "bob/user/timeLine/c",
                "bob/user/shopAdmin",
                "bob/shop/shop"], lang.hitch(this, function(timeLine, shopAdmin, shop){
                // 直接调用村友圈的save方法
                timeLine.save({
                    title: data.title,
                    images: data.thumb?[data.thumb]:[],
                    publisher: shopAdmin.getCurrentUserId(),
                    fromShopId: shop.getShopId(),
                    url: data.url||""
                }).then(lang.hitch(this, function(result){
                    if(result&&result.data&&result.data.ok==1){
                        alert("分享成功");
                    }else{
                        alert(result&&result.data&&result.data.message||"");
                    }
                }));
            }));
        },
        // 获取要分享店铺的地址url
        getShopLocation: function(shopId){
            var def = new Deferred();
            require(["bob/shop/shop"],function(shop){
                shop.loadById(shopId).then(function(item){
                    var baseUrl = config.getDataOrigin()+"/w/home/"+shopId;
                    if(config.fullURL){
                        baseUrl = config.fullURL;
                    }
                    if(item&&item.location){
                        baseUrl = item.location;
                    }
                    if(config.session&&config.session.userId){
                        if(baseUrl.indexOf("?")>-1){
                            baseUrl += "&suid="+config.session.userId;
                        }else{
                            baseUrl += "?suid="+config.session.userId;
                        }
                    }
                    def.resolve(baseUrl);
                });
            });
            return def;
        },
        getPageLoc:function(shopId,hash){
            var baseUrl = config.getDataOrigin()+"/w/home/"+shopId;
            if(config.fullURL){
                baseUrl = config.fullURL;
            }
            if(config.session&&config.session.userId){
                if(baseUrl.indexOf("?")>-1){
                    baseUrl += "&suid="+config.session.userId;
                }else{
                    baseUrl += "?suid="+config.session.userId;
                }
            }
            return baseUrl+hash;
        }
    });
    return new Share();
});