﻿define([
    "dojo/_base/lang",
    "dojo/date/locale",
    "cq/util/config/config"
],function(lang,locale,config){
    var start = null,serverStart = null;
    if(false===config.enableServerTime){
        start = serverStart = new Date();
    }else{
        require([
            "dojo/request/xhr",
            "cq/util/config/config"
        ],function(xhr,config){
            xhr.get(config.getDataOrigin()+"/biz/sys/now",{
                handleAs:"json"
            }).then(function(data){
                if(data&&data.now){
                    start = new Date();
                    lang.getObject("cq.util.sys",true).token = data.token;
                    serverStart = new Date(data.now);
                }
            });
        });
    }
    return lang.mixin(lang.getObject("cq.util.sys.date",true),{
        _now:function(){
            var now = new Date();
            if(serverStart){
                return new Date(serverStart.getTime()-start.getTime()+now.getTime());
            }
            return now;
        },
        now:function(){
            return this._now();
        },
        newDateISOString:function(){
            return this.formatToISOString(this.formatISODate(this._now()));
        },
        //当前时间的ISO格式字符串
        nowISOString:function(){
            return (this._now()).toISOString();
        },
        //将ISO日期字符串转换为当前日期"yyyy-mm-dd"
        formatISODate:function(isoString){
            var reg = /^\d{4}\-\d{2}\-\d{2}T\d{2}:\d{2}:\d{2}\.\d{3}Z$/;
            if(!reg.test(isoString)){
                isoString = isoString.replace(/\-/g,"/");
            }
            return locale.format(new Date(isoString),{
                selector:"date",
                datePattern:"yyyy-MM-dd"
            });
        },
        //将ISO日期字符串转换为当前日期"yyyy-mm-dd"
        formatISOMonthly:function(isoString){
            return locale.format(new Date(isoString),{
                selector:"date",
                datePattern:"yyyy-MM"
            });
        },
        //将ISO日期字符串转换为当前时间"HH:mm"
        formatISOTime:function(isoString){
            return locale.format(new Date(isoString),{
                selector:"date",
                datePattern:"HH:mm"
            });
        },
        //将ISO日期字符串转换为当前日期"yyyy-mm-dd:mm"
        formatISODateMinutes:function(isoString){
            return locale.format(new Date(isoString),{
                selector:"date",
                //干劲20161001：不要秒更合理？
                datePattern:"yyyy-MM-dd HH:mm"
            });
        },
        //将年月日"yyyy-mm-dd"字符串转为ISO日期字符串
        formatToISOString:function(dateString){
            if (dateString === "") {
                return "";
            }
            return locale.parse(dateString,{
                selector:"date",
                datePattern:"yyyy-MM-dd"
            }).toISOString();
        },
        //将年月日"yyyy-mm-dd HH:MM"字符串转为ISO日期字符串
        formatToISOStringWithMinutes:function(dateString){
            return locale.parse(dateString,{
                selector:"date",
                datePattern:"yyyy-MM-dd HH:mm"
            }).toISOString();
        },
        formatAs:function(dateTime,formatStr){
            if(formatStr==="timeStamp"){
                return new Date(dateTime).getTime();
            }else if("yyyyMMddhhmmss"===formatStr){
                return this.formatISODateMinutes(dateTime).replace("-","").replace(":","")+"00";
            }
            return locale.format(dateTime,{
                selector:"date",
                datePattern:formatStr
            }).toISOString();
        },
        parse:function(dateString,formatStr){
            if(formatStr==="timeStamp"){
                return new Date(dateString);
            }else if("yyyyMMddhhmmss"===formatStr){
                var n = new Date();
                n.setDate(1);
                n.setFullYear(dateString.substr(0, 4));
                n.setMonth(dateString.substr(4, 2) - 1);
                n.setDate(dateString.substr(6, 2) - 0);
                n.setHours(dateString.substr(8, 2) - 0, dateString.substr(10, 2) - 0, dateString.substr(12, 2), 0);
                return n;
            }
            return locale.parse(dateString,{
                selector:"date",
                //干劲20161001：不要秒更合理？
                datePattern:formatStr
            });
        },
        
        
        //事件处理函数：把时间格式处理为易读的类型
        //param:时间
        //返回值：处理事件后的一串字符串
        getTimeMoment:function(lastMsgTime){
            if(lastMsgTime){
                var time = new Date(lastMsgTime);
                var now  = this._now();
                //今日凌晨
                var today = this._now();
                today.setHours(0);
                today.setMinutes(0);
                today.setSeconds(0);
                today.setMilliseconds(0);
                var todayHours = now.getTime()-today.getTime();
                var difTime = now.getTime()-time.getTime();
                if(difTime<1*60*1000){
                    lastMsgTime = '刚刚';
                }else if(1*60*1000<=difTime&&difTime<3*60*1000){
                    lastMsgTime = '1分钟前';
                }else if (3*60*1000<=difTime&&difTime<5*60*1000){
                    lastMsgTime = '3分钟前';
                }else if(5*60*1000<=difTime&&difTime<30*60*1000){
                    lastMsgTime = '5分钟前';
                }else if (30*60*1000<= difTime&&difTime<60*60*1000){
                    lastMsgTime = '30分钟前';
                }else if(60*60*1000<=difTime&&difTime<todayHours){
                    if(time.getMinutes()!==0){
                        if(time.getMinutes()<10){
                            lastMsgTime = time.getHours()+':0'+time.getMinutes();
                        }else{
                            lastMsgTime = time.getHours()+':'+time.getMinutes();
                        }
                    }else{
                        if(time.getHours()<=12){
                            lastMsgTime = '早上'+time.getHours()+'点';
                        }else{
                            lastMsgTime = '下午'+time.getHours()-12+'点';
                        }
                    }
                }else if(time.getFullYear()!==now.getFullYear()){
                    lastMsgTime = time.getFullYear()+'年'+time.getMonth()+'月'+time.getDate()+'日'; 
                }else{
                    lastMsgTime = (time.getMonth()+1)+'月'+time.getDate()+'日';//+time.getHours()+':'+time.getMinutes();
                }
            }else{
                lastMsgTime =''; 
            }
            return lastMsgTime;
        }
    });
});
