define([
	"dojo/_base/lang",
    "dojo/Deferred",
    "cq/util/sys/userAgent"
],function(lang,Deferred,userAgent){
	var EARTH_RADIUS = 6378137.0;    //单位M
    var PI = Math.PI;
    
    function getRad(d){
        return d*PI/180.0;
    }
    
    /**
     * caculate the great circle distance
     * @param {Object} lat1
     * @param {Object} lng1
     * @param {Object} lat2
     * @param {Object} lng2
     */
    var getGreatCircleDistance = function(lat1,lng1,lat2,lng2){
        var radLat1 = getRad(lat1);
        var radLat2 = getRad(lat2);
        
        var a = radLat1 - radLat2;
        var b = getRad(lng1) - getRad(lng2);
        
        var s = 2*Math.asin(Math.sqrt(Math.pow(Math.sin(a/2),2) + Math.cos(radLat1)*Math.cos(radLat2)*Math.pow(Math.sin(b/2),2)));
        s = s*EARTH_RADIUS;
        s = Math.round(s*10000)/10000.0;
                
        return s;
    };
	
    return {
		_asked:false,
        //高德地图当前地理位置的信息
        currentPosition:"",
        //获取当前经纬度，若没有高德地图当前位置信息，返回null，否则{lon:xxx,lat:xxx}
        getCoordinate:function(){
            if(this.currentPosition&&this.currentPosition.position){
                return {
                    lon:this.currentPosition.position.lng,
                    lat:this.currentPosition.position.lat,
                };
            }else{
                return null;
            }
        },
        
        //获取当前区划名称信息，同上
        //      返回内容{province:省,city:市,county:县,address:地址简称,addrDetail:详细地址}
        getDistrict:function(){
            if(this.currentPosition&&this.currentPosition.position){
                return {
                    province:this.currentPosition.addressComponent.province||"",
                    city:this.currentPosition.addressComponent.city||"",
                    county:this.currentPosition.addressComponent.district||this.currentPosition.addressComponent.county||"",
                    address:this.currentPosition.addressComponent.building||"",
                    addrDetail:this.currentPosition.formattedAddress||""
                };
            }else{
                return null;
            }
        },
        getSynCurrentPosition:function(){
            this.getCurrentPosition().then(lang.hitch(this,function(data){
                return data;
            }));
        },
		getCurrentPosition:function(){
            var def = new Deferred();
            if(userAgent.isWeixin()){
                require([
                    "cq/util/sys/wx",   
                ],function(wx){
                    if(wx.isReady){
                        wx.getLocation().then(lang.hitch(this,function(pos){
                            if(pos&&pos.errMsg){
                                def.resolve(null);
                                return;
                            }
                            this.currentPosition = pos;
                            def.resolve(pos);
                        }));    
                    }
                });    
            }else{
                if(!this.encoder){
                    if(window.AMap&&AMap.Geolocation){
                        this.encoder = new AMap.Geolocation();
                    }else{
                        AMap.plugin("AMap.Geolocation",lang.hitch(this,function(){
                            this.getCurrentPosition().then(function(loc){
                                def.resolve(loc);
                            });
                        }));
                    }
                }
                if(this.encoder){
                    this.encoder.getCurrentPosition(lang.hitch(this,function(status,pos){
                        this.currentPosition = pos;
                        def.resolve(pos);
                    }),function(err){
                        def.resolve(null);
                    });
                }
            }
            return def;
		},
		getDistance:function(lat1,lng1,lat2,lng2){
			var f = getRad((lat1 + lat2)/2);
			var g = getRad((lat1 - lat2)/2);
			var l = getRad((lng1 - lng2)/2);
			
			var sg = Math.sin(g);
			var sl = Math.sin(l);
			var sf = Math.sin(f);
			
			var s,c,w,r,d,h1,h2;
			var a = EARTH_RADIUS;
			var fl = 1/298.257;
			
			sg = sg*sg;
			sl = sl*sl;
			sf = sf*sf;
			
			s = sg*(1-sl) + (1-sf)*sl;
			c = (1-sg)*(1-sl) + sf*sl;
			
			w = Math.atan(Math.sqrt(s/c));
			r = Math.sqrt(s*c)/w;
			d = 2*w*a;
			h1 = (3*r -1)/2/c;
			h2 = (3*r +1)/2/s;
			
			return d*(1 + fl*(h1*sf*(1-sg) - h2*(1-sf)*sg));
		},
        
        //TODO: 新增route方法
		//@arguments startPoint : 
		//			startPoint.lon : 起点经度
		//			startPoint.lat : 起点纬度
		//@arguments endPoint : 
		//			endPoint.lon : 终点经度
		//			endPoint.lat : 终点纬度
		//@arguments onSuccess : 成功的回调函数
		//@arguments onError : 失败的回调函数
		//返回值 results ：
		//		results.routes : 返回的路线
		//		results.routes.length: 返回个数
		//		results.routes[v].steps : 
		//				results[v].routes.steps.length : 返回驾车步骤个数
		//		results.routes[v].distance : 返回行车距离（米）
		//				
		getDrivingRoute:function(startPoint, endPoint, onSuccess, onError, waypoints){
			var MDrive,
				start_xy = new AMap.LngLat(startPoint.lon, startPoint.lat),//起点经纬度
				end_xy = new AMap.LngLat(endPoint.lon, endPoint.lat);//终点经纬度
			AMap.service(["AMap.Driving"], function() {
				var DrivingOption = {
					//驾车策略，包括 LEAST_TIME，LEAST_FEE, LEAST_DISTANCE,REAL_TRAFFIC
					policy: AMap.DrivingPolicy.LEAST_TIME 
				};        
				MDrive = new AMap.Driving(DrivingOption); //构造驾车导航类 
				//根据起终点坐标规划驾车路线
				MDrive.search(start_xy, end_xy, {
                    waypoints:(waypoints||[]).map(function(point){
                        return new AMap.LngLat(point.lon,point.lat);
                    })
                }, function(status, results){
					if(status === 'complete' && results.info === 'OK'){
						onSuccess(results);
					}else{
						onError(results);
					}
				}); 
		    });
		}
    };
});