﻿define([
    "dojo/_base/declare",
    "dojo/_base/lang",
    "dojo/_base/array",
    'dojo/dom-construct',
    "dojo/Deferred",
    "dojo/number",
    "dojox/charting/Chart",
    "dojox/charting/widget/Legend",
    "cq/app/widget/_WidgetBase",
    "dijit/_WidgetBase",
    "dijit/registry",
    "dojox/charting/Theme",
    "dojox/charting/themes/Tom",
    "dojox/charting/axis2d/Default",
    "dojox/charting/plot2d/Pie",
    "dojox/charting/action2d/Tooltip",
    "dojox/charting/action2d/Highlight",
    "dojo/has!touch?dojox/charting/action2d/TouchIndicator:dojox/charting/action2d/MouseIndicator",
],function(declare, lang, array, domConstruct, Deferred, number,Chart, Legend,cqWidgetBase,_WidgetBase,registry,Theme,Tom,Default,Pie,Tooltip,Highlight,Indicator){
    /*
    dataProvider：为本对象提供数据的模块名称
	dataOptions：调用dataMethod加载数据时额外的参数
	dataMethod：读取数据的方法名，参数同Chart
	yField：数据字段
	labelField：对应数据块显示的标签
	maxNum：最多显示多少块，默认为10(没有设置或设置的值小于2时)
		若数据行数小于等于最大块数，全部显示，否则所有后边的合并为其他，保证总的数量不超过maxNum

    
    */
    
    var props = "'dataProvider:\"{dataProvider}\","+
        "dataMethod:\"{dataMethod}\",tooltip:\"{tooltip}\",dataOptions:\{j:dataOptions}\,"+
        "maxNum:\"{maxNum}\","+
        "labelField:\"{labelField}\",yField:\"{yField}\"'";
    return declare('',[_WidgetBase,cqWidgetBase],{
        
        
        htmlTemp:'<div id="{id}" class="cq-from-Pie {className}" data-dojo-type="cq/widgets/chart/Pie" data-dojo-props='+props+'></div>',
        
        //配置的class属性
        className:'',
        //数据提供者
        dataProvider:'',
        
        //调用dataMethod加载数据时额外的参数
        dataOptions:{},
        
        //获取数据的方法
        dataMethod:'',
        
        //数据字段
        yField:'',
        //对应数据块显示的标签
        labelField:'',
        //
        tooltip:'',
        //最多显示多少块，默认为10(没有设置或设置的值小于2时)
		//若数据行数小于等于最大块数，全部显示，否则所有后边的合并为其他，保证总的数量不超过maxNum
        maxNum:10,
        buildRendering:function(){
            this.inherited(arguments);
            this.maxNum = this.maxNum>=2?this.maxNum:10;
            //this._initPieAndGetData();
            this.cqRefresh();
        },
        _initPieAndGetData:function(){
            var layoutData = this.getLayoutData();
            //数据加载完成后依据数据生成视图
            if(layoutData){
                if(layoutData.then){
                    layoutData.then(lang.hitch(this,this._initPieView));
                }else{
                    this._initPieView(layoutData);
                }
            }
        },
        
        getLayoutData:function(){
            if(this.dataProvider&&this.dataMethod){
                if(lang.isString(this.dataProvider)){
                    var def = new Deferred();
                    require([this.dataProvider],lang.hitch(this,function(dataProvider){
                        this.dataProvider = dataProvider;
                        var data = this.dataProvider[this.dataMethod](this.cqRefInfo,this,this.dataOptions);
                        if(data&&data.then){
                            data.then(function(result){
                                def.resolve(result);
                            });
                        }else{
                            def.resolve(data);
                        }
                    }));
                    return def;
                }else{
                    return this.dataProvider[this.dataMethod](this.cqRefInfo,this,this.dataOptions);
                }
            }else{
                if(this.dataProvider){
                    require([this.dataProvider],lang.hitch(this,function(dataProvider){
                        this.dataProvider = dataProvider;
                    }));
                }
                return this.inherited(arguments);
            }
        },
        _initPieView:function(items){
            //数据返回的格式
            //[{},{},{}]
            this.idByName = {}
            //排序有问题,最大值不在最前面
            var pieDatas = this.pieData = lang.clone(items);
            pieDatas.sort(lang.hitch(this,function(a,b){
                return a[this.yField]>b[this.yField]?-1:1;
            }));
            var dataItem = [],total = 0;
            for(var i=0;i<pieDatas.length;i++){
                total += parseFloat(pieDatas[i][this.yField],10)||0;
                if(i>=this.maxNum){
                    dataItem[this.maxNum-1][this.yField] += parseFloat(pieDatas[i],10)||0;
                }else{
                    dataItem[i] = pieDatas[i];
                }
            }
            this.maxNum = this.maxNum>0?this.maxNum:10;
            if(pieDatas.length>this.maxNum){
                dataItem[dataItem.length-1][this.labelField] = "其他";
                dataItem[dataItem.length-1][this.yField] = parseFloat(dataItem[dataItem.length-1][this.yField],10).toFixed(2);
            }
            dataItem = dataItem.map(lang.hitch(this,function(data){
                return {
                    y:data[this.yField],
                    id:data["id"],
                    "text":this.tooltip.replace("{data}",data[this.yField])+
                    ":"+Math.round(data[this.yField]/total*10000) / 100.00 + "%",
                    tooltip:data[this.labelField]+":"+this.tooltip.replace("{data}",data[this.yField]),
                    legend:data[this.labelField]
                };
            }));
            
            //创建饼图的节点
            if(!this.PieChartNode){
                this.PieChartNode = domConstruct.create('div',{
                    'class':'pieChart',
                },this.domNode);
            }
            if(!this.pieChartNodeChild){
                this.pieChartNodeChild = domConstruct.create('div',{
                    'class':'pieChild',
                    onclick:lang.hitch(this,function(e){
                        //console.log(registry.getEnclosingWidget(e.target));
                    })
                },this.PieChartNode);
            }
            var pieChart = this._PieChart = new Chart(this.pieChartNodeChild);
            pieChart.addPlot('default',{
                type:'Pie',
                font: "normal normal 10pt Tahoma",
                fontColor: "#000",
                labelWiring: "#000",
                radius: 100,
                labelStyle: "columns",
                htmlLabels: true,
                startAngle: -10,
                labelOffset:40
            });
            pieChart.addSeries('series',dataItem);
            var pieAction = new Tooltip(pieChart, "default",{
                mouseOver:false
            });
            pieAction.onClick =function(){};
            new Highlight(pieChart,"default");
            pieChart.connectToPlot('default','slice',lang.hitch(this,function(e){
                if(e.type=='onclick'){
                    //点击块在饼图中的序列
                    var index = e.index;
                    //饼图中的数据
                    var datas = e.run.data;
                    var value = datas[index];
                    this.onCqChanged(this,value);
                }
                if(e.type ==="onmouseover"){
                    setTimeout(lang.hitch(this,function(){
                        if(e.type ==="onmouseover"){
                            var index = e.index;
                            var datas = e.run.data;
                            var value = lang.clone(datas[index]);
                            
                            this.onCqChanged(this,value);
                            
                        }
                    }),600);
                }
            }));
            var g = Theme.generateGradient,defaultFill = {type: "linear", space: "shape", x1: 0, y1: 0, x2: 0, y2: 100};
            pieChart.setTheme(new Theme({
                chart: {
                    pageStyle: { backgroundImage: "none", color: "#eaf2cb"}
                },
                axis:{
                    stroke:	{ // the axis itself
                        color: "#a0a68b",
                        width: 1
                    },
                    tick: {	// used as a foundation for all ticks
                        color:     "#888c76",
                        position:  "center",
                        font:      "normal normal normal 7pt Helvetica, Arial, sans-serif",	// labels on axis
                        fontColor: "#888c76"	// color of labels
                    }
                },
                series: {
                    stroke:  {width: 2.5, color: "#eaf2cb"},
                    outline: null,
                    font: "normal normal normal 8pt Helvetica, Arial, sans-serif",
                    fontColor: "#eaf2cb"
                },
                marker: {
                    stroke:  {width: 1.25, color: "#eaf2cb"},
                    outline: {width: 1.25, color: "#eaf2cb"},
                    font: "normal normal normal 8pt Helvetica, Arial, sans-serif",
                    fontColor: "#eaf2cb"
                },
                seriesThemes: [
                    {fill: g(defaultFill, "#bf9e0a", "#ecc20c")},
                    {fill: g(defaultFill, "#73b086", "#95e5af")},
                    {fill: g(defaultFill, "#c7212d", "#ed2835")},
                    {fill: g(defaultFill, "#87ab41", "#b6e557")},
                    {fill: g(defaultFill, "#b86c25", "#d37d2a")},
                    {fill: g(defaultFill, "#2f2727", "#5c5050")},
                    {fill: g(defaultFill, "#4057a4", "#8396d7")},
                    {fill: g(defaultFill, "#642875", "#b58cc1")},
                    {fill: g(defaultFill, "#1d869c", "#4abad1")},
                    {fill: g(defaultFill, "#021418", "#05181c")},
                ],
                markerThemes: [
                    {fill: "#bf9e0a", stroke: {color: "#ecc20c"}},
                    {fill: "#73b086", stroke: {color: "#95e5af"}},
                    {fill: "#c7212d", stroke: {color: "#ed2835"}},
                    {fill: "#87ab41", stroke: {color: "#b6e557"}},
                    {fill: "#b86c25", stroke: {color: "#d37d2a"}},
                    {fill: "#2f2727", stroke: {color: "#5c5050"}},
                    {fill: "#4057a4", stroke: {color: "#8396d7"}},
                    {fill: "#642875", stroke: {color: "#b58cc1"}},
                    {fill: "#1d869c", stroke: {color: "#4abad1"}},
                    {fill: "#021418", stroke: {color: "#05181c"}},
                ]
            }));
            pieChart.render();
            var lengendNode = domConstruct.create('div',{
                'class':'legend',
                style:'float:right;display:inline-block'
            },this.PieChartNode);
            var lengendNodeChild = domConstruct.create('div',{
                'class':'lengendChild'
            },lengendNode);
            this.legend = new Legend({
                chart: pieChart,
                horizontal:false
            },lengendNodeChild);
            this.canRefrsh = true;
        },
        cqRefresh:function(){
            //控制过快刷新导致产生两个饼图
            if(this.canRefrsh&&this.isFirstload ){
                this.canRefrsh = false;
                if(this._PieChart){
                    this._PieChart.destroy();
                }
                if(this.legend){
                    this.legend.destroy();
                }
                this._initPieAndGetData();
            }
            if(!this.isFirstload){
                this.isFirstload = true;
                this._initPieAndGetData();
            }
        }
    });
});