//购物车的基类

//  选中按钮的控制
//      点击选中按钮做选中或全选
//          遍历所有下级，执行选中或取消选中的操作
//  下级变化之后
//      数量改变
//          向上触发商品发生了变化
//      选中状态改变
//          向上触发商品发生了变化
//  商品发生了变化
//      刷新选中状态
//      刷新数量信息
define([
    "dojo/_base/declare",
    "dojo/_base/lang",
    "dojo/_base/array",
    "dojo/dom-attr",
    "dojo/topic",
    "dojo/Deferred"
],function(declare,lang,array,domAttr,topic,Deferred){
    return declare("",null,{
        //列表的成员
        //  若不分供货商，列表成员应该是商品，否则是供货商
        ItemRender:null,
        //列表项的配置
        //  列表项若为供货商，需要配置商品的呈现组件
        renderOption:null,
        //购物车对象，提供读取购物车商品信息等
        cart:null,
        //列表的商家的活动
        //初始化过程
        //  初始化header部分
        //  加载依赖的模块
        //  执行初始化完成的动作
        buildRendering:function(){
            this.inherited(arguments);
            this.initCartHeader();
            var def = new Deferred();
            var requires = [];
            if(!this.cart){
                this.cart = "bob/ts/cart";
            }
            if(!this.renderOption.cart){
                this.renderOption.cart = "bob/ts/cart";
            }
            if(this.cart&&"string"===typeof(this.cart)){
                //购物车对象，C端购物车对象、B端购物车对象(B2B采购订单)
                requires.push(this.cart);
            }
            if(this.ItemRender&&"string"===typeof(this.ItemRender)){
                requires.push(this.ItemRender);
            }
            require(requires,lang.hitch(this,function(){
                var idx = 0;
                if(this.cart&&"string"===typeof(this.cart)){
                    this.cart = arguments[idx];
                    idx++;
                }
                if(this.ItemRender&&"string"===typeof(this.ItemRender)){
                    this.ItemRender = arguments[idx];
                    idx++;
                }
                def.resolve();
            }));
            def.then(lang.hitch(this,function(){
                if(this.renderOption&&this.renderOption.ItemType&&this.renderOption.cart&&"string"===typeof(this.renderOption.ItemType)){
                    require([
                        this.renderOption.ItemType,
                        this.renderOption.cart
                    ],lang.hitch(this,function(ItemType,cart){
                        this.renderOption.ItemType = ItemType;
                        this.renderOption.cart = cart;
                        this.onRenderTypeReady();
                    }));
                }else{
                    this.onRenderTypeReady();
                }
            }));
        },
        //初始化完成之后的处理
        //  做标志
        //  有条件的执行刷新操作
        onRenderTypeReady:function(){
            this._itemRenderReady = true;
            if(this._refreshOnRender){
                this.cqRefresh();
            }
        },
        
        //初始化购物车标题
        //  移动版的全选、编辑、完成等，以及点击事件
        //  桌面版的顶部标题栏等，以及点击事件
        initCartHeader:function(){
            
        },
        
        //处理商品选中
        //全选状态
        selected:true,
        //全选全不选的checkbox节点
        selectNode:null,
        //选中节点点击事件
        onSelectClick:function(e){
            this.onSelectClicked(!this.selected);
        },
        //点击全选或取消全选
        onSelectClicked:function(selected){
            this.setSelected(selected);
            this.getChildren().forEach(function(item){
                item.onSelectClicked(selected);
            });
        },
        
        //设置全选状态
        setSelected:function(selected){
            this.selected = selected;
            if(this.selectNode){
                domAttr.set(this.selectNode,"checked",selected);
            }
            //处理底部刷新
            this.onMatChanged();
        },
        //获取当前选中状态
        getSelected:function(){
            return this.selected;
        },
        //下级选中发生变化之后的响应
        onItemSelectChange:function(){
            if(this._matSelectTimer){
                clearTimeout(this._matSelectTimer);
            }
            this._matSelectTimer = setTimeout(lang.hitch(this,this._onItemSelectChange),300);
        },
        //下级列表项的选中状态发生变化的事件
        _onItemSelectChange:function(){
            //当this.getChildren()的长度为0 时，直接select为false
            if(!this.getChildren().length){
                this.setSelected(false);
            }else{
                this.setSelected(!array.some(this.getChildren(),function(item){
                    return !item.getSelected();
                }));
            }
            
        },
        
        //编辑和完成的处理
        //  这部分功能是可选的，网站版本的程序没有此项功能
        //  若启用编辑功能，子类需要做如下工作
        //      购物车的head区域要有编辑和完成按钮
        //      创建head区域时要真挺编辑和完成按钮的点击事件，调用startEdit和endEdit
        //      需要重写startEdit和endEdit方法，完成head区域状态的刷新
        //开始编辑
        startEdit:function(){
            this.getChildren().forEach(function(item){
                item.startEdit();
            });
        },
        //停止编辑
        endEdit:function(){
            this.getChildren().forEach(function(item){
                item.endEdit();
            });
        },
        //删除选中
        deleteSelected:function(){
            this.getChildren().forEach(function(item){
                item.deleteSelected();
            });
        },
        //删除全部
        deleteAll:function(){
            this.getChildren().forEach(function(item){
                item.deleteAll();
            });
        },
        //商品数量变化相关
        onMatChanged:function(){
            //this._onItemSelectChange();
            if(this._matChangeTimer){
                clearTimeout(this._matChangeTimer);
            }
            this._matChangeTimer = setTimeout(lang.hitch(this,function(){
                topic.publish("/cq/widget/item/cart/matChanged",this.getMatInfo());
            }),300);
        },
        //获取购物车中商品的信息
        getMatInfo:function(){
            var matInfo = {
                //已选中总金额
                selectedAmt:0,
                //已选中商品总数
                selectedNum:0,
                //已选中商品种数
                selectedTypes:0,
                //总金额
                totalAmt:0,
                //商品总数
                totalNum:0,
                //商品种数
                totalType:0
            };
            this.getChildren().forEach(function(item){
                var info = item.getMatInfo();
                matInfo.selectedAmt += (info.selectedAmt-0)||0;
                matInfo.selectedNum += (info.selectedNum-0)||0;
                matInfo.selectedTypes += (info.selectedTypes-0)||0;
                matInfo.totalAmt += (info.totalAmt-0)||0;
                matInfo.totalNum += (info.totalNum-0)||0;
                matInfo.totalType += (info.totalType-0)||0;
            });
            return matInfo;
        },
        
        //初始化购物车内容相关
        //是否在ItemRender准备好之后执行一次刷新操作
        _refreshOnRender:false,
        //执行刷新
        cqRefresh:function(){
            if(this._itemRenderReady){
                this.loadCartData().then(lang.hitch(this,function(date){
                    this.onDataReady(date);
                }));
            }else{
                this._refreshOnRender = true;
            }
        },
        //依据购物车的信息加载购物车数据，加工成本对象需要的结构
        //  按照商家分组的和不按照商家分组的，构造的数据及饿哦股不同
        loadCartData:function(){
            
        },
        //依据本对象需要的结构，刷新界面元素
        onDataReady:function(cartData){
            //销毁原有列表项
            this.getChildren().forEach(function(item){
                item.destroyRecursive();
            });
            //创建下级列表项
            var item = null;
            //对cartData做处理
            array.forEach(cartData,function(data){
                item = new this.ItemRender(this.renderOption);
                this.addChild(item);
                item.onDataReady(data);
            },this);
            this.endEdit();
            
            //刷新选中状态和购物车信息
            //处理全选
            this.onItemSelectChange();
            //处理选中商品信息
            this.onMatChanged();
        },
        destroyRecursive:function(){
            var node = this.domNode;
            this.inherited(arguments);
            if(node.parentNode){
                node.parentNode.removeChild(node);
            }
        }
    });
});