//购物车中一行商品的基类
define([
    "dojo/_base/declare",
    "dojo/dom-attr"
],function(declare,domAttr){
    return declare("",null,{
        //商品信息
        mid:"",
        logo:"",
        mName:"",
        //金额
        amt:0,
        //单价
        price:0,
        //数量
        qty:0,
        // 商品限购数量
        limitedQty: 0,
        disabled:false,
        //商品销售控制属性
        //最小起订量，小于等于零时不做控制
        minQty:0,
        //最大销售数量，通常是当前库存
        maxQty:999,
        //商品规格
        specs:{},
        //选中相关
        //全选状态
        selected:true,
        //全选全不选的checkbox节点
        selectNode:null,
        cart:null,
        //选中节点点击之后的响应
        onSelectClick:function(e){
            this.onSelectClicked(!this.selected);
            this.getParent().onItemSelectChange();
            
        },
        //点击全选或取消全选
        onSelectClicked:function(selected){
            this.setSelected(selected);
        },
        //选中
        setSelected:function(selected){
            this.selected = selected;
            domAttr.set(this.selectNode,"checked",selected);
            //处理购物车中选中状态
            //this.updateMatSelected();
            this.cart.updateMatSelected(this.mid,this.selected)
        },
        getSelected:function(){
            return this.selected;
        },
        getMatInfo:function(){
            return {
                //已选中总金额
                selectedAmt:this.selected?this.amt:0,
                //已选中商品总数
                selectedNum:this.selected?this.qty:0,
                //已选中商品种数
                selectedTypes:this.selected?1:0,
                //总金额
                totalAmt:this.amt,
                //商品总数
                totalNum:this.qty,
                //商品种数
                totalType:1,
                mid:this.mid,
                mName:this.mName
            };
        },
        //编辑相关 
        startEdit:function(){
        },
        endEdit:function(){
        },
        deleteSelected:function(){
            if(this.selected){
                this.deleteAll();
            }
        },
        
        //删除全部，删除按钮可以直接调用这个方法，无条件删除
        deleteAll:function(){
            //删除购物车中对应商品
            //this.deleteMatById();
            this.cart.deleteMid(this.mid);
            //找到父节点
            var parent = this.getParent();
            //销毁当前对象
            this.destroyRecursive();
            //调用父节点的商品发生变化的事件
            if(parent){
                parent.onMatChanged();
                //删除导致的需要关联上级选中问题
                parent.onItemSelectChange();
            }
        },
        
        //数量编辑相关
        //点击数量增加按钮的响应
        qtyIncClick:function(){
            if(!this.disabled){
                if(this.limitedQty>0){
                    if(this.limitedQty==this.qty){
                        this.qty = this.limitedQty
                        alert("该商品限购"+this.limitedQty+"个");
                        return;
                    }
                    //maxQty = Math.min(this.limitedQty, this.qty);
                }
                //this.minQty 是最大的库存
                if(this.maxQty>this.qty){
                    this.qty++;
                    //修改购物车中的商品数量，并刷新显示
                    this.onQtyChanged();
                }else{
                    this.qty = this.maxQty;
                    alert("库存不够啦");
                    return;
                    //是否给出提示，先不提示
                }
            }
        },
        //点击数量增加按钮的事件响应
        qtySubClick:function(){
            if(!this.disabled){
                if(this.minQty>0){
                    if(this.qty<=this.minQty){
                        if (!confirm("商品数量已经小于最小起订量，是否删除？")) return;
                        this.deleteAll();
                    }else{
                        this.qty--;
                        this.onQtyChanged();
                    }
                }else if(this.qty<=1){
                    if (!confirm("商品数量不能在减少了，是否删除？")) return;
                    this.deleteAll();
                }else{
                    //修改购物车中的商品数量，并刷新显示
                    //放在里面，因为deleteAll已经删除了直接节点和通知了上级节点
                    this.qty--;
                    this.onQtyChanged();
                }  
            }
        },
        //用户手工修改了商品数量——通过文本框输入
        qtyInputChange:function(newQty){
            if(!this.disabled){
                newQty = newQty.target.value-1;
                if(newQty){
                    //输入的必须是数字
                    if(newQty>=this.minQty&&newQty<=this.maxQty){
                        this.qty = newQty;
                    }
                }
                //修改购物车中的商品数量，并刷新显示
                this.onQtyChanged();
            }
        },
        onQtyChanged:function(){
            //修改购物车中商品信息
            //this.setMatQty();
            this.cart.setMatQty(this.mid,this.qty)
            //计算小计
            this.amt = this.qty*this.price;
            //修改数量和总金额的显示
            this.updateMatInfo();
            //若数量等于1或者等于最小起订量，禁止点击减少按钮
            this.setSubButtonDisabled(this.qty<=1||this.qty<=this.minQty);
            //若数量等于最大订货数量，禁止点击增加按钮
            this.setIncButtonDisabled(this.qty>=this.maxQty);
            //通知上级商品数量发生了变化
            if(this.getParent()){
                this.getParent().onMatChanged();
            }
        },
        //刷新商品的显示
        updateMatInfo:function(){},
        //启用或禁用减少按钮
        setSubButtonDisabled:function(disabled){},
        //启用或禁用增加按钮
        setIncButtonDisabled:function(disabled){},
        
        //依据数据刷新
        onDataReady:function(data,cart){
            this.cart = cart;
            this.price = data.retailPrice;
            if(Number(data.limitedQty,0)>0){
                this.qty = Math.min(data.qty,data.limitedQty);
            }else{
                this.qty = data.qty;
            }
            this.mid = data._id;
            this.mName = data.mName;
            this.minQty = data.minQty;
            this.maxQty = data.stockQty||0;
            this.logo = data.logo;
            this.limitedQty = data.limitedQty||0;
            this.specs = data.spec&&data.spec._id&&data.spec;
            //每次进入购物车设置为全不选状态
            data.selected = false;
            this.setSelected(!!data.selected);
            //记录感兴趣的信息
            this.onQtyChanged();
        },
        destroyRecursive:function(){
            var node = this.domNode;
            this.inherited(arguments);
            if(node.parentNode){
                node.parentNode.removeChild(node);
            }
        }
    });
});