//购物车中一个供货商的基类
define([
    "dojo/_base/declare",
    "dojo/_base/lang",
    "dojo/_base/array",
    "dojo/dom-attr"
],function(declare,lang,array,domAttr){
    return declare("",null,{
        ItemType:null,
        //商家店铺信息
        shopInfos:{},
        disabled:false,
        buildRendering:function(){
            this.inherited(arguments);
            this.initHeader();
        },
        initHeader:function(){
        },
        //选中相关
        //全选状态
        selected:true,
        //全选全不选的checkbox节点
        selectNode:null,
        //选中节点点击之后的响应
        onSelectClick:function(e){
            //选中自己
            this.onSelectClicked(!this.selected);
            //改变上级
            this.onItemSelectChange();
        },
        //点击全选或取消全选
        onSelectClicked:function(selected){
            this.setSelected(selected);
            //全选的时候看子节点是否全选
            this.getChildren().forEach(function(item){
                //子节点是disabled的话不允许设置
                //if(!this.getChildren().disabled){}
                if(item.disabled){
                    item.onSelectClicked(false);
                }else{
                    item.onSelectClicked(selected);
                }
            });
            //this.getParent().onItemSelectChange();
        },
        //设置全选状态
        setSelected:function(selected){
            if(this.getChildren().length&&this.getChildren()[0].disabled){
                domAttr.set(this.selectNode,"checked",false);
            }else if(this.selectNode){
                this.selected = selected;
                domAttr.set(this.selectNode,"checked",selected);
            }
            
        },
        //上级改变导致
        onItemSelectChange:function(){
            //修改。只查找item的disabled 为false 的子节点,隐藏的不计算
            //如果节点不存在
            if(this.domNode){
                this.setSelected(!array.some(this.getChildren(),function(item){
                    if(!item.disabled){
                        return !item.getSelected();
                    }
                }));
                var parent = this.getParent();
                if(parent){
                    parent.onItemSelectChange();
                }
            }
        },
        getSelected:function(){
            return this.selected;
        },
        
        
        //编辑相关接口
        //编辑和完成的处理
        //  这部分功能是可选的，网站版本的程序没有此项功能
        //  若启用编辑功能，子类需要做如下工作
        //      购物车的head区域要有编辑和完成按钮
        //      创建head区域时要真挺编辑和完成按钮的点击事件，调用startEdit和endEdit
        //      需要重写startEdit和endEdit方法，完成head区域状态的刷新
        //开始编辑
        startEdit:function(){
            this.getChildren().forEach(function(item){
                item.startEdit();
            });
        },
        //停止编辑
        endEdit:function(){
            this.getChildren().forEach(function(item){
                item.endEdit();
            });
        },
        //删除选中
        deleteSelected:function(){
            this.getChildren().forEach(function(item){
                item.deleteSelected();
            });
            this.onMatChanged();
        },
        //删除全部
        deleteAll:function(){
            this.getChildren().forEach(function(item){
                item.deleteAll();
            });
            this.onMatChanged();
        },
        
        
        //商品数量发生变化相关
        //商品数量变化相关
        onMatChanged:function(){
            var children = this.getChildren(),parent = this.domNode&&this.getParent();
            //若自己已经没有下级商品了，销毁自己
            if(!children.length&&this.domNode){
                this.destroyRecursive();
            }
            if(parent){
                //通知上级(购物车)商品发生了变化
                parent.onMatChanged();
                parent._onItemSelectChange();
            }
        },
        //获取购物车中商品的信息
        getMatInfo:function(){
            var matInfo = {
                //已选中总金额
                selectedAmt:0,
                //已选中商品总数
                selectedNum:0,
                //已选中商品种数
                selectedTypes:0,
                //总金额
                totalAmt:0,
                //商品总数
                totalNum:0,
                //商品种数
                totalType:0,
                //各商品对应的金额
                amount:{}
            };
            this.getChildren().forEach(function(item){
                var info = item.getMatInfo();
                matInfo.selectedAmt += (info.selectedAmt-0)||0;
                matInfo.selectedNum += (info.selectedNum-0)||0;
                matInfo.selectedTypes += (info.selectedTypes-0)||0;
                matInfo.totalAmt += (info.totalAmt-0)||0;
                matInfo.totalNum += (info.totalNum-0)||0;
                matInfo.totalType += (info.totalType-0)||0;
                matInfo.amount[info.mid] = {amount:info.totalAmt,mName:info.mName};
            });
            return matInfo;
        },
        
        //依据数据初始化
        onDataReady:function(shopMatData){
            array.forEach(shopMatData.mats,function(data){
                var item = new this.ItemType();
                //var cart = this.cart;
                this.addChild(item);
                //感觉这样传递cart不是很好！！
                item.onDataReady(data,this.cart);
            },this);
            //处理全选
            this.onItemSelectChange();
            //处理选中商品信息
            this.onMatChanged();
        },
        
        destroyRecursive:function(){
            var node = this.domNode;
            this.inherited(arguments);
            if(node.parentNode){
                node.parentNode.removeChild(node);
            }
        }
    });
});