// 客服对话中的消息列表
//      展示对话过程
//      处理查看更多历史消息
//      执行页面刷新
//      监听并发送新消息
// 需要处理的问题
//      依据参数刷新
//      拖动加载更多
//      新消息的显示
//      发送消息并显示
//      
define([
    "dojo/_base/declare",
    "dojo/_base/lang",
    "dojo/dom-class",
    "dojo/dom-style",
    "dojo/Deferred",
    "dojo/store/JsonRest",
    "dojox/mobile/EdgeToEdgeList",
    "cq/app/widget/_WidgetBase",
    "cq/widgets/item/TalkListItem",
    "cq/util/sys/date",
    "cq/app/widget/utils/widget",
    "cq/widgets/bigImg"
],function(declare,lang,domClass,domStyle,Deferred,JsonRest,EdgeToEdgeList,cqWidgetBase,TalkListItem,date,widget,bigImg){
    //具体呈现对话消息的列表，参考svcDetail中list的处理
    //消息列表区域
    return declare("",[EdgeToEdgeList,cqWidgetBase],{
        //属性
        htmlTemp:"<div data-dojo-props=\"userProvider:{userProvider},msgProvider:{msgProvider},msgEvtProvider:{msgEvtProvider},msgPProvider:{msgPProvider},autoDefaultReply:{autoDefaultReply}\" id='{id}' class='{className}' data-dojo-type='cq/widgets/item/svc/MsgList'></div>",
        //当前数据加载到了第几行
        start:0,
        //当前对话所在商家
        currentShopId:"",
        //当前用户的信息
        currentUserInfo:{},
        //目标用户id
        targetUserId:"",
        // 是否自动回复，默认为false
        autoDefaultReply:false,
        
        _loadProvider:function(){
            if(!this._providerDef){
                this._providerDef = new Deferred();
                require([
                    this.msgProvider||"bob/im/msg/c",
                    this.msgEvtProvider||"bob/im/msg/c",
                    this.msgPProvider||"bob/im/msgP/c",
                    this.userProvider||"bob/user/consumer"
                ],lang.hitch(this,function(msg,msgEvt,msgP,user){
                    this.msgProvider = msg;
                    this.msgEvtProvider = msgEvt;
                    this.msgPProvider = msgP;
                    this.userProvider = user;
                    this._providerDef.resolve();
                }));
            }
            return this._providerDef;
        },
        //拖动加载更多
        startup:function(){
            if(!this._bindScroll){
                this._bindScroll = true;
                this._loadProvider().then(lang.hitch(this,function(){
                    this.scrollablePane = this.getParent();
                    this.connect(this.msgEvtProvider,"onShowMsg",function(msgData){
                        var senderId = msgData.senderId;
                        senderId = senderId&&senderId._id||senderId;
                        //若接收人为自己，且发送人为对话目标人
                        if(senderId==this.targetUserId){
                            //设置消息为已读
                            msgData.isUse = true;
                            //添加列表项到列表的最后
                            this.addChild(this.getListItemByData(msgData));
                            this.scrollToBottom();
                        }
                    });
                    this.msgEvtProvider.init();
                    //绑定向上拖动加载更多事件
                    this.connect(this.scrollablePane,"onBeforeScroll",function(evt){
                        if(!this._isloading&&evt.beforeTop){
                            this.loadMsgData();
                        }
                    });
                    //绑定点击事件
                    this.connect(this.scrollablePane.containerNode,"onclick",this.onTalkListClick);
                    this.connect(widget.getEnclosingView(this),"onBeforeTransitionOut",function(){
                        if(this.targetUserId&&this.currentShopId){
                            this.msgPProvider.clearPenddingMsg(this.targetUserId,this.currentShopId);
                        }
                    });
                }));
            }
            this.inherited(arguments);
        },
        //加载更多
        loadMore:function(evt){
            if(!this._isloading&&evt.beforeTop){
                this.loadMsgData();
            }
        },
        //每次进入时刷新
        cqRefresh:function(){
            
            this._isloading = false;
            //销毁所有消息
            this.getChildren().forEach(function(item){
                item.destroyRecursive();
            });
            //依据cqViewParam读取对话咪表客户和对话所在商家
            this.getAllIds().then(lang.hitch(this,function(){
                this.msgPProvider.clearPenddingMsg(this.targetUserId,this.currentShopId);
                this._refreshStamp = true;
                this.loadMsgData().then(lang.hitch(this,this.scrollToBottom));
            }));
        },
        //依据cqViewParam读取对话列表客户和对话所在商家
        getAllIds:function(){
            return this._loadProvider().then(lang.hitch(this,function(){
                var ids = this.cqRefInfo.cqViewParam.params.split(',');
                //改动：支持不传入shopId，当不传入shopId时，获取当前商家Id
                this.currentShopId = ids[1]||this.msgProvider.getCurrentShopId();
                this.targetUserId = ids[0];//senderId
                this.totalCount = 0;
                return this.userProvider.getCurrentUser().then(lang.hitch(this,function(data){
                    this.currentUserInfo = data;
                }));
            }));
        },
        //加载更多
        loadMsgData:function(){
            if(this.totalCount&&this.totalCount<=this.domNode.children.length){
                return;
            }
            if(!this.store){
                this.store = new JsonRest();
            }
            this.store.target = this.msgProvider.getSvcMsgUrl();
            var queryResult = this.store.query({
                targetShopId: this.currentShopId,//targetShopId
                userId1: this.currentUserInfo._id,
                userId2: this.targetUserId,
            },{
                start:this.getChildren().length,
                count:20
            });
            this._isloading = true;
            queryResult.total.then(lang.hitch(this,function(total){
                this.totalCount = total||-1;
            }));
            return queryResult.then(lang.hitch(this,this._onDataReady),lang.hitch(this,function(){
                this._isloading = false;
            }));
        },
        
        _onDataReady:function(datas){
            this._isloading = false;
            if(datas&&datas.items){
                datas = datas.items;
            }
            for(var i=0;i<datas.length;i++){
                this.addChild(this.getListItemByData(datas[i]),0);
            }
            // 如果没有数据，或者最后一条数据距离当前时间大于5分钟，
            // 调用服务器端的 defaultReplyAs接口，targetShopId为本次对话的目标店铺，svcId为本地对话的目标人
            if(this.autoDefaultReply&&this._refreshStamp&&(0==datas.length||(date.now()-new Date(datas[0].createAt))>5*60*1000)){
                this._refreshStamp = false;
                this.msgProvider.getDefaultReply(this.currentShopId, this.targetUserId);
            }
        },
        
        //监听并发送消息
        observe:function(widget,field){
            this.inherited(arguments);
            //有新消息，就刷新列表，应该还resize吧
            if("msgBar"===field){
                this.connect(widget,"onSubmitMsg",this.onSubmitMsg);
            }
        },
        //发送消息
        onSubmitMsg:function(type,values){
            //获取发送消息类型
            var msgType = "text";
            if(type == "image"){
                msgType = "image";
            }
            //保存信息
            var data = {
                shopId: this.currentShopId,//targetShopId
                targetShopId:this.currentShopId,
                senderId: this.currentUserInfo._id,
                receiverId: this.targetUserId,
                msgType: msgType,
                msgContent: values
            };
            this.msgProvider.save(data);
            //刷新显示发送消息
            //显示刚刚发送的消息
            data.senderId = this.currentUserInfo;
            data.receiverId = {_id:this.targetUserId};
            this.addChild(this.getListItemByData(data));
            this.scrollToBottom();
        },
        //依据一行数据得到一个消息列表项
        getListItemByData:function(data){
            //处理数据并传入talkListItem zkj
            var dataMsg = data&&data.data&&data.data.item||data;
            dataMsg.isHost = (dataMsg.senderId&&dataMsg.senderId._id)===this.currentUserInfo._id;
            dataMsg.createAt = dataMsg.createAt||date.nowISOString();
            return new TalkListItem({
                params:dataMsg
            });
        },
        //滚动到底部
        scrollToBottom:function(){
            if(this.scrollTimer){
                clearTimeout(this.scrollTimer);
            }
            this.scrollTimer = setTimeout(lang.hitch(this,function(){
                this.scrollablePane.scrollTo({
                    x:0,
                    y:Math.min(0,domStyle.get(this.scrollablePane.domNode,"height")-domStyle.get(this.scrollablePane.containerNode,"height"))
                });
            }),300);
        },
        //列表点击事件，点图看大图
        onTalkListClick:function(evt){
            if(evt.target.localName == "img"){
                if(domClass.contains(evt.target.parentNode, "msgContent")){
                    bigImg.show(evt.target.src);
                } 
            }
        }
    });
});