// 创建人:赵春辉
// 创建日期:2016-06-03
// 用途:酉阳网站数据源支持分页的列表
define([
    "dojo/_base/declare",
    "dojo/_base/lang",
    "dojo/dom-attr",
    "dojo/dom-construct",
    "dojo/dom-style",
    "dojo/store/JsonRest",
    "dojox/dtl/_base",
    "../Template"
],function(declare, lang, domAttr, domConstruct, domStyle, JsonRest, dtl, Template) {
    return declare("cq.widgets.list.RestList", Template, {
        // 分页大小
        pageSize : 20,
        // 当前显示的是第几页
        currentPage : 0,
        // 读取数据的目标地址
        target : "",
        // 附加的查询条件
        query : "",
        // 启用分页
        enablePager : false,
        // 初始化界面元素
        buildRendering : function() {
            this.inherited(arguments);
            if (!this.containerNode) {
                this.containerNode = this.domNode;
            }
            if (!this.tempNode) {
                this.tempNode = this.domNode;
            }
            if (!this.pagerNode) {
                this.pagerNode = domConstruct.create("div", {
                    "class" : "pager_page",
                    onclick : lang.hitch(this, this.onPagerClick)
                });
            }
        },
        startup : function() {
            this.inherited(arguments);
            setTimeout(lang.hitch(this, function() {
                //控制不透明度
                domStyle.set(this.domNode, "opacity", 1);
            }), 100);
        },
        refreshList : function(page) {
            if (!this.store) {
                if (!this.target) {
                    console.error("no target rest list:" + this.get("id"));
                    return;
                }
                this.store = new JsonRest({target : this.target});
            }
            this.store.target = this.target||this.store.target;
            this.currentPage = page || 0;
            this.pageSize = this.pageSize > 0 ? this.pageSize-0 : 20;
            var queryResult = this.store.query(this.getQuery(), {
                start : this.currentPage * this.pageSize,
                count : this.pageSize
            });
            queryResult.then(lang.hitch(this, function (data) {
                this.onDataReady(data);
            }));
            queryResult.total.then(lang.hitch(this, this.onTotalLoaded));
            return queryResult;
        },
        getQuery : function() {
            if (this.query && "string" == typeof(this.query)) {
                try {
                    return eval("("+this.query+")");
                } catch (e) {
                    console.error(e);
                }
            }
            return this.query || {};
        },
        setQuery : function(query) {
            this.query = query;
            return this.refreshList(0);
        },
        onTotalLoaded : function(totalCount) {
            if (this.enablePager) {
                //已知信息
                //  totalCount总行数
                //  this.pageSize页行数
                //  this.currentPage当前页，从0开始
                
                
                var strHtml = '<span></span>',
                    prevPage = this.currentPage-1,
                    nextPage = this.currentPage+1,
                    pageCount = Math.floor(parseFloat(totalCount)/parseFloat(this.pageSize)),
                    pageControl = parseFloat(totalCount)%parseFloat(this.pageSize);
                
                if (prevPage < 0) {
                    strHtml += '<span><前一页</span>';
                } else {
                    strHtml += '<span class="pagePadding"><a href="javascript:void(0);" name="'+prevPage+'"><前一页</a></span>';
                }
                strHtml += '<span class="pagePadding"><a href="javascript:void(0);" name="0">1</a></span>';
                if (this.currentPage >= 5){
                    strHtml += '<span class="pagePadding">...</span>';
                }
                if (pageCount > this.currentPage + 2) {
                    var endPage = this.currentPage + 2;
                } else {
                    var endPage = pageCount;
                }
                for (var i = this.currentPage - 2; i <= endPage; i++) {
                    if (i > 0) {
                        if (i == this.currentPage) {
                            strHtml += '<span class="pagePadding currentPage">' + (i+1) + '</span>';
                        } else {
                            if (i != 0 && i != pageCount) {
                                strHtml += '<span class="pagePadding"><a href="javascript:void(0);" name="' + i + '">' + (i+1) + '</a></span>';
                            }
                        }
                    }
                }
                if (this.currentPage + 3 < pageCount){
                    strHtml += '<span class="pagePadding">...</span>';
                }
                if (this.currentPage != pageCount&&pageControl!==0){
                    strHtml += '<span class="pagePadding"><a href="javascript:void(0);" name="' + pageCount + '">' + (pageCount+1) + '</a></span>';
                }
                // 如果pageControl存在,说明总页数需要加一
                if (nextPage >= (pageControl ? pageCount+1 : pageCount)) {
                    strHtml += '<span class="pagePadding">后一页></span>';
                } else {
                    strHtml += '<span class="pagePadding"><a href="javascript:void(0);" name="' + nextPage + '">后一页></a></span>';
                }
                strHtml += ('</span> <span class="totalCount totalCountRecord">总' + totalCount + ' 条记录<span>');
                
                this.pagerNode.innerHTML = strHtml;
                this.containerNode.appendChild(this.pagerNode);
            }
        },
        onPagerClick : function(e) {
            if (e && e.target && -1 < domAttr.get(e.target, "name")) {
                var pageIndex = parseInt(domAttr.get(e.target, "name"));
                if (pageIndex > -1) {
                    this.refreshList(pageIndex);
                }
            }
        },
        onDefRendered : function(data) {},
        onDataReady : function(data) {
            if (!data.data) {
                data = {data : data};
            }
            if(data.data.item){
                data.data.items = [data.data.item];
            }
            this.items = data.data.items || [];
            this.data = data;
            this.onDefRendered(data);
            if (!this._template){
                this._template = new dtl.Template(this._templateString);
            }
            var context = new dtl._Context(this);
            this.beforeGetContent();
            this.tempNode.innerHTML =
            this.onListHtmlReady(this._template.render(context));
            dojo.parser.parse(this.tempNode);
            this.afterGetContent();
            //控制不透明度
            domStyle.set(this.domNode, "opacity", 1);
        },
        beforeGetContent : function() {
            if (this.pagerNode && this.pagerNode.parentNode) {
                this.pagerNode.parentNode.removeChild(this.pagerNode);
            }
            this.destroyDescendants(false);
            this.containerNode.innerHTML = "";
        },
        afterGetContent : function() {
            if (this.enablePager && this.pagerNode) {
                this.containerNode.appendChild(this.pagerNode);
            }
        }
    });
    return List2;
});