﻿define([
    "dojo/_base/declare",
    "dojo/_base/lang",
    "dojo/_base/array",
    'dojo/dom-construct',
    "dojo/Deferred",
    "dojo/number",
    "cq/app/widget/_WidgetBase",
    "dijit/_WidgetBase",
    "dijit/registry",
    "./echarts"
],function(declare,lang,array,domConstruct,Deferred,number,_CQWidgetBase,_WidgetBase,registry,echarts){
    var props = "'dataProvider:\"{dataProvider}\","+
        "dataMethod:\"{dataMethod}\",tooltip:\"{tooltip}\",dataOptions:\{j:dataOptions}\,"+
        "maxNum:\"{maxNum}\","+
        "title:\"{title}\",subTitle:\"{subTitle}\",seriesName:\"{seriesName}\",labelField:\"{labelField}\",valueFields:\"{valueFields}\",yField:\"{yField}\",chartType:\"{chartType}\",legend:\"{legend}\"'";
    return declare('', [_WidgetBase, _CQWidgetBase], {
        htmlTemp : '<div id="{id}" style="" class="cq-form-charts {className}" data-dojo-type="echarts/Chart" data-dojo-props=' + props + '></div>',
        
        className:'',
        dataProvider:'',
        dataOptions:{},
        dataMethod:'',
        yField:'',
        labelField:'',
        valueFields:'',
        tooltip:'',
        maxNum:10,
        chartType:'',
        legend:'',
        seriesName:'',
        title:'',
        subTitle:'',
        buildRendering:function(){
            this.inherited(arguments);
            this.maxNum = this.maxNum>=2?this.maxNum:10;
            this.cqRefresh();
        },
        cqRefresh:function(){
            if(this.canRefresh&&this.isFirshload){
                this.canRefresh = false;
                if(this.echart){
                    this.echart.dispose();
                }
                this._initChart();
            }
            if(!this.isFirshload){
                this.isFirshload = true;
                this._initChart();
            }
        },
        _initChart:function(){
            var layoutData = this.getLayoutData();
            if(layoutData){
                if(layoutData.then){
                    layoutData.then(lang.hitch(this, this._initChartView));
                }else{
                    this._initChartView(layoutData);
                }
            }
        },
        getLayoutData:function(){
            if(this.dataProvider&&this.dataMethod){
                if(lang.isString(this.dataProvider)){
                    var def = new Deferred();
                    require([this.dataProvider], lang.hitch(this, function(dataProvider){
                        this.dataProvider = dataProvider;
                        var data = this.dataProvider[this.dataMethod](this.cqRefInfo, this, this.dataOptions);
                        if(data&&data.then){
                            data.then(function(result){
                                def.resolve(result);
                            });
                        }else{
                            def.resolve(data);
                        }
                    }));
                    return def;
                }else{
                    return this.dataProvider[this.dataMethod](this.cqRefInfo, this, this.dataOptions);
                }
            }else{
                if(this.dataProvider){
                    require([this.dataProvider], lang.hitch(this, function(dataProvider){
                        this.dataProvider = dataProvider;
                    }));
                }
                return this.inherited(arguments);
            }
        },
        _initChartView:function(items){
            if(!this.echartNode){
                this.echartNode = domConstruct.create("div", {
                    'class':'echart',
                    'style':'width:100%;height:100%;'
                }, this.domNode);
            }
            var echart = this.echart = echarts.init(this.echartNode);
            
            this._initChartByType(items);
            this.canRefresh = true;
        },
        _initChartByType:function(items){
            switch(this.chartType){
                case "bar":this._initBarChart(items);break;
                case "pie":this._initPieChart(items);break;
                case "line":this._initLineChart(items);break;
                case "scatter":this._initScatterChart(items);break;
                default:break;
            }
        },
        _initScatterChart:function(items){
            if(lang.isString(this.valueFields)){
                this.valueFields = this.valueFields.split(",");
            }
            if(lang.isString(this.legend)){
                this.legend = this.legend.split(",");
            }
            var option = {
                title:{
                    text:this.title,
                    subtext:this.subTitle
                },
                xAxis: {
                    type:'value',
                    splitLine:{
                        lineStyle:{
                            type:'dashed'
                        }
                    }
                },
                yAxis: {
                    type:'value',
                    splitLine:{
                        lineStyle:{
                            type:'dashed'
                        }
                    }
                },
                tooltip:{
                    trigger: 'axis',
                    axisPointer: {
                        type: 'cross'
                    }
                },
                series:[]
            };
            if(this.valueFields.length >= 2){
                option.legend = {
                    data:this.legend
                };
            }
            for(var index = 0;index < items.length;index++){
                for(var j = 0;j < this.valueFields.length;j++){
                    if(!option.series[j]){
                        option.series[j] = {
                            type:this.chartType,
                            data:[],
                            name:this.legend&&this.legend[j]
                        }
                    }
                    option.series[j].data.push([items[index][this.labelField],items[index][this.valueFields[j]]]);
                }
            }
            this.echart.setOption(option);
        },
        _initBarChart:function(items){
            if(lang.isString(this.valueFields)){
                this.valueFields = this.valueFields.split(",");
            }
            if(lang.isString(this.legend)){
                this.legend = this.legend.split(",");
            }
            var option = {
                title:{
                    text:this.title,
                    subtext:this.subTitle
                },
                legend:{
                    data:this.legend
                },
                tooltip:{},
                series:[],
                xAxis:{
                    type:"category",
                    data:[]
                },
                yAxis:{
                    type:"value"
                }
            };
            for(var index = 0;index < items.length;index++){
                option.xAxis.data.push(items[index][this.labelField]);
                for(var j = 0;j < this.valueFields.length;j++){
                    if(!option.series[j]){
                        option.series[j] = {
                            name:this.legend&&this.legend[j]?this.legend[j]:"",
                            type:this.chartType,
                            data:[]
                        };
                    }
                    option.series[j].data.push(items[index][this.valueFields[j]]);
                }
            }
            this.echart.setOption(option);
        },
        _initPieChart:function(items){
            var option = {
                title:{
                    text:this.title,
                    subtext:this.subTitle
                },
                tooltip:{
                    trigger:'item',
                    formatter:"{a} <br/>{b} : {c} ({d}%)"
                },
                series:[{
                    name:this.seriesName,
                    type:this.chartType,
                    data:[],
                    center:['40%','50%'],
                    radius:'55%'
                }],
                legend:{
                    data:[],
                    right:10,
                    top:20,
                    bottom:20,
                    orient:'vertical'
                }
            };
            var pieData = lang.clone(items);
            pieData.sort(lang.hitch(this, function(a, b){
                return b[this.valueFields] - a[this.valueFields];
            }));
            for(var index = 0;index < pieData.length;index++){
                if(index >= (this.maxNum - 1)){
                    if(index == (this.maxNum - 1)){
                        option.legend.data.push("其它");
                        option.series[0].data.push({
                            name:"其它",
                            value:pieData[index][this.valueFields]
                        });
                    }else{
                        option.series[0].data[this.maxNum - 1].value += pieData[index][this.valueFields];
                    }
                }else{
                    option.legend.data.push(pieData[index][this.labelField]);
                    option.series[0].data.push({
                        value:pieData[index][this.valueFields],
                        name:pieData[index][this.labelField]
                    });
                }
            }
            this.echart.setOption(option);
        },
        _initLineChart:function(items){
            if(lang.isString(this.valueFields)){
                this.valueFields = this.valueFields.split(",");
            }
            if(lang.isString(this.legend)){
                this.legend = this.legend.split(",");
            }
            var option = {
                title:{
                    text:this.title,
                    subtext:this.subTitle
                },
                legend:{
                    data:this.legend
                },
                tooltip:{},
                series:[],
                xAxis:{
                    type:"category",
                    data:[]
                },
                yAxis:{
                    type:"value"
                }
            };
            for(var index = 0;index < items.length;index++){
                option.xAxis.data.push(items[index][this.labelField]);
                for(var j = 0;j < this.valueFields.length;j++){
                    if(!option.series[j]){
                        option.series[j] = {
                            name:this.legend&&this.legend[j]?this.legend[j]:"",
                            type:this.chartType,
                            data:[]
                        };
                    }
                    option.series[j].data.push(items[index][this.valueFields[j]]);
                }
            }
            this.echart.setOption(option);
        },
        resize:function(){
            if(this.echart){
                this.echart.resize();
            }
        }
    });
});