﻿define([
    "dojo/_base/declare",
    "dojo/_base/lang",
    "dojo/_base/array",
    "dojo/_base/xhr",
    "dojo/dom-construct",
    "dojo/dom-style",
    "dojo/dom-class",
    "dojo/Deferred",
    "dojo/number",
    "cq/app/widget/_WidgetBase",
    "dijit/_WidgetBase",
    "dijit/registry",
    "./echarts"
],function(declare,lang,array,xhr,domConstruct,domStyle,domClass,Deferred,number,_CQWidgetBase,_WidgetBase,registry,echarts){
    var props = "'dataProvider:\"{dataProvider}\","+
        "dataMethod:\"{dataMethod}\",dataOptions:\{j:dataOptions}\,'";
    return declare('', [_WidgetBase, _CQWidgetBase], {
        htmlTemp : '<div id="{id}" style="width:100%;" class="cq-form-progressbar {className}" data-dojo-type="echarts/ProgressBar" data-dojo-props=' + props + '></div>',
        
        className:'',
        dataProvider:'',
        dataOptions:{},
        // 数据获取方法
        dataMethod:'',
        
        buildRendering:function(){
            this.inherited(arguments);
            
            this.cqRefresh();
        },
        cqRefresh:function(){
            if(this.canRefresh&&this.isFirshload){
                this.canRefresh = false;
                if(this.echart){
                    this.echart.dispose();
                }
                this._initChart();
            }
            if(!this.isFirshload){
                this.isFirshload = true;
                this._initChart();
            }
        },
        _initChart:function(){
            var layoutData = this.getLayoutData();
            
            if(layoutData){
                if(layoutData.then){
                    layoutData.then(lang.hitch(this, this._initChartView));
                }else{
                    this._initChartView(layoutData);
                }
            }
        },
        getLayoutData:function(){
            if(this.dataProvider&&this.dataMethod){
                if(lang.isString(this.dataProvider)){
                    var def = new Deferred();
                    require([this.dataProvider], lang.hitch(this, function(dataProvider){
                        this.dataProvider = dataProvider;
                        var data = this.dataProvider[this.dataMethod](this.cqRefInfo, this, this.dataOptions);
                        if(data&&data.then){
                            data.then(function(result){
                                def.resolve(result);
                            });
                        }else{
                            def.resolve(data);
                        }
                    }));
                    return def;
                }else{
                    return this.dataProvider[this.dataMethod](this.cqRefInfo, this, this.dataOptions);
                }
            }else{
                if(this.dataProvider){
                    require([this.dataProvider], lang.hitch(this, function(dataProvider){
                        this.dataProvider = dataProvider;
                    }));
                }
                return this.inherited(arguments);
            }
        },
        _initChartView:function(items){
            if(!this.echartNode){
                this.echartNode = domConstruct.create("div", {
                    'class':'progressbar',
                    'style':'width:100%;height:100%;'
                }, this.domNode);
            }
            domStyle.set(this.domNode, {
                height:items.length * 56 + 'px'
            });
            var echart = this.echart = echarts.init(this.echartNode);
            
            var series = [{
                    type:'bar',
                    id:"series1",
                    stack:'stack0',
                    label:{
                        show:true
                    },
                    data:[],
                    barWidth:12
                },{
                    type:'bar',
                    stack:'stack1',
                    id:"series2",
                    data:[],
                    label:{
                        show:true,
                        formatter:function(params){
                            console.log(params);
                            if(params.data.label.color == "orange"){
                                return '-' + params.value;
                            }else if(params.value == 0){
                                return 0;
                            }else{
                                return '+' + params.value;
                            }
                        }
                    },
                    barWidth:12
                },{
                    type:'bar',
                    stack:'stack2',
                    id:"series3",
                    data:[],
                    label:{
                        show:true
                    },
                    barWidth:12
                },{
                    type:'bar',
                    stack:'stack2',
                    id:"series4",
                    data:[],
                    label:{
                        show:true,
                        formatter:function(params){
                            console.log(params.value);
                            if(params.value == 0){
                                return "";
                            }
                        },
                        position:'insideLeft'
                    },
                    barGap:0,
                    barCategoryGap:36,
                    barWidth:12
                },{
                    type:'scatter',
                    stack:'stack3',
                    id:"series5",
                    symbol:'rect',
                    z:20,
                    itemStyle:{
                        color:'black'
                    },
                    symbolSize:[2,12],
                    symbolOffset:[0, '100%'],
                    bottom:0,
                    data:[],
                    animation:false
                }];
            var cats = [];
            var max = 0;
            this.datas = {};
            for(var index = 0;index < items.length;index++){
                var item = items[index];
                var diff = item.sale-item.total;
                this.datas[item.name] = {
                    total:item.total,
                    sale:item.sale,
                    salePer:(item.sale/item.total*100).toFixed(2) + "%",
                    diff:diff>0?("+" + diff):diff,
                    diffPer:(diff>0?"+":"")+(diff/item.total*100).toFixed(2) + "%",
                    saleShowType:"num",
                    diffShowType:"num",
                    totalShowType:"num"
                };
                if(!max){
                    max = item.total>item.sale?item.total:item.sale;
                }else{
                    max = Math.max(max, item.total>item.sale?item.total:item.sale);
                }
                cats.push(item.name);
                series[0].data.push({
                    value:item.sale,
                    label:{
                        color:(item.sale>=item.total)?'blue':'orange'
                    },
                    itemStyle:{
                        color:(item.sale>=item.total)?'green':'red'
                    }
                });
                series[1].data.push({
                    // value:Math.abs(item.sale - item.total),
                    label:{
                        color:(item.sale>=item.total)?'blue':'orange'
                    },
                    itemStyle:{
                        color:'white'
                    }
                });
                series[2].data.push({
                    value:(item.sale >= item.total)?item.total:item.sale,
                    label:{
                        color:'blue'
                    },
                    itemStyle:{
                        color:'green'
                    }
                });
                series[3].data.push({
                    value:Math.abs(item.sale - item.total),
                    label:{
                        color:(item.sale>=item.total)?'blue':'orange'
                    },
                    itemStyle:{
                        color:(item.sale>=item.total)?'green':'red'
                    }
                });
                series[4].data.push(item.total);
            }
            for(var index = 0;index < items.length;index++){
                series[1].data[index].value = max;
            }
            series[0].label.formatter = lang.hitch(this, function(params){
                if(this.datas[params.name].saleShowType == "num"){
                    return this.datas[params.name].sale;
                }else{
                    return this.datas[params.name].salePer;
                }
            });
            series[1].label.formatter = lang.hitch(this, function(params){
                if(this.datas[params.name].diffShowType == "num"){
                    return this.datas[params.name].diff;
                }else{
                    return this.datas[params.name].diffPer;
                }
            });
            series[2].label.formatter = lang.hitch(this, function(params){
                if(this.datas[params.name].totalShowType == "num"){
                    return this.datas[params.name].total>this.datas[params.name].sale?this.datas[params.name].sale:this.datas[params.name].total;
                }else{
                    return this.datas[params.name].total>this.datas[params.name].sale?this.datas[params.name].salePer:"100%";
                }
            });
            series[3].label.formatter = lang.hitch(this, function(params){
                if(this.datas[params.name].totalShowType == "num"){
                    if(this.datas[params.name].diff!=0){
                        return Math.abs(this.datas[params.name].diff);
                    }else{
                        return "";
                    }
                }else{
                    if(this.datas[params.name].diff != 0){
                        return Math.abs(this.datas[params.name].diff/this.datas[params.name].total * 100).toFixed(2) + "%";
                    }else{
                        return "";
                    }
                }
            });
            var options = this.options = {
                title:{},
                tooltip:{
                    show:false
                },
                grid:{
                    left:12,
                    right:12,
                    top:10,
                    bottom:0
                },
                backgroundColor:'white',
                xAxis:{
                    type:'value',
                    show:false,
                    axisLine:{
                        show:false
                    },
                    axisTick:{
                        show:false
                    },
                    axisLabel:{
                        show:false
                    },
                    splitLine:{
                        show:false
                    }
                },
                yAxis:{
                    type:'category',
                    data:cats,
                    axisLine:{
                        show:false
                    },
                    axisTick:{
                        show:false
                    },
                    splitLine:{
                        show:false
                    },
                    axisLabel:{
                        align:'left',
                        verticalAlign:'bottom',
                        margin:-5,
                        padding:[0, 0, 22, 0],
                        interval:0
                    }
                },
                series:series
            };
            
            this.echart.setOption(options);
            
            this._initEvent();
            this.canRefresh = true;
        },
        _initEvent:function(){
            this.echart.on('click', lang.hitch(this, function(params){
                console.log(params);
                var options = lang.clone(this.options);
                if(params.seriesId&&params.seriesId == "series1"){
                    console.log("点击完成柱子");
                    this.datas[params.name].saleShowType = this.datas[params.name].saleShowType=="num"?"per":"num";
                }else if(params.seriesId&&params.seriesId == "series2"){
                    console.log("点击完成数字或比例");
                    this.datas[params.name].diffShowType = this.datas[params.name].diffShowType=="num"?"per":"num";
                }else if(params.seriesId&&(params.seriesId == "series3"||params.seriesId == "series4")){
                    console.log("点击完成进度");
                    this.datas[params.name].totalShowType = this.datas[params.name].totalShowType=="num"?"per":"num";
                }
                this.echart.resize();
            }));
        },
        resize:function(){
            
            // 获取根节点
            /*var parent = this.domNode.parentNode;
            while(!domClass.contains(parent, "mblView")){
                parent = parent.parentNode;
            }
            var bNode = parent.childNodes[0].childNodes[0];
            var bNodeHeight = domStyle.get(bNode, "height");
            
            domStyle.set(this.domNode, {
                'min-height':domStyle.get(parent, 'height') - bNodeHeight + "px"
            });
            */
            if(this.echart)
                this.echart.resize();
        }
    });
});